# -*- coding: utf-8 -*- 

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import current
from gluon.serializers import json

from menuhandler import MenuHandler

@restful()
@auth.rest_login()
def user():
    def GET(*args, **vars):
        '''
        URL: http://url_to_cais/api/user/user_identifier
        Method: GET
        Authorization: Needs HTTP Basic Authorization
        HTTP-Accept: application/json
        Parameters: 
           + user_identifier: email
        Returns:
           + 200: OK [+data]
             - identifier
             - first_name
             - last_name
             - email
           + 400: Bad Request
           + 401: Unauthorized
           + 404: Not Found
        '''
        # Needed 1 argument
        if len(args) != 1:
            raise HTTP(400)
        # Content-Type control
        if request.extension not in _config.get('rest_type_allowed'):
            raise HTTP(400)            
        
        # Get data
        from users import Users
        users = Users(_config, db)
        user = users.get(args[0])
        
        # Response
        if user is None:
            raise HTTP(404)
        else:
            return user
        
    def POST(*args, **vars): 
        '''
        URL: http://url_to_cais/api/user
        Method: POST
        Authorization: Needs HTTP Basic Authorization
        HTTP-Accept: application/json
        Parameters: None
        Data: JSON format
           + first_name
           + last_name
           + email
           + password [plain password]
        Returns:
           + 200: OK [+data]
             - identifier
             - first_name
             - last_name
             - email
           + 400: Bad Request
           + 401: Unauthorized
           + 404: Not Found
        '''
        # Needed None argument
        if len(args) != 0:
            raise HTTP(400) 
        
        data = json.loads(request.body.read())
        
        # Set data
        from users import Users
        users = Users(_config, db)
        added = users.add(data['first_name'],
                          data['last_name'],
                          data['email'],
                          data['password']) 
        
        # Get data
        if added:
            user = users.get(data['email'])
        
            # Response
            if user is None:
                raise HTTP(404)
            else:
                return user
        else:
            raise HTTP(404)
        
    def PUT(*args, **vars): 
        '''
        URL: http://url_to_cais/api/user/user_identifier
        Method: POST
        Authorization: Needs HTTP Basic Authorization
        HTTP-Accept: application/json
        Parameters: 
           + user_identifier: email
        Data: JSON format
           + first_name
           + last_name
           + email
           + password [plain password]
        Returns:
           + 200: OK [+data]
             - identifier
             - first_name
             - last_name
             - email
           + 400: Bad Request
           + 401: Unauthorized
           + 404: Not Found
        '''
        # Needed None argument
        if len(args) != 1:
            raise HTTP(400) 
        
        data = json.loads(request.body.read())
        
        # Set data
        from users import Users
        users = Users(_config, db)
        updated = users.update(args[0], data) 
        
        # Get data
        if updated:
            user = users.get(args[0])
        
            # Response
            if user is None:
                raise HTTP(404)
            else:
                return user
        else:
            raise HTTP(404)
        
    def DELETE(*args, **vars): 
        '''
        URL: http://url_to_cais/api/user/user_identifier
        Method: DELETE
        Authorization: Needs HTTP Basic Authorization
        HTTP-Accept: application/json
        Parameters: 
           + user_identifier: email
        Returns:
           + 204: OK
           + 400: Bad Request
           + 401: Unauthorized
           + 404: Not Found
        '''
        # Needed None argument
        if len(args) != 1:
            raise HTTP(400) 
        
        # Delete
        from users import Users
        users = Users(_config, db)
        deleted = users.delete(args[0]) 
        
        # Get data
        if deleted:
            raise HTTP(204)            
        else:
            raise HTTP(404)
            
        
    return locals()

@request.restful()
@auth.requires_login()  
def menu():
    def GET(*args, **vars):        
        # Get data
        mn = MenuHandler()
        
        current.response.headers['Content-Type'] = 'application/json'
        return XML(json(mn.get_menu_rest()))          
        
    return locals()
        
#    def POST(*args, **vars): 
#        '''
#        URL: http://url_to_cais/api/user
#        Method: POST
#        Authorization: Needs HTTP Basic Authorization
#        HTTP-Accept: application/json
#        Parameters: None
#        Data: JSON format
#           + first_name
#           + last_name
#           + email
#           + password [plain password]
#        Returns:
#           + 200: OK [+data]
#             - identifier
#             - first_name
#             - last_name
#             - email
#           + 400: Bad Request
#           + 401: Unauthorized
#           + 404: Not Found
#        '''
#        # Needed None argument
#        if len(args) != 0:
#            raise HTTP(400) 
#        
#        data = json.loads(request.body.read())
#        
#        # Set data
#        from users import Users
#        users = Users(_config, db)
#        added = users.add(data['first_name'],
#                          data['last_name'],
#                          data['email'],
#                          data['password']) 
#        
#        # Get data
#        if added:
#            user = users.get(data['email'])
#        
#            # Response
#            if user is None:
#                raise HTTP(404)
#            else:
#                return user
#        else:
#            raise HTTP(404)
#        
#    def PUT(*args, **vars): 
#        '''
#        URL: http://url_to_cais/api/user/user_identifier
#        Method: POST
#        Authorization: Needs HTTP Basic Authorization
#        HTTP-Accept: application/json
#        Parameters: 
#           + user_identifier: email
#        Data: JSON format
#           + first_name
#           + last_name
#           + email
#           + password [plain password]
#        Returns:
#           + 200: OK [+data]
#             - identifier
#             - first_name
#             - last_name
#             - email
#           + 400: Bad Request
#           + 401: Unauthorized
#           + 404: Not Found
#        '''
#        # Needed None argument
#        if len(args) != 1:
#            raise HTTP(400) 
#        
#        data = json.loads(request.body.read())
#        
#        # Set data
#        from users import Users
#        users = Users(_config, db)
#        updated = users.update(args[0], data) 
#        
#        # Get data
#        if updated:
#            user = users.get(args[0])
#        
#            # Response
#            if user is None:
#                raise HTTP(404)
#            else:
#                return user
#        else:
#            raise HTTP(404)
#        
#    def DELETE(*args, **vars): 
#        '''
#        URL: http://url_to_cais/api/user/user_identifier
#        Method: DELETE
#        Authorization: Needs HTTP Basic Authorization
#        HTTP-Accept: application/json
#        Parameters: 
#           + user_identifier: email
#        Returns:
#           + 204: OK
#           + 400: Bad Request
#           + 401: Unauthorized
#           + 404: Not Found
#        '''
#        # Needed None argument
#        if len(args) != 1:
#            raise HTTP(400) 
#        
#        # Delete
#        from users import Users
#        users = Users(_config, db)
#        deleted = users.delete(args[0]) 
#        
#        # Get data
#        if deleted:
#            raise HTTP(204)            
#        else:
#            raise HTTP(404)
#            
#        
#    return locals()