# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from calendar import CalendarHandler

@auth.requires_login()
def index():
    from mil.tools import get_request_args
    [type, name, params] = get_request_args(2)
    
    _menu.show()
    active = False
    # Validation
    if type == 'project':
        from project import ProjectHandler       
        ProjectHandler.user_set_project(name)
        if not _government.has_permission('calendar', 'calendar_access', application='project'):
            response.flash = T('You have not permission')
            active = False
        else:    
            # Check type
            from core import SubApp
            SubApp.check_type(type, name) 
            active = True
    
    return dict(css_classes = CalendarHandler.get_css_classes(),
                type=type,
                active=active)

@auth.requires_login()
@service.json
def getcalendardata():
    import time       
    from datetime import datetime
    from calendar.models.event import EventModel
    insKlass = EventModel(current.db)
    rows = insKlass.get_events_from_range(current._user.id, 
                                          current.request.vars.start, 
                                          current.request.vars.end)
    data = []
    for row in rows:
        start = time.mktime(datetime(year = row.event.init_date.year, 
                                     month = row.event.init_date.month, 
                                     day = row.event.init_date.day, 
                                     hour = row.event.init_date.hour, 
                                     minute = row.event.init_date.minute, 
                                     second=0).timetuple())
        end = time.mktime(datetime(year = row.event.end_date.year, 
                                   month = row.event.end_date.month, 
                                   day = row.event.end_date.day, 
                                   hour = row.event.end_date.hour, 
                                   minute = row.event.end_date.minute, 
                                   second=0).timetuple())
        data += [{"title": row.event.title, 
                  "id": row.event.id, 
                  "start": start, 
                  "end": end, 
                  "allDay": row.event.all_day, 
                  "className": row.user_calendar.fullcalendar_class, 
                  "url": str(URL(current.request.application, 
                                 'calendar',
                                 "event",
                                 args=['formread', str(row.event.id)],
                                 extension='html'))}]
    return dict(events=data)

@auth.requires_login()
def modifyevent():
    #init response.gritter
    if not response.gritter:
        response.gritter = []


    #get event
    event = get_event(request.vars.event)

    #check perm
    if can_modify(event):
        #modify

        _modify_event(event, request.vars.dayDelta, request.vars.minuteDelta, request.vars.allDay, request.vars.resizeOrDrop)
        response.gritter += [{"title": T('Calendar'), "text": "Evento modificado correctamente", "sticky": False}]
    else:
        #not modify
        response.gritter += [{"title": T('Calendar'), "text": "No tienes permiso para modificar este evento", "sticky": False}]

    return dict()

@auth.requires_login()
def management():
    from calendar import CalendarManagement
    iklass = CalendarManagement()   
    return iklass()

@auth.requires_login()
def user():
    from calendar import UserCalendarManagement
    iklass = UserCalendarManagement()   
    return iklass()

@auth.requires_login()
def event():
    from calendar import CalendarEvent
    iklass = CalendarEvent()   
    return iklass()

@auth.requires_login()
def reminder():
    from calendar import CalendarReminder
    iklass = CalendarReminder()   
    return iklass()