# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon.storage import Storage
from gluon.serializers import json

from masters.languagehandler import LanguageHandler
from masters.visualthemehandler import VisualThemeHandler

@_menu.showme()
def index():                   
     # Language
    try:
        preferences=_user.preferences
        user_lang=preferences['lang']
    except:
        preferences={}
        user_lang=''
    langHnd = LanguageHandler(db)
    languages = langHnd.get_all_as_dict()
       
    element_options=[[TR(TD(SPAN(languages[elem])),TD(INPUT(_type='radio',_name='language',_value=elem,value=user_lang)))] for elem in languages.keys()]
    form = FORM(DIV(TABLE(
                          TR(TD(H3(T('Select language')))),
                          element_options,
                          TR(TD(INPUT(_type="submit",_name="submit",_value=T('Change'))), TD(''))),_class="formfield"))
    if request.vars.submit:
            preferences = _user.preferences
            if not "lang" in preferences.keys() or request.vars.language != preferences['lang']:
                preferences = _user.preferences
                preferences['lang'] = request.vars.language  
                _user.set_preferences(preferences)
                _user.set_language()
            
            session.flash = T('Configuration updated')
            redirect(URL(r=request,c='default',f='index'))
    
    # Password   
    form_p = FORM(DIV(TABLE(
                          TR(TD(H3(T('Change password')))),
                          TR(TD(SPAN(T('New password')),TD(INPUT(_type="password",_value='',_name="pass_1",requires=IS_NOT_EMPTY())))),
                          TR(TD(SPAN(T('Confirm password')),TD(INPUT(_type="password",_value='',_name="pass_2",requires= IS_EQUAL_TO(request.vars.pass_1,error_message=T('Passwords do not match')))))),
                          TR(TD(INPUT(_type="submit",_name="submit_p",_value=T('Change'))), TD(''))),_class="formfield"))
  
    if request.vars.submit_p:
        if request.vars.pass_1 == request.vars.pass_2:
            # Prepare data
            data=Storage()
            data.email=_user.email
            data.password = request.vars.pass_1
            # Update password
            from masters.users import Users
            users = Users(_config, db)
            users.update(_user.email, data)

            session.flash = T('Password changed')
            redirect(URL(r=request,c='desktop',f='index'))
        elif form_p.errors:
            response.flash = T('Password strings must be equal')            
                
    return dict(lang_form=form,pass_form=form_p)

@_menu.showme()
def platform():
    # Checks update form
    if request.vars.submit_plt:
        _config.set('title',request.vars.title)
        _config.set('subtitle',request.vars.subtitle)
        try:
            if request.vars.mylogo.file:
                ext = request.vars.mylogo.filename.split(".")[1] or 'png'
                import shutil
                shutil.copyfileobj(request.vars.mylogo.file,
                                   open(os.path.join(request.folder,
                                                     'static',
                                                     'mylogo.%s' % ext),
                                        'wb'))
                _config.set('logo', URL(r=request,
                                        c='static', 
                                        f='mylogo.%s' % ext))
        except Exception, e:
            pass     
        try:   
            if request.vars.myfavicon.file:
                ext = request.vars.myfavicon.filename.split(".")[1] or 'ico'
                import shutil
                shutil.copyfileobj(request.vars.myfavicon.file,
                                   open(os.path.join(request.folder,
                                                     'static',
                                                     'myfavicon.%s' % ext),
                                        'wb'))
                _config.set('favicon', URL(r=request,
                                           c='static', 
                                           f='myfavicon.%s' % ext))      
        except Exception, e:
            pass     
        
        if request.vars.language == '':
            language = 'en-en'
        else:
            language = request.vars.language
        _config.set('language',language)
        if request.vars.visualtheme:
            _config.set('visual_style',request.vars.visualtheme)
        
        
        if request.vars.status == 'private':
            _config.set('forge_private', True)
        else:
            _config.set('forge_private', False)
            
        if request.vars.gbook == 'active':
            _config.set('force_guestbook', True)
        else:
            _config.set('force_guestbook', False)
            
        if request.vars.autoregister == 'active':
            _config.set('auto_register', True)
        else:
            _config.set('auto_register', False)
        
        # Activate changes: title, logo, language and visual theme
        response.title = _config.get('title')
        response.logo =  _config.get('logo')
        response.favicon =  _config.get('favicon')
        response.language = _config.get('language')
        response.visual_style = _config.get('visual_style') or 'base'
    
        response.flash = T('Update done!')
          
          
    # Default values
    langHnd = LanguageHandler(db)
    vtHnd = VisualThemeHandler(db)
    register = _config.get('register')
    default = {
        'title': _config.get('title') or response.title,
        'subtitle':_config.get('subtitle') or response.subtitle,
        'logo': _config.get('logo') or URL(r=request,c='static',f='images/omw_logo_30h.png'),
        'favicon': _config.get('favicon') or URL(r=request,c='static',f='images/favicon.ico'),
        'status': _config.get('forge_private') or False,
        'gbook': _config.get('force_guestbook') or False,
        'languages': langHnd.get_all_as_dict(),
        'vthemes': vtHnd.get_all_as_dict(),
        'autoregister': _config.get('auto_register') or False,
    }
    
    form = SQLFORM.factory()
    
    return dict(form=form, default=default, platform_language = _config.get("language") )
