# -*- coding: utf-8 -*- 

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import current, IS_IN_SET, redirect, URL
from gluon.storage import Storage
from gluon.sql import SQLField
from gluon.sqlhtml import form_factory

from mil.tools import get_request_args   

@auth.requires_login()
@auth.requires_membership('admin')
def index():
    redirect(URL(c='government', f='services'))

@auth.requires_login()
@auth.requires_membership('admin')
def services():
    _menu.show()
    
    # Get administration services    
    from services import ServiceManager
    smng = ServiceManager()
    service_list=[]
    for item in smng.services.keys():
        service = smng.services[item]
        if service is not None:
            tmp = Storage()
            tmp.id = item
            try:
                tmp.name = service['literal']
            except:
                tmp.name = '...'
                
            service_list.append(tmp)
            
    if current._user.is_admin:
        from core.governhandler import GovernmentHandler
        govern_files = GovernmentHandler.get_government_files()
    else:
        govern_files = None
   
    return dict(services=service_list, 
                selected=[],
                govern_files=govern_files)

@auth.requires_login()
@auth.requires_membership('admin')
def profiles():
    _menu.show()
       
    [id, params] = get_request_args(1)

    from masters.profiles import Profiles
    
    insKlss = Profiles(_config, db)
    all_data = insKlss.get_all_by_subapp(id)
    
    data = []
    for profile in all_data:
        data.append(Storage(id=profile.auth_group.id,
                            is_structural=profile.auth_profile.is_structural,
                            description=profile.auth_group.description))

    return dict(id=id, data=data)

@auth.requires_login()
@auth.requires_membership('admin')
def profile():       
    [app_id, type, id, params] = get_request_args(3)   
     
    _menu.show()   
    
    from core.profilehandler import ProfileHandler     
    
    if type == 'configure':
        # Profile data
        from masters.profiles import Profiles
        insKlss = Profiles(_config, db)
        profile_data = insKlss.get_by_id(id)
        
        insKlss = Profiles(_config, db)
        profile_data = insKlss.get_by_id(id)
        
        data = ProfileHandler.configure(db, app_id, id, profile_data) 
        return dict(id=app_id, 
                    action=type, 
                    profile=profile_data, 
                    data_list=data)
    elif type == 'edit':
        # Profile data
        from masters.profiles import Profiles
        insKlss = Profiles(_config, db)
        profile_data = insKlss.get_by_id(id)
        
        form = ProfileHandler.edit(db, app_id, id, profile_data)
        return dict(id=app_id, 
                    action=type,
                    profile=profile_data, 
                    form=form)
    elif type == 'delete':
        ProfileHandler.delete(_config, db, app_id, id)
        session.flash = current.T('Role deleted')
        redirect(URL(c='government', f='profiles', args=[app_id]))
    else:
        return dict(id=app_id, 
                    action=type)

@auth.requires_login()
@auth.requires_membership('admin')
def new_profile():
    _menu.show()
       
    [id, params] = get_request_args(1)
    
    form = form_factory(SQLField('profile', length=512, default='',
                                 label=T('Role')),
                        SQLField('description', 'text',
                                 label=T('Description')))
    
    if form.accepts(request.vars, session):
        # Check args
        profile = request.vars.profile
        description = request.vars.description
        if profile:
            try:          
                # Add web2py group
                gp_id = auth.add_group(profile, description)
                
                # Add profile to subapplication
                if id:
                    from masters.profiles import Profiles
                    inKlass = Profiles(_config, db)
                    row_id = inKlass.add_profile(id, gp_id)
                    
                    redirect(URL(c='government', f='profiles', args=[id]))
            except Exception, e:
                pass            
        else:
            response.flash = T('You must assign a role')

    return dict(id=id, form=form)

@auth.requires_login()
@auth.requires_membership('admin')
def subapplication():   
    _menu.show()
       
    [id, params] = get_request_args(1)
    
    from services import ServiceManager
    smng = ServiceManager()
    
    from system import AuthConfiguration
    auth_mng = AuthConfiguration(db)
    data = None
    
    if len(request.vars) > 0:
        data = smng.services[id]
        for action in data['configuration']:
            if action['id'] in request.vars:
                auth_mng.add(id, 
                             id, 
                             action['id'], 
                             request.vars[action['id']]) # Two ids

    
    data = None
    if id in smng.services.keys():
        all_data = smng.services[id] 
        data = {}
        data['controller'] = all_data['controller']
        data['literal'] = all_data['literal']
        data['permissions'] = all_data['permissions']
        data['configuration'] = []
        for action in all_data['configuration']:
            d_action = {}
            d_action['id'] = action['id']
            d_action['label'] = action['label'] 
            d_action['options'] = []
            value = auth_mng.get_value(id, id, action['id']) # Two ids
            for option in action['options']:
                d_option = option 
                d_option['checked'] = False
                if value is None:
                    if option['default']:
                        d_option['checked'] = True
                else:
                    if option['id'] == value:
                        d_option['checked'] = True
                d_action['options'].append(d_option)
            data['configuration'].append(d_action)
        data['services'] = []
        # TODO: prepare services configuration
#        for service in all_data['services'].keys():
#            tmp = {'label': all_data['services'][service]['label'],
#                   'control': all_data['services'][service]['control']}
#            data['services'] .append(tmp)
            
    return dict(id=id, data=data)


@auth.requires_login()
@auth.requires_membership('admin')
def configure():      
    [app_id, service_id, params] = get_request_args(2)
    
    from services import ServiceManager
    smng = ServiceManager()
    
    from system import AuthConfiguration
    auth_mng = AuthConfiguration(db)
    data = None
    
    if app_id in smng.services.keys():
        app_data = smng.services[app_id] 
        
        if len(request.vars) > 0:
            data = app_data['services'][service_id] 
            for action in data['configuration']:
                if action['id'] in request.vars:
                    # Radio button
                    if action['type'] == 'only_one':
                        auth_mng.add(app_id, 
                                     service_id, 
                                     action['id'], 
                                     request.vars[action['id']])
                    # Check button
                    elif action['type'] == 'multi_value':
                        if not isinstance(request.vars[action['id']], list):
                            list_items = [request.vars[action['id']]]
                        else:
                            list_items = request.vars[action['id']]
                        for item in list_items:
                            auth_mng.add(app_id, 
                                         service_id, 
                                         action['id'], 
                                         item)
        
        data = None
        if service_id in app_data['services'].keys():
            all_data = app_data['services'][service_id] 
            data = {}
            data['literal'] = all_data['label']
            data['configuration'] = []
            try:
                for action in all_data['configuration']:
                    d_action = {}
                    d_action['id'] = action['id']
                    d_action['label'] = action['label'] 
                    d_action['type'] = action['type'] 
                    d_action['options'] = []
                    
                    # Radio button
                    if action['type'] == 'only_one':
                        value = auth_mng.get_value(app_id, 
                                                   service_id, 
                                                   action['id'])
                        
                    for option in action['options']:
                        # Check button
                        if action['type'] == 'multi_value':
                            value = auth_mng.get_value(app_id, 
                                                       service_id, 
                                                       action['id'],
                                                       option['id'])
                        d_option = option 
                        d_option['checked'] = False
                        d_option['disabled'] = False
                        if value is None:
                            if option['default'] == 'disabled':
                                d_option['disabled'] = True
                            else:
                                if option['default']:
                                    d_option['checked'] = True
                        else:
                            if option['id'] == value:
                                d_option['checked'] = True
                        d_action['options'].append(d_option)
                        
                    data['configuration'].append(d_action)
                        
            except:
                pass
        
    return dict(id=app_id, service_id=service_id, data=data)
    
@auth.requires_login()
@auth.requires_membership('admin')
def export():  
    try:  
        import StringIO
        from system import AuthConfiguration
        
        # File data
        auth_mng = AuthConfiguration(db)
        # File
        s = StringIO.StringIO()
        
        s.write(auth_mng.export_data())
        response.headers['Content-Type'] = 'text/csv'
        response.headers['Content-disposition'] = 'attachment; filename=data.csv'
        return s.getvalue()
    except Exception, e:
        return "Error: %s"%e
     
@auth.requires_login()
@auth.requires_membership('admin')    
def import_poe():
    form = FORM(INPUT(_type='file', _name='data'), 
                INPUT(_type='submit',_class='btn'),
                _id="importform")
    if form.process(session=None).accepted:
        try:
            if form.vars.data != '':
                from system import AuthConfiguration
                auth_mng = AuthConfiguration(db)
                auth_mng.import_data(form.vars.data.file)
                msg = T('Data imported correctly')
                action = "jQuery('#import_area').hide();"
            else:
                msg = T('You must select a file')
                action = ''
        except Exception, e:
            msg = "%s: %s" % (T('Error'), str(e))
            action = ''
            
        lit = "jQuery('#flash').html('%s');jQuery('#flash').show();%s" % (msg, action)
        return XML(lit, sanitize=True)
    return response.render('government/import_poe.load',form=form)   

@auth.requires_login()
@auth.requires_membership('admin')
def import_govfile():  
    [file, params] = get_request_args(1)
    
    try:
        from core.governhandler import GovernmentHandler
        GovernmentHandler.process_government_file(file)
        session.flash = current.T('Government File loaded')
    except Exception, e:
        session.flash = str(e)
            
    
    redirect(URL(c='government', f='services'))
    
@auth.requires_login()
@auth.requires_membership('admin')
def export_govfile():  
    [app, params] = get_request_args(1)
    
    try:
        from core.governhandler import GovernmentHandler
        data = GovernmentHandler.get_government_file(app)
        
        # File 
        import StringIO
        s = StringIO.StringIO()
        
        s.write(data)
        response.headers['Content-Type'] = 'text/yaml'
        response.headers['Content-disposition'] = 'attachment; filename=data.yaml'
        return s.getvalue()
    except Exception, e:
        return "Error: %s"%e
    

@auth.requires_login()
@auth.requires_membership('admin')
def system():   
    _menu.show()    
    
    from services import ServiceManager
    smng = ServiceManager(True)
    
    from system import AuthConfiguration
    auth_mng = AuthConfiguration(db)
    data = None
    
    sys_id = 'forge'
    
    if len(request.vars) > 0:
        data = smng.system
        for item in data['configuration']:       
            for action in data['configuration'][item]['actions']:
                if action['id'] in request.vars: 
                    auth_mng.add(sys_id, 
                                 item, 
                                 action['id'], 
                                 request.vars[action['id']]) 

    
    data = None
    if smng.system.keys():
        all_data = smng.system
        data = {}
        data['literal'] = all_data['literal']
        data['configuration'] = []
        for key in all_data['configuration']:
            item = all_data['configuration'][key]
            d_group = {}
            d_group['id'] = item['id']
            d_group['label'] = item['label'] 
            d_group['configuration'] = []            
            for action in item['actions']:
                d_action = {}
                d_action['id'] = action['id']
                d_action['label'] = action['label'] 
                d_action['options'] = []
                value = auth_mng.get_value(sys_id, item['id'], action['id'])
                for option in action['options']:
                    d_option = option 
                    d_option['checked'] = False
                    if value is None:
                        if option['default']:
                            d_option['checked'] = True
                    else:
                        if option['id'] == value:
                            d_option['checked'] = True
                    d_action['options'].append(d_option)
                d_group['configuration'].append(d_action)
            data['configuration'].append(d_group)
            
    return dict(data=data)