# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from mil.tools import get_request_args

from masters.guestbook import *
define_table(db)

crud.settings.create_next = URL('index')
crud.settings.delete_next = URL('index')
crud.settings.update_next = URL('index')

@auth.requires_login()
def index(): 
    # Menu
    _menu.controller={'name': T('Guestbook'),'module': 'guestbook','function': 'index'}
    _menu.show()
    
    
    #Check guestbook view (admin, or project)
    [name, params] = get_request_args(1)
    project = name
    
    #we force a comparison to check if the global var force_guestbook is set in the config.
    is_guestbook_active()
    
    if name is not None:
        pid = _user.get_pid_from_name(name)
        # User in project
        _user.in_project(pid)        
        if not _user.ejec:
            redirect(URL(r=request,c="desktop"))
    else:
        if auth.has_membership(user_id=_user.id, role='admin') is False:
            redirect(URL(r=request,c="desktop"))
    """
    if name == None:
        #if is admin view:
        #check if the user is admin
        # _user.check_is_admin() changed by next lines
        if auth.has_membership(user_id=_user.id, role='admin') is False:
            redirect(URL(r=request,c="desktop"))
        #check if the admin wants to force the guestbook in all the projects
        
        #get all the downloads
        response.write("es zona admin")
    else:
        #if is project view:
        #    [name, params] = get_request_args(1)
        
        # Project id from project name
        #    pid = _user.get_pid_from_name(name)
        # User in project
        #    _user.in_project(pid)
        #check if the user is the project admin
        
        #check if the proyect admin wants to add the guestbook (or the platform addmin has forced it)
        #get all the downloads
        response.write("no es zona admin")
    """
    
    
    
    
    
    # Check administration permission
    #_user.check_is_admin()
    
    return dict(project = project)


@auth.requires_login()
def check():
    
    [name, params] = get_request_args(1)
    show_submit = True
    if name is not None:
        pid = get_pid_from_name(name)
        # User in project
        _user.in_project(name, pid)        
        if _user.ejec:
            active = is_guestbook_active(pid) # 2: forced by admin, 1: active in project, 0 not actived
            
            message = T("Guestbook inactive")
            default_value = False
            is_writable = True
            if active == 2:
                message = T("Guestbook forced by forge admin")
                is_writable = False
                default_value = True
                show_submit = False
            elif active == 1:
                message = T("Guestbook activated")
                default_value = True
                
            form=SQLFORM.factory(
                Field('activate', 'boolean', default = default_value, writable=is_writable))
            
            
            if form.accepts(request.vars,session):
                if form.vars.activate == True:
                    enable_guestbook_in_project(pid)
                    message = T("Guestbook activated")
                    active = 1
                    form.custom.widget.activate.attributes["_checked"] = "checked"
                else:
                    disable_guestbook_in_project(pid)
                    active = 0
                    del form.custom.widget.activate.attributes["_checked"]
                    message = T("Guestbook inactive")

        else:            
            redirect(URL(r=request,c="desktop"))
        
    else:
        if auth.has_membership(user_id=_user.id, role='admin') is False:
            redirect(URL(r=request,c="desktop"))
        active = is_guestbook_active() # 2: forced by admin, 1: active in project, 0 not actived
            
        default_value = False
        message = T("Guestbook not forced in all platform")
        if active == 2:
            message = T("Guestbook forced in all platform")
            default_value = True
            
        form=SQLFORM.factory(
            Field('activate', 'boolean', default = default_value))   
        
    
        if form.accepts(request.vars,session):
            if form.vars.activate == True:
                message = T("Guestbook forced in all platform")
                enable_guestbook_in_platform()
                active = 2
                form.custom.widget.activate.attributes["_checked"] = "checked"
            else:
                message = T("Guestbook not forced in all platform")
                active = 0
                del form.custom.widget.activate.attributes["_checked"]
                disable_guestbook_in_platform()
    
    return dict(form = form, message=message, show_submit = show_submit)

@auth.requires_login()
def list():
    [name, params] = get_request_args(1)
    
    query = None
    
    if name is not None:
        pid = _user.get_pid_from_name(name)
        # User in project
        _user.in_project(pid)        
        if not _user.ejec:
            redirect(URL(r=request,c="desktop"))
        query = (db.guestbook.project == pid)
    else:
        if auth.has_membership(user_id=_user.id, role='admin') is False:
            redirect(URL(r=request,c="desktop"))
        
     #if is admin view:
     
    
    
    form = crud.select(db.guestbook, 
                           linkto=None,
                           query = query,
                           fields = ['guestbook.project','guestbook.mail','guestbook.data','guestbook.file','guestbook.is_platform_user','guestbook.date' ], 
                           headers = {'guestbook.project': T("Project"), 'guestbook.mail': T('Email'),'guestbook.data': T('Data'),'guestbook.file': T('File'),'guestbook.is_platform_user': T('Is platform user'), 'guestbook.date': T('Date')}
                           )
     
     
    return dict(form=form)

def sign():
    
    from datetime import datetime
    
    now = datetime.now()
    
    #get the project id        
    pid = request.vars.pid
    url = request.vars.proc_url
    file = request.vars.file
    
    data = None    
    
    if not is_guestbook_data_in_session(pid):
        data = get_guestbook_data_session(pid)    
    
    
    db.guestbook.date.default = now
    db.guestbook.project.default = pid
    db.guestbook.file.default = file
    
    if data is not None:
        db.guestbook.mail.default = data.mail
        db.guestbook.is_platform_user.default = data.is_platform_user
    else:
        db.guestbook.is_platform_user.default = _user.logged
        if _user.logged:
            db.guestbook.mail.default = _user.email
        
    form= SQLFORM(db.guestbook)   
    
    
    if form.accepts(request.vars,session):
        session.guestbook_skip_first = True
        set_guestbook_data_session()
        url_proc = url.split("/")
        controller = url_proc[2]
        if url_proc.__len__() <= 3:
            function = None
        arguments = []
        if url_proc.__len__() >= 4:
            function = url_proc[3]
            
            for n in range(4,url_proc.__len__()):
                arguments.append(n)
        
            
        redirect_url = URL(r=request, c=controller, f=function, args=arguments, vars=dict(file=file) )
        redirect(redirect_url)
        
    return dict(form=form)