# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import os.path

from mil.base import File, YAMLFile

def index():
    if _config is not None:
        if auth.has_membership('admin'):
            _menu.show()
        else:
            redirect(URL(r=request,c="default"))
    
    # Checks files
    if not File.exists(static_file):
        template_file = os.path.join(skels_path, 'forge_config.yaml')
        File.copy(template_file, static_file)
    if not File.exists(dynamic_file):
        template_file = os.path.join(skels_path, 'config.yaml')
        File.copy(template_file, dynamic_file)
        
    # Import yaml data
    config_data = YAMLFile.load(static_file)

    if request.vars:        
        def set_data(key, parent=None):
            if key in request.vars.keys():
                if parent is not None:
                    config_data[parent][key] = request.vars[key]
                else:
                    config_data[key] = request.vars[key]
        # System        
        if 'dal_migrate' in request.vars.keys():
            config_data['dal_migrate'] = False
            if request.vars['dal_migrate'] == 'true':
                config_data['dal_migrate'] = True
        set_data('filesystem_path')
        if _config is not None:
            # Create filesystem base
            base_path = config_data['filesystem_path']
            if File.check_dir(base_path):
                File.make_dir(os.path.join(base_path, 'data'))
                File.make_dir(os.path.join(base_path, 'logs'))
        if 'has_cas_provider' in request.vars.keys():
            set_data('cas_provider')
        else:
            config_data['cas_provider'] = None
            
        # Database
        if 'dal' in request.vars.keys():
            set_data('dal')
        
        # Admin user
        set_data('first_name', 'admin')
        set_data('last_name', 'admin')
        set_data('email', 'admin')
        set_data('password', 'admin')
        
        # Registration
        if 'registration_automatic' in request.vars.keys():
            config_data['register']['registration_automatic'] = True
        else:
            config_data['register']['registration_automatic'] = False
        config_data['register']['min_password_length'] = int(request.vars['min_password_length'])
        if 'registration_approval' in request.vars.keys():
            config_data['register']['registration_approval'] = True
        else:
            config_data['register']['registration_approval'] = False
        if 'registration_validation' in request.vars.keys():
            config_data['register']['registration_verification'] = True
        else:
            config_data['register']['registration_verification'] = False
        if 'reset_password' in request.vars.keys():
            config_data['register']['reset_password_verification'] = True
        else:
            config_data['register']['reset_password_verification'] = False
            
        # Notification: email    
        if 'actived' in request.vars.keys():
            config_data['mailer']['active'] = True
            set_data('server', 'mailer')
            set_data('sender', 'mailer')
            set_data('login', 'mailer')
        else:
            config_data['mailer']['active'] = False
            
        # External
        if 'dav' in request.vars.keys():    
            set_data('webdav_server')
            if _config is not None:
                # Create dir in filesystem base
                if File.check_dir(base_path):
                    File.make_dir(os.path.join(base_path, 'dav'))
        else:
            config_data['webdav_server'] = None
        if 'svn' in request.vars.keys():  
            set_data('svn_server')
            if _config is not None:
                # Create dir in filesystem base
                if File.check_dir(base_path):
                    File.make_dir(os.path.join(base_path, 'svn'))
        else:
            config_data['svn_server'] = None
            
        # Save into file
        YAMLFile.save(static_file,config_data)
        
        # Redirection
        redirect(URL(r=request,c='default',f='index'))
    
    # Checks files
    return dict(data = config_data)
        
def info():
    import sys
    
    # System
    system = {}    
    # Python version
    try:
        version = sys.version[:5] 
        
        if sys.version_info < (2, 6):
            status = str(T('Invalid'))
        else:
            status = str(T('Valid'))
        system['Python'] = (True, '%s: %s' % (version, status))
    except:
        system['Python'] = (False, str(T('Unknown')))
        
    # Libraries
    libraries = {}
    # MIL
    try:
        from mil.base import File
        libraries['mil'] = (True, str(T('Installed')))
    except:    
        libraries['mil'] = (False, str(T('Error')))
    # Yaml
    try:
        from yaml import dump
        libraries['yaml'] = (True, str(T('Installed')))
    except:    
        libraries['yaml'] = (False, str(T('Error')))
    # Python webdav
    try:
        from python_webdav.client import Client
        libraries['python_webdav'] = (True, str(T('Installed')))
    except:    
        libraries['python_webdav'] = (False, str(T('Error')))
        
    # Connections
    connections = {}
    # MySQL
    try:
        from gluon import DAL
        db_conn = DAL(_config.get('dal'))
        connections['MySQL'] = (True, str(T('Online')))
    except:    
        connections['MySQL'] = (False, str(T('Offline')))
    # DAV
    from mil.service import http
    try:
        con = http(_config.get('webdav_server'))
        connections['Dav'] = (True, str(T('Online')))
    except Exception, e:
        if str(e) in ['401', '403']:
            connections['Dav'] = (True, str(T('Online')))
        else:
            connections['Dav'] = (False, 'Offline: %s' % e)
    # Svn
    try: 
        con = http(_config.get('svn_server'))
        connections['Svn'] = (True, str(T('Online')))
    except Exception, e:   
        if str(e) in ['401', '403']: 
            connections['Svn'] = (True, str(T('Online')))
        else:
            connections['svn'] =  (False, 'Offline: %s' % e)
     
    
    
    return dict(system=system, libs=libraries, conn=connections)
