# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

@auth.requires_login()
@auth.requires_membership('admin')
def index():
    _menu.show()
    
    # All projects. Admin mode
    from project import ProjectHandler
    from project.models.card import ProjectCardModel
    
    # Prepare table
    ProjectCardModel.define_table(db)
    
    # Get rows
    rows = db(db.project_card.id > 0).select(db.project_card.id)
    
    # Prepare rows to view output
    list = []
    for row in rows:            
        list.append(ProjectHandler.prepare_card(row.id))
        
    return dict(list=list) 

@auth.requires_login()
@auth.requires_membership('admin')
def create():
    _menu.show()
    
    from project.models.card import ProjectCardModel
    from project.models.team import ProjectTeamModel
    cardKlass = ProjectCardModel(db)
    teamKlass = ProjectTeamModel(db)
        
    # Team leaders         
    from masters.users import Users   
    team_leaders = Users(current._config, current.db).get_all()
    options = [(str(x.id), "%s %s (%s)" % (x.first_name, x.last_name, x.email) ) for x in team_leaders]

    # Hide files
    db.project_card.resume.writable=False
    db.project_card.resume.readable=False
    db.project_card.objetive.writable=False
    db.project_card.objetive.readable=False
    form = SQLFORM.factory(db.project_card,
                           Field('leader',
                                 label=T('Leader'), 
                                 requires=IS_IN_SET(options, multiple=False)))
    
    if form.process().accepted:
        # Add Card
        id = cardKlass.add(form.vars)
        
        # Add leader
        teamKlass.set_leader(id, form.vars.leader)
        # Activate services
        from project import ProjectHandler
        ProjectHandler.join_member_as(id, form.vars.leader)
        leader = Users(current._config, current.db).get_by_id(form.vars.leader)
        
        # Add services by project type
        from project.models.type_service import ProjectTypeServicesModel
        ptsKlass = ProjectTypeServicesModel(db)
        c_services = ptsKlass.get_by_type(form.vars.type).services
        from project.models.service import ProjectServicesModel
        psKlass = ProjectServicesModel(db)
        psKlass.set(id, c_services)
        
        
        from project.subapp import ProjectInstance
        sapp_prj = ProjectInstance()
        services = sapp_prj.get_services()
        activate_actions =[]
        for service in services:
            # Checks if service was active
            if service.id in c_services:
                activate_actions.append({'id': service.id, 'action': 'enable'})
            else:
                activate_actions.append({'id': service.id, 'action': 'none'})
                
        # Activate services
        from project import ProjectHandler
        ProjectHandler.activate_services(id, 
                                         activate_actions, 
                                         leader)
        
        redirect(URL('dashboard','index'))
        
    return dict(form=form)
    
@auth.requires_login()
@auth.requires_membership('admin')
def lock():
    from mil.tools import get_request_args
    [name, params] = get_request_args(1)
    
    from project import ProjectHandler
    ProjectHandler.lock(name)
    
    redirect(URL('project','index'))
    
@auth.requires_login()
@auth.requires_membership('admin')
def delete():
    session.flash=T('Deletion Project functionality not enabled')
    redirect(URL('project','index'))
    
    from mil.tools import get_request_args
    [name, params] = get_request_args(1)
    
    from project import ProjectHandler
    ProjectHandler.delete(name)
    
    redirect(URL('project','index'))
    
    
# Sub application modules
#

# Admin mode

@auth.requires_login()
@auth.requires_membership('admin')
def type():
    from project import ProjectType
    iklass = ProjectType()    
    return iklass()

@auth.requires_login()
@auth.requires_membership('admin')
def status():
    from project import ProjectStatus
    iklass = ProjectStatus()    
    return iklass()

@auth.requires_login()
@auth.requires_membership('admin')
def profile():
    from project import ProjectProfile
    iklass = ProjectProfile()    
    return iklass()

@auth.requires_login()
@auth.requires_membership('admin')
def typeservices():
    from project import ProjectTypeService
    iklass = ProjectTypeService()    
    return iklass()

@auth.requires_login()
@auth.requires_membership('admin')
def configure():      
    from mil.tools import get_request_args 
    [id, params] = get_request_args(1)
    
    from services import ServiceManager
    smng = ServiceManager("masters")
    
    from system import AuthConfiguration
    auth_mng = AuthConfiguration(db)
    data = None
    
    if len(request.vars) > 0:
        data = smng.services[id]
        auth_mng.delete(id)
        for action in data['actions']:
            if action['id'] in request.vars:
                auth_mng.add(id, action['id'], request.vars[action['id']])
        redirect(URL('index'))
    
    data = None
    if id in smng.services.keys():
        all_data = smng.services[id] 
        data = {}
        data['controller'] = all_data['controller']
        data['literal'] = all_data['literal']
        data['permissions'] = all_data['permissions']
        data['actions'] = []
        for action in all_data['actions']:
            d_action = {}
            d_action['id'] = action['id']
            d_action['label'] = action['label'] 
            d_action['options'] = []
            value = auth_mng.get_value(id, action['id'])
            for option in action['options']:
                d_option = option 
                d_option['checked'] = False
                if value is None:
                    if option['default']:
                        d_option['checked'] = True
                else:
                    if option['id'] == value:
                        d_option['checked'] = True
                d_action['options'].append(d_option)
            data['actions'].append(d_action)
    return dict(id=id, data=data)

# User mode
def card():
    from project import ProjectCard
    iklass = ProjectCard()    
    return iklass()

def board():
    from project import ProjectBoard
    iklass = ProjectBoard()    
    return iklass()

def service():
    from project import ProjectService
    iklass = ProjectService()    
    return iklass()

def team():
    from project import ProjectTeam
    iklass = ProjectTeam()    
    return iklass()

#@auth.requires_login()
#@auth.requires_membership('admin')
#def request():
##    from project import ProjectRequest
#    iklass = ProjectTeam()    
#    return iklass()

def roadmap():
    from project import ProjectRoadmap
    iklass = ProjectRoadmap()    
    return iklass()

def task():
    from project import ProjectTask
    iklass = ProjectTask()    
    return iklass()

def resume():
    from project import ProjectResume
    iklass = ProjectResume()    
    return iklass()

def bug():
    from project import ProjectBug
    iklass = ProjectBug()    
    return iklass()

def incidence():
    from project import ProjectIncidence
    iklass = ProjectIncidence()    
    return iklass()

def join():
    from project import ProjectJoin
    iklass = ProjectJoin()    
    return iklass()

# Common services
def webdav():
    from mil.tools import get_request_args
    [name, params] = get_request_args(1)  
     
    redirect(URL(r=request,c='webdav',f='index', args=['project', name]))

def repository():
    from mil.tools import get_request_args
    [name, params] = get_request_args(1)  
     
    redirect(URL(r=request,c='repository',f='index', args=['project', name]))

def wiki():
    from mil.tools import get_request_args
    [name, params] = get_request_args(1)  
     
    redirect(URL(r=request,c='wiki',f='index', args=['project', name]))

@auth.requires_login()
def calendar():
    from mil.tools import get_request_args
    [name, params] = get_request_args(1)  
     
    redirect(URL(r=request,c='calendar',f='index', args=['project', name]))


#
#
#@auth.requires_login()
#@auth.requires_membership('admin')
#def manage():
#    _menu.controller={'name': T('Project Manager'),
#                      'module': 'project',
#                      'function': 'manage'}
#    _menu.show()
#    
#    from project.models import ProjectCardModel
#    insKlss = ProjectCardModel(db) 
#    return dict(form=insKlss.grid())
#    
#
#@auth.requires_login()
#@auth.requires_membership('admin')
#def logs():
#    _menu.controller={'name': T('Logs'),
#                      'module': 'master',
#                      'function': 'logs'}
#    _menu.show()
#    _syslog.log('Master.logs access')
#    from masters.logs import Logs
#    insKlss = Logs(db) 
#    return dict(form=insKlss.grid())
#
#
#@auth.requires_login()
#@auth.requires_membership('admin')
#def configure():
#    from service import ServiceManager
#    from system import AuthConfiguration
#
#    id='project'
#    smng = ServiceManager("masters")
#    auth_mng = AuthConfiguration(db)
#    data = None
#    
#    if len(request.vars) > 0:
#        data = smng.get(id)
#        auth_mng.delete(id)
#        for action in data['actions']:
#            if action['id'] in request.vars:
#                auth_mng.add(id, action['id'], request.vars[action['id']])
#    
#    data = None
#    if id in smng.services.keys():
#        data = smng.get_as_dict(id, auth_mng) 
#    return dict(id=id, data=data)
#
#
#@request.restful()
#@auth.requires_login()  
#def api(self):
#    def GET(*args, **vars):  
#        data = {'message': 'Ok'}  
#        
#        current.response.headers['Content-Type'] = 'application/json'
#        return XML(data.content)    
#        
#    def POST(*args, **vars):             
#        json_data = current.request.body.read()
#        # Check if json string is valid
#        is_valid = False
#        import gluon.contrib.simplejson as json
#        try:
#            data = json.loads(json_data)
#            is_valid = True
#        except Exception, e:
#            data = None
#        if is_valid:
#            raise HTTP(200)
#        else:
#            raise HTTP(400) 
#        
#    return locals()