# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

@_menu.showme() 
def index():    
    from mil.tools import get_request_args
    [type, name, params] = get_request_args(2)
        
    # Validation
    if type == 'project':
        from project import ProjectHandler       
        ProjectHandler.user_set_project(name)
        if not _government.has_permission('repository', 'repository_access', application='project'):
            response.flash = T('You have not permission')
            active = False
        else:     
            # Check type
            from core import SubApp
            SubApp.check_type(type, name) 
            
            if _config.get('svn_server') is None:
                response.flash = T('Svn server not defined')
                active = False
            else:
                active = True
        
    return dict(active=active, svn_type= type, svn_name=name)

def files(): 
    from mil.tools import get_request_args
    [name, params] = get_request_args(1)
    
    from project import ProjectHandler       
    ProjectHandler.user_set_project(name)
    
    repo_server =  _config.get('svn_server')
    if not repo_server.endswith('/'):
        repo_server += "/"
    
    elf_opts = {
         'URL': URL(r=request,c='repository',f='file'),
         'debug': False, 
         'fileURL': True,
         'repo_server': repo_server,
         'repo_path': name,
         'repo_user': _user.email, 
         'repo_pswd': _user.get_external_password('svn'),
         'rootAlias': _user.project.title
    }
 
    from elfinder import elFinderConnector, ELFINDERDRIVER
    elf = elFinderConnector(ELFINDERDRIVER.REPO_SVN, elf_opts)
    
    # fetch only needed GET/POST parameters
    httpRequest = {}
    for field in elf.get_http_allowed_parameters():
        if field in request.vars:
            httpRequest[field] = request.vars[field]
            if field == 'upload[]':
                upFiles = {}
                cgiUploadFiles = request.vars['upload[]']
                for up in cgiUploadFiles:
                    if up.filename:
                        upFiles[up.filename] = up.file 
                httpRequest['upload[]'] = upFiles
            
    # run connector with parameters
    try:
        elf_status, elf_header, elf_response = elf.run(httpRequest)
    except Exception, e:
        elf_status = 500 
        elf_header = []
        elf_response = None       
    
    # Output 
    if elf_status == 200:
        data = elf_response
    else:
        data = {'error': 'Error code %s' % elf_status}
        
    for h, v in elf_header.iteritems():
        response.headers[h] = v
            
    from gluon.serializers import json
    if elf_header['Content-type'] == 'application/json':    
        return json(data)
    else:   
        return dict(data=json(data))        

def file():
    import pysvn
    
    if len(request.args) > 0:
        parameters = "/".join(request.args)
        
    path = '%s/%s' % (_config.get('svn_server'), parameters)
    
    try:
        ## Connection
        client = pysvn.Client()
        client.set_default_username(_user.email)
        client.set_default_password(_user.get_external_password('svn'))
        
        # Get file
        data = client.cat(path)
        
        from gluon.contenttype import contenttype
        file_name = os.path.basename(parameters)
        file_type = contenttype(file_name) or 'txt'
        response.headers['Content-Type'] = file_type    
        return data
    except Exception, e:
        from gluon.http import HTTP
        raise HTTP(404) 
        