# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

@_menu.showme() 
def index():        
    from mil.tools import get_request_args
    [type, name, params] = get_request_args(2)
        
    # Validation
    if type == 'project':
        from project import ProjectHandler       
        ProjectHandler.user_set_project(name)
        if not _government.has_permission('webdav', 'webdav_access', application='project'):
            response.flash = T('You have not permission')
            active = False
        else:            
            # Check type
            from core import SubApp
            SubApp.check_type(type, name) 
            
            if _config.get('webdav_server') is None:
                response.flash = T('Webdav server not defined')
                active = False
            else:
                active = True
        
    return dict(active=active, dav_type=type, dav_name=name)

def files():        
    from mil.tools import get_request_args
    [name, params] = get_request_args(1)
    
    from project import ProjectHandler       
    ProjectHandler.user_set_project(name)
    
    elf_opts = {
         'URL': URL(r=request,c='webdav',f='file', args=[name]),
         'debug': False, 
         'fileURL': True,
         'dav_server': _config.get('webdav_server'),
         'dav_path': name,
         'dav_user': _user.email, 
         'dav_pswd': _user.get_external_password('dav'),
         'rootAlias': _user.project.title,
         'uploadMaxSize': 20480
    }
 
    from elfinder import elFinderConnector, ELFINDERDRIVER
    elf = elFinderConnector(ELFINDERDRIVER.WEBDAV, elf_opts)
    
    # fetch only needed GET/POST parameters
    httpRequest = {}
    for field in elf.get_http_allowed_parameters():
        if field in request.vars:
            httpRequest[field] = request.vars[field]
            if field == 'upload[]':
                upFiles = {}
                cgiUploadFiles = request.vars['upload[]']
                for up in cgiUploadFiles:
                    if up.filename:
                        upFiles[up.filename] = up.file 
                httpRequest['upload[]'] = upFiles
            
    # run connector with parameters
    try:
        elf_status, elf_header, elf_response = elf.run(httpRequest)
    except Exception, e:
        elf_status = 500 
        elf_header = {'Content-type': 'application/json'}
        elf_response = None       
    
    # Output 
    if elf_status == 200:
        data = elf_response
        
        for h, v in elf_header.iteritems():
            response.headers[h] = v
    else:
        data = {'error': 'Error code %s' % elf_status}
                    
    from gluon.serializers import json
    if elf_header['Content-type'] == 'application/json':    
        return json(data)
    else:   
        return dict(data=json(data))        

def file():
    from mil.tools import get_request_args
    [name, params] = get_request_args(1)
    
    from system.webdav_client import WebdavClient
            
    # Prepare password
    dav_password = _user.get_external_password('dav')
    if current._config.get('webdav_type') == 'pywebdav':
        if len(request.args) > 0:
            parameters = "/".join(request.args)
    else:
        if len(request.args) > 0:
            pre_pars = request.args.insert(0, _config.get('webdav_server'))
            parameters = "/".join(request.args)
        
    ## Connection
    con = WebdavClient(_config.get('webdav_server'))
    [resp, data] = con.getFile(parameters,
                               user=_user.email, 
                               passwd=dav_password) 
    
    if resp['status']=='200':  
        from gluon.contenttype import contenttype
        file_name = os.path.basename(parameters)
        file_type = contenttype(file_name) or 'text'
        response.headers['Content-Type'] = file_type  
        # If file not showable, download it
        if file_type.find('text') == -1:   
            response.headers['Content-disposition'] = 'attachment; filename=%s' % (file_name)
        return data
    else:
        from gluon.http import HTTP
        raise HTTP(404) 
 