# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon.http import HTTP

from mil.tools import get_request_args

from core import SubApp

# Three levels: 1,2,3. 
# At level 1, pages can only include text, images, audio and video. 
# At level 2, pages can also include widgets . 
# At level 3, pages can also include web2py template code.
_config.wiki_level = 1 
# 'markmin' or 'html' for wysiwyg editor
_config.wiki_mode = 'markmin'

import wiki.core
from wiki.name_space import WikiNS

##########################################################
# code to handle wiki pages
##########################################################

def index():    
    [type, name, params] = get_request_args(2)  
    
    current._menu.show()
    
    # Validation
    if type == 'project':
        from project import ProjectHandler       
        ProjectHandler.user_set_project(name)
        if not _government.has_permission('wiki', 'wiki_access', application='project'):
            response.flash = T('You have not permission')
            active = False
        else:     
            if name is None:
                name = 'forge'
                
            active = True
            session.wiki_type = type
            redirect(URL(r=request,f='page',args=[name]))
    else:
        if name is None:
            name = 'forge'
        active = True
        session.wiki_type = type
        redirect(URL(r=request,f='page',args=[name]))
            
    return dict(active=active, wiki_type= type, wiki_name=name)
 
def pages_map():
    [name, params] = get_request_args(1)  
    
    current._menu.show()
    
    # Check type
    SubApp.check_type(session.wiki_type, name) 
     
    wiki_editor = wiki.core.wiki_get_editor()
        
    wiki_ns=WikiNS()
    wiki_ns.show(name)
    
    # Models
    from wiki.models.wiki_page import WikiPageModel        
    WikiPageModel.define_table(current.db)

    w = db.wiki_page
    if wiki_editor:
        pages = db((w.id>0)&(w.ns==name)).select(orderby=w.slug)
    else:
        pages = db((w.active==True)&(w.public==True)&(w.ns==name)).select(orderby=w.slug)
        
    for item in pages:
        item.modified_by=db.wiki_page.modified_by.represent(item.modified_by)
        
    if wiki_editor:
        form=SQLFORM.factory(Field('ns',default=name,writable=False,requires=db.wiki_page.ns.requires),
                             Field('slug',requires=db.wiki_page.slug.requires),
                             Field('from_template',requires=IS_EMPTY_OR(IS_IN_DB(db,db.wiki_page.slug))))
        if form.accepts(request.vars):
            redirect(URL(r=request,f='page',args=[name,form.vars.slug],vars=dict(template=request.vars.from_template or '')))
    else:
        form=''
        
    return dict(ns=name,is_editor=wiki_editor,pages=pages,form=form)

def page():
    """
    shows a page
    """
    [name,slug,params] = get_request_args(2)   
    
    current._menu.show()
    
    # Check type
    SubApp.check_type(session.wiki_type, name) 
     
    wiki_editor = wiki.core.wiki_get_editor()
    
    wiki_ns=WikiNS()
    wiki_ns.show(name)
    
    if slug is None or slug=='':
        slug = 'index'
        
    # Models
    from wiki.models.wiki_page import WikiPageModel        
    WikiPageModel.define_table(current.db)
    
    w = db.wiki_page
    page = w(ns=name, slug=slug)
    if not wiki_editor and (not page or not page.public or not page.active):
        page=None
    
    if request.extension=='load':
        return wiki_ns.render(page.body)
    if request.extension=='html':         
        return dict(is_editor=wiki_editor,
                    ns=name,
                    page=page,
                    slug=slug,
                    page_url=URL(r=request,f='page'),
                    wiki_ns=wiki_ns)
    return MARKMIN(page.body,extra={'page': (lambda text: ''),
                                    'widget':(lambda code:''),
                                    'template':(lambda template:'')})

def page_archive():
    """
    shows and old version of a page
    """
    [name,id,params] = get_request_args(2)   
    
    current._menu.show()
    
    # Check type
    SubApp.check_type(session.wiki_type, name) 
     
    wiki_editor = wiki.core.wiki_get_editor()
    wiki_ns=WikiNS()
    wiki_ns.show()
    
    # Models
    from wiki.models.wiki_page_archive import WikiPageArchiveModel        
    WikiPageArchiveModel.define_table(current.db)

    h = db.wiki_page_archive
    page = h(id)
    if not page or (not wiki_editor and (not page.public or not page.active)):
        raise HTTP(404)
    if request.extension!='html': return page.body
    return dict(ns=name, page=page, wiki_ns=wiki_ns)

@auth.requires_login()
def page_edit():
    """
    edit a page
    """
    [name,slug,params] = get_request_args(2)   
    
    current._menu.show()
    
    # Check type
    SubApp.check_type(session.wiki_type, name) 
     
    wiki_editor = wiki.core.wiki_get_editor()
        
    wiki_ns=WikiNS()
    wiki_ns.show()
    
    if slug == None:
        slug = 'index'
        
    # Models
    from wiki.models.wiki_page import WikiPageModel        
    WikiPageModel.define_table(current.db)

    w = db.wiki_page
    page = w(ns=name,slug=slug)
    if not page:
        page = w.insert(ns=name,
                        slug=slug, 
                        title=slug.replace('-',' ').capitalize(),
                        body=request.vars.template and w(slug=request.vars.template).body or '')
    form = crud.update(w, page, onaccept=crud.archive,
                       next=URL(r=request,f='page',args=[name,slug]))
    return dict(ns=name, 
                form=form, 
                page=page, 
                wiki_ns=wiki_ns)


def page_history():
    """
    show page changelog
    """
    [name,slug,params] = get_request_args(2)   
    
    current._menu.show()
    
    # Check type
    SubApp.check_type(session.wiki_type, name) 
     
    wiki_editor = wiki.core.wiki_get_editor()

    wiki_ns=WikiNS()
    wiki_ns.show()
    
    # Models
    from wiki.models.wiki_page import WikiPageModel        
    WikiPageModel.define_table(current.db)
    
    
    from wiki.models.wiki_page_archive import WikiPageArchiveModel        
    WikiPageArchiveModel.define_table(current.db)

    w = db.wiki_page
    h = db.wiki_page_archive
    page = w(ns=name, slug=slug)
    page.modified_by = db.wiki_page.modified_by.represent(page.modified_by)
    history = db(h.current_record==page.id).select(orderby=~h.modified_on)
    for item in history:
        item.modified_by=db.wiki_page.modified_by.represent(item.modified_by)
    
    return dict(ns=name, page=page, history=history)


##########################################################
# ajax callbacks
##########################################################
@auth.requires_login()
def attachments():
    """
    allows to edit page attachments
    """
    # Models
    from wiki.models.wiki_attachment import WikiAttachmentModel       
    WikiAttachmentModel.define_table(current.db)
    
    a=db.wiki_attachment
    a.tablename.default=tablename=request.args(0)
    a.record_id.default=record_id=request.args(1)
    #if request.args(2): a.file.writable=False
    form=crud.update(a,request.args(2),
                     next=URL(r=request,args=request.args[:2]))
    if request.vars.list_all:
        query = a.id>0
    else:
        query = (a.tablename==tablename)&(a.record_id==record_id)
    rows=db(query).select(orderby=a.name)
    return dict(form=form,rows=rows)

def attachment():
    """
    displays an attachments
    """
    # Models
    from wiki.models.wiki_attachment import WikiAttachmentModel       
    WikiAttachmentModel.define_table(current.db)
    
    short=request.args(0)
#    if wiki_authorize_attachments and \
#            not short in session.wiki_attachments:
#        raise HTTP(400)
    a=db.wiki_attachment
    record=a(short.split('.')[0])
    if not record: raise HTTP(400)
    request.args[0]=record.file
    return response.download(request,db,attachment=False)

def comment():
    """
    post a comment
    """
    tablename, record_id = request.args(0), request.args(1)
    table=db.wiki_comment
    if record_id=='None': record_id=0
    table.tablename.default=tablename
    table.record_id.default=record_id
    if auth.user:
        form = crud.create(table)
    else:
        form = A(T('login to comment'),_href=auth.settings.login_url)
    comments=db(table.tablename==tablename)\
        (table.record_id==record_id).select()
    return dict(form = form,comments=comments)

@auth.requires_login()
def jqgrid():
    """
    jqgrid callback retrieves records
    http://trirand.com/blog/jqgrid/server.php?q=1&_search=false&nd=1267835445772&rows=10&page=1&sidx=amount&sord=asc&searchField=&searchString=&searchOper=
    """
    from gluon.serializers import json
    import cgi
    tablename = request.vars.tablename or error()
    columns = (request.vars.columns or error()).split(',')
    rows=int(request.vars.rows or 25)
    page=int(request.vars.page or 0)
    sidx=request.vars.sidx or 'id'
    sord=request.vars.sord or 'asc'
    searchField=request.vars.searchField
    searchString=request.vars.searchString
    searchOper={'eq':lambda a,b: a==b,
                'nq':lambda a,b: a!=b,
                'gt':lambda a,b: a>b,
                'ge':lambda a,b: a>=b,
                'lt':lambda a,b: a<b,
                'le':lambda a,b: a<=b,
                'bw':lambda a,b: a.like(b+'%'),
                'bn':lambda a,b: ~a.like(b+'%'),
                'ew':lambda a,b: a.like('%'+b),
                'en':lambda a,b: ~a.like('%'+b),
                'cn':lambda a,b: a.like('%'+b+'%'),
                'nc':lambda a,b: ~a.like('%'+b+'%'),
                'in':lambda a,b: a.belongs(b.split()),
                'ni':lambda a,b: ~a.belongs(b.split())}\
                [request.vars.searchOper or 'eq']
    table=db[tablename]
    if request.vars.fieldname:
        dbset = table._db(table[request.vars.fieldname]==request.vars.fieldvalue)
    else:
        dbset = table._db(table.id>0)
    if searchField: dbset=dbset(searchOper(table[searchField],searchString))
    orderby = table[sidx]
    if sord=='desc': orderby=~orderby
    limitby=(rows*(page-1),rows*page)
    fields = [table[f] for f in columns]
    records = dbset.select(orderby=orderby,limitby=limitby,*fields)
    nrecords = dbset.count()
    items = {}
    items['page']=page
    items['total']=int((nrecords+(rows-1))/rows)
    items['records']=nrecords
    readable_fields=[f.name for f in fields if f.readable]
    def f(value,fieldname):
        r = table[fieldname].represent
        if r: value=r(value)
        try: return value.xml()
        except: return cgi.escape(str(value))
    items['rows']=[{'id':r.id,'cell':[f(r[x],x) for x in readable_fields]} \
                       for r in records]
    return json(items)

def tags():
    import re
    db_tag = db.wiki_tag
    db_link = db.wiki_link
    table_name=request.args(0)
    record_id=request.args(1)
    if not auth.user_id:
        return ''
    form = SQLFORM.factory(Field('tag_name',requires=IS_SLUG()))
    if request.vars.tag_name:
        for item in request.vars.tag_name.split(','):
            tag_name = re.compile('\s+').sub(' ',item).strip()
            tag_exists = tag = db(db_tag.name==tag_name).select().first()
            if not tag_exists:
                tag = db_tag.insert(name=tag_name, links=1)
            link = db(db_link.tag==tag.id)\
                (db_link.table_name==table_name)\
                (db_link.record_id==record_id).select().first()
            if not link:
                db_link.insert(tag=tag.id,
                               table_name=table_name,record_id=record_id)
                if tag_exists:
                    tag.update_record(links=tag.links+1)
    for key in request.vars:
        if key[:6]=='delete':
            link_id=key[6:]
            link=db_link[link_id]
            del db_link[link_id]
            db_tag[link.tag] = dict(links=db_tag[link.tag].links-1)
    links = db(db_link.table_name==table_name)\
              (db_link.record_id==record_id).select()\
              .sort(lambda row: row.tag.name.upper())
    return dict(links=links, form=form)

def cloud():
    tags = db(db.wiki_tag.links>0).select(limitby=(0,20))
    if tags:
        mc = max([tag.links for tag in tags])
    return DIV(_class='wiki_tag_cloud',*[SPAN(A(tag.name,_href=URL(r=request,c='wiki',f='page',args=('tag',tag.id))),_style='font-size:%sem' % (0.8+1.0*tag.links/mc)) for tag in tags])

@auth.requires_login()
def widget_builder():
    """
    >> inspect.getargspec(PluginWikiWidgets.tags)
    (['table', 'record_id'], None, None, ('None', None))
    >>> dir(PluginWikiWidgets)
    """
    import inspect
    name=request.vars.name
    if wiki_widgets=='all':
        widgets = ['']+[item for item in dir(PluginWikiWidgets) if item[0]!='_']
    else:
        widgets = wiki_widgets
    form=FORM(LABEL('Widget Name: '), SELECT(_name='name',value=name,
                     _onchange="jQuery(this).parent().submit()",
                     *widgets))
    widget_code=''
    if name in widgets: 
        a,b,c,d=inspect.getargspec(getattr(PluginWikiWidgets,name))
        a,d=a or [],d or []
        null = lambda:None
        d=[null]*(len(a)-len(d))+[x for x in d]
        ESC='x'
        fields = [Field(ESC+a[i],label=a[i],default=d[i]!=null and d[i] or '',
                        requires=(d[i]==null) and IS_NOT_EMPTY() or None,
                        comment=(d[i]==null) and 'required' or '') \
                      for i in range(len(a))]
        form_widget=SQLFORM.factory(hidden=dict(name=name),*fields)
        doc = getattr(PluginWikiWidgets,name).func_doc or ''
        if form_widget.accepts(request.vars):
            keys=['name: %s' % request.vars.name]
            for name in a:
                if request.vars[ESC+name]:
                    keys.append(name+': %s' % request.vars[ESC+name])
            widget_code=CODE('``\n%s\n``:widget' % '\n'.join(keys))
    else:
        doc=''
        form_widget=''
    return dict(form=form,form_widget=form_widget,doc=doc,
                widget_code=widget_code)
