# -*- coding: utf-8 -*- 

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import os
import datetime

from mil.base import File
from mil.base.jit import JIT

from gluon import current

from mil.core import file_configuration

from core.application import Application
from core.notifier import startSystemNotifier, SystemNotifier 
from core.userhandler import UserHandler
from core.government import Government

from system import ErrorToFile, LogToTable

# Base path. It is different depending if is called from web or script
if request.env.web2py_path:                     # Web
    w2p_path=request.env.web2py_path
else:                                           # Script
    w2p_path=os.getcwd()
app_path = os.path.join(w2p_path,request.folder)
if request.env.http_host:                       # Web
    w2p_url='http://'+request.env.http_host   
else:                                           # Script
    w2p_url=''
    
  
# Config files
static_file = os.path.join(app_path, 
                           'private', 
                           'configuration', 
                           'forge_config.yaml')
dynamic_file = os.path.join(app_path, 
                            'private', 
                            'configuration', 
                            'config.yaml')
skels_path = os.path.join(app_path, 'private', 'skels')  
       
# Configuration system
try:
    _config = file_configuration(static_file, dynamic_file, skels_path)
    
    # Start Bus System
    startSystemNotifier()
            
    # Launch system log
    _syslog = JIT(SystemNotifier)  
    _syslog.add_event_type('flow')
                
    # Listener: error
    error_dir = os.path.join(_config.get('filesystem_path'), 'logs')
    if not File.exists(error_dir):
        File.make_dir(error_dir)
    error_listener = ErrorToFile(os.path.join(error_dir, 'error.log'))  
    _syslog.subscribe(error_listener,'error')   
                
    # Listener: log
    _syslog.subscribe(LogToTable,'log')
    
    # Listener: flow
    from system import Flow
    _syslog.subscribe(Flow, 'flow')
    
    # System Log available using web2py current instance
    current._syslog = _syslog
    
    
    # Installation info instantiation
    if request. controller == 'installation' and request.function == 'info':
        # Configuration available using web2py current instance
        current._config = _config
        # User
        _user = None 
    else:
        # Application
        app = Application(app_path, _config)
        db = app.db
        _environ = app.environment

        # Disabble application
        if  _config.get('active') is False or _config.get('active') is None:
            if request.controller != 'default':
                redirect(URL(r=request,c='default',f='info', args=['disable']))
            else:
                if request.function != 'info':
                    redirect(URL(r=request,c='default',
                                 f='info', args=['disable']))
                else:
                    if len(request.args) > 0 and request.args[0] != 'disable':
                        redirect(URL(r=request,c='default',
                                     f='info', args=['disable']))
        
        # Defined elements
        _config.app_path = app_path
        _config.app_url = '%s/%s' % (w2p_url, request.application)
        
        #: Default datetime value
        _config.default_date = datetime.datetime.strptime('0001-01-01 00:00:00',
                                                          "%Y-%m-%d %H:%M:%S")
        #: Default date format  
        _config.default_datetime_format = "%Y-%m-%d %H:%M:%S"                                
        #: Default date format  
        _config.default_date_format = "%Y-%m-%d" 
                          
        #: Default DDBB migrate option
        _config.migrate = True
        
        # DAL available using web2py current instance
        current.db = db
        # Configuration available using web2py current instance
        current._config = _config
        # Environment available using web2py current instance
        current._environ = _environ
        
        # Authorization
        auth = app.auth
        # Auth available using web2py current instance
        current.auth = auth
            
        # User
        _user = UserHandler(auth.user)
        # User available using web2py current instance
        current._user = _user
        
        # Gobernment 
        _config.government_path = os.path.join(app_path, 
                                               'private', 'government') 
        _government = Government()
        # Government available using web2py current instance
        current._government = _government
        
        # Generic elements. This line is needed
        from gluon.tools import Crud, Service, PluginManager, prettydate
        crud, service, plugins = Crud(db), Service(), PluginManager()
        
        
        
        # Instantiate mail
        try:
            if _config.get('mailer')['active'] :
                from core.email import mailService
                mail = mailService()
                mail.settings.server =  _config.get('mailer')['server']  # your SMTP server
                mail.settings.sender =  _config.get('mailer') ['sender'] # your email
                mail.settings.login = _config.get('mailer')['login']     # your credentials
                
                auth.settings.mailer = mail
            else:
                mail = None
        except:
            mail = None
        current.mail = mail
        
        #try:
        #    if isinstance(session.token,dict):
        #        _user.load(session.token['user'],
        #                   session.token['email'],
        #                   session.token['username']) #,session.token['password'])
        #    else:
        #        _user.load(session.token[0], 
        #                   session.token[1], 
        #                   session.token[2]) #, session.token[3])
        #except Exception, e:
        #    pass
        #    
        ### Guest mode
        ##if vip_user.email is None:
        ##    vip_user.load('guest@localhost', False)
        ##_user = User()
        ## User available using web2py current instance
        #current._user = _user
        
        # Login flow
        from system.workflowrunner import FlowRunner
        fi =  FlowRunner('login')
        fi.run()
except Exception, e:
    # Guardar en fichero
    try:
        label = "%s [error] db.py: %s \n" % (datetime.datetime.today(), str(e))
        open(os.path.join(error_dir, 'error.log'),"a").write(label)
    except Exception, e:
        pass

    # Installation info instantiation
    if request. controller == 'installation' and request.function == 'index':
        # Configuration available using web2py current instance
        _config = None
        current._config = _config
        # User
        _user = None 
    else:
        redirect(URL(r=request, c='installation', f='index'))      
    
    