# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import datetime

from gluon import *

from calendar.models.calendar import CalendarModel

class EventModel(object):
    @staticmethod
    def define_table(db):        
        if 'event' in db.tables:
            return
        
        CalendarModel.define_table(db)
        db.define_table('event',
                        Field('title', 'string', required=True,  notnull=True, 
                              label=current.T('Title')),
                        Field('place', 'string', 
                              label=current.T('Place')),
                        Field('description', 'text', 
                              label=current.T('Description')),
                        Field('init_date', 'datetime', required=True, 
                              label=current.T('Init date')),
                        Field('end_date', 'datetime', required=True, 
                              label=current.T('End date')),
                        Field('calendar_id', 'reference calendar', 
                              label=current.T('Calendar')),
                        Field('all_day', 'boolean', default=False, 
                              required=True, label=current.T('All day')),
                        format='%(title)s'
                        )

    def __init__(self, db):
        self.db = db
        
        EventModel.define_table(db)
        
    def grid(self):
        form = SQLFORM.grid(self.db.event, 
                            searchable=False,
                            details=False,
                            csv=False, 
                            user_signature = False,
                            maxtextlength=100)
        return form
    
    def add(self, **kwargs):
        calendar_id = kwargs.get('calendar_id')
        if calendar_id is None:
            raise Exception ('calendar identifier needed')
        title = kwargs.get('title', "...")
        place = kwargs.get('place', "...")
        description = kwargs.get('description', "...")
        init_date = kwargs.get('init_date', datetime.datetime.now())
        end_date = kwargs.get('end_date', datetime.datetime.now() + datetime.timedelta(hours=1))
        all_day = kwargs.get('all_day',False)

        return self.db.event.insert(title=title,
                                    place=place,
                                    description=description,
                                    init_date=init_date,
                                    end_date=end_date,
                                    calendar_id=calendar_id,
                                    all_day=all_day)
    
    def get_events_from_range(self, user_id, start, end):        
        from datetime import datetime
        
        from calendar.models.user_calendar import UserCalendarModel
        UserCalendarModel.define_table(self.db)
        
        init_date = datetime.fromtimestamp(int(start))    
        end_date = datetime.fromtimestamp(int(end))
    
        query =((self.db.calendar.id == self.db.event.calendar_id) \
                & (self.db.user_calendar.calendar == self.db.calendar.id) \
                & (self.db.user_calendar.user_id == user_id) \
                & (self.db.event.init_date >= init_date) \
                & (self.db.event.init_date < end_date))
        return self.db(query).select(self.db.event.id, 
                                     self.db.event.title,
                                     self.db.event.all_day, 
                                     self.db.event.init_date, 
                                     self.db.event.end_date, 
                                     self.db.user_calendar.fullcalendar_class)
        
    def form_read(self, id):
        from gluon.tools import Crud
        return Crud(self.db).read(self.db.event, id)
    
    def delete_by_calendar(self, calendar_id):
        return self.db(self.db.event.calendar_id == calendar_id).delete()
    
    