# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import *

from calendar.models.calendar import CalendarModel

class UserCalendarModel(object):
    @staticmethod
    def define_table(db):        
        if 'user_calendar' in db.tables:
            return
        
        CalendarModel.define_table(db)
        db.define_table('user_calendar',
                        Field('calendar', 'reference calendar', 
                              required=True, notnull=True, 
                              label=current.T('Calendar')),
                        Field('color', 'string', default="blue", required=True, 
                              label=current.T('Color')),
                        Field('fullcalendar_class', 'string', required=True,
                              default="blablabla",
                              writable=False, readable=False),
                        Field('permissions', 'string', required=True, 
                              label=current.T('Permissions'), 
                              requires=IS_IN_SET(['r', 'rw', 'owner'], zero=None)),
                        Field('user_id', 'reference auth_user', required=True, 
                              notnull=True, label=current.T('User')),
                        format='')

    def __init__(self, db):
        self.db = db
        
        UserCalendarModel.define_table(db)
        
    def grid(self):
        form = SQLFORM.grid(self.db.user_calendar,
                            searchable=False, 
                            details=False,
                            csv=False, 
                            user_signature = False,
                            maxtextlength=100)
        return form
    
    def add_owner(self, user_id, calendar_id):
        return self.add_user(user_id, calendar_id, 'owner')
    
    def delete_by_calendar(self, calendar_id):
        return self.db(self.db.user_calendar.calendar == calendar_id).delete()
    
    def add_user(self, user_id, calendar_id, permission='r'):
        # Checks if exists
        query= ((self.db.user_calendar.calendar == calendar_id) & \
                (self.db.user_calendar.user_id == user_id))
        try:
            result = self.db(query).select(self.db.user_calendar.id).first().id
        except:
            result = None
        
        if result:
            return result
        return self.db.user_calendar.insert(calendar=calendar_id,
                                            permissions=permission,
                                            color="blue",
                                            fullcalendar_class="blablabla",
                                            user_id=user_id)
    
    def remove_user(self, user_id, calendar_id):
        # Checks if exists
        query= ((self.db.user_calendar.calendar == calendar_id) & \
                (self.db.user_calendar.user_id == user_id))
        self.db(query).delete()    
    
    def get_permission(self, calendar_id, user_id):
        query= ((self.db.user_calendar.calendar == calendar_id) & \
                (self.db.user_calendar.user_id == user_id))
        result = self.db(query).select(self.db.user_calendar.permissions)
        if len(result) == 1:
            if result[0].permissions == "owner" or result[0].permissions == "rw":
                return True
        return False
    
    def configure_form(self, cal_id):
        from calendar import CalendarHandler
        if not CalendarHandler.can_modify_calendar(cal_id):
              self.db.calendar.name.writable = False
        
        calendar = self.db((self.db.calendar.id == cal_id) &
                           (self.db.user_calendar.calendar == cal_id)).select()[0]
        
        #generate the defaults
        self.db.calendar.id.default = calendar.calendar.id
        self.db.calendar.name.default = calendar.calendar.name
        self.db.calendar.parent_id.default = calendar.calendar.parent_id
        self.db.user_calendar.calendar.default = calendar.user_calendar.calendar
        self.db.user_calendar.color.default = calendar.user_calendar.color
        self.db.user_calendar.fullcalendar_class.default = calendar.user_calendar.fullcalendar_class
        self.db.user_calendar.permissions.default = calendar.user_calendar.permissions
        self.db.user_calendar.user_id.default = calendar.user_calendar.user_id
        
        
        self.db.calendar.id.readable = self.db.calendar.id.writable = False
        self.db.calendar.parent_id.readable = self.db.calendar.parent_id.writable = False
        self.db.user_calendar.calendar.readable = self.db.user_calendar.calendar.writable = False
        self.db.user_calendar.fullcalendar_class.readable = self.db.user_calendar.fullcalendar_class.writable = False
        self.db.user_calendar.permissions.readable = self.db.user_calendar.permissions.writable = False
        self.db.user_calendar.user_id.readable = self.db.user_calendar.user_id.writable = False
        
        return SQLFORM.factory(self.db.calendar, self.db.user_calendar)

    def configure_update(self, cal_id, vars):
        calendar = self.db((self.db.calendar.id == cal_id) &
                           (self.db.user_calendar.calendar == cal_id)).select()[0]
        calendar.calendar.update_record(vars)
        calendar.user_calendar.update_record(vars)
