# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import *
from gluon.utils import web2py_uuid

from mil.core import RESTAuth
from mil.crypt import MD5PACKED

class OMWAuth(RESTAuth): 
    def __init__(self, environment=None, db=None, mailer=True,
                 hmac_key=None, controller='default', cas_provider=None):
        RESTAuth.__init__(self, environment, db, mailer, hmac_key, controller, 
                          cas_provider)
                
        ## create all tables needed by auth if not custom tables
        self.settings.extra_fields['auth_user'] = [
                                        Field('identifier', 
                                              'string',
                                              length=36, 
                                              default=web2py_uuid(), 
                                              notnull=True, 
                                              label=current.T('Identifier'), 
                                              writable=False, 
                                              readable=False),
                                        Field('preferences', 
                                              'string', 
                                              length=400, 
                                              default='', 
                                              label=current.T('Preferences'), 
                                              writable=False, 
                                              readable=False),
                                        Field('md5word', 
                                              'password', 
                                              length=512, 
                                              writable=False, 
                                              readable=False, 
                                              label=current.T('MD5 Password'))
                                                   ]
        self.settings.create_user_groups = False
        self.define_tables() 
        db.auth_user.md5word.requires=MD5PACKED(hmac_key)
        
        def on_validation(form):
            from mil.crypt import md5packed_crypt
            if form.vars.password_two:                
                form.vars.md5word = md5packed_crypt(self.settings.hmac_key,
                                                    form.vars.password_two)
                
#        def on_accept(form):
#            # Add user into htpasswd file
#            from system import htpasswd_add
#            htpasswd_add(form.vars.email, form.vars.password_two)
            
        self.settings.register_onvalidation = on_validation
#        self.settings.register_onaccept = on_accept
        
        # Login url
        self.settings.login_url = URL(c='default', f='user', args=['login'])
        
        # Change auth_user and auth_group format
        try:
            db.auth_user._format = '%(first_name)s %(last_name)s (%(email)s)'
            db.auth_group._format = '%(description)s'
        except Exception, e:
            pass

        
    def external_login(self, user, passwd, path=None):        
        table_user = self.settings.table_user
        if self.settings.login_userfield:
            username = self.settings.login_userfield
        elif 'username' in table_user.fields:
            username = 'username'
        else:
            username = 'email'
        if username in table_user.fields or not self.settings.login_email_validate:
            tmpvalidator = IS_NOT_EMPTY(error_message=self.messages.is_empty)
        else:
            tmpvalidator = IS_EMAIL(error_message=self.messages.invalid_email)
        old_requires = table_user[username].requires
        table_user[username].requires = tmpvalidator
        
        passfield = self.settings.password_field
        try: table_user[passfield].requires[-1].min_length = 0
        except: pass
                
        vars = {username: user, passfield: passwd}
        
        # check for username in db
        user = self.db(table_user[username] == vars[username]).select().first()
        if user:
            # user in db, check if registration pending or disabled
            temp_user = user
            if temp_user.registration_key == 'pending':
                user = None
            elif temp_user.registration_key in ('disabled','blocked'):
                user = None
            elif not temp_user.registration_key is None and \
                     temp_user.registration_key.strip():
                user = None
                
            # Checks password
            if temp_user[passfield] != vars.get(passfield, ''):
                # Checks if password is not crypt
                crypt = CRYPT(key=self.settings.hmac_key, 
                              min_length=self.settings.password_min_length)
                if temp_user[passfield] != crypt(vars.get(passfield))[0]:
                    user = None
                    
            # Path validation
            if user: 
                from gluon.storage import Storage
                
                # First, we check if user is administrator
                if self.has_membership(user_id=user.id, role='admin'):
                    return Storage(read=True, write=True, admin=True) 
                
                # Get project == path
                current.T = lambda s: s
                current.auth = self
                from project.models.card import ProjectCardModel
                ProjectCardModel.define_table(self.db)
                project = self.db(self.db.project_card.name == path).select().first()
                
                # Get user acl  
                from project.models.profile import ProjectProfileModel
                from project.models.team import ProjectTeamModel
                ProjectProfileModel.define_table(self.db)
                ProjectTeamModel.define_table(self.db)
                
                acl = Storage(read=False, write=False, admin=False)
                try:
                    # Search user's profile (in project)
                    query = ((self.db.project_user_profile.id == self.db.project_team.profile) & \
                             (self.db.project_team.project == project.id) &\
                             (self.db.project_team.user==user.id))
                    data = self.db(query).select(self.db.project_user_profile.read_auth, 
                                                 self.db.project_user_profile.write_auth, 
                                                 self.db.project_user_profile.exec_auth).first()
                    if data:
                         acl.read = data.read_auth
                         acl.write = data.write_auth
                         acl.admin = data.exec_auth
                except Exception, e:
                    pass
                return acl
        return None