# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import os.path 

from gluon import current
from gluon.storage import Storage
from gluon.tools import Mail

from mil.base import File, YAMLFile

class mailTemplate:
    filepath=''
    data=None
    
    def __init__(self,name):
        self.mail_templates_path = os.path.join(current._config.app_path, 
                                                'static', 
                                                'email')
        self.filepath=os.path.join(self.mail_templates_path, '%s.yaml' % name)
        if File.exists(self.filepath) is False:
            raise Exception('File not exists')
        
        self._load()
        
    def _load(self):
        try:
            data = YAMLFile.load(self.filepath)
        except Exception,e:
            data = {}
        self.data = Storage(data)
        
    def get(self, lang):
        if lang in self.data.template.keys():
            return str(self.data.template[lang])
        else:
            return str(self.data.template[self.data.default])
        
    def get_subject(self, lang):
        if lang in self.data.template.keys():
            return str(self.data.subject[lang])
        else:
            return str(self.data.subject[self.data.default])
        
             

class mailService(Mail): 
    def is_enabled(self):
        try:
            if not isinstance(self.settings.server, str):
                raise Exception('Server address not specified')
            if not isinstance(self.settings.sender, str):
                raise Exception('Sender address not specified')
            if not isinstance(self.settings.login, str):
                raise Exception('Login data not specified')
            return current._config.get('mailer')['active']
        except:
            return False
           
    def send(self, to, subject, message):          
        if self.is_enabled():
            return Mail.send(self, to, subject, message)
        else:
            if self.settings.server == 'mock':
                return self.mock_send(to, subject, message)
           
    def send_from_template(self, **kwargs ):         
        from gluon.template import render
        from gluon.storage import Storage
        
        # Arguments
        to = kwargs.get('to', None)    
        template = kwargs.get('template', None) 
        data = kwargs.get('data', {})     
        lang = kwargs.get('lang', '')      
        subject = kwargs.get('subject', '') 
        
        if to is None or template is None:
            current._syslog.error('Mail.send_from_template: needed to and template', 
                                  'warning') 
            return False
        
        # Data to template
        diccionario = dict()  
        diccionario["T"] = current.T    
        diccionario["data"] = data
            
        # Template
        template = mailTemplate(template)
        message = render(content=template.get(lang), context=diccionario)
        # Subject
        if subject == '':
            subject = render(content=template.get_subject(lang), 
                             context=diccionario)
        
        if self.is_enabled():
            return self.send(to, subject, message)
        else:
            if self.settings.server == 'mock':
                return self.mock_send(to, subject, message)
    
    def mock_send(self, to, subject, message):
        from mil.tools.common import get_date_timestamp
        
        data = "To: %s\nSubject: %s\n\n%s"%(to, subject, message)
        filename=os.path.join(current._config.app_path, 
                              'private', 
                              'data', 
                              'test', 
                              'test_mail_%s' % get_date_timestamp())
        File.save(filename, data)
        return True