# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import current

from mil.tools import str2bool

class Government(object):
    def __init__(self):
        pass   
    
    def check(self, module, funct):
        def wrapper(action,self=self):
            def f(_action=action,_self=self,*a,**b):
                # Check permission
                return _action()
            f.__doc__ = action.__doc__
            f.__name__ = action.__name__
            return f
        return wrapper
    
    def check_configuration(self, module, action, value, **kwargs ):
        # Arguments
        subapp = kwargs.get('application', current.request.controller)
        profile = 0 # Database requirement
        
        try:
            # Auth management
            auth_data = self._get_permissions(profile)
            
            # Checking
            key = '%s_%s_%s' % (subapp, module, action)
            if  auth_data.has_key(key):
                if auth_data[key] == value:
                    return True
            return False
        except Exception, e:
            current._syslog.error('Government.check_configuration: %s' % str(e), 
                                  'warning') 
        
        # Default return
        return False
    
    def has_permission(self, module, action, **kwargs):
        # Arguments
        subapp = kwargs.get('application', current.request.controller)
        profile = kwargs.get('profile', None)

        # User information
        user = current._user
        # If user is administrator has access always
        if user.is_admin:
            return True
                
        # Profile management
        if profile is None:
            if user.has_key(subapp):
                profile = user[subapp]['profile']
            
        # Auth management
        if user.has_key(subapp):
            auth_data = user[subapp]['auth']
        else:
            auth_data = self._get_permissions(profile)
        
        # Checking
        try:
            if profile is not None:
                key = '%s_%s_%s' % (subapp, module, action)
                if  auth_data.has_key(key):
                    if str2bool(auth_data[key]):
                        return True
            return False
        except Exception, e:
            current._syslog.error('Government.has_permission: %s' % str(e), 
                                  'warning') 
       
        # Default return
        return False
    
    def _get_permissions(self, profile):
        from system import AuthConfiguration
        auth_mng = AuthConfiguration(current.db, profile)
        all_data = auth_mng.get_all()
        
        data = {}
        if all_data:
            for row in all_data:
                key = '%s_%s_%s' % (row.application, row.module, row.action)
                data[key] = row.value
        return data
    
    
    def get_module_permissions(self, module, **kwargs):
        # Arguments
        subapp = kwargs.get('application', current.request.controller)
        profile = kwargs.get('profile', None)

        # User information
        user = current._user
                
        # Profile management
        if profile is None:
            if user.has_key(subapp):
                profile = user[subapp]['profile']
            
        # Auth management
        if user.has_key(subapp):
            auth_data = user[subapp]['auth']
        else:
            auth_data = self._get_permissions(profile)
        
        # Checking
        try:
            if profile is not None:
                key = '%s_%s_' % (subapp, module)
                
                data = {}
                for pkey in auth_data.keys(): 
                    if pkey.startswith(key):
                        nkey = pkey.replace(key, '')
                        # If user is administrator has access always
                        if user.is_admin:
                            data[nkey] = True
                        else:
                            data[nkey] = str2bool(auth_data[pkey])
                return data
        except Exception, e:
            current._syslog.error('Government.get_all_permissions: %s' % str(e), 
                                  'warning') 
       
        # Default return
        return False    
    def get_module_permissions(self, module, **kwargs):
        # Arguments
        subapp = kwargs.get('application', current.request.controller)
        profile = kwargs.get('profile', None)

        # User information
        user = current._user
                
        # Profile management
        if profile is None:
            if user.has_key(subapp):
                profile = user[subapp]['profile']
            
        # Auth management
        if user.has_key(subapp):
            auth_data = user[subapp]['auth']
        else:
            auth_data = self._get_permissions(profile)
        
        # Checking
        try:
            if profile is not None:
                key = '%s_%s_' % (subapp, module)
                
                data = {}
                for pkey in auth_data.keys(): 
                    if pkey.startswith(key):
                        nkey = pkey.replace(key, '')
                        # If user is administrator has access always
                        if user.is_admin:
                            data[nkey] = True
                        else:
                            data[nkey] = str2bool(auth_data[pkey])
                return data
        except Exception, e:
            current._syslog.error('Government.get_all_permissions: %s' % str(e), 
                                  'warning') 
       
        # Default return
        return False
        
    def get_module_permissions_by_profile(self, module, **kwargs):
        # Arguments
        subapp = kwargs.get('application', current.request.controller)
        profile = kwargs.get('profile', None)

        # User information
        user = current._user
                
        # Profile management
        if profile is None:
            if user.has_key(subapp):
                profile = user[subapp]['profile']
            
        # Auth management
        if user.has_key(subapp):
            auth_data = user[subapp]['auth']
        else:
            auth_data = self._get_permissions(profile)
        
        # Checking
        try:
            if profile is not None:
                key = '%s_%s_' % (subapp, module)
                
                data = {}
                for pkey in auth_data.keys(): 
                    if pkey.startswith(key):
                        nkey = pkey.replace(key, '')
                        data[nkey] = str2bool(auth_data[pkey])
                return data
        except Exception, e:
            current._syslog.error('Government.get_all_permissions: %s' % str(e), 
                                  'warning') 
       
        # Default return
        return False