# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import current, IS_IN_SET, redirect, URL
from gluon.storage import Storage
from gluon.sql import SQLField
from gluon.sqlhtml import form_factory

class ProfileHandler(object):
    @staticmethod
    def edit(db, app_id, id, profile_data):        
        form = form_factory(SQLField('profile', length=512, writable=False,
                                     default=profile_data.role,
                                     label=current.T('Role')),
                            SQLField('description', 'text',
                                     default=profile_data.label,
                                     label=current.T('Description')))
    
        if form.accepts(current.request.vars, current.session):
            # Check args
            description = current.request.vars.description
            try:          
                # Change web2py group description
                db((db.auth_group.id==id)).update(description=description)
                current.session.flash = current.T('Role changed')
                   
                redirect(URL(c='government', f='profiles', args=[app_id]))
            except Exception, e:
                pass  
        return form
    
    @staticmethod
    def delete(_config, db, app_id, id):
        # Delete Configuration
        from system import AuthConfiguration
        auth_mng = AuthConfiguration(db, id)
        auth_mng.delete_by_profile()
        
        # Delete Project profile
        from masters.profiles import Profiles
        inKlass = Profiles(_config, db)
        inKlass.delete_subapp_profile(app_id, id)
        
        # Delete Group
        current.auth.del_group(id)
     
    @staticmethod
    def configure(db, app_id, id, profile_data):
        from services import ServiceManager
        smng = ServiceManager()
        
        from system import AuthConfiguration
        auth_mng = AuthConfiguration(db, id)
        data = None
        
        if app_id in smng.services.keys():
            app_data = smng.services[app_id]  
            
            if len(current.request.vars) > 0:             
                def prepare_vars(subapp, name, action):
                    d_action = []
                    try:
                        type = action['type']
                    except:
                        type = 'alone'
                    d_action.append([subapp, name, action['id'], type])
                    
                    try:
                        for option in action['options']:
                            d_action.extend(prepare_vars(subapp, name, option))
                    except:
                        pass
                      
                    return d_action
                
                # Subapp actions
                tmp = []
                for action in app_data['actions']:                            
                    tmp.extend(prepare_vars(app_id, app_id, action))
                #Services
                for service_id in app_data['services'].keys():
                    all_data = app_data['services'][service_id] 
                    try:
                        for action in all_data['actions']:                            
                            tmp.extend(prepare_vars(app_id, service_id, action)) 
                    except:
                        pass
                    
                if len(tmp) > 0:
                    for item in tmp:
                        id = '%s_%s_%s' % (item[0], item[1], item[2])
                        if item[3] == 'alone':
                            value = False
                            if id in current.request.vars:
                                value = True
                            else:
                                value = False
                        else:
                            value = False
                            if id in current.request.vars:
                                value = current.request.vars['%s_option' % id]
                            else:
                                value = None
                            
                        # Add new row
                        auth_mng.add(item[0], item[1], item[2], value)
                                
            def prepare_action(subapp, name, action, value=None):
                d_action = {}
                d_action['id'] = '%s_%s_%s' % (subapp, name, action['id'])
                d_action['label'] = action['label']
                try:
                    d_action['type'] = action['type']
                except:
                    d_action['type'] = 'alone'
                
                if value is None:
                    value = auth_mng.get_value(subapp, name, action['id'])
                    
                d_action['checked'] = 'False'
                if value is None:
                    try:
                        if action['default']:
                            d_action['checked'] = 'True'
                    except:
                        pass
                else:
                    if d_action['type'] == 'alone':
                        d_action['checked'] = value
                        value = None
                    else:
                        if d_action['type'] == 'option':
                            d_action['checked'] = 'False'
                            if value == d_action['id']:
                                d_action['checked'] = 'True'
                        else:
                            d_action['checked'] = 'True'
                    
                try:
                    d_action['options'] = []
                    for option in action['options']:
                        d_action['options'].append(prepare_action(subapp, 
                                                                  name,
                                                                  option,
                                                                  value))
                           
                except:
                    pass
                return d_action
            
            data = []
            # Subapp actions
            tmp = {'literal': app_data['literal'], 'actions': [ ]}
            for action in app_data['actions']:                            
                tmp['actions'].append(prepare_action(app_id, app_id, action))
            data.append(tmp)
            #Services
            for service_id in app_data['services'].keys():
                all_data = app_data['services'][service_id] 
                tmp = {'literal': all_data['label'], 'actions': []}
                try:
                    for action in all_data['actions']:                            
                        tmp['actions'].append(prepare_action(app_id, 
                                                             service_id,
                                                             action))                          
                except:
                    pass
                data.append(tmp)
            
        return data