# -*- coding: utf-8 -*- 

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import inspect
import urllib

from mil.tools import get_request_args

from gluon import current, redirect, URL, HTTP
from gluon.storage import Settings, Storage
from gluon.tools import call_or_redirect

class SubAppModule(object):
    settings = Settings()
    def __init__(self, controller, module, view=None):
        self.settings.controller = controller
        self.module = module
        if view is None:
            self.t_view = module
        else:
            self.t_view = view            
        
    def __call__(self):
        request = current.request
        args = request.args
        if not args:
            redirect(SubApp.url(self.settings.controller,
                                args='index',
                                vars=current.request.vars))
        
        f_request = args[0]
            
        if hasattr(self, args[0]):
            function = args[0]
            del current.request.args[0]
        else:
            function = 'index'
        
        try:
            SubApp.set_view(self.t_view, function)
            return getattr(self, function)()
        except Exception, e:
            current._syslog.error('SubAppModule._call__: %s' % str(e), 'warning')
            if current.response.view.rfind('html') > -1:
                message_1 = "%s: '%s/%s/%s'" % (
                                      current.T('Your request is not available'), 
                                      self.settings.controller,
                                      self.module,
                                      f_request)
                message_2 = "%s: '%s/index'" % (
                                      current.T('You had been redirected to'), 
                                      self.settings.controller)
                current.session.flash = "%s: %s. %s" % (current.T('Warning'), 
                                                        message_1,
                                                        message_2)
                redirect(URL(c=self.settings.controller, f='index'))
            else:
                raise HTTP(405, 'no json')
            
#        if function == 'api':
#            return self.api()
#        else:
#            SubApp.set_view(self.module, function)
#            return getattr(self, function)()

    def redirect(self, data):
        from system.workflowrunner import workflow_redirect
        # Do not try except next sentence.
        workflow_redirect(data)
    
class SubAppInstance(object):
    name = None
    parametric_services = {}
    services = {}
    def __init__(self, name):
        self.name = name
        self.services = self.set_services()
        
    def set_services(self):
        self.services = []

    def get_menu(self):
        return None
    
    def get_actions(self):
        return None
    
    def get_services(self):
        services = []
        for service in self.services:
            if service.default == False:
                tmp = Storage(id=service.id,
                              function=service.function, 
                              label=service.label, 
                              default=service.default)
                services.append(tmp)
        return services
        
    def get_services_id(self):
        key = []
        for service in self.services:
            key.append(service.id)
        return key
        
    def get_service(self, id):
        ret_val = None
        for service in self.services:
            if service.id == id:
                ret_val = service

        return ret_val
    
    def get_configuration(self):
        try:            
            data = {}
            data['controller'] = self.name
        except Exception, e:
            data = None
            
        return data
    
    def has_permission(self, **kwargs):
        return True
    

class SubApp:    
    """
    This class implements the basic statics methods to handle a Subapplication.
    """
    @staticmethod        
    def set_view(module_name, function):
        if function is None:            
            current.response.view = "%s/%s.%s" % (module_name, 
                                                  function, 
                                                  current.request.extension)
        else:
            current.response.view = "%s/%s/%s.%s" % (current.request.controller, 
                                                     module_name,
                                                     function, 
                                                     current.request.extension)
        
    @staticmethod
    def url(c, f=None, args=None, vars=None):
        if args is None: args=[]
        if vars is None: vars={}
        return URL(c=c, f=f, args=args, vars=vars)
    
    @staticmethod
    def check_type(type, name):
        if type == 'project':
            from project import ProjectHandler
            ProjectHandler.user_set_project(name) 
            
    @staticmethod   
    def requires(condition, requires_login=True):

        def decorator(action):

            def f(*a, **b):          
                
                basic_allowed, basic_accepted, user = current.auth.basic()
                user = user or current.auth.user
                if requires_login:
                    if not user:
                        if current.auth.settings.allow_basic_login_only or \
                                basic_accepted or current.request.is_restful:
                            raise HTTP(403,"Not authorized")
                        elif current.request.ajax:
                            return A('login',
                                     _href=current.auth.settings.login_url)
                        else:
                            next = current.auth.here()
                            current.session.flash = current.response.flash
                            return call_or_redirect(
                                current.auth.settings.on_failed_authentication,
                                current.auth.settings.login_url+\
                                    '?_next='+urllib.quote(next))

                if callable(condition):
                    flag = condition()
                else:
                    flag = condition
                if not flag:
                    current.session.flash = current.auth.messages.access_denied
                    return call_or_redirect(URL(c='dashboard', f='index'))

                return action(*a, **b)
            
            f.__doc__ = action.__doc__
            f.__name__ = action.__name__
            f.__dict__.update(action.__dict__)
            return f

        return decorator

    @staticmethod   
    def requires_login():
        """
        decorator that prevents access to action if not logged in
        """
        return SubApp.requires(True)

    @staticmethod   
    def requires_membership(role=None, group_id=None):
        """
        decorator that prevents access to action if not logged in or
        if user logged in is not a member of group_id.
        If role is provided instead of group_id then the
        group_id is calculated.
        """
        return SubApp.requires(lambda: current.auth.has_membership(
                                                           group_id=group_id, 
                                                           role=role))
    