# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import os.path
import base64

from gluon import *
from gluon.storage import Storage
import gluon.contrib.simplejson as json

class UserHandler(Storage):
    def __init__(self, user=None):
        if isinstance(user, Storage):
            for key in user.keys():
                if key != 'preferences':
                    self[key] = user[key]
            self._preferences()
            self.is_logged = True
            if current.auth.has_membership(user_id=self.id, role='admin'):
            	self.is_admin = True
            else:
            	self.is_admin = False
#            self.set_acl(True, True, True)
            self.set_language()
        else:
            self.first_name = ''
            self.last_name = ''
            self.email = 'annonymous@host.ext'
            self.identifier = None
            self.preferences=dict()
            self.is_logged = False
            
#            self.set_acl(False, False, False)
    
    def __str__(self): 
#        read_auth = '_'
#        write_auth = '_'
#        ejec_auth = '_'
#        if self.read:
#            read_auth = 'r'
#        if self.write:
#            write_auth = 'w'
#        if self.ejec:
#            ejec_auth = 'x'
#        auth = "%s%s%s" %(read_auth, write_auth, ejec_auth)
        auth = ''
        is_admin = '_'
        if self.is_admin:
            is_admin = 'A'
        return "(%s) %s %s - %s [%s%s]" % (self.identifier, 
                                    self.first_name, 
                                    self.last_name, 
                                    self.email,
                                    is_admin,
                                    auth)
        
    def _preferences(self):
        query = (current.db.auth_user.identifier==self.identifier)
        pf = current.db(query).select(current.db.auth_user.preferences).first()
        try:
            self.preferences=json.loads(pf.preferences)
        except:
            self.preferences=dict()                
            
    def set_preferences(self, preferences):
        self.preferences = preferences
        prefs = json.dumps(self.preferences)
        query = (current.db.auth_user.identifier==self.identifier)
        current.db(query).update(preferences=prefs)
        
    def set_language(self):
        try:
            if self.preferences['lang']:
                if self.preferences['lang'] != '':
                    T.force(self.preferences['lang'])
        except:
            pass
        
    def get_folder_path(self):
        from mil.base import File
        # Root path from user
        fs_path = current._config.get('filesystem_path')
        if fs_path.startswith('{$app}'):
            rel = fs_path.replace('{$app}', '')
            if rel.startswith('/'):
                rel = rel[1:]
            relative_path = os.path.join(current._config.app_path, rel)
        else:
            relative_path = fs_path
            
        user_path = os.path.join(relative_path,
                                 'users',
                                 self.email)
        # Checks if path exists. If not, it will create it
        File.make_dir(user_path)
        
        return user_path
    
    def get_clean_password(self):
        try:
            from mil.crypt import md5packed_decrypt
            query = (current.db.auth_user.identifier == self.identifier)
            pc = current.db(query).select(current.db.auth_user.md5word).first().md5word
            return md5packed_decrypt(current._config.get('hmac_key'), pc)
        except:
            if self.email == 'annonymous@host.ext':
                return 'annonymous'
            else:
                return None
    
    def get_external_password(self, type=None):
        return self.get_clean_password()
#        ext_password = self.password
#        if type == 'dav':
#            if current._config.get('webdav_type') != 'pywebdav':
#                if current._config.get('auth_type') == 'authz':
#                    ext_password = self.get_clean_password()
#            
#        else:
#            if current._config.get('auth_type') == 'authz':
#                ext_password = self.get_clean_password()
#
#        return ext_password
    
#    def check_is_admin(self):
#        if current.auth.has_membership(user_id=current._user.id, role='admin'):
#        if self.is_admin is False:
#            redirect(MAIN)

#    def check_is_config_admin(self,is_admin=False):
#        # Check if is default admin
#        if is_admin is False:  
#            try:
#                pos=_config.get('admin_email').index(self.email)
#                is_admin=True 
#            except:
#                is_admin=False
#        self.is_admin=is_admin
#        return is_admin
        
#    def set_password(self,passwd):
#        self.password=passwd
#        db(db.user.email==self.email).update(password=self.password)
        
                    
#    def crud_represent(self, id):
#        # Prepare DIV
#        params = []
#        params.append(id)
#            
#        div = DIV() 
#        if self.write:
#            div.append(A(current.T("Edit"), _href=URL(r=request, f='update', args=params)))
#            div.append(" ")
#        if  self.read:
#            div.append(A(current.T("Show"), _href=URL(r=request, f='read', args=params)))
#            div.append(" ") 
#        if  self.write:        
#            div.append(A(current.T("Delete"), _href=URL(r=request, f='delete', args=params), _onclick="return confirm('" + current.T("Are you sure to delete?")+"')"))
#        return div
   
    def prefs(self, request, response, items=None):
        if self.is_logged:
            if items is None:
                items=[('logout',
                        current.T('exit'),
                        URL(request.application,
                            c='default',
                            f='user',
                            args='logout'))]
            user_data=SPAN(self.email,
                           ' ',
                           IMG(_src=URL(request.application,
                                        'static',
                                        response.visual_style+'/img/flecha_d.png'),
                               _name="act_prefs"),_class="user_prefs")
            user_prefs=DIV(
                           UL(*[LI(A(item,_href=action,_name=name)) for (name,item,action) in items]),
                           _class="prefs")
            bar=DIV(user_data,
                    user_prefs,
                    _class="auth_navbar",
                    _name='user_preferences')
            
            return TAG[''](bar,SCRIPT("prefs();"))
        else:
#            return auth.navbar(action=URL(request.application,'default','user'))
            action=URL(request.application,'default','user')
            login=A(current.T('login'),_href=action+'/login',_name='login')
            register=A(current.T('register'),_href=action+'/register')
            retrieve_username=A(current.T('forgot username?'),
                            _href=action+'/retrieve_username',
                            _name='retrive')
            lost_password=A(current.T('lost password?'),
                            _href=action+'/request_reset_password',
                            _name='lost_password')
            bar = SPAN('[ ',login,' ]',_class='auth_navbar')

            if not 'register' in current.auth.settings.actions_disabled:
                bar.insert(2, ' | ')
                bar.insert(3, register)
            if 'username' in current.auth.settings.table_user.fields() and \
                    not 'retrieve_username' in current.auth.settings.actions_disabled:
                bar.insert(-1, ' | ')
                bar.insert(-1, retrieve_username)
            if not 'request_reset_password' in current.auth.settings.actions_disabled:
                bar.insert(-1, ' | ')
                bar.insert(-1, lost_password)
            return bar
        
    # Forward compatibility
    def set_language(self):
        try:
            if self.preferences['lang']:
                if self.preferences['lang'] != '':
                    T.force(self.preferences['lang'])
        except:
            pass
        
    def set_acl(self, acl_read, acl_write, acl_exec):
        self.read = acl_read
        self.write = acl_write
        self.ejec = acl_exec        
        
    def clean_acl(self):
        self.read = False
        self.write = False
        self.ejec = False 
        
    @staticmethod
    def get_by_email(email):
        return current.db(current.db.auth_user.email == email).select().first()
    
    @staticmethod
    def get_by_id(id):
        return current.db(current.db.auth_user.id == id).select().first()
        
        
#    def get_http_auth(self):
#        if self.crypt is None:
#            return "basic %s" % base64.encodestring('%s:%s' % (self.email, self.password))[:-1]
#        else:
#            return "crypt %s" % self.crypt
#    
#    def get_http_basic(self):
#        return "basic %s" % self.crypt
#    
#    def get_admin_auth(self,type='cas'):
#        if type=='cas':
#            return "admin %s" % base64.b64encode(_config.get('cas_passphrase'))
#        elif type=='calendar':
#            return "admin %s" % base64.b64encode(_config.get('calendar_passphrase'))
#        elif type=='blog':
#            return "admin %s" % base64.b64encode(_config.get('blog_passphrase'))
#        else:
#            return None