# -*- coding: utf-8 -*-
#
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Based on Connector for elFinder File Manager
# author Troex Nevelin <troex@fury.scancode.ru>

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from .drivers import FileSystemDriver, WebdavDriver, SvnDriver

class ELFINDERDRIVER():
     FS = 'filesystem'
     WEBDAV = 'webdav'
     REPO_SVN = 'svn'

class elFinderConnector():
    """Connector for elFinder"""
    _allowed_drivers = ['filesystem', 'webdav', 'svn']
    _drivers = {'filesystem': 'FileSystemDriver',
                'webdav': 'WebdavDriver',
                'svn': 'SvnDriver'}
    
    _driver = None
        
    def __init__(self, driver, opts):
        if driver in self._allowed_drivers:
            constructor = globals()[self._drivers[driver]]
            self._driver = constructor(opts)            

    def run(self, httpRequest=[]):
        """main function"""
        self._driver.reset()        
        return self._driver.run(httpRequest)
    
    def get_http_allowed_parameters(self):
        return self._driver.httpAllowedParameters
        
        
