# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import time

from gluon import *

from mil.tools.common import get_unique_id, to_list_of_dict

class Actions(object):
    @staticmethod
    def define_table(db):
        db.define_table('pending_action',
            Field('action', 'integer', length=5, required=True, 
                  label=current.T('Action')),
            Field('params', 'string', length=255, required=True, 
                  label=current.T('Parameters')),
            Field('status', 'string', length=1, default=0, required=True, 
                  label=current.T('Status')),
            Field('start_date', 'integer', length=11, required=True, 
                  default=int(time.time()),
                  label=current.T('Start')),
            Field('end_date', 'integer', length=11, required=True, 
                  label=current.T('End')),
            Field('comment', 'text', required=True, default='',
                  label=current.T('Comments')),
            format='')
        
    def __init__(self, db):
        self.db = db
        
        Actions.define_table(db)
        
    def grid(self):
        form = SQLFORM.grid(self.db.pending_action, 
                            searchable=False,
                            csv=False, 
                            user_signature = False,
                            maxtextlength=100)
        return form
    
    def insert(self, action, params):
        return self.db.pending_action.insert(action=action,
                                             params=params,
                                             status=0,
                                             start_date=int(time.time()),
                                             end_date=int(time.time()),
                                             comment='')
    
    # Libforge interface
    def get_pending(self):
        query = (self.db.pending_action.status == 0)
        rows = self.db(query).select(orderby=self.db.pending_action.action)
        
        actions = []
        for row in rows:
            tmp = {}
            tmp['id'] = row.id
            tmp['type'] = row.action
            tmp['params'] = row.params.split()      # SPLIT Without separator
            
            actions.append(tmp)
        return actions    
    
    def update(self, id, stts, comment):
        if id is None:
            return False
        if stts:
            status = '1'
        else:
            status = '2'
        
        if comment is None:
            comment = ''
        
        row = self.db(self.db.pending_action.id == id).select().first()
        try:
            row.update_record(status=status,
                              end_date=int(time.time()),
                              comment=comment)
            return True
        except:
            return False  
    
    