# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import *

from mil.tools.common import to_list_of_dict

class Boxes(object):
    @staticmethod
    def define_table(db):
        db.define_table('available_boxes',
            Field('box_type', 'string', required=True, 
                  label=current.T('Box Type')),
            Field('list_url', 'string', required=True, 
                  label=current.T('List URL')),
            Field('content_url', 'string', required=True, 
                  label=current.T('Content URL')),
            Field('default_name', 'string', required=True, 
                  label=current.T('Default Name')),
            Field('modification_date', 'datetime', 
                  label=current.T('Modification date'), 
                  requires=IS_DATE(format=current.T("%Y-%m-%d %H:%M:%S"))),
            format='%(default_name)s')

        try:
            if db(db.available_boxes.id >= 1).count() == 0:
                db.available_boxes.insert(box_type='task', 
                                          list_url='',
                                          content_url='api/project/%s/task',
                                          default_name='Tareas') 
                db.available_boxes.insert(box_type='my_task', 
                                          list_url='api/task',
                                          content_url='api/project/%s/task',
                                          default_name='Mis Tareas')   
        except:
            pass
        
    def __init__(self, db):
        self.db = db
        
        Boxes.define_table(db)
        
    def grid(self):
        form = SQLFORM.grid(self.db.available_boxes, 
                            searchable=False,
                            csv=False, 
                            user_signature = False,
                            maxtextlength=100)
        return form
    

    def get_available(self):    
        rows = self.db().select(self.db.available_boxes.ALL) 
        return to_list_of_dict(rows.as_dict())