# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import *

from mil.tools.common import get_unique_id

class Flows(object):
    @staticmethod
    def define_table(db):
        db.define_table('flow',
            Field('identifier', 'string', length=20, 
                  default=get_unique_id(2,4), required=False, 
                  label=current.T('Identifier')),
            Field('name', 'string', length=50, requires=IS_ALPHANUMERIC(), 
                  required=True, notnull=True, unique=True, 
                  label=current.T('Name')),
            Field('description', 'text', label=current.T('Description')),
            format='%(name)s')
        
        try:
            if db(db.flow.id >= 1).count() == 0:
                db.flow.insert(identifier=get_unique_id(2,4),
                               name='create_project',
                               description='Alta de nuevo Proyecto')
        except:
            pass
        
    def __init__(self, db):
        self.db = db
        
        Flows.define_table(db)
        
    def grid(self):
        form = SQLFORM.grid(self.db.flow, 
                            searchable=False,
                            csv=False, 
                            user_signature = False,
                            maxtextlength=100)
        return form
    
    def get_all(self):
        return  self.db(self.db.flow.id > 0).select()

class Steps(object):
    @staticmethod
    def define_table(db):
        db.define_table('flow_step',
            Field('identifier', 'string', length=20, 
                  default=get_unique_id(2,4), required=False, 
                  label=current.T('Identifier')),
            Field('name', 'string', length=50, requires=IS_ALPHANUMERIC(), 
                  required=True, notnull=True, unique=True, 
                  label=current.T('Name')),
            Field('action', 'string', length=200, required=True, notnull=True, 
                  label=current.T('Action')),
            Field('parameters', 'string', length=200, 
                  label=current.T('Parameters')),
            Field('emit_params', 'string', length=200, 
                  label=current.T('Emit')),
            Field('receive_params', 'string', length=200, 
                  label=current.T('Receive')),
            format='%(name)s')
        
        try:
            if db(db.flow_step.id >= 1).count() == 0:
                db.flow_step.insert(identifier=get_unique_id(2,4),
                                    name='create_project_card',
                                    action='project_card',
                                    parameters='create',
                                    emit_params='controller,function,flow,step',
                                    receive_params='')
                db.flow_step.insert(identifier=get_unique_id(2,4),
                                    name='configure_services',
                                    action='project_service',
                                    parameters='configure',
                                    emit_params='controller,function,flow,step',
                                    receive_params='id')
                db.flow_step.insert(identifier=get_unique_id(2,4),
                                    name='add_project_lider',
                                    action='project_team',
                                    parameters='create',
                                    emit_params='controller,function,flow,step',
                                    receive_params='id,1')
                db.flow_step.insert(identifier=get_unique_id(2,4),
                                    name='desktop',
                                    action='desktop',
                                    parameters='index',
                                    emit_params='',
                                    receive_params='')
                db.flow_step.insert(identifier=get_unique_id(2,4),
                                    name='create_project',
                                    action='project',
                                    parameters='create',
                                    emit_params='controller,function,flow,step',
                                    receive_params='id')
        except:
            pass
        
    def __init__(self, db):
        self.db = db
        
        Steps.define_table(db)
        
    def grid(self):
        form = SQLFORM.grid(self.db.flow_step, 
                            searchable=False,
                            csv=False, 
                            user_signature = False,
                            maxtextlength=100)
        return form
    
    def get_all(self):
        return  self.db(self.db.flow_step.id > 0).select()
    
class FlowSteps(object):
    @staticmethod
    def define_table(db):
        db.define_table('flow_has_flow_step',
                        Field('key_primary', 'reference flow'),
                        Field('key_secondary', 'list:reference flow_step'),
                        format='') 
        
#        try:
#            if db(db.flow_has_flow_step.id >= 1).count() == 0:
#                if current._config.get('installation_type') == "lite":
#                    db.flow_has_flow_step.insert(key_primary='1',
#                                                    key_secondary='1|2|4|5')
#                    db.flow_has_flow_step.insert(key_primary='2',
#                                                    service='1|2|3|4|5|')
#                    db.flow_has_flow_step.insert(key_primary='3',
#                                                    service='1|2|3|4|5')
#                else:
#                    db.flow_has_flow_step.insert(key_primary='1',
#                                                    key_secondary='1|2|4|5|7|8')
#                    db.flow_has_flow_step.insert(key_primary='2',
#                                                    service='1|2|3|4|5|7|8')
#                    db.flow_has_flow_step.insert(key_primary='3',
#                                                    service='1|2|3|4|5|6|7|8')
#        except:
#            pass 
        
    def __init__(self, db):
        self.db = db

        FlowSteps.define_table(self.db)
        
    def get(self):
        insPrimary =  Flows(self.db)
        insSecondary = Steps(self.db)
        
        ret_dict = dict()
        ret_dict["key_primary"] = insPrimary.get_all()
        list_key_primary = dict()
        for item in ret_dict["key_primary"] :
            list_key_primary[item.id] = item.name
        ret_dict["list_key_primary"] = list_key_primary
        
        ret_dict["key_secondary"] = insSecondary.get_all()
        list_key_secondary = dict()
        for item in ret_dict["key_secondary"]:
            list_key_secondary[item.id] = item.name
        ret_dict["list_key_secondary"] = list_key_secondary
        
        ret_dict["proposal_items"] = \
            self.db(self.db.flow_has_flow_step.id > 0).select()
        
        return ret_dict
    
    
    def save_data(self):
        from gluon.contrib import simplejson
        from gluon.storage import Storage
        
        # This two lines are needed to define referenced tables
        insPrimary =  Flows(self.db)
        insSecondary = Steps(self.db)
        
        data = simplejson.loads(current.request.body.read())
        
        # First, we process current items        
        rows = self.db(self.db.flow_has_flow_step.id > 0).select()
        items_list = []
        for key in data["items"].keys():
            items_list.append(int(key))
        item_added = [] 
        for row in rows:
            if row.key_primary in items_list:  # Update data
                item_added.append(row.key_primary)
                list_items = []             
                for item in data["items"][str(row.key_primary)]:
                    list_items.append(int(item))                
                row.update_record(key_secondary = list_items)
            else:   # Delete row
                row.delete_record()
        
        # Finally, we added new items
        for key,val in data["items"].items():
            if int(key) not in item_added:
                list_items = []             
                for item in val:
                    list_items.append(int(item))            
                self.db.flow_has_flow_step.insert(key_primary = int(key), 
                                                  key_secondary = list_items)

