# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import os

from gluon import *

def define_table(db):
       db.define_table('guestbook',
           Field('project', 'reference project_card', 
                 label=current.T('Project'), 
                 writable=False, readable=False),
           Field('file','string', label=current.T('File'), 
                 writable=False, readable=False),
           Field('mail', 'string', required=True, 
                 label=current.T('E-mail')),
           Field('data', 'text', required=True, 
                 label=current.T('Objetive')),
           Field('is_platform_user', 'boolean', 
                 writable=False, readable=False),
           Field('date', 'datetime', label=current.T('Date'), 
                 writable=False, readable=False, 
                 requires=IS_DATE(format=current.T("%Y-%m-%d %H:%M:%S"))),    
           format='')
  
def is_guestbook_active(pid = None):
    """
    return 2 = guestbook forced by admin
    return 1 = active in project
    return 0 = not active
    """
    #first of all, we check if the global "_config.get('force_guestbook')" is set in our configuration system, if not, we add it.
    if current._config.get('force_guestbook') is None:
        current._config.set('force_guestbook', False)
#        add_force_guestbook_var_to_config()
    
    #first of all check if the guestbook is forced by the forge administrator
    if current._config.get('force_guestbook'):
        return 2   
    
    #check guestbook in the project
    if is_guestbook_active_in_project(pid):
        return 1
    
    return 0

def is_guestbook_active_in_project(pid):
    if pid == None:
        return False
    #check for the record
    reg = current.db(current.db.project_has_guestbook.project==pid).select().first()
    
    if reg == None:
        #if none we create a registry
        current.db.project_has_guestbook.insert(project=pid)
        #and return false
        return False
    return reg.guestbook_is_active
    

#def add_force_guestbook_var_to_config():
#    web2py_path=os.path.join(request.env.web2py_path,'applications')
#    config_name=os.path.join(web2py_path,request.application,'models','config.py')
#
#    data=[]
#    force_guestbook_value = False
#    if os.path.isfile(config_name):
#        # Read file    
#        fh = open(config_name, "r")
#        for line in fh.readlines():
#            line=line.strip(' \t\n\r')
#            data.append(line)
#        data.append("%s=%s" % ("FORCE_GUESTBOOK",force_guestbook_value))
#            
#        fh.close()
#    if data is not []:  
#        fh = open(config_name, "w")
#        fh.write('\n'.join(data))
#        fh.close()
#    #we add the FORCE_GUESTBOOK to globals() for this first try
#    globals()["FORCE_GUESTBOOK"] = force_guestbook_value


def enable_guestbook_in_project(project):
    return enable_disable_guestbook_in_project(project, True)
    
def disable_guestbook_in_project(project):
    return enable_disable_guestbook_in_project(project, False)

def enable_disable_guestbook_in_project(project, enable):
    current.db(current.db.project_has_guestbook.project == project).update(guestbook_is_active=enable)    

def enable_guestbook_in_platform():
    return enable_disable_guestbook_in_platform(True)    

def disable_guestbook_in_platform():
    return enable_disable_guestbook_in_platform(False)

def enable_disable_guestbook_in_platform(enable):
    current._config.set('force_guestbook', enable)
#    data = None
#
#    web2py_path=os.path.join(request.env.web2py_path,'applications')
#    config_name=os.path.join(web2py_path,request.application,'models','config.py')
#
#    # checks if config.py file exists
#    if os.path.isfile(config_name):
#        data=get_config_file_items(config_name)
#        retdata = []
#        for item in data:
#            retdata.append(item)
#            if item["name"] == "force_guestbook":
#                retdata[-1]["value"] = enable 
#
#        set_config_file_items(config_name, retdata)

def must_show_form(pid):
    
    #if the guestbook is not active, we do nothing 
    if is_guestbook_active(pid) == 0:
         return False    
    
    #if the guestbook is active and theres not data in session, we show the form
    if not is_guestbook_data_in_session(pid):
        return True
    #if the guestbook is active and there is data saved in the session
    #save automatically the data and allow the download process
    elif is_guestbook_data_in_session(pid):
        save_guestbook_data_for_project(request.vars.file,pid)
    return False
    

def is_guestbook_data_in_session(pid):
    try:
        if pid in session.guestbook.projects.keys():
            return True        
        return False
    except:     # in first access session.guestbook.projects not exists
        add_guestbook_data_to_session()
        return False
    
def add_guestbook_data_to_session():
    session.guestbook = Storage()
    session.guestbook.user_data = Storage()
    #session.guestbook.projects = list() # ERROR: in set_guestbook_data_session()
    session.guestbook.projects = dict()
    

def set_guestbook_data_session():
    from gluon.storage import Storage
    pid = str(request.vars.pid)
    if not "guestbook" in session:
        add_guestbook_data_to_session()
        
    session.guestbook.user_data.mail = request.vars.mail
    session.guestbook.user_data.is_platform_user = vip_user.logged
    
    #if not request.vars.pid in session.guestbook.projects: Changed to dict()
    if not request.vars.pid in session.guestbook.projects.keys():
        session.guestbook.projects[pid] = Storage() # ERROR session.guestbook.projects is a list ??
        session.guestbook.projects[pid].data = request.vars.data
            
    return
   
def get_guestbook_data_session(pid):
    from gluon.storage import Storage
    try:
        data = Storage()
        data.mail = session.guestbook.user_data.mail
        data.is_platform_user = session.guestbook.user_data.is_platform_user
        data.data = session.guestbook.user_data.project[pid].data
        
        return data
    except:
        return None


def process_guestbook():
    
    url = request.url
    file = request.vars.file    
    
    #get the project name from the file
    project_name = file.split("/")[2]
    
    pid = get_pid_from_name(project_name)
    
    if must_show_form(str(pid)):
        #redirect to form
        redirect(URL(r=request, c="guestbook", f="sign", vars=dict(file=file, proc_url = url, pid=pid)))
        return False
    #if not, we allow to continue the download process
    
    return


def save_guestbook_data_for_project(file,pid):
    
    
    if session.guestbook_skip_first:
        session.guestbook_skip_first = False
        return
    
    import datetime
    
    mail = session.guestbook.user_data.mail    
    is_platform_user = session.guestbook.user_data.is_platform_user
    data = session.guestbook.projects[pid].data
    date = datetime.datetime.now()
    project = pid  
    
    current.db.guestbook.insert(mail = mail, project = pid, file = file, data = data, date = date, is_platform_user = is_platform_user )

    return           