# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import *

class LanguageHandler:
    @staticmethod
    def define_table(db):
        db.define_table('language',
            Field('label', 'string', length=50, label=current.T('Description')),
            Field('encoding', 'string', length=5, label=current.T('Code')),
            Field('enabled', 'boolean', label=current.T('Enabled')),
            format='%(label)s')
        
        try:
            if db(db.language.id >= 1).count() == 0:
                db.language.insert(label='English',
                                   encoding='en-en',
                                   enabled=True)
                db.language.insert(label='Español',
                                   encoding='es-es',
                                   enabled=True)
                db.language.insert(label='Euskera',
                                   encoding='eu-es',
                                   enabled=True)
        except:
            pass 
        
    def __init__(self, db):
        self.db = db
        LanguageHandler.define_table(db)
        
    def get_all(self):
        rows=self.db((self.db.language.enabled==True)).select()
        
        ret_val=[]
        for row in rows:
            ret_val.append(row.label)
            
        return ret_val
    
    def get_all_as_dict(self):
        rows=self.db((self.db.language.enabled==True)).select()
        
        ret_val={}
        for row in rows:
            ret_val[row.encoding]=row.label
            
        return ret_val

    def get_all_languages(self):
        from gluon.fileutils import listdir
        path = os.path.join(APP_PATH, 'languages/')
        langs = []
#        langs=[DEFAULT_LANGUAGE]
        for language in listdir(path, '^\w+(\-\w+)?\.py$'):
            langs.append(language[:-3])
        return langs
            
    def get_encoding(self, id):
        row=self.db((self.db.language.id==id)).select().first()
        
        if row:
            return row.encoding
        else:
            return ''