# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################
import datetime

from gluon import *

from mil.tools.common import get_unique_id, to_list_of_dict

class Logs(object):    
    @staticmethod
    def define_table(db):
        if not 'log' in db.tables:
            db.define_table('log',
                Field('event', 'string', length=20, required=True, 
                      label=current.T('Event Type')),            
                Field('severity', 'string', length=20, required=True, 
                      label=current.T('Severity')),
                Field('message', 'text', required=True, 
                      label=current.T('Message')),
                Field('timestamp', 'datetime', label=current.T('Timestamp'), 
                      writable=False, readable=True, 
                      default=datetime.datetime.now(),
                      requires=IS_DATE(format=current.T("%Y-%m-%d %H:%M:%S"))), 
                format='%(value)s')
        
    def __init__(self, db):
        self.db = db
        
        Logs.define_table(db)
        
    def grid(self):
        form = SQLFORM.grid(self.db.log, 
                            searchable=False,
                            csv=False, 
                            user_signature = False,
                            maxtextlength=100)
        return form
    
    def insert(self, event, severity, message):
        self.db.log.insert(event=event,
                           severity=severity,
                           message=message)
    