# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import *
from gluon.storage import Storage

class Profiles(object):    
    @staticmethod
    def define_table(db):      
        if 'auth_profile' in db.tables:
            return
        
        db.define_table('auth_profile',
                        Field('application', 'string', length=200, 
                              required=True, notnull=True, 
                              label=current.T('Application')),
                        Field('group_id', 'reference auth_group', 
                              required=True, notnull=True, 
                              label=current.T('Group')),
                        Field('is_structural', 'boolean', default=True,
                              required=True, notnull=True, 
                              label=current.T('Structural')),
                        format='%(application)s')       
        
    def __init__(self, config, db):
        self.config = config
        self.db = db
        
        Profiles.define_table(db)
        
    def grid(self, subapp=None):
#        links = [lambda row: A(current.T('Configure'),
#                               _href=URL("profile","configure",args=[row.id]))]
        literal = "jQuery('#%(id)s').fadeIn();jQuery('#c%(id)s').html('%(label)s');web2py_component('%(callback)s', 'c%(id)s'); return false"
        links = [lambda row: A(current.T('Configure'),
                               _onclick=literal % dict(id='mm_add',
                                                       label=current.T('Loading ...'),
                                                       callback=URL("profile",
                                                                    "configure",
                                                                    args=[row.id])))]
        if subapp is None:
            query = (self.db.auth_group.id > 0)
        else:
            query = (self.db.auth_group.id == self.db.auth_profile.group_id) & \
                    (self.db.auth_profile.application==subapp)
        form = SQLFORM.grid(query, 
                            searchable=False,
                            csv=False, 
                            user_signature = False,
                            links=links,
                            maxtextlength=100)
        return form
    
    def get(self, id):
        query = (self.db.auth_group.id == id)
        return self.db(query).select().first()
    
    def get_all_by_subapp(self, subapp):
        query = (self.db.auth_group.id == self.db.auth_profile.group_id) & \
                (self.db.auth_profile.application==subapp)
        return self.db(query).select()
    
    def get_subapp_profiles(self, name):
        rows = self.get_all_by_subapp(name)
        data = []
        for row in rows:
            data.append(Storage(id=row.auth_group.id,
                                name=row.auth_group.description))
            
        return data
    
    def get_id_by_subapp_and_role(self, subapp, role):
        query = (self.db.auth_group.role == role) & \
                (self.db.auth_group.id == self.db.auth_profile.group_id) & \
                (self.db.auth_profile.application==subapp)
        row = self.db(query).select().first()
        
        if row:
            return row.auth_group.id
        else:
            return None
    
    def get_by_id(self, id):
        query = (self.db.auth_group.id == id)
        data = self.db(query).select().first()
        
        if data:
            return Storage(id=data.id,
                           role=data.role,
                           label=data.description)
        else:
            return None
    
    def add_profile(self, app, gp_id, structural=False):
        return self.db.auth_profile.insert(application=app, 
                                           group_id=gp_id,
                                           is_structural=structural)
        
    def delete_subapp_profile(self, app, id):
        query = (self.db.auth_profile.application==app) & \
                (self.db.auth_profile.group_id==id)
        self.db(query).delete()
