# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import *

from mil.tools.common import get_unique_id, normalize

from services.iservice import IService

from project.models.type_service import ProjectTypeServicesModel

_SERVICES = ['Project']

class Project(IService):
    _actions = [{'label': 'Alta de Proyecto', 
                 'id': 'proy_alta',
                 'options': [{'id': 'proy_alta_auth', 
                              'label': 'Automática', 
                              'default': True},
                             {'id': 'proy_alta_man', 
                              'label': 'Manual', 
                              'default': False}]},
                {'label': 'Validación', 
                 'id': 'proy_valid',
                 'options': [{'id': 'proy_valid_none', 
                              'label': 'No se usa validación', 
                              'default': True},
                             {'id': 'proy_valid_use', 
                              'label': 'Se usa Validación', 
                              'default': False}]},
                ]
    _controller = ''
    _literal = 'Projects'
    _permissions = ['add']
    
    @staticmethod
    def define_table(db): 
        ProjectTypeServicesModel.define_table(db)

        db.define_table('project',
                        Field('identifier', 'string', length=20, 
                              default=get_unique_id(2,4), required=True, 
                              notnull=True, label=current.T('Identifier'), 
                              readable=True, writable=False),
                        Field('title', 'string', length=200, required=True, 
                              notnull=True, label=current.T('Title')),
                        Field('name', 'string', length=200, 
                              requires=IS_ALPHANUMERIC(), required=False, 
                              notnull=False, unique=True, 
                              label=current.T('Name'), 
                              compute=lambda r:normalize(r.title)),
                        Field('type', 'reference project_type', required=True, 
                              notnull=True, label=current.T('Type')),
                        Field('private', 'boolean', required=True, notnull=True, 
                              default=False, label=current.T('Private')),
                        Field('deleted', 'boolean', readable=False, 
                              writable=False, notnull=True, default=False, 
                              label=current.T('Check to delete')),
                        Field('parameters', 'text', readable=False, 
                              writable=False, label=current.T('Data')),
                        format='%(name)s')       
        
    def __init__(self, db):
        self.db = db
        
        Project.define_table(db)
        
    def grid(self):
        def launch_flow(data):
            current._syslog.flow('new_project',
                                 current.auth.user.identifier, 
                                 identifier=data.custom.inpval.identifier)
        form = SQLFORM.grid(self.db.project, 
                            searchable=False,
                            csv=False, 
                            user_signature = False,
                            oncreate=lambda form: launch_flow(form),
                            maxtextlength=100)
        return form
    
    def get_all(self):
        return  self.db(self.db.project.id > 0).select()
    
   