# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import *
from gluon.myregex import regex_expose

from mil.base import File

class Services(object):
    @staticmethod
    def define_table(db):
        db.define_table('service',
                        Field('name', 'string', length=200, required=True, 
                              notnull=True, label=current.T('Name')),
                        Field('action', 'string', length=400, required=True, 
                              notnull=True, label=current.T('Action')),
                        Field('type', 'string', 
                              requires=IS_IN_SET([current.T('internal'), 
                                                  current.T('server'),
                                                  current.T('external')]), 
                              required=True, notnull=True, 
                              label=current.T('Type')),
                        format='%(name)s')

        
        try:
            if db(db.service.id >= 1).count() == 0:
                db.service.insert(name='Projects',
                                  action='project',
                                  type='internal')
                db.service.insert(name='Tasks',
                                  action='task',
                                  type='internal')
                db.service.insert(name='Incidences',
                                  action='incidence',
                                  type='internal')
                db.service.insert(name='Bugs',
                                  action='bug',
                                  type='internal') 
                db.service.insert(name='Roadmap',
                                  action='roadmap',
                                  type='internal')
                if current._config.get('installation_type') == "enterprise":
                    db.service.insert(name='Webdav',
                                      action='webdav',
                                      type='internal')    
                    db.service.insert(name='Code Repository (svn)',
                                      action='code',
                                      type='internal')
                db.service.insert(name='Calendars',
                                  action='calendar',
                                  type='internal')
                if current._config.get('installation_type') == "enterprise":
                    db.service.insert(name='ETR',
                                      action='etr',
                                      type='internal')
                db.service.insert(name='Wiki',
                                  action='wiki',
                                  type='internal')
                db.service.insert(name='File',
                                  action='file',
                                  type='internal')
        except:
            pass 
        
    def __init__(self, db):
        self.db = db
        
        Services.define_table(db)
        
    def grid(self):
        form = SQLFORM.grid(self.db.service, 
                            searchable=False,
                            csv=False, 
                            user_signature = False,
                            maxtextlength=100)
        return form
    
    def get_all(self):
        return  self.db(self.db.service.id > 0).select()
    
    def get_by_action(self, action):
        return  self.db(self.db.service.action == action).select().first()
    
    def get_all_from_code(self, dir_path):
        import os, inspect, random
        services = {}
        for fileName in os.listdir(dir_path):
            if fileName.endswith('.py') & (fileName != 'master.py'):
                moduleName = fileName[0:fileName.index('.py')]
                services[moduleName] = []
                
                data = File.read(os.path.join(dir_path,fileName))
                exposed = regex_expose.findall(data)
                for function in exposed:
                    services[moduleName].append(function)
        return services
    