# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import current, A, CRYPT, SQLFORM, URL
from gluon.storage import Storage

class Users(object):
    def __init__(self, config, db):
        self.config = config
        self.db = db
        
    def grid(self):
        def link_aprove(r):
            if r.registration_key == 'pending':
                return A(current.T('Aprove'),
                         _href=URL(r=current.request, 
                                   c="manage", 
                                   f="aprove", args=[r.id]))
            else:
                return ''
        def link_reject(r):
            if r.registration_key == 'pending':
                return A(current.T('Reject'),
                         _href=URL(r=current.request, 
                                   c="manage", 
                                   f="reject", args=[r.id]))
            else:
                return ''
        def link_enable(r):
            if r.id == 1:
                return ''
            if r.registration_key == 'pending':
                return ''
            if r.registration_key == 'disabled':    #'disabled','blocked'
                return A(current.T('Enable'),
                         _href=URL(r=current.request, 
                                   c="manage", 
                                   f="enable", args=[r.id]))
            else:
                return A(current.T('Disable'),
                         _href=URL(r=current.request, 
                                   c="manage", 
                                   f="disable", args=[r.id]))
        def link_admin(r):
            if r.id == 1:
                return ''
            if current.auth.has_membership('admin', r.id):    # is admin
                return A(current.T('User'),
                         _href=URL(r=current.request, 
                                   c="manage", 
                                   f="set_admin", args=[r.id]))
            else:
                return A(current.T('Administrator'),
                         _href=URL(r=current.request, 
                                   c="manage", 
                                   f="set_admin", args=[r.id]))
                
        def on_validation(form):
            from mil.crypt import md5packed_crypt
            if form.latest.password:
                form.vars.password_two = form.latest.password
                form.vars.md5word = md5packed_crypt(current._config.get('hmac_key'),
                                                    form.latest.password)
                
        def on_create(form):
            # Notification
            if current._government.check_configuration('email',
                                                       'add_user',
                                                       'True',
                                                       application='forge'):
                # Prepare data
                data= Storage(name="%s %s" % (form.vars.first_name,
                                              form.vars. last_name),
                              forge=current._config.get('title'),
                              user=form.vars.email,
                              password=form.vars.password_two,
                              forge_url=current._config.app_url)
                try:
                    current.mail.send_from_template(to=form.vars.email, 
                                                   template='add_user', 
                                                   data=data,
                                                   lang=current._config.get('language'))
                except Exception, e:
                    current._syslog.error('Users.on_create.send_mail: %s' % str(e), 
                                          'warning')
                
#            # Add user into htpasswd file
#            if form.vars.password_two:
#                from system import htpasswd_add
#                htpasswd_add(form.vars.email, form.vars.password_two)
                
        def on_delete(table, id):
            # Get user info
            row = self.db(self.db.auth_user.id == id).select().first()
            
            # Notification
            if current._government.check_configuration('email',
                                                       'del_user',
                                                       'True',
                                                       application='forge'):
                # Prepare data
                data= Storage(name="%s %s" % (row.first_name, row.last_name),
                              forge=current._config.get('title'))
                try:
                    current.mail.send_from_template(to=row.email, 
                                                   template='del_user', 
                                                   data=data,
                                                   lang=current._config.get('language'))
                except Exception, e:
                    current._syslog.error('Users.delete.send_mail: %s' % str(e), 
                                          'warning')
                    
#            # Delete user into htpasswd file
#            from system import htpasswd_del
#            htpasswd_del(row.email)    
        
        try:
            form = SQLFORM.grid(self.db.auth_user, 
                                searchable=False,
                                csv=False, 
                                user_signature = False,
                                maxtextlength=100,
                                onvalidation=on_validation,
                                oncreate=on_create,
                                ondelete=on_delete,
                                links=[lambda r:link_aprove(r), 
                                       lambda r:link_reject(r),
                                       lambda r:link_enable(r),
                                       lambda r:link_admin(r)])
        except Exception, e:
            pass
        
        return form
        
    def get(self, id):
        if id is None:
            return None
        
        ret_val = {}
        row = self.db(self.db.auth_user.email == id).select().first()
        
        if row:
            ret_val['id'] = row.id
            ret_val['identifier'] = row.identifier
            ret_val['first_name'] = row.first_name
            ret_val['last_name'] = row.last_name
            ret_val['email'] = row.email            
        
        return ret_val if ret_val else None
    
    def get_by_id(self, id):
        if id is None:
            return None
        
        ret_val = None
        row = self.db(self.db.auth_user.id == id).select().first()
        
        if row:
            ret_val = Storage(identifier = row.identifier,
                              first_name = row.first_name,
                              last_name = row.last_name,
                              email = row.email)
        
        return ret_val if ret_val else None
        
    def get_all(self):        
        ret_val = []
        rows = self.db(self.db.auth_user.id > 0).select()        
        for row in rows:
            tmp = Storage(id=row.id,
                          identifier=row.identifier,
                          first_name=row.first_name,
                          last_name=row.last_name,
                          email=row.email)    
            ret_val.append(tmp)        
        return ret_val
    
    # Web    
    def aprove(self, id):
        if id is None:
            return False
        
        row = self.db(self.db.auth_user.id == id).select().first()
        try:
            row.update_record(registration_key='')
            return True
        except:
            return False  
         
    def reject(self, id):
        if id is None:
            return False
        
        try:
            self.db(self.db.auth_user.id == id).delete()
            return True
        except:
            return False   
        
    def enable(self, id, status):
        if id is None:
            return False
        
        row = self.db(self.db.auth_user.id == id).select().first()
        try:
            if status:
                value = ''
            else:
                value = 'disabled'
            row.update_record(registration_key=value)
            return True
        except:
            return False  
        
    
    # Manual and REST
    def add_forced(self, identifier, first_name, last_name, email, password):
        try:
            # Checks if user exists
            user = self.get(email)
            if user is not None:
                return user['id']
            
            # Add user into database
            from mil.crypt import md5packed_crypt
            min_len = self.config.get('register')['min_password_length']
            crypt = CRYPT(key=self.config.get('hmac_key'), min_length=min_len)
            
            md5_word = md5packed_crypt(self.config.get('hmac_key'), password)
            
            ret_val = self.db.auth_user.insert(identifier=identifier,
                                               first_name=first_name,
                                               last_name=last_name,
                                               email=email,
                                               password=crypt(password)[0],
                                               md5word=md5_word)
            return ret_val
        except TypeError:
            return None
    
    def add(self, first_name, last_name, email, password, notify=True):
        try:
            # Checks if user exists
            user = self.get(email)
            if user is not None:
                return user['id']
            
            # Add user into database
            from mil.crypt import md5packed_crypt
            min_len = self.config.get('register')['min_password_length']
            crypt = CRYPT(key=self.config.get('hmac_key'), min_length=min_len)
            
            md5_word = md5packed_crypt(self.config.get('hmac_key'), password)
            
            ret_val = self.db.auth_user.insert(first_name=first_name,
                                               last_name=last_name,
                                               email=email,
                                               password=crypt(password)[0],
                                               md5word=md5_word)
            
            # Notification
            if notify:
                if current._government.check_configuration('email',
                                                           'add_user',
                                                           'True',
                                                           application='forge'):
                    # Prepare data
                    data= Storage(name="%s %s" % (first_name, last_name),
                                  forge=current._config.get('title'),
                                  user=email,
                                  password=password,
                                  forge_url=current._config.app_url)
                    try:
                        current.mail.send_from_template(to=email, 
                                                       template='add_user', 
                                                       data=data,
                                                       lang=current._config.get('language'))
                    except Exception, e:
                        current._syslog.error('Users.add.send_mail: %s' % str(e), 
                                              'warning')
                
#            # Add user into htpasswd file
#            # TODO: use configuration setting
#            if self.config.get('auth_htpasswd'):
#                from system.htpasswd import htpasswd_add
#                htpasswd_add(email, password)
            
            return ret_val
        except TypeError:
            return None
    
    def update(self, id, data):
        if id is None:
            return False
        
        if 'password' in data.keys():
            clean_password = data['password']
            from mil.crypt import md5packed_crypt            
            data['md5word'] = md5packed_crypt(self.config.get('hmac_key'), 
                                               data['password'])
            
            min_len = self.config.get('register')['min_password_length']
            crypt = CRYPT(key=self.config.get('hmac_key'), min_length=min_len)
            data['password'] = crypt(data['password'])[0]
          
        row = self.db(self.db.auth_user.email == id).select().first()
        try:
            # Update table
            row.update_record(**data)
            
            # Notification
            if 'password' in data.keys():
                if current._government.check_configuration('email',
                                                           'changed_password',
                                                           'True',
                                                           application='forge'):
                    # Prepare data
                    data= Storage(name="%s %s" % (row.first_name, row.last_name),
                                  forge=current._config.get('title'),
                                  user=row.email,
                                  password=clean_password,
                                  forge_url=current._config.app_url)
                    try:
                        current.mail.send_from_template(to=row.email, 
                                                       template='changed_password', 
                                                       data=data,
                                                       lang=current._config.get('language'))
                    except Exception, e:
                        current._syslog.error('Users.update.send_mail: %s' % str(e), 
                                              'warning')
            
#            # Update AuthZ
#            if 'password' in data.keys():
#                from system import htpasswd_update
#                htpasswd_update(row.email, clean_password )
            
            return True
        except:
            return False            
        
    def delete(self, id):
        if id is None:
            return False
        
        try:
            row = self.db(self.db.auth_user.email == id).select().first()
            
            # Notification
            if current._government.check_configuration('email',
                                                       'del_user',
                                                       'True',
                                                       application='forge'):
                # Prepare data
                data= Storage(name="%s %s" % (row.first_name, row.last_name),
                              forge=current._config.get('title'))
                try:
                    current.mail.send_from_template(to=row.email, 
                                                   template='del_user', 
                                                   data=data,
                                                   lang=current._config.get('language'))
                except Exception, e:
                    current._syslog.error('Users.delete.send_mail: %s' % str(e), 
                                          'warning')
                
            
#            # Delete user from htpasswd file
#            from system import httpasswd_add
#            httpasswd_del(row.email)
            
            # Delete user from database
            self.db(self.db.auth_user.email == id).delete()
            
            return True
        except:
            return False 
        