# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import *

from mil.tools.common import get_unique_id

class WorkFlows:
    @staticmethod
    def define_table(db):
        # Check if table is handled
        if not 'workflow' in db.tables:
            db.define_table('workflow',
                Field('identifier', 'string', length=20, 
                  default=get_unique_id(2,4), required=False, 
                  label=current.T('Identifier')),
                Field('name', 'string', length=50, 
                      label=current.T('Name')),
                Field('description', 'string', length=100, 
                      label=current.T('Description')),
                Field('content', 'text', label=current.T('Code')),
                format='%(label)s')
        
            try:
                if db(db.workflow.id >= 1).count() == 0:
                    data = [{'name': 'new_project',
                             'description': 'Create a new Project into forge',
                             'content': '''{"node_1":{"label":"Has Permission","type":"boolean","param":"action: _government.has_permission()","pos":{"top":234,"left":285},"connector":{"input":{"con_in1":{"label":"in","connection":""}},"output":{"con_outt":{"label":"true","connection":"node_2.con_in1"},"con_outf":{"label":"false","connection":"node_3.con_in1"}}}},"node_2":{"label":"Card","type":"connector","param":"url: project, card, create","pos":{"top":197,"left":469},"connector":{"input":{"con_in1":{"label":"in","connection":"node_1.con_outt"}},"output":{"con_out1":{"label":"out","connection":"node_4.con_in1"}}}},"node_3":{"label":"Dashboard","type":"end","param":"url: dashboard","pos":{"top":285,"left":467},"connector":{"input":{"con_in1":{"label":"in","connection":"node_1.con_outf"}},"output":{}}},"node_4":{"label":"Leader","type":"connector","param":"url: project,team,leader","pos":{"top":198,"left":583},"connector":{"input":{"con_in1":{"label":"in","connection":"node_2.con_out1"}},"output":{"con_out1":{"label":"out","connection":"node_5.con_in1"}}}},"node_5":{"label":"Service","type":"connector","param":"url: project,service","pos":{"top":197,"left":714},"connector":{"input":{"con_in1":{"label":"in","connection":"node_4.con_out1"}},"output":{"con_out1":{"label":"out","connection":"node_9.con_in1"}}}},"node_9":{"label":"End","type":"end","param":"url: dashboard","pos":{"top":197,"left":833},"connector":{"input":{"con_in1":{"label":"in","connection":"node_5.con_out1"}},"output":{}}}}'''}, 
                            {'name': 'login',
                             'description': 'Forge login process',
                             'content': '''{"node_1":{"label":"Is User Logged in","type":"boolean","param":"action: auth.user, is, None","pos":{"top":195,"left":285},"connector":{"input":{"con_in1":{"label":"in","connection":""}},"output":{"con_outt":{"label":"true","connection":"node_3.con_in1"},"con_outf":{"label":"false","connection":"node_2.con_in1"}}}},"node_2":{"label":"Selected or Dashboard Page","type":"end","param":"","pos":{"top":142,"left":467},"connector":{"input":{"con_in1":{"label":"in","connection":"node_1.con_outt"}},"output":{}}},"node_3":{"label":"Is Login proccess active","type":"boolean","param":"cond: {request.controller, eq, 'default'} and {request.function, eq, 'user'} and {request.args[0], eq, 'login'}","pos":{"top":243,"left":472},"connector":{"input":{"con_in1":{"label":"in","connection":"node_1.con_outf"}},"output":{"con_outt":{"label":"true","connection":"node_4.con_in1"},"con_outf":{"label":"false","connection":"node_5.con_in1"}}}},"node_4":{"label":"End","type":"end","param":"","pos":{"top":209,"left":701},"connector":{"input":{"con_in1":{"label":"in","connection":"node_3.con_outt"}},"output":{}}},"node_5":{"label":"is Private Forge","type":"boolean","param":"action: _config.get('forge_private')","pos":{"top":289,"left":699},"connector":{"input":{"con_in1":{"label":"in","connection":"node_3.con_outf"}},"output":{"con_outt":{"label":"true","connection":"node_6.con_in1"},"con_outf":{"label":"false","connection":"node_7.con_in1"}}}},"node_6":{"label":"Login","type":"end","param":"url: default, user, login","pos":{"top":265,"left":878},"connector":{"input":{"con_in1":{"label":"in","connection":"node_5.con_outt"}},"output":{}}},"node_7":{"label":"Index","type":"end","param":"url: default","pos":{"top":343,"left":876},"connector":{"input":{"con_in1":{"label":"in","connection":"node_5.con_outf"}},"output":{}}}}'''}
                            ]
                    for item in data:
                        db.workflow.insert(name=item['name'],
                                           description=item['description'],
                                           content=item['content'])
            except:
                pass 
        
    def __init__(self, db):
        self.db = db
        WorkFlows.define_table(db)
        
    def grid(self): 
        links = [lambda row: A(current.T('Configure'),
                               _href=URL("workflow","configure", 
                                         args=[row.id]))]
        form = SQLFORM.grid(self.db.workflow, 
                            searchable=False,
                            fields=[self.db.workflow.identifier,
                                    self.db.workflow.description],
                            csv=False, 
                            user_signature = False,
                            links=links,
                            maxtextlength=100)
        return form
    
    def get_all(self):
        return  self.db(self.db.workflow.id > 0).select()
    
    def get(self, name):
        return self.db(self.db.workflow.name == name).select().first() or None
    
    def get_by_ident(self, ident):
        return self.db(self.db.workflow.identifier == ident).select().first() or None
    
    def get_by_name(self, ident):
        return self.db(self.db.workflow.name == ident).select().first() or None
    
    def get_by_id(self, ident):
        return self.db(self.db.workflow.id == ident).select().first() or None
    
    def set_by_id(self, ident, workflow):
        self.db(self.db.workflow.id == ident).update(content=workflow)
        
    def insert(self, name, description, content):        
        return self.db.workflow.insert(name=name,
                                       description=description,
                                       content=content)
