# -*- coding: utf-8 -*-

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import base64

from gluon import *
from gluon.storage import Storage
from gluon.tools import Auth
from gluon.utils import web2py_uuid

class RESTAuth(Auth): 
    def __init__(self, environment=None, db=None, mailer=True,
                 hmac_key=None, controller='default', cas_provider=None):
        Auth.__init__(self, environment, db, mailer, hmac_key, controller, 
                      cas_provider)
        
    def rest_basic(self):
        basic = current.request.env.http_authorization
        if not basic or not basic[:6].lower() == 'basic ':
            return False
        (username, password) = base64.b64decode(basic[6:]).split(':')
        return self.login_rest(username, password)

    def login_rest(self, username, password):
        """
        REST service logins user. 
        
        This function is like login_bare(), but it not store auth in session
        """

        request = current.request
        session = current.session
        table_user = self.settings.table_user
        if self.settings.login_userfield:
            userfield = self.settings.login_userfield
        elif 'username' in table_user.fields:
            userfield = 'username'
        else:
            userfield = 'email'
        passfield = self.settings.password_field
        user = self.db(table_user[userfield] == username).select().first()
        password = table_user[passfield].validate(password)[0]
        if user:
            if not user.registration_key and user[passfield] == password:
                user = Storage(table_user._filter_fields(user, id=True))
                self.user = user
                # All app available
                current.auth.user = user
                return user
        return False     
    
    def rest_login(self):
        """
        decorator that prevents access to REST action if not logged in
        """ 
        def decorator(action):

            def f(*a, **b):

                if not self.rest_basic():
                    raise HTTP(403,"Not authorized")
                return action(*a, **b)
            f.__doc__ = action.__doc__
            f.__name__ = action.__name__
            f.__dict__.update(action.__dict__)
            return f

        return decorator
    
    
#def rest_service_authorization():
#    """
#    Execute REST petition
#    Use HTTP Basic authorization method
#    """
#    try:
#        auth_data = request.env.http_authorization
#        if not auth_data:
#            RESTError(400)
#        if auth_data[:6].lower() == 'basic ':
#            type, basic = auth_data.split(' ')
#            serve_rest = rest_basic_login(auth_data[6:])
#        if auth_data[:6].lower() == 'crypt ':
#            type, basic = auth_data.split(' ')
#            serve_rest = rest_crypt_login(auth_data[6:])                
#        if auth_data[:6].lower() == 'admin ':
#            type, basic = auth_data.split(' ')
#            serve_rest = rest_admin_login(auth_data[6:])
#
#        if serve_rest is False:
#            RESTError(401)
#    
#    except:
#        RESTError(401)
#        
#def rest_basic_login(auth_data):
#    """
#    logins user from CAS service
#    """
#    try:
#        url = "%s/basic_login?auth=%s" % (auth.settings.login_form.urlbase, 
#                                          auth_data)
#    except:
#        url = "%s/basic_login?auth=%s" % (CAS.rest_url, auth_data)
#    
#    data = urllib.urlopen(url).read().split('\n')
#    if data[0]=='yes': 
#        (email, password) = base64.b64decode(auth_data).split(':')
#        crypt = CRYPT(digest_alg='sha1')
#        session.vip_user=User(None,email,None,crypt(password)[0])
#        return True
#    return False        
#    
#def rest_crypt_login(auth_data):
#    """
#    logins user from CAS service
#    """
#    try:
#        url = "%s/crypt_login?auth=%s" % (auth.settings.login_form.urlbase, auth_data)
#    except:
#        url = "%s/crypt_login?auth=%s" % (CAS.rest_url, auth_data)
#    data = urllib.urlopen(url).read().split('\n')
#    if data[0]=='yes':             
#        (email, password) = base64.b64decode(auth_data).split(':')
#        session.vip_user=User(None,email,None,password)
#        return True
#    return False              
#    
#def rest_admin_login(auth_data):
#    """
#    logins user from CAS service
#    """
#    try:
#        url = "%s/admin_login?auth=%s" % (auth.settings.login_form.urlbase, auth_data)
#    except:
#        url = "%s/admin_login?auth=%s" % (CAS.rest_url, auth_data)
#    data = urllib.urlopen(url).read().split('\n')
#    if data[0]=='yes': 
#        return True
#    return False
