# -*- coding: utf-8 -*- 

from handler import Handler
from mil.base import File, FileNotExists, Storage

class fileHandler(Handler):
    filepath=None
    type='File'
    
    def __init__(self, **kwargs):
        filepath = kwargs.get('filepath', None)
        auto = kwargs.get('auto', False)
           
        if filepath is not None:
            if File.exists(filepath):
                self.filepath=filepath
            else:
                if auto:
                    File.touch(filepath)
                    self.filepath=filepath
                else:
                    raise FileNotExists(filepath)
            
    def load(self, id):  
        data=None      
        if self.filepath is not None:
            if File.exists(self.filepath):
                data=File.load(self.filepath)
                        
        if data is not None:
            ret_val = Storage(data) 
        else:
            ret_val = Storage()             

        return ret_val
   
    def save(self, data):
        if self.filepath is not None:
            File.save(self.filepath, data)
            
    def erase(self, id):
        if self.filepath is not None:
            File.remove(self.filepath)
            
    def __repr__(self):
        return "<Handler: %s (%s) >" % (self.type, self.filepath)        
            
from mil.base.yamlfile import YAMLFile
class yamlHandler(fileHandler):    
    def __init__(self, **kwargs):
        fileHandler.__init__(self, **kwargs)
        self.type='YAMLFile'
         
    def load(self):  
        data=None      
        if self.filepath is not None:
            if File.exists(self.filepath):
                data=YAMLFile.load(self.filepath)

        if data is not None:
            ret_val = Storage(data) 
        else:
            ret_val = Storage()            

        return ret_val
   
    def save(self, data):
        if self.filepath is not None:
            YAMLFile.save(self.filepath,dict(data))
                     
                  