# -*- coding: utf-8 -*- 
# Este fichero forma parte de mil
#
# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'licence' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.#!/usr/bin/env python

__all__ = ['webHandler']

import os
import urllib
import urllib2

from handler import Handler

class webRequest:
    code  = None
    error = None
    data  = None
    
    def __str__(self):
        "Shows code an message error"
        return "%s - %s"%(self.code, self.error) 

class webHandler(Handler):
    @staticmethod
    def url_auth(url, user=None, password=None):
        request = webHandler.url_get(url)
        if request.code == 401: # Server requests authentication
            request = webHandler.basic_auth(None, os.path.dirname(url), user, password, url)
        return request
    
    @staticmethod
    def url_get(url, headers={}):
        req = urllib2.Request(url, headers=headers)
        return webHandler._get_url(req)
    
    @staticmethod
    def basic_auth(realm, uri, username, password, url):
        pass_manager = urllib2.HTTPPasswordMgrWithDefaultRealm()
        pass_manager.add_password(realm, uri, username, password)        
        
        auth_handler = urllib2.HTTPBasicAuthHandler(pass_manager)
        
        opener = urllib2.build_opener(auth_handler)
        # ...and install it globally so it can be used with urlopen.
        urllib2.install_opener(opener)
    
        return webHandler._get_url(url)    
    
    @staticmethod         
    def _get_url(url):
        request = webRequest()
        try:
            response = urllib2.urlopen(url)
            request.code = 200
        except IOError, e:
            if hasattr(e, 'reason'):
                request.code = 400
                request.error = 'We failed to reach a server. Reason: %s' % e.reason
            elif hasattr(e, 'code'):
                request.code = e.code
        except Exception, e:
            request.code = 400
            request.error = e
        else:
            # everything is fine
            request.data=response.read()
        return request
    
    

REST_CONTENT_TYPE = {
    'html': 'html',
    'text/html': 'html',
    'application/html': 'html',
    'json': 'json',
    'application/json': 'json',
    'xml': 'xml',
    'application/xml': 'xml'
    }

def rest(url, method='get', headers={}, data=None):
    req = urllib2.Request(url, headers=headers)
    req.get_method = lambda: method.upper()
    if method.upper() == 'POST' or method.upper() == 'PUT':
        req.add_data(data)
        
    try:
        response = urllib2.urlopen(req)
    except IOError, e:
        if hasattr(e, 'reason'):
            return '[ERROR] We failed to reach a server. Reason: %s' % e.reason
        elif hasattr(e, 'code'):
            return '[ERROR] The server couldn\'t fulfill the request. Error code: %i' % e.code
    else:
        # everything is fine
        return response.read()
      

def rest_RC(code):
    raise HTTP(code, defined_status[code])

def rest_service_format():
    """
    Checks output format
    """
    # Only checks output format if default format (html) is declared in headers.
    if request.extension == 'html':
        # http acept properties: Accept
        http_accept = {}
        # Format
        try:
            formats = request.env.http_accept.split(';')
            http_accept['format'] = formats[0].split(',')
        except:
            http_accept['format'] = 'application/json'
         
        for ctype in http_accept['format']:
            if REST_CONTENT_TYPE.has_key(ctype):  
                request.extension=REST_CONTENT_TYPE[ctype]
                response.view="api/%s.%s"%(request.function,REST_CONTENT_TYPE[ctype])
                return
    
def rest_service_authorization():
    """
    Execute REST petition
    Use HTTP Basic authorization method
    """
    try:
        auth_data = request.env.http_authorization
        if not auth_data:
            rest_RC(400)
        if auth_data[:6].lower() == 'basic ':
            type, basic = auth_data.split(' ')
            serve_rest = rest_basic_login(auth_data[6:])
        if auth_data[:6].lower() == 'crypt ':
            type, basic = auth_data.split(' ')
            serve_rest = rest_crypt_login(auth_data[6:])                
        if auth_data[:6].lower() == 'admin ':
            type, basic = auth_data.split(' ')
            serve_rest = rest_admin_login(auth_data[6:])

        if serve_rest is False:
            rest_RC(401)
    
    except:
        rest_RC(401)
        
def rest_basic_login(auth_data):
    """
    logins user from CAS service
    """
    try:
        url = "%s/basic_login?auth=%s" % (auth.settings.login_form.urlbase, auth_data)
    except:
        url = "%s/basic_login?auth=%s" % (CAS.rest_url, auth_data)
    
    data = urllib.urlopen(url).read().split('\n')
    if data[0]=='yes': 
        (email, password) = base64.b64decode(auth_data).split(':')
        crypt = CRYPT(digest_alg='sha1')
        session.vip_user=User(None,email,None,crypt(password)[0])
        return True
    return False        
    
def rest_crypt_login(auth_data):
    """
    logins user from CAS service
    """
    try:
        url = "%s/crypt_login?auth=%s" % (auth.settings.login_form.urlbase, auth_data)
    except:
        url = "%s/crypt_login?auth=%s" % (CAS.rest_url, auth_data)
    data = urllib.urlopen(url).read().split('\n')
    if data[0]=='yes':             
        (email, password) = base64.b64decode(auth_data).split(':')
        session.vip_user=User(None,email,None,password)
        return True
    return False              
    
def rest_admin_login(auth_data):
    """
    logins user from CAS service
    """
    try:
        url = "%s/admin_login?auth=%s" % (auth.settings.login_form.urlbase, auth_data)
    except:
        url = "%s/admin_login?auth=%s" % (CAS.rest_url, auth_data)
    data = urllib.urlopen(url).read().split('\n')
    if data[0]=='yes': 
        return True
    return False