# -*- coding: utf-8 -*-

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import thread
try:
    import louie
except:
    import mil.libs.louie as louie

def startBusSystem():
    louie.reset()

class MessageBus(object):
    """
    Messaging subsystem. Has one universal message posting function.
    Also allows a simple subscription mechanism, so interested listeners
    can keep up to date.
    """        
    def __init__(self):
        self.busaccess = thread.allocate_lock()
        
    def subscribe(self, callback, signal = "signal.default"):
        """
        subscribe a function of the form:
        receive_message(message,timestamp)
        
        @par aSection: a regexp for filtering messages
        """
        return louie.connect(callback, signal)
        
    def unsubscribe(self, callback, signal = "signal.default"):
        """unsubscribe a registered function"""
        return louie.disconnect(callback, signal)
        
    
    def post(self, signal, *args, **kwargs):
        """Send a message to the given signal"""
        self.busaccess.acquire()
        ret_val = louie.send(signal, None, *args, **kwargs)
        self.busaccess.release()
        
        return ret_val
