# -*- coding: utf-8 -*-

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from mil.base import File
from mil.tools.system import System

class Apache(System):
    def __init__(self):
        System.__init__(self)
        
        self.path = '/etc/init.d/apache2'
        
    def reload(self):
        """
        Reload apache server
        """
        command = [self.path, "reload"]
        self.call(command, 'unable to reload apache server')
    
    def add_user(self, user, group, filename):
        try:
            groups = self._read_groups(filename)
            if not self._exists_user_in_group(groups, user, group):
                if not group in groups.keys():
                    groups[group] = []
                groups[group].append(user)
            self._write_groups(groups, filename)
        except Exception, e:
            raise Exception(str(e))
    
    def del_user(self, user, group, filename):
        try:
            groups = self._read_groups(filename)
            if self._exists_user_in_group(groups, user, group):
                 groups[group].remove(user)
            self._write_groups(groups, filename)
        except Exception, e:
            raise Exception(str(e))
        
    def _exists_user_in_group(self, groups, user, group):
        if group in groups.keys():
            try:
                pos = groups[group].index(user)
                return True
            except:
                return False
        else:
            return False
        
    
    def _read_groups(self, filename):
        groups = {}
        try:
            if File.exists(filename):
                for line in File.readlines(filename):
                    line = line.rstrip("\r\n")
                    # Delete or not use commented lines
                    import re
                    if not re.match('^\s*#', line):
                        members = line.split(" ")
                        header = members.pop(0).replace(':','')
                        groups[header] = members
        except Exception, e:
            raise Exception('(Apache) read groups: %s' % str(e))
        return groups
            
    def _write_groups(self, groups, filename):
        try:
            lines = []
            for group in groups.keys():
                lines.append('%s: %s' % (group, ' '.join(groups[group])))
            File.write(filename, "\r\n".join(lines))
        except Exception, e:
            raise Exception('(Apache) write groups: %s' % str(e))
            
            