# -*- coding: utf-8 -*-

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import os

from mil.base import File
from mil.tools.system import System

class BaseRepo(System):
    type = 'BASE'
    def __init__(self, name, **kwargs):
        System.__init__(self)
        
        self.name = name
        
        self.acl = kwargs.get('acl', None)
        
        if not isinstance(self.acl, dict):
            raise Exception('acl parameter must be a dictionary')
        
        self.private = kwargs.get('private', False)
        
    def set_acl(self, private=None):
        if private is None:
            private = self.private
        try:
            acl = self._get_apache_acl(private=private, 
                                       name=self.acl['group_name'], 
                                       path=self.acl['group_path'])
            if acl:
                acl_path = os.path.join(self.acl['acl_path'], 
                                        '%s.%s' % (self.name, self.type))
                if File.exists(acl_path):
                    File.remove(acl_path)
                File.write(acl_path, acl)
        except Exception, e:
            return "(%s:set_acl) %s" % (self.type, str(e))
        
    def del_acl(self):
        try:
            acl_path = os.path.join(self.acl['acl_path'], 
                                    '%s.%s' % (self.name, self.type))
            File.remove(acl_path)
        except Exception, e:
            return "(%s:del_acl) %s" % (self.type, str(e))
                    
    def _get_apache_acl(self, **kwargs):         
        private = kwargs.get('private', False)
        group_name = kwargs.get('name', None)
        group_path = kwargs.get('path', None)
        
        directives = "GET HEAD OPTIONS PROPFIND REPORT"
        content = []
        content.append(" " )
        content.append("### Service %s %s ###" % (self.type, self.name))
        content.append('<Location /%s/%s/>' % (self.type.lower(), self.name))
        if private:
            content.append("  <Limit %s>" % directives)
            content.append("    AuthGroupFile %s" % group_path)
            content.append("    Require group %s" % group_name)
            content.append("  </Limit>")
        
        content.append("  <LimitExcept %s>" % directives)
        content.append("    AuthGroupFile %s" % group_path)
        content.append("    Require group %s" % group_name)
        content.append("  </LimitExcept>")
        content.append("</Location>")        
        content.append("### Service %s %s ###" % (self.type, self.name))
        
        return "\n".join(content)

                        