# -*- coding: utf-8 -*-

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from mil.base import File
from mil.tools.system import System

class Repository(System):
    _repo_types = ['bdb', 'fsfs']
    
    def __init__(self, dirname, fstype='fsfs'):
        System.__init__(self)
        
        self.dirname = dirname
        if fstype in self._repo_types:
            self.fstype = fstype
        else:
            raise Exception('repository type not allowed')

    def create(self):
        """
        Initialize a repo of the type :attr:`self.fstype`.
        """
        if not File.exists(self.dirname):
            command = ["svnadmin", "create", "--fs-type", 
                       self.fstype, 
                       self.dirname]
            self.call(command,'unable to create subversion repository')
        

    def delete(self):
        """The deletion of the dirname path will remove the repo as well."""
        if File.exists(self.dirname):
            File.remove_dir(self.dirname)

    def backup(self, backup_path):
        """$ svnadmin hotcopy /repo-path /repo-backup-path
            Berkeley DB repository
           $ svnadmin hotcopy --clean-logs /repo-path /repo-backup-path
        """
        if File.check_dir(backup_path) is False:
            raise Exception('must be defined a backup path')
        
        command = ["svnadmin", "hotcopy"]
        if self.fstype == 'bdb':
            command.append("--clean-logs")
        command.append(self.dirname)
        command.append(backup_path)      
        self.call(command,'unable to create subversion repository')
       


   