# -*- coding: utf-8 -*-

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import os
import time

from mil.base import File

from baserepo import BaseRepo

class DAV(BaseRepo):
    type = 'DAV'
    def __init__(self, name, **kwargs):
        BaseRepo.__init__(self, name, **kwargs)
        
        self.name = name
        
        self.config = kwargs.get('config', None)
        
        if not isinstance(self.config, dict):
            raise Exception('config parameter must be a dictionary')
        
        self.dirname = os.path.join(self.config['base_path'], self.name)
        # Configure skeleton folder
        self.config['copy_path'] = [os.path.join(self.config['tmpl_path'], 
                                                'repo_skel')]

    def create(self):
        # Create DAV structure
        # Main folder
        try:
            File.make_dir(self.dirname)
        except Exception, e:
            return "(DAV:create) %s" % str(e)
        # Create DAV subfolders
        try:
            if self.config['create'] is not None:
                for item in self.config['create']:
                    File.make_dir(os.path.join(self.dirname, item))
        except Exception, e:
            return "(DAV:create) %s" % str(e)
        
        # Copy DAV contents
        try:
            if self.config['copy']:
                for item in self.config['copy_path']:
                    destiny = os.path.join(self.dirname, 
                                           os.path.basename(item))
                    # Copy folder it it not exixts
                    if File.check_dir(destiny) is False:
                        File.copy_dir(item, destiny)
        except Exception, e:
            return "(DAV:create) %s" % str(e)
        
        # Set DAV ACL
        msg = self.set_acl()
        if msg:
            return "(DAV:create) %s" % str(msg)
        # Change DAV owner
        try:
            if self.config['user_owner']:
                command = ['chown','-Rf', 
                           self.config['user_owner'], 
                           self.dirname]
                self.call(command,
                          'unable to change subversion repository owner')
        except Exception, e:
            return "(DAV:create) %s" % str(e)
         
    def delete(self):
        if self.config['delete_path']:
            if os.path.isdir(self.config['delete_path']):
                # Copy folder to delete path
                destiny = os.path.join(self.config['delete_path'],
                                       "%s.%s" % (self.name, int(time.time())))
                File.copy_dir(self.dirname, destiny)
                # Delete folder
                File.remove_dir(self.dirname)
                # Delete ACL File
                msg = self.del_acl()
                if msg:
                    return "(DAV:delete) %s" % str(msg)
            else:
                return "(DAV:delete) delete_path is not a folder"
            