# -*- coding: utf-8 -*-

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import os
import time

from mil.base import File
from mil.tools.system import System

class Mailman(System):
    def __init__(self, **kwargs):
        System.__init__(self)
        
        self.params = kwargs.get('params', None)
        if not isinstance(self.params, dict): 
            raise Exception('params parameter must be a dictionary')
        
        self.config = kwargs.get('config', None)        
        if not isinstance(self.config, dict):
            raise Exception('config parameter must be a dictionary')
        
        self.password = kwargs.get('password', None)
        if self.password is None:
            raise Exception('password parameter must be defined')            
        
    def create(self):
        list_name = '%s\@%s' % (self.params['service_name'],
                                self.config['list_domain'])
        command = ["/usr/sbin/newlist", 
                   "-l", self.config['lang'],
                   "-q", list_name,
                   self.params['mail'],
                   self.password]
        msg = self.call(command, 'unable to create list:' % list_name)
        if msg:
            return "(Mailman:create) %s" % str(msg)
    
    def backup(self):
        list_name = self.params['service_name']
        if self.config['virtual_hack']:
            list_name = '%s-%s' % (list_name, self.config['list_domain'])
            
        if self.config['delete_path']:
            if os.path.isdir(self.config['delete_path']):
                # Copy folder to delete path
                archives_path = os.path.join(self.config['archives'], 
                                             list_name)
                destiny = os.path.join(self.config['delete_path'],
                                       "%s.%s" % (list_name, int(time.time())))
                try:
                    File.copy_dir(archives_path, destiny)
                except Exception, e:
                    return "(Mailman:backup) %s" % str(e)
                
                # Copy mbox folder to delete path
                archives_path = os.path.join(self.config['archives'], 
                                             '%s.mbox' % list_name)
                destiny = os.path.join(self.config['delete_path'],
                                       "%s.%s" % ('%s.mbox' % list_name, 
                                                  int(time.time())))
                try:
                    File.copy_dir(archives_path, destiny)
                except Exception, e:
                    return "(Mailman:backup) %s" % str(e)
            else:
                message =  'can not find historic files of mail list'
                return "(Mailman:backup) %s" % message
    
    def delete(self):
        msg = self.backup()
        if msg:
            return "(Mailman:delete) %s" % str(msg)
        
        list_name = self.params['service_name']
        if self.config['virtual_hack']:
            list_name = '%s-%s' %(list_name, self.config['list_domain'])
        
        # Delete list
        command = ["/usr/sbin/rmlist", "-a", list_name]
        msg = self.call(command, 'unable to create list:' % list_name)
        if msg:
            return "(Mailman:delete) %s" % str(msg)
    
    def add_user(self):
        list_name = self.params['service_name']
        if self.config['virtual_hack']:
            list_name = '%s-%s' %(list_name, self.config['list_domain'])
        
        # Delete list
        command = ["echo", self.params['mail'],
                   "| /usr/sbin/add_members", "-r", '- %s' % list_name]
        msg = self.call(command, 'unable to create list: %s' % list_name)
        if msg:
            return "(Mailman:add_user) %s" % str(msg)
    
    def upd_user(self):
        list_name = self.params['service_name']
        if self.config['virtual_hack']:
            list_name = '%s-%s' %(list_name, self.config['list_domain'])
        
        # Update email in all lists
        command = ["/usr/sbin/clone_member", "-r", "-a", 
                   self.params['old_mail'],
                   self.params['new_mail']]
        msg = self.call(command, 'unable to update new email')
        if msg:
            return "(Mailman:upd_user) %s" % str(msg)
    
    def del_user(self, user):
        list_name = self.params['service_name']
        if self.config['virtual_hack']:
            list_name = '%s-%s' %(list_name, self.config['list_domain'])
        
        # Delete user from a list
        command = ["/usr/sbin/remove_members", 
                   list_name, 
                   self.params['mail']]
        tm = 'unable to remove user "%s" from list: %s' % (self.params['mail'], 
                                                           list_name)
        msg = self.call(command, tm)
        if msg:
            return "(Mailman:del_user) %s" % str(msg)
    
    def get_data(self):
        list_name = self.params['service_name']
        if self.config['virtual_hack']:
            list_name = '%s-%s' %(list_name, self.config['list_domain'])
            
        output = {'list_password': self.password,
                  'list_domain': self.config['list_domain'],
                  'list_name': list_name}
        return output