# -*- coding: utf-8 -*-

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import os

from mil.base import File

from mil.repos.apache import Apache
from mil.repos.dav import DAV
from mil.repos.svnrepo import SVNRepo

class SubApp(object):
    service_class = {'SVN': 'SVNRepo', 'DAV': 'DAV'}
    def __init__(self, **kwargs):   
        params = kwargs.get('params', None)
        if not isinstance(params, dict): 
            raise Exception('params parameter must be a dictionary')
        self.param = params

        self.config = kwargs.get('config', None)        
        if not isinstance(self.config, dict):
            raise Exception('config parameter must be a dictionary')
        
        self.acl = kwargs.get('acl', None)        
        if not isinstance(self.acl, dict):
            raise Exception('acl parameter must be a dictionary')
        
        self.private = kwargs.get('private', False)

    def create(self):
        # Install subapplication services
        if self.config['app']['default_services'] is not None:
            for service in self.config['app']['default_services']:
                msg = self.add_service(service)
                if msg:
                    return "(SubApp:create) %s" % str(msg)
            
        # Create apache group file
        try:
            acl_path = os.path.join(self.acl['group_path'], self.param['name'])
            File.write(acl_path, '%s:' % (self.acl['group_name']))
        except Exception, e:
            return "(SubApp:create) %s" % str(e)
        
        # Add subapplication administrator user
        if self.param['user'] is not None:
            msg = self.add_member(self.param['user'])
            if msg:
                return "(SubApp:create) %s" % str(msg)
         
    def delete(self):
        # Delete all subapplication services
        if self.config['app']['default_services'] is not None:
            for service in self.config['app']['default_services']:
                msg = self.del_service(service)
                if msg:
                    return "(SubApp:delete) %s" % str(msg)
            
        # Delete apache groups file
        File.remove(os.path.join(self.acl['group_path'], self.param['name']))
    
    def add_member(self, member):
        # Add member to all subapplication services
        try:
            apache = Apache()
            apache.add_user(member, 
                            self.acl['group_name'], 
                            os.path.join(self.acl['group_path'], self.param['name']))
        except Exception, e:
            return "(SubApp:add_member) %s" % str(e)
    
    def del_member(self, member):
        # Delete member from all subapplication services
        try:
            apache = Apache()
            apache.del_user(member, 
                            self.acl['group_name'], 
                            os.path.join(self.acl['group_path'], self.param['name']))
        except Exception, e:
            return "(SubApp:del_member) %s" % str(e)
    
    def add_service(self, service): 
        # Checks if service is configured. If not, install an configure it.
        test_path = os.path.join(self.acl['acl_path'], 
                                 '%s.%s' % (self.param['name'], service))
        if File.exists(test_path) is False:
            # Instance handler
            try:
                classname = self.service_class[service]
                config = self.config['services'][service]
                config['tmpl_path'] = self.config['private_path']
                s_ins = globals()[classname](self.param['name'], 
                                             acl=self.acl, 
                                             config=config,
                                             private=self.private) 
                return s_ins.create()
            except Exception, e:
                return "(SubApp:add_service) Service: %s > %s" % (service,
                                                                  str(e))                
    
    def del_service(self, service):
        # Checks if service is configured. If yes, delete it.
        test_path = os.path.join(self.acl['acl_path'], 
                                 '%s.%s' % (self.param['name'], service))
        if File.exists(test_path):
            # Instance handler
            try:
                classname = self.service_class[service]
                config = self.config['services'][service]
                config['tmpl_path'] = self.config['private_path']
                s_ins = globals()[classname](self.param['name'], 
                                             acl=self.acl, 
                                             config=config,
                                             private=self.private) 
                return s_ins.delete()
            except Exception, e:
                return "(SubApp:del_service) Service: %s > %s" % (service,
                                                                  str(e))
                
    def auth_service(self, service, auth):
        if not auth in ['private', 'public']:
            return 'unknown service auth'
        
        if auth == 'private':
            self.private = True
        else:
            self.private = False
        
        # Checks if service is configured. If not, install an configure it.
        test_path = os.path.join(self.acl['acl_path'], 
                                 '%s.%s' % (self.param['name'], service))
        if File.exists(test_path):
            # Instance handler
            try:
                classname = self.service_class[service]
                config = self.config['services'][service]
                config['tmpl_path'] = self.config['private_path']
                s_ins = globals()[classname](self.param['name'], 
                                             acl=self.acl, 
                                             config=config,
                                             private=self.private) 
                return s_ins.set_acl()
            except Exception, e:
                return "(SubApp:add_service) Service: %s > %s" % (service,
                                                                  str(e))  
        
       
    def __str__(self):
        output = 'Name: %s\n' % self.param['name']
        output = '%sACL: %s\n' % (output, self.acl)
        output = '%sConfig: %s' % (output, self.config)
        return output
        
    def __repr__(self):
        return str(self)
        
        