# -*- coding: utf-8 -*-

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import os
import time

from mil.base import File
from mil.tools.strings import template

from mil.repos.baserepo import BaseRepo
from mil.repos.core.svn import Repository

class SVNRepo(BaseRepo):
    type = 'SVN'
    def __init__(self, name, **kwargs):
        BaseRepo.__init__(self, name, **kwargs)
        
        self.name = name
        
        self.config = kwargs.get('config', None)
        self.data = kwargs.get('data', {})
        
        if not isinstance(self.config, dict):
            raise Exception('config parameter must be a dictionary')
        
        self.dirname = os.path.join(self.config['base_path'], self.name)
        # Configure hooks
        self.config['hooks'] = os.path.join(self.config['tmpl_path'], 
                                            'svn_hooks')

    def create(self):
        # Create repository
        try:
            repo = Repository(self.dirname, self.config['fstype'])
            repo.create()
        except Exception, e:
            return "(SVN:create) %s" % str(e)
        
        # Add hooks
        try:
            if self.config['hooks'] is not None:
                self._add_hooks()
        except Exception, e:
            return "(SVN:create) %s" % str(e)
        
        # Set ACL
        msg = self.set_acl()
        if msg:
            return "(SVN:create) %s" % str(msg)
        
        # Change owner
        try:
            if self.config['user_owner']:
                command = ['chown','-Rf', 
                           self.config['user_owner'], 
                           self.dirname]
                self.call(command,
                          'unable to change subversion repository owner')
        except Exception, e:
            return "(SVN:create) %s" % str(e)
        
    def delete(self):
        if self.config['delete_path']:
            if os.path.isdir(self.config['delete_path']):
                # Copy folder to delete path
                destiny = os.path.join(self.config['delete_path'],
                                       "%s.%s" % (self.name, int(time.time())))
                File.copy_dir(self.dirname, destiny)
                # Delete folder
                File.remove_dir(self.dirname)
                # Delete ACL File
                msg = self.del_acl()
                if msg:
                    return "(SVN:delete) %s" % str(msg)
            else:
                return "(SVN:delete) delete_path is not a folder"
                
    def _add_hooks(self):
        # if self.hooks is directory, read hooks and process them
        if os.path.isdir(self.config['hooks']):
            for root, dirs, files in os.walk(self.config['hooks']):
                # for each file
                for f in files:
                    # Process file as template
                    template_content = File.read(os.path.join(root, f))
                    content = template(template_content, self.data)
                    if content is None:
                        raise Exception('invalid format in hook template')
                    # Save content into repository hooks folder
                    hook_path = os.path.join(self.dirname, 'hooks', f)
                    bol_ok, err = File.write(hook_path, content)
                    if err:
                        raise Exception(err)
                    # Change permissions (+x)
                    command = ['chmod','0744', hook_path]
                    self.call(command, 
                              'unable to change hook file permissions')
                    