# -*- coding: utf-8 -*- 

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import sys
import traceback
import exceptions
import urllib
import urllib2

from gluon import *
from gluon.contenttype import contenttype

class RESTError(exceptions.Exception):
    filepath = ''
    def __init__(self, msg):
        self.message = msg
        
    def __str__(self):
        return "REST service ERROR: %s"%self.message
        
def rest(url, method='get', headers={}, data=None):
    req = urllib2.Request(url, headers=headers)
    req.get_method = lambda: method.upper()
    if method.upper() == 'POST' or method.upper() == 'PUT':
        req.add_data(data)
        
    try:
        response = urllib2.urlopen(req)
    except IOError, e:
        if hasattr(e, 'reason'):
            raise RESTError('Failed to reach a server. Reason: %s' % e.reason)
        elif hasattr(e, 'code'):
            raise RESTError('Server Error code: %i' % e.code)
    else:
        # everything is fine
        return response.read()
 
def basic_authorization(username, password):     
    import base64
    base64string = base64.encodestring('%s:%s' % (username, password))[:-1]
    return  "Basic %s" % base64string


def restful():
    def wrapper(action):
        def f(_action=action, *a, **b):
            _request = current.request
            _request.is_restful = True
            method = _request.env.request_method
            
            # User
            from core.userhandler import UserHandler
            current._user = UserHandler(current.auth.user)
            
            # Content-Type
            if len(_request.env.http_accept):
                ctype = _request.env.http_accept.split(';')[0].split(',')[0]
                i = ctype.rfind('/')
                if i>=0:
                    type, extension = ctype.split('/')
                else:
                    extension = ctype  
                    
            # Extension        
            if extension.lower() == 'html':
                # URL "extension"
                if len(_request.args) and '.' in _request.args[-1]:
                    _request.args[-1], extension = \
                        _request.args[-1].rsplit('.',1)                        
            _request.extension = extension
            
            current.response.headers['Content-Type'] = contenttype('.%s' % extension.lower())    
            
            # Content-Type control
            if _request.extension not in current._config.get('rest_type_allowed'):
                raise HTTP(400)
            else:
                current.response.view="api/generic.%s" % (_request.extension)
               
            # Method Control
            if not method in ['GET','POST','DELETE','PUT']:
                raise HTTP(400,"invalid method")
            
            rest_action = _action().get(method,None)
            if not rest_action:
                raise HTTP(400,"method not supported")
            try:
                return rest_action(*_request.args, **_request.vars)
            except TypeError, e:
                exc_type, exc_value, exc_traceback = sys.exc_info()
                if len(traceback.extract_tb(exc_traceback))==1:
                    raise HTTP(400,"invalid arguments")
                else:
                    raise e
        f.__doc__ = action.__doc__
        f.__name__ = action.__name__
        return f
    return wrapper
