# -*- coding: utf-8 -*-

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from mil.base import File

class SMFlow(object):
    def __init__(self, name='flow'):
        self.name = name
        self.connections = {}

    def __str__(self):
        return '{name : %s, connections : %s}' % (self.name, self.connections)

    __repr__ = __str__
    
    def get_name(self):
        return self.name
            
    def get_ports(self):
        return self.connections.keys()
        
    def _prepare(self, string):
        self.connections.clear()
        
        lines = string.split("\n")
        for line in lines:
            if line.find("=>") == -1:
                raise Exception('Bad formed workflow')
            
            line = line.replace(' ','')
            source, destination = line.split("=>", 2)

            port_index = source.find(".")
            if port_index > 0:
                port = source[port_index+1:]
                source = source[0:port_index]
            else:
                port = 'output'

            if not source in self.connections:
                self.connections[source] = {}
            if not destination in self.connections:
                self.connections[destination] = {}

            mappings = self.connections[source]
            mappings[port] = destination
        return True
            
    def fromFile(self, file_path):
        return self._prepare(File.load(file_path))
    
    def fromString(self, string):
        return self._prepare(string)
        
            