# -*- coding: utf-8 -*-

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from mil.tools import get_unique_id
from mil.handler import Handler, isNotHandler

from smworkflow import SMWorkFlow

END_OF_FLOW = '<eof/>'

class SMInstance(object):
    def __init__(self, engine, **kwargs): 
        if isinstance(engine, SMWorkFlow):
            self.engine = engine
        else:
            raise TypeError('engine must be SMEngine instance')    
        
        # Optional parameters
        self.identifier = kwargs.get('identifier', None)
        if self.identifier is None:
            self.identifier = get_unique_id(2,3)
        handler = kwargs.get('handler', None)       
        if isinstance(handler, Handler):
            self.handler = handler
        else:
            raise isNotHandler('handler must be Handler instance')
        data = self.handler.load(self.identifier)
        if data:
            self.step = data['step']
            self.data = data['data']
        else: 
            self.step = None
            self.data = None
        # First step
        if self.step is None:
            self.step = self.engine.first_node
            self.commit()     

    def __str__(self):
        return '{engine : %s, path : %s}' % (repr(self.engine), 
                                             repr(self.handler))

    __repr__ = __str__
        
    def commit(self):
        data = {'step': self.step,
                'data': self.data}
        self.handler.save(self.identifier, data)
        
    def close(self):
        self.handler.delete(self.identifier)
        return END_OF_FLOW
    
    def run(self, values = None):
        try:
            if self.step is None:
                return self.close()
            
            try:
                if self.data['values'] is not None:
                    values = self.data['values']
            except:
                pass
            try:
                returned = self.engine.step(self.step, values) 
                if returned.error:
                    raise Exception(returned.error)     
            except Exception, e:
                raise Exception ('SMInstance.run: %s' % str(e)) 
             
            self.step = returned.connection
            self.data = {'next': returned.next_step,
                         'values': values}
            self.commit()
            return (returned.next_step, returned.called_page)
        except Exception, e:
            raise e
        
    def update_step(self, step, values=None):
        try:
            if step is None:
                return 
            
            self.data['values'] = values
            self.commit()
        except Exception, e:
            raise e    
        
#    def next_step(self, step, values=None):
#        try:
#            if step is None:
#                return 
#
#            returned = self.engine.step(step, values)
#            self.step = returned.connection
#            self.data = {'next': returned.next_step,
#                         'values': values}
#            self.commit()
#        except Exception, e:
#            raise e
        