# -*- coding: utf-8 -*-

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from smobject import SMObject
from smdatastructure import SMDataStructure

try:
    from gluon import redirect
except:
    def redirect(url):
        print url
        
class SMComponent(SMObject):
    """Base class for state machine workflow components."""
    type = 'Base'
    def __init__(self, workflow):
        self.connections = {}
        self.connections['output'] = None
        self.workflow = workflow
        self.data = None
        self.output = 'output'
        self.error = None
        
    def __call__(self, values):
        """
        It must return a tuple: component, data
        """ 
        if isinstance(values, SMDataStructure):
            self.data = values.data
        else:
            self.data = values
            
        if self._sensor():
            self._control()
            self._working()
        
        return self._return()

    def connect(self, component, name='output'):
        self.connections[name] = component
        
    def set_output(self, name):
        self.output = name

    def get_connection(self, name=None):
        if name is None:
            name = self.output
        return self.connections[name]

    def output_ports(self):
        return sorted(self.connections.keys())
    
    def set_error(self, message):
        self.error = message
        print 'ERROR: %s' % message
        
    def _control(self):
        return True
        
    def _working(self):
        self.set_output('output')

    def _return(self):
        return SMDataStructure(connection = self.get_connection(),
                               data = self.data,
                               error = self.error)
   
class SMOpenNode(SMComponent):    
    def _sensor(self):
        if not 'output' in self.connections.keys():
            self.set_error('You must define Output Component')
            return False
        return True

   
class SMBooleanNode(SMComponent):    
    def _sensor(self):
        if not 'true' in self.connections.keys():
            self.set_error('You must define True Component')
            return False
        if not 'false' in self.connections.keys():
            self.set_error('You must define False Component')
            return False
        return True
    
    def _control(self):
        self.status = False
    
    def _working(self):
        if self.status:
            self.set_output('true')
        else:
            self.set_output('false')
   
class SMConnectorNode(SMComponent):    
    def _sensor(self):
        if not 'input' in self.connections.keys():
            self.set_error('You must define Input Component')
            return False
        if not 'output' in self.connections.keys():
            self.set_error('You must define Output Component')
            return False
        return True

class SMPageviewNode(SMComponent):
    def _sensor(self):
        return True
    
class SMPagecallNode(SMComponent):
    def _sensor(self):
        return True
        
    def _control(self):
        self.url = None
        
    def _working(self):
        if self.url:
            redirect(self.url)   
    
class SMEndNode(SMComponent):  
    pass