# -*- coding: utf-8 -*-

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from mil.exceptions import EventNotAllowed
from mil.messagebus import startBusSystem, MessageBus, Message, IBusListener

def startNotifierSystem():
    startBusSystem()
    
class SystemMessage(object):
    """A class that contains a system message and its properties"""
    def __init__(self, message, event='log', severity='info'):
        self.event = event
        self.severity = severity
        self.message = message
        
    def __str__(self):
        return "[%s](%s) %s" % (self.event, self.severity, self.message)
    
    __repr__ = __str__
  
class Notifier(MessageBus):    
    def __init__(self):
        MessageBus.__init__(self)
        
        self.event_types = ['log', 'error']
    
    def add_event_type(self, event=None):
        if event is not None:
            self.event_types.append(event)

    def emit(self, *args, **kwargs):
        content = args[0] #kwargs.get('content', '')
        event = kwargs.get('event', 'log')
        severity = kwargs.get('delimiter', 'info')
        self._post(content, event, severity)        
        
    def log(self, content):
        self._post(content, 'log', 'info')
        
    def error(self, content, severity):
        self._post(content, 'error', severity)
        
    def _post(self, content, event, severity):
        message = SystemMessage(content, event, severity)
        try:
            MessageBus.post(self, "event.system.%s" % event,  message)
        except Exception, e:
            import pprint; pprint.pprint(e)
        
    def subscribe(self, listener, event):
        if not event in self.event_types:
            raise EventNotAllowed(event)
        MessageBus.subscribe(self, listener.receipt, 
                             "event.system.%s" % event)
        
    def unsubscribe(self, listener, event):
        if not event in self.event_types:
            raise EventNotAllowed(event)
        MessageBus.unsubscribe(self, listener.receipt, 
                               "event.system.%s" % event)
        