# -*- coding: utf-8 -*- 

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import os
import subprocess

# An error is raised by subprocess if we ever pass close_fds=True on Windows.
# We want it to be True on all other platforms to not leak file descriptors.
close_fds = True
if os.name == 'nt':
    close_fds = False
    
def runCommand(command, timeout=None):
    '''
    Execute a command.
    
    Execute a command, read the output and return it back.
    
    @param cmd: command to execute
    @param timeout: process timeout in seconds
    @return: a tuple of three: exit code, stdout, then stderr
    @raise OSError: on missing command or if a timeout was reached
    '''    
    ret_out = None # process output
    ret_err = None # stderr
    ret_ret = None # return code
    
    p = subprocess.Popen(command, shell=True, 
                         stdout=subprocess.PIPE, 
                         stderr=subprocess.PIPE)
    # if timeout is not set, wait for process to complete
    if not timeout:
        ret_ret = p.wait()
    else:
        fin_time = time.time() + timeout
        while p.poll() == None and fin_time > time.time():
            time.sleep(1)
    
        # if timeout reached, raise an exception
        if fin_time < time.time():
            os.kill(p.pid, signal.SIGKILL)
            raise OSError("Process timeout has been reached")
    
        ret_ret = p.returncode    
    ret_out, ret_err = p.communicate()    
    return (ret_out, ret_err, ret_ret)

class System(object):    
    def call(self, command, message='Error'):
        if isinstance(command, list):
            command = " ".join(command)
        (out, err, ret) = runCommand(command)
        if ret:
            raise Exception(message)