# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import current

from mil.tools import get_request_args

from core.subapplication import SubApp, SubAppModule

from project import ProjectHandler
from project.models.card import ProjectCardModel

class ProjectCard(SubAppModule):
    def __init__(self, controller='project'):
        SubAppModule.__init__(self, 'project', 'card')
    
    @SubApp.requires_membership('admin')
    def index(self):
        current._menu.show()
        insKlss = ProjectCardModel(current.db) 
        return dict(form=insKlss.grid())
    
    @SubApp.requires_login()   
    def create(self):
        current._menu.show()
        insKlss = ProjectCardModel(current.db) 
        form = insKlss.form()
        if form.process().accepted:
            project = insKlss.get_one_by_id(form.vars.id)
            # Checks flow redirection
            self.redirect(project.name)
        return dict(form=form)
    
    @SubApp.requires_login() 
    def edit(self):
        [id, params] = get_request_args(1)  
        current._menu.show()
        
        ProjectHandler.user_set_project(int(id)) 
        if current._government.has_permission('project', 'edit_card'):       
            
            insKlss = ProjectCardModel(current.db) 
            fg = insKlss.form(id)
            if fg.process().accepted:
                current.request.flash = current.T('Data updated')
            else:
                current.request.flash = current.T('Data not updated')
                
            return dict(form=fg)
        else:
            return dict(form=current.T('You have not permission'))
    