# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import current, redirect, URL
from gluon.storage import Storage

from mil.tools import get_request_args

from core.subapplication import SubApp, SubAppModule

from project import ProjectHandler
from project.models.card import ProjectCardModel
from project.models.candidate import ProjectCandidateModel

class ProjectJoin(SubAppModule):
    def __init__(self):
        SubAppModule.__init__(self, 'project', 'join')
    
    def index(self):
        current._menu.show()
        
        prjKlss = ProjectCardModel(current.db)
        insKlss = ProjectCandidateModel(current.db)
        
        # Public projects
        publics = prjKlss.get_all_public_ids()
        
        # Requested projects
        projects = insKlss.get_user_request_ids(current._user)
        
        joined_projects=[]
        for project in projects:
            joined_projects.append(project.project)
        
        list_p = []
        list_r = []
        for row in publics:
            try:
                has_petition=joined_projects.index(row.id)
                list_r.append(ProjectHandler.prepare_card(row.id))
            except:
                list_p.append(ProjectHandler.prepare_card(row.id))
            
        return dict(list_p=list_p, list_r=list_r, user=current._user)  
    
    def to(self):
        [p_name, params] = get_request_args(1)
            
        # Menu
        current._menu.show()
               
        # Project id from project name
        project = ProjectHandler.get_by_name(p_name)
               
        # Form properties
        insKlss = ProjectCandidateModel(current.db)
        form = insKlss.join_to_form()
        
        form.vars.project = project.id
        form.vars.user = current._user.id
        form.custom.widget.user = "%s %s (%s)" % (current._user.first_name,
                                                  current._user.last_name,
                                                  current._user.email)
        
        if form.accepts(current.request.vars, current.session):
            current.session.flash = current.T('Request sended to leader')
            redirect(URL(current.request.application, 'project', 'join'))
        elif form.errors:
            current.response.flash = current.T('Fill the form')
        
        return dict(form=form, title=project.title)    

    def management(self):
        [p_name, params] = get_request_args(1)
            
        # Menu
        current._menu.show()
               
        # Project id from project name
        project = ProjectHandler.get_by_name(p_name)
        ProjectHandler.user_set_project(p_name)
               
        # Form properties
        insKlss = ProjectCandidateModel(current.db)
        
        form = insKlss.grid(project.id)
        
        return dict(form=form, pid=project.id, user=current._user)    
    
    def unjoin(self):
        [p_name, params] = get_request_args(1)
            
        # Menu
        current._menu.show()
               
        # Project id from project name
        project = ProjectHandler.get_by_name(p_name)
               
        # Delete database row
        insKlss = ProjectCandidateModel(current.db)    
        insKlss.delete(project.id, current._user.id)
        
        # Return
        current.session.flash = "%s %s" % (current.T('Unjoin project'), 
                                           project.name)
        redirect(URL(current.request.application, 
                     'project', 'join', args=['index']))
        
    def reject(self):
        [id, params] = get_request_args(1)
            
        # Menu
        current._menu.show()                       
        
        try:
            # Data
            insKlss = ProjectCandidateModel(current.db)  
            candidate = insKlss.get_one(id)
            
            # Project id from project name
            project = ProjectHandler.get_by_id(candidate.project)
            
            # Reject
            insKlss.reject(candidate.id)
            
            current.session.flash = current.T('Petition rejected')
        except:
            current.session.flash = current.T('Unknown identifier')
            
        redirect(URL(current.request.application, 
                     'project', 'join', args=['management', project.name]))        
                    
    def accept(self):
        [id, params] = get_request_args(1)
            
        # Menu
        current._menu.show()                       
        
        try:
            # Data
            insKlss = ProjectCandidateModel(current.db)  
            candidate = insKlss.get_one(id)
            
            # Project id from project name
            project = ProjectHandler.get_by_id(candidate.project)
            
            # Accept
            insKlss.accept(candidate.id)
            
            # Join to project as guest
            from project.models.team import ProjectTeamModel
            teamKlass = ProjectTeamModel(current.db)
            # Collaborator guest
            from masters.profiles import Profiles  
            profile = Profiles(current._config, current.db).get_id_by_subapp_and_role('project', 'guest')    
            ProjectTeamModel(current.db).set_member(project.id, 
                                                    candidate.user, 
                                                    profile)
            # Activate services
            ProjectHandler.join_member_as(project.id, candidate.user, profile)
                    
            
            current.session.flash = current.T('Petition accepted')
        except:
            current.session.flash = current.T('Unknown identifier')
            
        redirect(URL(current.request.application, 
                     'project', 'join', args=['management', project.name]))    
        
    def info(self):
        [mail, params] = get_request_args(1)
            
        # Menu
        current._menu.show() 
        
        try:
            # Get id
            if mail == current._user.email:
                user = current._user
            else:
                # TODO: enable for administration searching
                user = None
            
            if user is None:
                data = []
            else:
                # Data
                insKlss = ProjectCandidateModel(current.db)  
                requests = insKlss.get_user_requests(user)
                
                data = []
                for request in requests:
                    # Project id from project name
                    project = ProjectHandler.get_by_id(request.project)
                    
                    data.append(Storage(title=project.title,
                                        objetive=request.objetive,
                                        processed=request.processed,
                                        accepted=request.accepted))
        except Exception, e:
            data = []
            current._syslog.error('project.join.info: %s' % str(e), 
                                  'warning')
        
        return dict(data=data)                    
        