# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################
import datetime

from gluon import *

from card import ProjectCardModel

class ProjectCandidateModel(object):    
    @staticmethod
    def define_table(db):                
        if 'candidate' in db.tables:
            return
        
        db.define_table('candidate',
            Field('project', 'reference project_card', 
                  required=True, label=current.T('Project'), 
                  writable=False, readable=False),
            Field('user', 'reference auth_user', 
                  required=True, label=current.T('User'), 
                  writable=False, readable=True),
            Field('objetive', 'text', required=True, 
                  requires=IS_NOT_EMPTY(), label=current.T('Objetive')),
            Field('request_date', 'datetime', default=datetime.datetime.now(), 
                  required=True, label=current.T('Request date'), 
                  writable=False, readable=True, 
                  requires=IS_DATETIME(format=current.T(current._config.default_datetime_format))),
            Field('processed','boolean', default=False, required=True, 
                  writable=False, readable=False),
            Field('accepted','boolean', default=False, required=True, 
                  writable=False, readable=False),
            Field('action_date', 'datetime', label=current.T('Action date'), 
                  writable=False, readable=False, 
                  requires=IS_DATETIME(format=current.T(current._config.default_datetime_format))),
            format='')
                
    def __init__(self, db):
        self.db = db
        
        ProjectCardModel.define_table(self.db)
        ProjectCandidateModel.define_table(self.db)
        
    def grid(self, id=None):
        # Links
        def link_accept(r):
            return A(current.T('Accept'),
                     _href=URL(r=current.request, 
                               c="project", 
                               f="join", 
                               args=['accept', r.id]))
        def link_reject(r):
            return A(current.T('Reject'),
                     _href=URL(r=current.request, 
                               c="project", 
                               f="join", 
                               args=['reject', r.id]))

        if id is None:
            query = self.db.candidate
        else:            
            query = ((self.db.candidate.project == id) & \
                     (self.db.candidate.processed != True))
            
        fields = (self.db.candidate.id,
                  self.db.candidate.user,
                  self.db.candidate.objetive,
                  self.db.candidate.request_date)
        form = SQLFORM.grid(query, 
                            searchable=False,
                            csv=False, 
                            fields=fields,
                            create=False,           # Can not create
                            deletable=False,        # Can not delete
                            editable=False,         # Can not edit
                            details=False,          # Can not show
                            user_signature = False,
                            maxtextlength=100,
                            links=[lambda r:link_accept(r), 
                                   lambda r:link_reject(r)])
        return form
            
    def join_to_form(self):
        # Form
        form = SQLFORM(self.db.candidate)
        
        return form
    
    def get_user_requests(self, user):
        query = (self.db.candidate.user == user.id)
        return self.db(query).select()
    
    def get_user_request_ids(self, user):
        query = ((self.db.candidate.user == user.id) & \
                 (self.db.candidate.processed == False))
        return self.db(query).select(self.db.candidate.project)
    
    def get_one(self, id):        
        return self.db(self.db.candidate.id == id).select().first()
    
    def delete(self, project, user):
        query = (self.db.candidate.project == project) & \
                (self.db.candidate.user == user)
        return self.db(query).delete()
        
    def reject(self, id):
        query = (self.db.candidate.id == id)
        self.db(query).update(action_date=datetime.datetime.now(),
                              processed=True)
        
    def accept(self, id):
        query = (self.db.candidate.id == id)
        self.db(query).update(action_date=datetime.datetime.now(),
                              processed=True,
                              accepted=True)
