# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import *

from mil.tools.common import get_unique_id, normalize

from type import ProjectTypeModel

class ProjectCardModel(object):
    @staticmethod
    def define_table(db):         
        if 'project_card' in db.tables:
            return
        
        ProjectTypeModel.define_table(db)
        
        db.define_table('project_card',
                        Field('identifier', 'string', length=20, 
                              default=get_unique_id(2,4), required=False, 
                              notnull=True, label=current.T('Identifier'), 
                              writable=False),
                        Field('name', 'string', length=200, 
                              requires=IS_ALPHANUMERIC(), required=False, 
                              notnull=False, unique=True, 
                              label=current.T('Name'), 
                              compute=lambda r:normalize(r.title)),
                        Field('title', 'string', length=200, required=True, 
                              notnull=True, label=current.T('Title')),
                        Field('type', 'reference project_type', required=True, 
                              notnull=True, label=current.T('Type')),
                        Field('private', 'boolean', required=True, notnull=True, 
                              default=False, label=current.T('Private')),
                        Field('description', 'text', 
                              label=current.T('Description')),
                        Field('start_date', 'date', required=True, notnull=True, 
                              label=current.T('Start'), 
                              requires=IS_DATE(format=current.T("%Y-%m-%d"))),
                        Field('end_date', 'date', required=True, notnull=True, 
                              label=current.T('End'), 
                              requires=IS_DATE(format=current.T("%Y-%m-%d"))),
                        Field('objetive', 'text', label=current.T('Objetive')),
                        Field('resume', 'text', label=current.T('Resume')),
                        Field('active', 'boolean', readable=False, 
                              writable=False, notnull=True, default=False, 
                              label=current.T('Active')),
                        Field('deleted', 'boolean', readable=False, 
                              writable=False, notnull=True, default=False, 
                              label=current.T('Check to delete')),
                        Field('create_lib', 'boolean', readable=False, 
                              writable=False, notnull=True, default=False, 
                              label=current.T('Processed')),
                        format='%(name)s')       
        
    def __init__(self, db):
        self.db = db
        
        ProjectCardModel.define_table(db)
        
    def grid(self):
        fields = (self.db.project_card.id,
                  self.db.project_card.title,
                  self.db.project_card.type,
                  self.db.project_card.private,
                  self.db.project_card.start_date,
                  self.db.project_card.end_date)
        form = SQLFORM.grid(self.db.project_card, 
                            searchable=False,
                            csv=False, 
                            fields=fields,
                            user_signature = False,
                            maxtextlength=100)
        return form
    
    def form(self, record = None):
        form = SQLFORM(self.db.project_card, record)
        return form        
    
    def get_all(self):
        return self.db(self.db.project_card.id > 0).select()
    
    def get_one_by_id(self, id):
        return self.db(self.db.project_card.id == id).select().first()
    
    def get_all_public_ids(self):
        # Public projects
        query = ((self.db.project_card.deleted == False) & \
                 (self.db.project_card.private == False))
        return self.db(query).select(self.db.project_card.id)
    
    def add(self, data):
        return self.db.project_card.insert(**self.db.project_card._filter_fields(data))
    
    def lock(self, id):        
        row = self.db(self.db.project_card.id == id).select().first()
        try:            
            if row.active is False:
                active = True
            else:
                active = False
                
            row.update_record(active=active)
            return True
        except:
            return False  
        return 
    
    def delete(self, id): 
        row = self.db(self.db.project_card.id == id).select().first()
        row.update_record(deleted=True)
#        return self.db(self.db.project_card.id == id).delete()
    