# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import *

class ProjectProfileModel(object):    
    @staticmethod
    def define_table(db):   
        if 'project_user_profile' in db.tables:
            return
        
        db.define_table('project_user_profile',
                        Field('name', 'string', length=100, required=True, 
                              notnull=True, label=current.T('Profile')),
                        Field('read_auth', 'boolean', required=True, 
                              notnull=True, label=current.T('Read')),
                        Field('write_auth', 'boolean', required=True, 
                              notnull=True, label=current.T('Write')),
                        Field('exec_auth', 'boolean', required=True, 
                              notnull=True, label=current.T('Execution')),
                        format='%(name)s')   
        
        try:
            if db(db.project_user_profile.id >= 1).count() == 0:
                db.project_user_profile.insert(name='Líder',
                                               read_auth=True,
                                               write_auth=True,
                                               exec_auth=True)
                db.project_user_profile.insert(name='Colaborador',
                                               read_auth=True,
                                               write_auth=True,
                                               exec_auth=False)
                db.project_user_profile.insert(name='Invitado',
                                               read_auth=True,
                                               write_auth=False,
                                               exec_auth=False)
        except:
            pass 
        
    def __init__(self, db):
        self.db = db
        
        ProjectProfileModel.define_table(db)
        
    def get(self):        
        return self.db(self.db.project_user_profile.id > 0).select()
        
    def grid(self):
        form = SQLFORM.grid(self.db.project_user_profile, 
                            searchable=False,
                            csv=False, 
                            user_signature = False,
                            maxtextlength=100)
        return form