# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import datetime

from gluon import *

from mil.tools.common import get_unique_id

from card import ProjectCardModel

class RoadmapModel(object):
    @staticmethod
    def define_table(db):        
        if 'roadmap' in db.tables:
            return
        
        db.define_table('roadmap',
            Field('identifier', 'string', length=20, 
                  default=get_unique_id(2,4), notnull=True, 
                  label=current.T('Identifier'), 
                  writable=False, readable=False),
            Field('project', 'string', length=20, notnull=True, 
                  label=current.T('Project')),
            Field('sumary', 'string', length=250, required=True, notnull=True, 
                  requires = CLEANUP(),
                  label=current.T('Sumary')),
            Field('content', 'text', required=True, notnull=True, 
                  label=current.T('Description')),
            Field('start_date', 'datetime', default=datetime.datetime.now(), 
                  notnull=True, label=current.T('Creation Date'), 
                  writable=False, readable=False, 
                  requires=IS_DATE(format=current.T(current._config.default_datetime_format))),
            Field('creator', 'string', length=200, default='', notnull=True, 
                  label=current.T('Creator')),
            Field('update_date', 'datetime', label=current.T('Update Date'), 
                  requires=IS_DATE(format=current.T(current._config.default_datetime_format))),
            Field('responsible', 'string', length=150, 
                  label=current.T('Assigned to')),
            Field('close_date', 'datetime', 
                  label=current.T('Close Date'), 
                  requires=IS_DATE(format=current.T(current._config.default_datetime_format))),
            Field('status', 'boolean', default=False, notnull=True, 
                  label=current.T('Finished')),
            Field('priority', 'integer', default=0, notnull=True, 
                  label=current.T('Priority')),
            Field('init_date', 'date', default=datetime.datetime.now(), 
                  notnull=True, label=current.T('Initial Date'), 
                  writable=False, readable=False, 
                  requires=IS_DATE(format=current.T("%Y-%m-%d"))),
            Field('end_date', 'date', default=datetime.datetime.now(), 
                  notnull=True, label=current.T('End Date'), 
                  writable=False, readable=False, 
                  requires=IS_DATE(format=current.T("%Y-%m-%d"))),
            format='%(sumary)s')
        
        
    def __init__(self, db):
        self.db = db
        
        RoadmapModel.define_table(db)
        
    def grid(self):
        form = SQLFORM.grid(self.db.roadmap, 
                            searchable=False,
                            csv=False, 
                            user_signature = False,
                            maxtextlength=100)
        return form