# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import *
from gluon.storage import Storage

from project.subapp import ProjectInstance

class ProjectServicesModel(object):    
    @staticmethod
    def define_table(db):       
        if 'project_service' in db.tables:
            return
        
        db.define_table('project_service',
                        Field('project', 'reference project_card'),
                        Field('services', 'list:string'),
                        format='') 
        
        prj = ProjectInstance()
        services = prj.get_services_id()
        db.project_service.services.requires = IS_IN_SET(services)
                
    def __init__(self, db):
        self.db = db

        ProjectServicesModel.define_table(self.db)
        
    def get(self):        
        return self.db(self.db.project_service.id > 0).select()
        
    def get_one(self, id):        
        return self.db(self.db.project_service.id == id).select().first()
    
    def get_one_by_project(self, id):        
        return self.db(self.db.project_service.project == id).select().first()
        
    def get_services(self, id): 
        # Get Seervices       
        data = self.db(self.db.project_service.id == id).select().first()
        
        # Services Handler
        prj = ProjectInstance()
        
        ret_val = []
        for service in data.services:
            srv_data = prj.get_service(service)
            tmp = Storage(label=srv_data.label,
                          controller=prj.name,
                          function=srv_data.function)
            ret_val.append(tmp)
            
        return ret_val
    
    def get_services_by_project(self, id): 
        # Get Seervices       
        data = self.db(self.db.project_service.project == id).select().first()
        
        # Services Handler
        prj = ProjectInstance()
        
        ret_val = []
        for service in data.services:
            srv_data = prj.get_service(service)
            tmp = Storage(label=srv_data.label,
                          controller=prj.name,
                          function=srv_data.function)
            ret_val.append(tmp)
            
        return ret_val
    
    def set(self, id, services):
        self.db.project_service.update_or_insert(
                    self.db.project_service.project==id,
                    project=id,
                    services=services)
        
    def delete_by_project(self, id):
        return self.db(self.db.project_service.project == id).delete()
        