# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import datetime

from gluon import *

from mil.tools.common import get_unique_id

class TaskModel(object):
    @staticmethod
    def define_table(db):        
        if 'task' in db.tables:
            return

        db.define_table('task',
                        Field('identifier', 'string', length=20, 
                              default=get_unique_id(2,4), notnull=True, 
                              label=current.T('Identifier'), 
                              writable=False, readable=False),
                        Field('project', 'string', length=20, notnull=True, 
                              label=current.T('Project')),
                        Field('sumary', 'string', length=250, required=True, 
                              requires = CLEANUP(),
                              notnull=True, label=current.T('Sumary')),
                        Field('content', 'text', required=True, notnull=True, 
                              label=current.T('Description')),
                        Field('start_date', 'datetime', 
                              default=datetime.datetime.now(), notnull=True, 
                              label=current.T('Creation Date'), 
                              writable=False, readable=False, 
                              requires=IS_DATE(format=current.T(current._config.default_datetime_format))),
                        Field('date', 'datetime', 
                              label=current.T('Date'), 
                              writable=True, readable=True, 
                              requires=IS_DATE(format=current.T(current._config.default_datetime_format))),
                        Field('creator', 'string', length=200, default='', 
                              notnull=True, label=current.T('Creator')),
                        Field('update_date', 'datetime',
                               label=current.T('Update Date'), 
                               requires=IS_DATE(format=current.T(current._config.default_datetime_format))),
                        Field('responsible', 'string', length=150, 
                              label=current.T('Assigned to')),
                        Field('close_date', 'datetime', 
                              label=current.T('Close Date'), 
                              requires=IS_DATE(format=current.T(current._config.default_datetime_format))),
                        Field('status', 'boolean', default=False, notnull=True, 
                              label=current.T('Finished')),
                        Field('priority', 'integer', default=0, notnull=True, 
                              label=current.T('Priority')),
                        format='%(sumary)s')
                
    def __init__(self, db):
        self.db = db
        
        TaskModel.define_table(db)
        
    def grid(self):
        form = SQLFORM.grid(self.db.task, 
                            searchable=False,
                            csv=False, 
                            user_signature = False,
                            maxtextlength=100)
        return form
        
    def user_data(self, user, status=None):
        # Project Card table
        from project.models.card import ProjectCardModel
        ProjectCardModel.define_table(self.db)
        
        query = ((self.db.project_card.identifier==self.db.task.project) & \
                 (self.db.task.responsible==user))
        if status is not None:
            if status:
                query = query & (self.db.task.status==True)
        data = self.db(query).select(self.db.task.id, 
                                     self.db.project_card.name,
                                     self.db.project_card.title,
                                     self.db.task.sumary, 
                                     self.db.task.priority, 
                                     orderby=~self.db.task.priority|self.db.task.date)
        
        list = []
        for row in data:
            list.append({'id': row.task.id,
                         'pname': row.project_card.name,
                         'ptitle': row.project_card.title,
                         'title': row.task.sumary,
                         'priority': str(row.task.priority)})
        return list
        
    def user_grid(self, user):
        query = (self.db.task.responsible==user)
        
        fields = (self.db.task.id, 
                  self.db.task.sumary,
                  self.db.task.priority)     
        default_sort_order=[self.db.task.date, self.db.task.priority]
        
        def link_show(r):
            return A(current.T('Show'),
                     _href=URL(r=current.request, 
                               c="project", 
                               f="task", 
                               args=['read', r.project, r.id],
                               extension='html'))
                
        form = SQLFORM.grid(query, 
                            fields=fields, 
                            orderby=default_sort_order,
                            searchable=False,
                            csv=False, 
                            details=False,
                            create=False,
                            editable=False,
                            sortable=False,
                            deletable=False,
                            user_signature = False,
                            maxtextlength=100,
                            links=[lambda r:link_show(r)])
        return form