# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import *

from masters.profiles import Profiles

from project.models.card import ProjectCardModel
from project.models.profile import ProjectProfileModel

class ProjectTeamModel(object):    
    @staticmethod
    def define_table(db):     
        if 'project_team' in db.tables:
            return
        
        ProjectCardModel.define_table(db)
        ProjectProfileModel.define_table(db)
        
        db.define_table('project_team',
                        Field('project', 'reference project_card', 
                              readable=False, writable=False,
                              required=False, label=current.T('Project')),
                        Field('user', 'reference auth_user', 
                              required=False, label=current.T('User')),
                        Field('profile', 'reference auth_group', 
                              required=False, label=current.T('Profile')),
                        format='')
                
    def __init__(self, db):
        self.db = db
        
        ProjectTeamModel.define_table(self.db)
        
    def grid(self, id=None):
        if id is None:
            query = self.db.project_team
        else:            
            query = (self.db.project_team.project == id)
            
        # Profile 
        iKls = Profiles(current._config, self.db)
          
#        def link_add(r):  
#            has_permission = False
#            if current._user.is_admin:
#                has_permission = True
#            else:
#                if current._government.has_permission('project', 'add_member'):
#                    if r.profile == iKls.get_id_by_subapp_and_role('project', 'leader'): 
#                        has_permission = True
#                
#            if has_permission:         
#                return A(current.T('Add member'),
#                         _href=URL(r=current.request, 
#                                   c="project", 
#                                   f="team", 
#                                   args=['join', r.project]))
#            else:
#                return ''         
          
        def link_edit(r):  
            has_permission = False
            if current._user.is_admin:
                has_permission = True
            else:
                if current._government.has_permission('project', 'edit_member'):
                    if r.profile == iKls.get_id_by_subapp_and_role('project', 'leader'): 
                        has_permission = True
                
            if has_permission:         
                return A(current.T('Edit'),
                         _href=URL(r=current.request, 
                                   c="project", 
                                   f="team", 
                                   args=['edit', r.project, r.user, r.profile]))
            else:
                return ''         
          
        def link_delete(r):
            has_permission = False
            if current._user.is_admin:
                has_permission = True
            else:
                if current._government.has_permission('project', 'add_member'):
                    if r.profile != iKls.get_id_by_subapp_and_role('project', 'leader'): 
                        has_permission = True
                
            if has_permission:
                return A(current.T('Delete'),
                         _href=URL(r=current.request, 
                                   c="project", 
                                   f="team", 
                                   args=['unjoin', r.project, r.user]))
            else:
                return ''
                  
        form = SQLFORM.grid(query, 
                            searchable=False,
                            csv=False, 
                            details=False,
                            create=False,
                            editable=False,
                            deletable=False,
                            user_signature = False,
                            maxtextlength=100,
                            links=[lambda r:link_edit(r),
                                   lambda r:link_delete(r)])
        return form
    
    def set_leader(self, project_id, leader):
        iKls = Profiles(current._config, self.db)
        leader_id = iKls.get_id_by_subapp_and_role('project', 'leader')
        
        query = (self.db.project_team.project == project_id)
        self.db.project_team.update_or_insert(query,
                                              project=project_id,
                                              user=leader,
                                              profile=leader_id)
        
    def set_member(self, project_id, member, profile):
        self.add_as(project_id, member, profile)  
        
    def get_leader(self, project_id):  
        # Profile 
        iKls = Profiles(current._config, self.db)
        leader_id = iKls.get_id_by_subapp_and_role('project', 'leader')
        query = (self.db.auth_user.id == self.db.project_team.user) & \
                (self.db.project_team.profile == leader_id) & \
                (self.db.project_team.project == project_id)
        
        return self.db(query).select(self.db.auth_user.first_name,
                                     self.db.auth_user.last_name,
                                     self.db.auth_user.email).first()
    
    def get_leader_id(self, project_id):
        query = (self.db.project_team.project == project_id)
        return self.db(query).select(self.db.project_team.user).first().user
        
    def add_as(self, project, user, profile=2):
        return  self.db.project_team.insert(project=project,
                                            user=user,
                                            profile=profile)
        
    def get_user_projects(self, user_id):
        query = (self.db.project_team.user == user_id)
        rows = self.db(query).select(self.db.project_team.project)
        
        ret_val = []
        for row in rows:
            ret_val.append(row.project)
            
        return ret_val
    
    def delete_by_project(self, id):
        return self.db(self.db.project_team.project == id).delete()
    
    def delete_user_project(self, user, project):
        query = (self.db.project_team.user == user) & \
                (self.db.project_team.project == project)
        return self.db(query).delete()
        
    def change_to(self, project, user, profile):
        query = (self.db.project_team.user == user) & \
                (self.db.project_team.project == project)
        return  self.db.project_team.update_or_insert(query,
                                                      project=project,
                                                      user=user,
                                                      profile=profile)
