# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import *

from masters.services import Services

from services.iservice import IService

_SERVICES = ['ProjectTypeModel']

class ProjectTypeModel(IService):
    _actions = [{'label': 'Alta de Tipo de Proyecto', 
                 'id': 'proy_tipo_alta',
                 'options': [{'id': 'proy_tipo_auth', 
                              'label': 'Se permite definir nuevo tipo', 
                              'default': False},
                             {'id': 'proy_tipo_none', 
                              'label': 'No se permite definir nuevo tipo', 
                              'default': True}]}
                ]
    _controller = ''
    _literal = 'Project Types'
    _permissions = ['add']
    
    @staticmethod
    def define_table(db):    
        if 'project_type' in db.tables:  
            return
          
        db.define_table('project_type',
                        Field('name', 'string', length=200, required=True, 
                              notnull=True, label=current.T('Name')),
                        Field('description', 'text', 
                              label=current.T('Description')),
                        format='%(name)s')
        
        try:
            if db(db.project_type.id >= 1).count() == 0:
                db.project_type.insert(name='Consultoría',
                    description='Proyecto de Consultoría.')
                db.project_type.insert(name='Publicación',
                    description='Proyecto para publicar desarrollos.')
                db.project_type.insert(name='Liberación',
                    description='Proyecto para la liberación de desarrollos.')
        except:
            pass 
        
    def __init__(self, db):
        self.db = db
        
        ProjectTypeModel.define_table(db)
        
    def grid(self):
        form = SQLFORM.grid(self.db.project_type, 
                            searchable=False,
                            csv=False, 
                            user_signature = False,
                            maxtextlength=100)
        return form
    
    def get_all(self):
        return self.db(self.db.project_type.id > 0).select()
    
    def get_keys(self):
        rows = self.db(self.db.project_type.id > 0).select()
        data = {}
        for row in rows:
            data[row.id] = row.name
            
        return data
    