# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import *

from project.subapp import ProjectInstance

class ProjectTypeServicesModel(object):    
    @staticmethod
    def define_table(db):       
        if 'project_type_has_service' in db.tables:
            return

        db.define_table('project_type_has_service',
                        Field('project_type', 'reference project_type'),
                        Field('services', 'list:string'),
                        format='') 
        
        prj = ProjectInstance()
        services = prj.get_services_id()
        db.project_type_has_service.services.requires = IS_IN_SET(services)
        
        try:
            if db(db.project_type_has_service.id >= 1).count() == 0:
                if current._config.get('installation_type') == "lite":
                    db.project_type_has_service.insert(project_type='1',
                                                    services=['roadmap', 'task', 'incidence'])
                    db.project_type_has_service.insert(project_type='2',
                                                    services=['roadmap', 'task', 'bug'])
                    db.project_type_has_service.insert(project_type='3',
                                                    services=['board', 'roadmap', 'task'])
                else:
                    db.project_type_has_service.insert(project_type='1',
                                                    services=['roadmap', 'task', 'incidence'])
                    db.project_type_has_service.insert(project_type='2',
                                                    services=['roadmap', 'task', 'bug'])
                    db.project_type_has_service.insert(project_type='3',
                                                    services=['board', 'roadmap', 'task'])
        except Exception, e:
            pass 

    def __init__(self, db):
        self.db = db

        ProjectTypeServicesModel.define_table(self.db)
        
    def get(self):        
        return self.db(self.db.project_type_has_service.id > 0).select()
        
    def get_by_type(self, type):   
        query = (self.db.project_type_has_service.project_type == type)
        return self.db(query).select().first()
    
    def set(self, id, services):
        self.db.project_type_has_service.update_or_insert(
                    self.db.project_type_has_service.project_type==id,
                    project_type=id,
                    services=services)
        
    def get_old(self):
        insPType =  ProjectType(self.db)
        insServ = Services(self.db)
        
        ret_dict = dict()
        ret_dict["types"] = insPType.get_all()
        list_types = dict()
        for item in ret_dict["types"] :
            list_types[item.id] = item.name
        ret_dict["list_types"] = list_types
        
        ret_dict["services"] = insServ.get_all()
        list_services = dict()
        for item in ret_dict["services"]:
            list_services[item.id] = item.name
        ret_dict["list_services"] = list_services
        
        ret_dict["proposal_items"] = \
            self.db(self.db.project_type_has_service.id > 0).select()
        
        return ret_dict
    
    
    def save_data_old(self):
        from gluon.contrib import simplejson
        from gluon.storage import Storage
        
        # This two lines are needed to define referenced tables
        insPType =  ProjectType(self.db)
        insServ = Services(self.db)
        
        data = simplejson.loads(current.request.body.read())
        
        # First, we process current items        
        rows = self.db(self.db.project_type_has_service.id > 0).select()
        items_list = []
        for key in data["items"].keys():
            items_list.append(int(key))
        item_added = [] 
        for row in rows:
            if row.project_type in items_list:  # Update data
                item_added.append(row.project_type)
                list_items = []             
                for item in data["items"][str(row.project_type)]:
                    list_items.append(int(item))                
                row.update_record(services = list_items)
            else:   # Delete row
                row.delete_record()
        
        # Finally, we added new items
        for key,val in data["items"].items():
            if int(key) not in item_added:
                list_items = []             
                for item in val:
                    list_items.append(int(item))            
                self.db.project_type_has_service.insert(project_type = int(key), 
                                                        services = list_items)

