# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from datetime import datetime

from gluon import *
from gluon.storage import Storage
import gluon.contrib.simplejson as json

from mil.tools import get_request_args
from mil.tools.common import return2json, escape2html

from core.subapplication import SubApp, SubAppModule

from project import ProjectHandler
from project.visual import VisualOL
from project.visual.element_ol import OLElement

class OLController(SubAppModule):
    def __init__(self, controller, type, view=None):
        SubAppModule.__init__(self, controller, type, view)
        
        self.type = type
        
        self.label = type.capitalize()
        self.label_list = '%s list' % type.capitalize()
 
    def index(self):
        current._menu.show()
        
        # Handle project
        [name, params] = get_request_args(1)
        ProjectHandler.user_set_project(name)
          
        # Elements options
        if params is None:
            return
        
        try:
            type = params[0]
        except IndexError:
            type = 'open'
            filter = 'all'
        try:
            filter = params[1]
        except IndexError:
            filter = 'all'
        try:
            range = params[2]
        except IndexError:
            range = ''
            
        if type:
            if type == 'open':
                sess_ol_type = 'open'
            if type == 'close':
                sess_ol_type = 'close'
            if type == 'all':
                sess_ol_type = 'all'
        else:
            sess_ol_type = ''
            
        if filter:
            if filter == 'all':
                sess_ol_filter = 'all'
            else:
                sess_ol_filter = 'my'
        else:
            sess_ol_filter = ''
            
        if range:
            if range == '':
                sess_ol_range = ''
            elif range == 'last10':
                sess_ol_range = 'last10'
            elif range == 'oneweek':
                sess_ol_range = 'oneweek'
            elif range == 'twoweeks':
                sess_ol_range = 'twoweeks'
            elif range == 'onemonth':
                sess_ol_range = 'onemonth'
            elif range == 'older':
                sess_ol_range = 'older'
            elif range == 'assigned':
                sess_ol_range = 'assigned'
            elif range == 'created':
                sess_ol_range = 'created'
            elif range == 'unassigned':
                sess_ol_range = 'unassigned'
        else:
            sess_ol_range = ''
    
        if sess_ol_type is None:
            sess_ol_type = 'open'
            
        # Save data into session
        current.session[self.type] = Storage(type=sess_ol_type,
                                             filter=sess_ol_filter,
                                             range=sess_ol_range)
            
        # Visual element
        visual = VisualOL(self.type)
        
        # Permission
        auth_data = current._government.get_module_permissions(self.type)
        auth = {'assignments': auth_data['access_assign_%s' % self.type],
                'assign_to': auth_data['change_assign_%s' % self.type],
                'tags': auth_data['access_tag_%s' % self.type],
                'create_tag': auth_data['add_tag_%s' % self.type],
                'delete_tag': auth_data['delete_tag_%s' % self.type],
                'documents': auth_data['access_file_%s' % self.type],
                'create_document': auth_data['add_file_%s' % self.type],
                'delete_document': auth_data['delete_file_%s' % self.type],
                'download_document': auth_data['download_file_%s' % self.type],
                'can_close': auth_data['change_status_%s' % self.type],
                'add_childs': False,
                'create': auth_data['add_%s' % self.type],
                'comments': auth_data['access_comment_%s' % self.type],
                'create_comment': auth_data['add_comment_%s' % self.type],
                'edit': auth_data['edit_%s' % self.type],
                'delete': auth_data['delete_%s' % self.type]
                }
        auth_json = XML(json.dumps(auth), sanitize=False)        
    
        (title, params) = visual.header()
        (type, data) = visual.menu()
        items = visual.get_data()
        return dict(module=self.type,
                    header_title=title,
                    header_params=params,
                    label=self.label,
                    label_list=self.label_list,
                    type=type,
                    numbers=data,
                    list_items=[],
                    items=items,
                    auth=auth_json,
                    refresh_lapse=5)
    
    def read(self):
        [name,id,params] = get_request_args(2)
        
        current._menu.show()
        
        # Handle project
        ProjectHandler.user_set_project(name)
        
        # Permission
        auth_data = current._government.get_module_permissions(self.type)
        auth = {'assigments': False,
                'assign_to': False,
                'tags': auth_data['access_tag_%s' % self.type],
                'create_tag': False,
                'delete_tag': False,
                'document': auth_data['access_file_%s' % self.type],
                'create_document': False,
                'delete_document': False,
                'download_document': auth_data['download_file_%s' % self.type]}
        auth_json = XML(json.dumps(auth), sanitize=False)   
        
        # Visual element
        visual=VisualOL(self.type)
        
        return dict(header_title=current.T(self.label),
                    header_params=[],
                    label=self.label,
                    items=visual.get_data(id),
                    auth=auth_json,
                    item_id=id)
    
              
    def box(self):        
        visual=VisualOL(self.type)         
        self.table = current.db[self.type] 
        self.table.id.represent = lambda id: DIV(A(current.T("Show"), _href="%s%s" % (HOST, URL(r=current.request, f='read', args=(id), extension=""))))
        form = current.db().select(self.table.id, self.table.name)
        return dict(form=form)
    
           
    def sort(self):   
        [name,identifier,params] = get_request_args(2)
        
        # Handle project
        ProjectHandler.user_set_project(name)
        
        elem=OLElement(self.type, identifier)
        elem.sort(current.request.vars, self.type)

           
    def modal(self):
        [name,action,params] = get_request_args(2)
        
#        # Handle project
#        ProjectHandler.user_set_project(name)
        
        if action is None:
            action='add'
            
        if action == 'add':
            load=LOAD('project', 
                      '%s.load' % self.type,                      
                      args=['add', name],
                      ajax=True)
        elif action == 'edit':
            load=LOAD('project', 
                      '%s.load' % self.type,                      
                      args=['edit', name, params[0]],
                      ajax=True)
        elif action == 'tag':
            load=LOAD('project', 
                      '%s.load' % self.type,                      
                      args=['tag', name, params[1], params[2]],
                      ajax=True)
        elif action == 'comment':
            load=LOAD('project', 
                      '%s.load' % self.type,                      
                      args=['comment', name, params[1], params[2]],
                      ajax=True)
        elif action == 'add_child':
            load=LOAD('project', 
                      '%s.load' % self.type,                      
                      args=['add', name, params[0], params[1]],
                      ajax=True)
        elif action == 'file':
            load=LOAD('project', 
                      '%s.load' % self.type,                      
                      args=['file', name, params[0], params[1], params[2]],
                      ajax=True)
        elif action == 'responsible':
            load=LOAD('project', 
                      '%s.load' % self.type,                      
                      args=['responsible', name, params[1], params[2]],
                      ajax=True)
        elif action == 'date':
            load=LOAD('project', 
                      '%s.load' % self.type,                      
                      args=['date', name, params[0]],
                      ajax=True)
        
        return dict(load_action=load)
    
           
    def menues(self):
        [name,identifier,params] = get_request_args(2)
        
        # Handle project
        ProjectHandler.user_set_project(name)
        
        # Visual element
        visual=VisualOL(self.type)
    
        (type,data)=visual.menu()  
        return dict(data=data)
    
           
    def add(self):      
        # Checks parameters
        [name, id, vid, params] = get_request_args(3)
               
         # Handle project
        ProjectHandler.user_set_project(name)
        
        # Visual element
        visual=VisualOL(self.type)          
        self.table = current.db[self.type]
        # Default values
        visual.add_db_parameters() 
        
        # Form
        form = SQLFORM(self.table)
        bol_ok=True
        if form.accepts(current.request.vars, current.session):
            (id,num)=visual.add_child(self.table.identifier.default)
            
            if current._user.id is not None:
                creator=current._user.email
            else:
                creator=''
    
            # Visual data
            current.db.commit()
            tmp = visual.get_child(self.table.identifier.default)
            tmp['responsible'] = ''
            tmp['description'] = escape2html(return2json(current.request.vars.content))
            
            params = {}
            params['div_id'] = 'mm_add'
            params['data'] = tmp
    
            bol_ok = False
            import gluon.contrib.simplejson
            form = XML(gluon.contrib.simplejson.dumps(params))
            
        return dict(form=form, show=bol_ok)
    
           
    def edit(self):   
        [name,identifier,params] = get_request_args(2)
        
        # Handle project
        ProjectHandler.user_set_project(name)
          
        # Visual element
        visual=VisualOL(self.type)       
        self.table = current.db[self.type] 
        # Default values
        visual.edit_db_parameters()
    
        elem = OLElement(self.type, identifier)
        elem_id = elem.get_id()
        # Form
        form = SQLFORM(self.table, elem_id)
        bol_ok=True
        if form.accepts(current.request.vars, current.session):  
            params={}
            
            # Visual data
            tmp = Storage()
            
            data=Storage()
            data.sumary=current.request.vars.sumary
            data.content=current.request.vars.content
            try:
                if current.request.vars.date:
                    data.date = datetime.strptime(current.request.vars.date, '%d-%m-%Y %H:%M:%S')
                    tmp['date']=current.request.vars.date
            except Exception, e:
                pass    
                  
            try:
                elem.edit(data)
            except Exception, e:
                pass  
            
            # Visual data
            tmp['tid'] = identifier
            tmp['type'] = self.type
            tmp['title'] = data.sumary
            tmp['description'] = escape2html(return2json(data.content))
            
            params['div_id'] = 'mm_add'
            params['data'] = tmp
    
            bol_ok=False
            import gluon.contrib.simplejson
            form=XML(gluon.contrib.simplejson.dumps(params))
            
        return dict(form=form,show=bol_ok)
    
           
    def responsible(self):
        [pid,tid,id,params] = get_request_args(3)
        
        # Handle project
        ProjectHandler.user_set_project(pid)          
            
        team=ProjectHandler.get_team(current._user.project.id)
        
        # Form
        from gluon import IS_IN_SET
        from gluon.sql import SQLField
        from gluon.sqlhtml import form_factory
        options=[str(x.email) for x in team]
        form=form_factory(SQLField('responsible',
                                   requires=IS_IN_SET(options,multiple=False)))
    
        bol_ok=True
        if form.accepts(current.request.vars, current.session):
            try:
                elem = OLElement(self.type, tid)
                elem.change_responsible(current.request.vars.responsible)
            except Exception, e:
                pass            
                    
            # Visual data
            tmp = Storage()
            tmp['tid']=id
            tmp['responsible']=current.request.vars.responsible
            params={}
            params['div_id']='mm_add'
            params['data']=tmp
            bol_ok=False
            import gluon.contrib.simplejson
            form=XML(gluon.contrib.simplejson.dumps(params))
            
        return dict(form=form,show=bol_ok)
    
           
    def priority(self):
        [pid,tid,value,params] = get_request_args(3)
        
        # Handle project
        ProjectHandler.user_set_project(pid)         
        
        elem=OLElement(self.type, tid)    
        return elem.change_priority(pid,tid,value)
    
           
    def close_item(self):
        [pname,tid,id,params] = get_request_args(3)
        
        # Handle project
        ProjectHandler.user_set_project(pname)         
        
        elem=OLElement(self.type, tid)    
        return elem.close()
    
           
    def tag(self):
        [pid,tid,id,params] = get_request_args(3)
        
        # Handle project
        ProjectHandler.user_set_project(pid)
        
        elem=OLElement(self.type, tid)    
        return elem.add_tag(pid,tid,id)
    
    def date(self):   
        [name,identifier,params] = get_request_args(2)
        
        # Handle project
        ProjectHandler.user_set_project(name)
          
        # Visual element
        visual=VisualOL(self.type)       
        self.table = current.db[self.type] 
        # Default values
        visual.edit_db_parameters()
    
        elem = OLElement(self.type, identifier)
        elem_id = elem.get_id()
            
        # Form
        form = SQLFORM(self.table, elem_id, fields=['date'])
        bol_ok=True
        if form.accepts(current.request.vars, current.session):
            params={}            
            
            # Visual data
            tmp = Storage()
            
            elem_data = elem.get(identifier)
            data=Storage()
            try:
                if current.request.vars.date:
                    data.date = datetime.strptime(current.request.vars.date, '%d-%m-%Y %H:%M:%S')
                    tmp['date']=current.request.vars.date
            except Exception, e:
                pass    
                  
            try:
                elem.edit(data)
            except Exception, e:
                pass  
            
            # Visual data
            tmp['tid']=identifier
            tmp['type']=self.type
            tmp['title']=elem_data['name']
            text=elem_data['desc'].replace("\r\n","<br/>")
            tmp['description']=text.replace("\n","<br/>")
            
            params['div_id']='mm_add'
            params['data']=tmp
    
            bol_ok=False
            import gluon.contrib.simplejson
            form=XML(gluon.contrib.simplejson.dumps(params))
            
        return dict(form=form,show=bol_ok)
    
    
           
    def delete_tag(self):
        [pid,tid,id,params] = get_request_args(3)
        
        # Handle project
        ProjectHandler.user_set_project(pid)
        
        elem=OLElement(self.type, tid)    
        return elem.delete_tag(id)
    
               
    def comment(self):
        [pid,tid,id,params] = get_request_args(3)  
        
        # Handle project
        ProjectHandler.user_set_project(pid)
          
        elem=OLElement(self.type, tid)    
        return elem.add_comment(pid,tid,id)    
    
           
    def document(self):    
        [pname,pid,id,tid,params] = get_request_args(4)
        
        # Handle project
        ProjectHandler.user_set_project(pname)
                
        elem=OLElement(self.type, id)    
        return dict(data=elem.add_document(tid))
    
           
    def delete_document(self):
        [pid,tid,id,params] = get_request_args(3)
        
        # Handle project
        ProjectHandler.user_set_project(pid)
        
        elem=OLElement(self.type, tid)    
        return elem.delete_document(id)
    
           
    def file(self):    
        [pname,pid,id,tid,params] = get_request_args(4)
        
        # Handle project
        ProjectHandler.user_set_project(pname)
                
        elem=OLElement(self.type, id)    
        return elem.add_file(tid)
     
              
    def delete_item(self):
        [pname,tid,id,params] = get_request_args(3)
        
        # Handle project
        ProjectHandler.user_set_project(pname)
        
        elem=OLElement(self.type, tid)    
        return elem.delete()
    
           
    def printer(self):
        [pname,id,params] = get_request_args(2)   
        
        # Handle project
        ProjectHandler.user_set_project(pname)
                
        # Visual element
        visual=VisualOL(self.type)
        project=ProjectHandler.get_by_name(pname)
    
        if current.request.extension=="pdf":    
            return visual.printer(project,current.T(self.label_list),id)
        elif current.request.extension=="txt":  
            return visual.writer_txt(project,current.T(self.label_list),id)
        elif  current.request.extension=="rtf": 
            return visual.writer_rtf(project,current.T(self.label_list),id)
        else:
            return visual.writer_txt(project,current.T(self.label_list),id)
        