# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import datetime
import time

from gluon import current, XML, URL, redirect
from gluon.storage import Storage

from mil.tools.common import normalize

from subapp import ProjectInstance

from project.models.card import ProjectCardModel
from project.models.type import ProjectTypeModel
from project.models.service import ProjectServicesModel
from project.models.profile import ProjectProfileModel
from project.models.team import ProjectTeamModel
from project.models.bug import BugModel
from project.models.task import TaskModel
from project.models.incidence import IncidenceModel
from project.models.roadmap import RoadmapModel
from project.models.element import ElementModel

class ProjectHandler(object):        
    @staticmethod
    def get(pid):
        ProjectCardModel.define_table(current.db)
        if pid:
            return current.db(current.db.project_card.id == pid).select().first()
        else:
            return None  
   
    @staticmethod
    def get_by_name(pid):
        if pid:
            ProjectCardModel.define_table(current.db)
            return current.db(current.db.project_card.name == pid).select().first()
        else:
            return None
        
    @staticmethod
    def get_by_id(id):
        if id:
            ProjectCardModel.define_table(current.db)
            return current.db(current.db.project_card.id == id).select().first()
        else:
            return None
        
    @staticmethod
    def get_by_identifier(id):
        if id:
            ProjectCardModel.define_table(current.db)
            return current.db(current.db.project_card.identifier == id).select().first()
        else:
            return None
        
        
    @staticmethod
    def get_identifier(pid):
        if pid:
            ProjectCardModel.define_table(current.db)
            return current.db(current.db.project_card.id == pid).select().first().identifier
        else:
            return None
        
    @staticmethod
    def prepare_card(pid):
        project = ProjectHandler.get(pid)
        if project.id is not None:            
            # database accesses
            # Type
            ProjectTypeModel.define_table(current.db)
            type_literal = current.db(current.db.project_type.id==project.type).select().first().name
            # Data of leader
            ptmKlass = ProjectTeamModel(current.db)
            leader = ptmKlass.get_leader(project.id)
            # Services
            services = ProjectHandler.get_services(project.id)  
            
            # Prepare data to view
            tmp = Storage()
            tmp['id'] = project.id
            tmp['name'] = project.name
            tmp['title'] = project.title
            tmp['type'] = type_literal
            tmp['start'] = project.start_date.strftime("%Y-%m-%d")
            tmp['end'] = project.end_date.strftime("%Y-%m-%d")
            if leader is not None:
                tmp['leader'] = "%s %s" %(leader.first_name, leader.last_name)
                tmp['leader_email'] = leader.email
                tmp['i_am_leader'] = False
                if current._user.email == leader.email:
                    tmp['i_am_leader'] = True
            else:
                tmp['leader'] = ""
                tmp['leader_email'] = ""
                tmp['i_am_leader'] = False
            tmp['status'] = ProjectHandler.status(project)
            tmp['profile'] = ProjectHandler.get_user_acl(project.id, current._user.id)  
            if project.private:
                tmp['private']='Private'
            else:
                tmp['private']='Public'
            tmp['services'] = services 
            tmp['blocked'] = project.active    
            tmp['deleted'] = project.deleted   
            return tmp
        else:
            return None
        
#    @staticmethod
#    def get_leader(project_id):
#        ProjectTeamModel.define_table(current.db)
#        data = current.db((current.db.auth_user.id==current.db.project_team.user)&
#                          (current.db.project_team.profile==1)&
#                          (current.db.project_team.project==project_id)).select(current.db.auth_user.id,
#                                                                                current.db.auth_user.first_name,
#                                                                                current.db.auth_user.last_name,
#                                                                                current.db.auth_user.email).first()
#        from gluon.storage import Storage
#        return Storage(id=data.id,
#                       first_name=data.first_name,
#                       last_name=data.last_name,
#                       email=data.email)
        
    @staticmethod
    def get_services(project_id):
        # Services
        psm = ProjectServicesModel(current.db)    
        try:        
            rows = psm.get_services_by_project(project_id)
        except:
            rows = []
        services = []
        for row in rows:
            tmp = Storage()  
            tmp['name'] = row.label
            tmp['controller'] =  row.controller
            tmp['function'] = row.function
            services.append(tmp)             
        return services
        
    @staticmethod
    def prepare_resume(pid, to_doc=False):
        project = ProjectHandler.get_by_name(pid)
        if project.id is not None:  
            # database accesses
            # Type
            ProjectTypeModel.define_table(current.db)
            type_literal = current.db(current.db.project_type.id==project.type).select().first().name
            # Data of leader
            ptmKlass = ProjectTeamModel(current.db)
            leader = ptmKlass.get_leader(project.id)
            # Services
            services = ProjectHandler.get_services(project.id)  
                          
            # Team
            query = ((current.db.project_user_profile.id==current.db.project_team.profile) & 
                     (current.db.auth_user.id==current.db.project_team.user) & 
                     (current.db.project_team.project==project.id))
            rows = current.db(query).select(current.db.auth_user.first_name, 
                                            current.db.auth_user.last_name, 
                                            current.db.auth_user.email, 
                                            current.db.project_user_profile.name)
            team = []
            for row in rows:
                tmp = Storage()  
                tmp['name'] = "%s %s" % (row.auth_user.first_name, 
                                         row.auth_user.last_name)
                tmp['email'] = row.auth_user.email
                tmp['profile'] = current.T(row.project_user_profile.name)
                team.append(tmp)                              
            
            # Prepare data to view
            tmp = Storage()
            tmp['id'] = project.id
            tmp['name'] = project.name
            tmp['title'] = project.title
            tmp['type'] = type_literal
            tmp['start'] = project.start_date.strftime("%Y-%m-%d")
            tmp['end'] = project.end_date.strftime("%Y-%m-%d")
            
            if leader is not None:
                tmp['leader'] = "%s %s" %(leader.first_name, leader.last_name)
                tmp['leader_email'] = leader.email
                tmp['i_am_leader'] = False
                if current._user.email == leader.email:
                    tmp['i_am_leader'] = True
            else:
                tmp['leader'] = ""
                tmp['leader_email'] = ""
                tmp['i_am_leader'] = False
                
            tmp['status'] = ProjectHandler.status(project)
            if project.description is not None:
                if to_doc:
                    tmp['description']=project.description
                else:
                    tmp['description']=XML(project.description.replace("\n","<br />"))
            else:
                tmp['description']=''
            if project.objetive is not None:
                if to_doc:
                    tmp['objetive']=project.objetive
                else:
                    tmp['objetive']=XML(project.objetive.replace("\n","<br />"))
            else:
                tmp['objetive']=''
            tmp['profile'] = ProjectHandler.get_user_acl(project.id, current._user.id)
#            profile = ProjectHandler.get_user_acl(project.id, current._user.id)
#            for item in profile.keys():
#                tmp['profile_%s'%item]=profile[item] 
            if project.private:
                tmp['private']='Private'
            else:
                tmp['private']='Public'
            tmp['services'] = services      
            tmp['team'] = team   
            return tmp
        else:
            return None
        
    @staticmethod
    def status(project):
        # Returns a string contains a 'css' class
        #
        #         start_date         -10      -4  end_date
        #  before             during     next     after
        #
        if project.id is not None: 
            today_ts = time.mktime(datetime.datetime.today().timetuple())
            start_ts = time.mktime(project.start_date.timetuple())
            next_date = project.end_date-datetime.timedelta(days=10)
            next_ts = time.mktime(next_date.timetuple())
            end_date = project.end_date-datetime.timedelta(days=4)
            end_ts = time.mktime(end_date.timetuple())
            # Check in which range today is
            if today_ts < start_ts:
                return 'before'
            elif start_ts <= today_ts and today_ts < next_ts:
                return 'during'
            elif next_ts <= today_ts and today_ts < end_ts:
                return 'next'
            else:
                return 'after'
        else:
            return ''
        
    @staticmethod
    def user_set_project(name=None):
        if name is None:
            u_prj = None
        else:
            if isinstance(name, int):
                project = ProjectHandler.get_by_id(name)                
            else:
                project = ProjectHandler.get_by_name(name)
                
            if project:
                # User profile           
                profile = ProjectHandler.get_user_acl(project.id, 
                                                      current._user.id)
                # Checks if try public access of private project
                if project.private:
                    from masters.profiles import Profiles 
                    guest_id = Profiles(current._config, current.db).get_id_by_subapp_and_role('project', 'public') 
                    if guest_id == profile:
                        current.session.flash = current.T('Access denied')
                        redirect(URL(r=current.request,
                                     c='default',
                                     f='index'))
                
                services = ProjectHandler.get_services(project.id) 
                # Checks if try public access of private project
                
                permissions = ProjectHandler.get_permissions(profile)
                u_prj = Storage(id=project.id,
                                ident=project.identifier,
                                name=project.name,
                                title=project.title,
                                profile=profile,
                                services=services,
                                auth=permissions)
            else:
                u_prj = None
        current._user.project = u_prj
        
    @staticmethod
    def get_permissions(profile):
        from system import AuthConfiguration
        auth_mng = AuthConfiguration(current.db, profile)
        all_data = auth_mng.get_all()
        
        data = {}
        if all_data:
            for row in all_data:
                key = '%s_%s_%s' % (row.application, row.module, row.action)
                data[key] = row.value
        return data
        
    @staticmethod
    def get_team(pid=None):
        if pid is None:
            return None
        
        ProjectTeamModel.define_table(current.db)
        
        project = ProjectHandler.get_by_id(pid)

        if project.id is not None:            
            # Team
            query = ((current.db.auth_group.id==current.db.project_team.profile) & \
                     (current.db.auth_user.id==current.db.project_team.user) & \
                     (current.db.project_team.project==project.id))
            rows = current.db(query).select(current.db.auth_user.first_name, 
                                            current.db.auth_user.last_name, 
                                            current.db.auth_user.email, 
                                            current.db.auth_group.description)
            team = []
            for row in rows:
                tmp = Storage()  
                tmp['name'] = "%s %s" %(row.auth_user.first_name,
                                        row.auth_user.last_name)
                tmp['email'] = row.auth_user.email
                tmp['profile'] = current.T(row.auth_group.description)
                team.append(tmp)                                 
            return team
        else:
            return None
        
    @staticmethod
    def get_team_list(pid=None):
        if pid is None:
            return None
        
        ProjectTeamModel.define_table(current.db)
        
        project = ProjectHandler.get_by_id(pid)

        if project.id is not None:            
            # Team
            query = ((current.db.auth_user.id==current.db.project_team.user) & \
                     (current.db.project_team.project==project.id))
            rows = current.db(query).select(current.db.auth_user.email)
            team = []
            for row in rows:
                team.append(row.email)                                 
            return team
        else:
            return None
        
    @staticmethod
    def get_team_leader(pid=None):
        if pid is None:
            return None
        
        ProjectTeamModel.define_table(current.db)
        
        project = ProjectHandler.get_by_id(pid)
        
        # Profile
        from masters.profiles import Profiles 
        profile = Profiles(current._config, current.db).get_id_by_subapp_and_role('project', 'leader')  

        if project.id is not None:            
            # Team
            query = ((current.db.auth_user.id==current.db.project_team.user) & \
                     (current.db.project_team.project==project.id) & \
                     (current.db.project_team.profile==profile))
            rows = current.db(query).select(current.db.auth_user.email)
            team = []
            for row in rows:
                team.append(row.email)                                 
            return team
        else:
            return None          
        
#    def in_project(self,name,pid):   
#        self.p_name=name 
#        self.pid=pid
#        self.p_identifier=get_identifier_from_pid(pid)
#        session.project_id=self.pid
#        self.clean_acl()
#        if self.pid is not None:
#            # Search user's profile (in project)
#            query = ((db.project_user_profile.id==db.project_team.profile) & (db.project_team.project==self.pid) & (db.project_team.user==self.id))
#            acl = db(query).select(db.project_user_profile.read_auth, db.project_user_profile.write_auth, db.project_user_profile.exec_auth)
#            try:
#                self.set_acl(acl[0].read_auth, acl[0].write_auth, acl[0].exec_auth)
#            except:
#                pass

    @staticmethod
    def get_user_acl(pid, user_id):  
        # initialize model        
        ProjectTeamModel.define_table(current.db)
        
        # Public role by default
        from masters.profiles import Profiles
        iKls = Profiles(current._config, current.db)
        acl = iKls.get_id_by_subapp_and_role('project', 'public')

        if pid:
            try:
                 # Search user's profile (in project)
                query = ((current.db.project_team.project == pid) &\
                         (current.db.project_team.user==user_id))
                data = current.db(query).select().first()
                if data:
                     acl = data.profile
            except Exception, e:
                pass
                
        return acl
                
#        # Initializa models          
#        ProjectProfileModel.define_table(current.db)
#        ProjectTeamModel.define_table(current.db)
#        
#        # Using project
#        acl = Storage(read_auth=False, write_auth=False, exec_auth=False)
#        if pid:
#            try:
#                # Search user's profile (in project)
#                query = ((current.db.project_user_profile.id == current.db.project_team.profile) & \
#                         (current.db.project_team.project == pid) &\
#                         (current.db.project_team.user==user_id))
#                data = current.db(query).select(current.db.project_user_profile.read_auth, 
#                                                current.db.project_user_profile.write_auth, 
#                                                current.db.project_user_profile.exec_auth).first()
#                if data:
#                     acl.read_auth=data.read_auth
#                     acl.write_auth=data.write_auth
#                     acl.exec_auth=data.exec_auth
#            except Exception, e:
#                pass
#   
#        return acl
    
    @staticmethod
    def get_project_tasks(pid, max_num=0):
        if pid is None:
            return None
        
        # Initializa models
        ElementModel.define_table(current.db)
        TaskModel.define_table(current.db)
        # Query
        query=((current.db.task.identifier==current.db.element_has_element.child)&
               (current.db.element_has_element.type=='task')&
               (current.db.element_has_element.parent==pid))
        if max_num > 0:
            rows=current.db(query).select(current.db.task.id,
                                  current.db.task.sumary,
                                  current.db.task.start_date,
                                  current.db.task.close_date,
                                  limitby=(0, max_num),
                                  orderby="task.update_date DESC")
        else:
            rows=current.db(query).select(current.db.task.id,
                                  current.db.task.sumary,
                                  current.db.task.start_date,
                                  current.db.task.close_date,
                                  orderby="task.update_date DESC")
            
        return rows
    
    @staticmethod
    def lock(pid):
        project = ProjectHandler.get_by_name(pid)
        if project.id is not None:             
            ProjectCardModel(current.db).lock(project.id)
            
    @staticmethod
    def delete(pid):
        project = ProjectHandler.get_by_name(pid)
        if project.id is not None:     
            # Delete data
            # TODO: delete all dependencies first
    
            # Delete services
            ProjectServicesModel(current.db).delete_by_project(project.id)
    
            # Delete team
            ProjectTeamModel(current.db).delete_by_project(project.id)
    
            # Delete card     
            ProjectCardModel(current.db).delete(project.id)
            
    @staticmethod
    def activate_services(pid, services, leader = None):
        project = ProjectHandler.get_by_id(pid)
        if project.id is not None:
            if type(services) == list:
                for service in services:
                    # Check action
                    if service['action'] == 'enable':                      
                        if service['id'] == 'repository':
                            try:
                                from system import SvnHandler
                                sh = SvnHandler(current._config.get('svn_server'))
                                sh.set_auth(current._user.email,
                                            current._user.get_clean_password())
                                sh.add_repo(project.name)
#                                sh.mkdir(project.name)
#                                
#                                if leader:
#                                    sh.assign_user_as_writer(project.name, 
#                                                             leader.email)                                        
                            except Exception, e:
                                current._syslog.error('Repository: %s' % str(e), 
                                                'error')
                                pass
                        if service['id'] == 'webdav':
                            try:
                                from system import WebdavHandler
                                wh = WebdavHandler(current._config.get('webdav_server'))
                                wh.set_auth(current._user.email,
                                            current._user.get_clean_password())
                                wh.mkdir(project.name)
                            except Exception, e:
                                current._syslog.error('Webdav: %s' % str(e), 
                                                'error')
                                pass
                        if service['id'] == 'calendar':
                            try:
                                from calendar import CalendarHandler
                                cal_id = CalendarHandler.create_calendar(project.name, 
                                                                         project.id)
                                if cal_id:
                                    if leader:
                                        CalendarHandler.assign_user_as_owner(cal_id,
                                                                             leader.id)
                                    # Add end event
                                    params = {'title': current.T('End of project'),
                                              'place': '',
                                              'description': '%s %s' % (current.T('End of project'),
                                                                        project.title),
                                              'init_date': project.end_date,
                                              'end_date': project.end_date,
                                              'calendar_id': cal_id,
                                              'all_day': True}
                                    CalendarHandler.add_event(**params)
                            except Exception, e:
                                current._syslog.error('Calendar: %s' % str(e), 
                                                'error')
                                pass                        
                    elif service['action'] == 'disable':                        
                        if service['id'] == 'repository':
                            try:
                                from system import SvnHandler
                                sh = SvnHandler(current._config.get('svn_server'))
                                sh.set_auth(current._user.email,
                                            current._user.get_clean_password())
                                sh.del_repo(project.name)
#                                sh.rmdir(project.name)
#                                
#                                sh.remove_auth(project.name)                                        
                            except Exception, e:
                                current._syslog.error('Repository: %s' % str(e), 
                                                'error')
                                pass
                        if service['id'] == 'webdav':
                            try:
                                from system import WebdavHandler
                                wh = WebdavHandler(current._config.get('webdav_server'))
                                wh.set_auth(current._user.email,
                                            current._user.get_clean_password())
                                wh.rmdir(project.name)
                            except Exception, e:
                                current._syslog.error('Webdav: %s' % str(e), 
                                                'error')
                                pass
                        if service['id'] == 'calendar':
                            try:
                                from calendar import CalendarHandler
                                CalendarHandler.delete_calendar(project.name)
                            except Exception, e:
                                current._syslog.error('Calendar: %s' % str(e), 
                                                'error')
                                pass
                                 
    @staticmethod
    def join_member_as(p_id, member, profile=None):
        # Project data             
        project = ProjectHandler.get_by_id(p_id)
        
        # User data
        from core.userhandler import UserHandler
        user = UserHandler.get_by_id(member)
        
        # Profile
        from masters.profiles import Profiles   
        if profile is None:
            iKls = Profiles(current._config, current.db)
            profile = iKls.get_id_by_subapp_and_role('project', 'leader')            
        
        # Project services
        psm = ProjectServicesModel(current.db)    
        try:        
            services = psm.get_one_by_project(project.id).services
        except:
            services = []
            
        for service in services:
            if service == 'calendar':   
                # Get calendar
                from calendar import CalendarHandler
                calendar = CalendarHandler.get_calendar_by_name(project.name)
                
                # Add user to calendar
                CalendarHandler.assign_user(calendar.id, user.id, 'r')
                
        # Profile             
        profile = Profiles(current._config, current.db).get_by_id(profile)
        
        # Notify
        if current._government.check_configuration('project', 
                                                   'email_join_project', 
                                                   'enabled'):            
            # Prepare data
            data= Storage(name="%s %s" % (user.first_name, user.last_name),
                          forge=current._config.get('title'),
                          project=project.title,
                          profile=profile.label)
            try:
                current.mail.send_from_template(to=user.email, 
                                               template='email_join_project', 
                                               data=data,
                                               lang=current._config.get('language'))
            except Exception, e:
                current._syslog.error('ProjectHandler.notify_member: %s' % str(e), 
                                      'warning')
                                 
    @staticmethod
    def unjoin_member(p_id, member):
        # Project data             
        project = ProjectHandler.get_by_id(p_id)
        
        # User data
        from core.userhandler import UserHandler
        user = UserHandler.get_by_id(member)
        
        # Project services
        psm = ProjectServicesModel(current.db)    
        try:        
            services = psm.get_one_by_project(project.id).services
        except:
            services = []
            
        for service in services:
            if service == 'calendar':                
                # Get calendar
                from calendar import CalendarHandler
                calendar = CalendarHandler.get_calendar_by_name(project.name)
                
                # Add user to calendar
                CalendarHandler.unassign_user(calendar.id, user.id)
                        
        # Notify
        if current._government.check_configuration('project', 
                                                   'email_unjoin_project', 
                                                   'enabled'):            
            # Prepare data
            data= Storage(name="%s %s" % (user.first_name, user.last_name),
                          forge=current._config.get('title'),
                          project=project.title)
            try:
                current.mail.send_from_template(to=user.email, 
                                               template='email_unjoin_project', 
                                               data=data,
                                               lang=current._config.get('language'))
            except Exception, e:
                current._syslog.error('ProjectHandler.notify_member: %s' % str(e), 
                                      'warning')
            
    ##
    ## REST Interface
    ##
    
    @staticmethod
    def get_all_by_user(user_id):           
        ProjectCardModel.define_table(current.db)
        ProjectTeamModel.define_table(current.db)
        
        query = ((current.db.project_card.deleted == False) & \
                 (current.db.project_card.id == current.db.project_team.project) & \
                 (current.db.project_team.user == user_id))
        return current.db(query).select(current.db.project_card.id,
                                        current.db.project_card.identifier,
                                        current.db.project_card.name,
                                        current.db.project_card.title,
                                        current.db.project_team.profile)
        
    @staticmethod
    def get_bugs(user_email, params=[]):
        ProjectCardModel.define_table(current.db)
        ElementModel.define_table(current.db)
        BugModel.define_table(current.db)
        
        query = ((current.db.project_card.identifier == current.db.element_has_element.parent) & \
                 (current.db.element_has_element.child == current.db.bug.identifier) & \
                 (current.db.element_has_element.type == 'bug') & \
                 (current.db.bug.responsible == user_email) )
        if len(params) >= 1: 
            if params[0] is not '':
                query = ((current.db.element_has_element.child == current.db.bug.identifier) & \
                         (current.db.element_has_element.type == 'bug') & \
                         (current.db.bug.identifier == params[0]))

        return current.db(query).select(current.db.bug.id,
                                        current.db.bug.identifier,
                                        current.db.bug.sumary,
                                        current.db.bug.start_date,
                                        current.db.bug.close_date,
                                        current.db.project_card.name)
        
    @staticmethod
    def get_project_roadmap(id):
        ElementModel.define_table(current.db)
        RoadmapModel.define_table(current.db)
        
        query = ((current.db.roadmap.identifier == current.db.element_has_element.child) & \
                 (current.db.element_has_element.type == 'roadmap') & \
                 (current.db.element_has_element.parent == id)) 
        return current.db(query).select(current.db.roadmap.id,
                                        current.db.roadmap.identifier,
                                        current.db.roadmap.status,
                                        current.db.roadmap.sumary,
                                        current.db.roadmap.creator,
                                        current.db.roadmap.responsible,
                                        current.db.roadmap.init_date,
                                        current.db.roadmap.end_date,
                                        current.db.roadmap.priority,
                                        current.db.roadmap.content,
                                        current.db.element_has_element.parent)

    @staticmethod
    def get_tasks(user_email, params=[]):
        ProjectCardModel.define_table(current.db)
        ElementModel.define_table(current.db)
        TaskModel.define_table(current.db)
        
        query = ((current.db.project_card.identifier == current.db.element_has_element.parent) & \
                 (current.db.element_has_element.child == current.db.task.identifier) & \
                 (current.db.element_has_element.type == 'task') & \
                 (current.db.task.responsible == user_email) )
        if len(params) >= 1: 
            if params[0] is not '':
                query = ((current.db.element_has_element.child == current.db.task.identifier) & \
                         (current.db.element_has_element.type == 'task') & \
                         (current.db.task.identifier == params[0]))

        return current.db(query).select(current.db.task.id,
                                        current.db.task.identifier,
                                        current.db.task.sumary,
                                        current.db.task.start_date,
                                        current.db.task.close_date,
                                        current.db.project_card.name)

    @staticmethod
    def get_incidences(user_email, params=[]):
        ProjectCardModel.define_table(current.db)
        ElementModel.define_table(current.db)
        IncidenceModel.define_table(current.db)
        
        query = ((current.db.project_card.identifier == current.db.element_has_element.parent) & \
                 (current.db.element_has_element.child == current.db.incidence.identifier) & \
                 (current.db.element_has_element.type == 'incidence') & \
                 (current.db.incidence.responsible == user_email) )
        if len(params) >= 1: 
            if params[0] is not '':
                query = ((current.db.element_has_element.child == current.db.incidence.identifier) & \
                         (current.db.element_has_element.type == 'incidence') & \
                         (current.db.bug.incidence == params[0]))

        return current.db(query).select(current.db.incidence.id,
                                        current.db.incidence.identifier,
                                        current.db.incidence.sumary,
                                        current.db.incidence.start_date,
                                        current.db.incidence.close_date,
                                        current.db.project_card.name)
                
#    @staticmethod
#    def get_project_bugs(pid, num_max=0):
#        if pid is None:
#            return None
#        
#        # Initializa models
#        ElementModel.define_table(current.db)
#        ProjectBugModel.define_table(current.db)
#        # Query
#        query=((current.db.task.identifier==current.db.element_has_element.child)&
#               (current.db.element_has_element.type=='task')&
#               (current.db.element_has_element.parent==pid))
#        rows=current.db(query).select(current.db.task.id,
#                              current.db.task.sumary,
#                              current.db.task.start_date,
#                              current.db.task.close_date,
#                              limitby=(0,max_num),
#                              orderby="task.update_date DESC")
#        pass
#    
#    @staticmethod
#    def get_project_incidences(pid, num_max=0):
#        if pid is None:
#            return None
#        
#        # Initializa models
#        ElementModel.define_table(current.db)
#        ProjectIncidenceModel.define_table(current.db)
#        # Query
#        query=((current.db.task.identifier==current.db.element_has_element.child)&
#               (current.db.element_has_element.type=='task')&
#               (current.db.element_has_element.parent==pid))
#        rows=current.db(query).select(current.db.task.id,
#                              current.db.task.sumary,
#                              current.db.task.start_date,
#                              current.db.task.close_date,
#                              limitby=(0,max_num),
#                              orderby="task.update_date DESC")
#        pass
#    