# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################
import yaml

DOCUMENT = """
  actions:
    - 
      label: 'Public panel - resume, team, news, ...'
      id: public_access
      options:
        - 
          label: 'Edit panel boxes'
          id: edit_public_boxes
    - 
      label: 'Private panel - project management'
      id: private_access
      options:
        - 
          label: 'Edit panel boxes'
          id: edit_private_boxes
    -
      label: 'Project Card'
      id: card_access
      options:
        - 
          label: 'Edit card'
          id: edit_card
    -
      label: 'Team'
      id: team_access
      options:
        - 
          label: 'Add'
          id: add_member
        - 
          label: 'Delete'
          id: delete_member
        - 
          label: 'Change profile'
          id: profile_member
        - 
          label: 'Manage join requirements'
          id: manage_join
    -
      label: 'Services'
      id: services_access
      options:
        - 
          label: 'Add'
          id: add_service
        - 
          label: 'Delete'
          id: delete_service
        - 
          label: 'Configure'
          id: configure_service
    -
      label: 'Profiles'
      id: profile_access
      options:
        - 
          label: 'Add'
          id: add_profile
        - 
          label: 'Delete'
          id: delete_profile
        - 
          label: 'Configure'
          id: configure_profile
          type: group
          options:
            -
              label: 'By profile'
              id: configure_by_profile
              default: true
              type: option
            -
              label: 'All profiles'
              id: configure_all_profiles
              default: false
              type: option
  configuration:
    -
      label: 'Create Project'
      id: create
      type: only_one
      options:
        -
          id: all
          label: 'All users can do' 
          default: false
        -
          id: admin 
          label: 'Only Administrators can do' 
          default: true
    -
      label: 'Close Project'
      id: close
      type: only_one
      options:
        -
          id: all
          label: 'All users can do' 
          default: false
        -
          id: admin 
          label: 'Only Administrators can do' 
          default: true
    -
      label: 'Automatic comments'
      id: add_comment
      type: only_one
      options:
        -
          id: disabled
          label: 'Do not generate' 
          default: true
        -
          id: enabled 
          label: 'Generate always' 
          default: false
    -
      label: 'Join Project'
      id: join
      type: only_one
      options:
        -
          id: disabled
          label: 'Disabled' 
          default: true
        -
          id: enabled 
          label: 'Enable' 
          default: false
    -
      label: 'Join to Project (inform the user)'
      id: email_join_project
      type: only_one
      options:
        -
          id: disabled
          label: 'Do not send notification' 
          default: true
        -
          id: enabled
          label: 'Send notification' 
          default: false
    -
      label: 'Unjoin from Project (inform the user)'
      id: email_unjoin_project
      type: only_one
      options:
        -
          id: disabled
          label: 'Do not send notification' 
          default: true
        -
          id: enabled
          label: 'Send notification' 
          default: false
    -
      label: 'Assign task, bug, incidence (inform the user)'
      id: email_assign_to
      type: only_one
      options:
        -
          id: disabled
          label: 'Do not send notification' 
          default: true
        -
          id: enabled
          label: 'Send notification' 
          default: false
    -
      label: 'Close task, bug, incidence (inform the leader)'
      id: email_inform_leader
      type: only_one
      options:
        -
          id: disabled
          label: 'Do not send notification' 
          default: true
        -
          id: enabled
          label: 'Send notification' 
          default: false
  literal: 'Project'
  permissions:
    - 'add'
  services:
    roadmap:
      actions:
        -
          label: 'Access'
          id: roadmap_access
          options:
            - 
              label: 'Add'
              id: add_roadmap
            - 
              label: 'Edit'
              id: edit_roadmap
            - 
              label: 'Delete'
              id: delete_roadmap
            - 
              label: 'Change date'
              id: change_date_roadmap
            - 
              label: 'Print'
              id: print_roadmap
            - 
              label: 'Validate'
              id: validate_roadmap
            -
              label: 'Comments'
              id: access_comment_roadmap
              options:
                - 
                  label: 'Add'
                  id: add_comment_roadmap
            -
              label: 'Tags'
              id: access_tag_roadmap
              options:
                - 
                  label: 'Add'
                  id: add_tag_roadmap
                - 
                  label: 'Delete'
                  id: delete_tag_roadmap
      id: roadmap
      control: roadmap
      label: 'Roadmap'
      configuration:
        -
          id: type
          label: 'Type'
          type: only_one
          options:
            -
              id: gantt
              label: 'Gantt'
              default: disabled
            -
              id: scrum
              label: 'Scrum'
              default: disabled
    task:
      actions:
        -
          label: 'Access'
          id: task_access
          options:
            - 
              label: 'Add'
              id: add_task
            - 
              label: 'Edit'
              id: edit_task
            - 
              label: 'Delete'
              id: delete_task
            - 
              label: 'Change priority'
              id: change_prority_task
            - 
              label: 'Change date'
              id: change_date_task
            - 
              label: 'Print'
              id: print_task
            - 
              label: 'Change status'
              id: change_status_task
              options:
                - 
                  label: 'Restore status'
                  id: restore_status_task
            - 
              label: 'Validate'
              id: validate_task
            - 
              label: 'Assign'
              id: access_assign_task
              options:
                - 
                  label: 'Change'
                  id: change_assign_task
            -
              label: 'Comments'
              id: access_comment_task
              options:
                - 
                  label: 'Add'
                  id: add_comment_task
            -
              label: 'Files'
              id: access_file_task
              options:
                - 
                  label: 'Add'
                  id: add_file_task
                - 
                  label: 'Delete'
                  id: delete_file_task
                - 
                  label: 'Download'
                  id: download_file_task
            -
              label: 'Tags'
              id: access_tag_task
              options:
                - 
                  label: 'Add'
                  id: add_tag_task
                - 
                  label: 'Delete'
                  id: delete_tag_task
      id: task
      control: task
      label: 'Tasks'
      configuration:
        - 
          id: type_assign_task
          label: 'Assign Type'
          type: only_one
          options:
            -
              id: auto_assign
              label: 'Auto assign'
              default: disabled
            -
              id: select_assign
              label: 'Select responsible'
              default: true
        -
          id: status
          label: 'Status'
          type: only_one
          options:
            -
              id: open_close
              label: 'Open/Close'
              default: true
            -
              id: multiple
              label: 'Various values for opened status and for closed status'
              default: disabled
        -
          id: validation
          label: 'Validation'
          type: only_one
          options:
            -
              id: no_validate
              label: 'Validation disabled'
              default: true
            -
              id: creator_validation
              label: 'Validation by the user which create the task'
              default: disabled
            -
              id: profile_validation
              label: 'Validation by a concrete profile'
              default: disabled
            -
              id: creator_or_profile_validation
              label: 'Validation by the user which create the task or by a concrete profile'
              default: disabled
        -
          id: comments
          label: 'Automatic Comments'
          type: multi_value
          options:
            -
              id: priority_changed
              label: 'Priority changed'
              default: false
            -
              id: responsible_changed
              label: 'Responsible changed'
              default: false
            -
              id: description_changed
              label: 'Description changed'
              default: false
            -
              id: date_changed
              label: 'Date changed'
              default: false
            -
              id: status_changed
              label: 'Status changed'
              default: false
        -
          id: email
          label: 'Email notifications'
          type: multi_value
          options:
            -
              id: added
              label: 'When a task is added'
              default: false
            -
              id: priority_changed
              label: 'When priority is changed'
              default: false
            -
              id: responsible_changed
              label: 'When responsible is changed'
              default: false
            -
              id: description_changed
              label: 'When description is changed'
              default: false
            -
              id: date_changed
              label: 'When date is changed'
              default: false
            -
              id: status_changed
              label: 'When status is changed'
              default: false
            -
              id: file_added
              label: 'When a file is uploaded'
              default: false
    bug:
      actions:
        -
          label: 'Access'
          id: bug_access
          options:
            - 
              label: 'Add'
              id: add_bug
            - 
              label: 'Edit'
              id: edit_bug
            - 
              label: 'Delete'
              id: delete_bug
            - 
              label: 'Change priority'
              id: change_prority_bug
            - 
              label: 'Change date'
              id: change_date_bug
            - 
              label: 'Print'
              id: print_bug
            - 
              label: 'Change status'
              id: change_status_bug
              options:
                - 
                  label: 'Restore status'
                  id: restore_status_bug
            - 
              label: 'Validate'
              id: validate_bug
            - 
              label: 'Assign'
              id: access_assign_bug
              options:
                - 
                  label: 'Change'
                  id: change_assign_bug
            -
              label: 'Comments'
              id: access_comment_bug
              options:
                - 
                  label: 'Add'
                  id: add_comment_bug
            -
              label: 'Files'
              id: access_file_bug
              options:
                - 
                  label: 'Add'
                  id: add_file_bug
                - 
                  label: 'Delete'
                  id: delete_file_bug
                - 
                  label: 'Download'
                  id: download_file_bug
            -
              label: 'Tags'
              id: access_tag_bug
              options:
                - 
                  label: 'Add'
                  id: add_tag_bug
                - 
                  label: 'Delete'
                  id: delete_tag_bug
      id: bug
      control: bug
      label: 'Bugs'
      configuration:
        - 
          id: type_assign_task
          label: 'Assign Type'
          type: only_one
          options:
            -
              id: auto_assign
              label: 'Auto assign'
              default: disabled
            -
              id: select_assign
              label: 'Select responsible'
              default: true
        -
          id: status
          label: 'Status'
          type: only_one
          options:
            -
              id: open_close
              label: 'Open/Close'
              default: true
            -
              id: multiple
              label: 'Various values for opened status and for closed status'
              default: disabled
        -
          id: validation
          label: 'Validation'
          type: only_one
          options:
            -
              id: no_validate
              label: 'Validation disabled'
              default: true
            -
              id: creator_validation
              label: 'Validation by the user which create the task'
              default: disabled
            -
              id: profile_validation
              label: 'Validation by a concrete profile'
              default: disabled
            -
              id: creator_or_profile_validation
              label: 'Validation by the user which create the task or by a concrete profile'
              default: disabled
        -
          id: comments
          label: 'Automatic Comments'
          type: multi_value
          options:
            -
              id: priority_changed
              label: 'Priority changed'
              default: false
            -
              id: responsible_changed
              label: 'Responsible changed'
              default: false
            -
              id: description_changed
              label: 'Description changed'
              default: false
            -
              id: date_changed
              label: 'Date changed'
              default: false
            -
              id: status_changed
              label: 'Status changed'
              default: false
        -
          id: email
          label: 'Email notifications'
          type: multi_value
          options:
            -
              id: added
              label: 'When a bug is added'
              default: false
            -
              id: priority_changed
              label: 'When priority is changed'
              default: false
            -
              id: responsible_changed
              label: 'When responsible is changed'
              default: false
            -
              id: description_changed
              label: 'When description is changed'
              default: false
            -
              id: date_changed
              label: 'When date is changed'
              default: false
            -
              id: status_changed
              label: 'When status is changed'
              default: false
            -
              id: file_added
              label: 'When a file is uploaded'
              default: false
    incidence:
      actions:
        -
          label: 'Access'
          id: incidence_access
          options:
            - 
              label: 'Add'
              id: add_incidence
            - 
              label: 'Edit'
              id: edit_incidence
            - 
              label: 'Delete'
              id: delete_incidence
            - 
              label: 'Change priority'
              id: change_prority_incidence
            - 
              label: 'Change date'
              id: change_date_incidence
            - 
              label: 'Print'
              id: print_incidence
            - 
              label: 'Change status'
              id: change_status_incidence
              options:
                - 
                  label: 'Restore status'
                  id: restore_status_incidence
            - 
              label: 'Validate'
              id: validate_incidence
            - 
              label: 'Assign'
              id: access_assign_incidence
              options:
                - 
                  label: 'Change'
                  id: change_assign_incidence
            -
              label: 'Comments'
              id: access_comment_incidence
              options:
                - 
                  label: 'Add'
                  id: add_comment_incidence
            -
              label: 'Files'
              id: access_file_incidence
              options:
                - 
                  label: 'Add'
                  id: add_file_incidence
                - 
                  label: 'Delete'
                  id: delete_file_incidence
                - 
                  label: 'Download'
                  id: download_file_incidence
            -
              label: 'Tags'
              id: access_tag_incidence
              options:
                - 
                  label: 'Add'
                  id: add_tag_incidence
                - 
                  label: 'Delete'
                  id: delete_tag_incidence
      id: incidence
      control: incidence
      label: 'Incidences'
      configuration:
        - 
          id: type_assign_task
          label: 'Assign Type'
          type: only_one
          options:
            -
              id: auto_assign
              label: 'Auto assign'
              default: disabled
            -
              id: select_assign
              label: 'Select responsible'
              default: true
        -
          id: status
          label: 'Status'
          type: only_one
          options:
            -
              id: open_close
              label: 'Open/Close'
              default: true
            -
              id: multiple
              label: 'Various values for opened status and for closed status'
              default: disabled
        -
          id: validation
          label: 'Validation'
          type: only_one
          options:
            -
              id: no_validate
              label: 'Validation disabled'
              default: true
            -
              id: creator_validation
              label: 'Validation by the user which create the task'
              default: disabled
            -
              id: profile_validation
              label: 'Validation by a concrete profile'
              default: disabled
            -
              id: creator_or_profile_validation
              label: 'Validation by the user which create the task or by a concrete profile'
              default: disabled
        -
          id: comments
          label: 'Automatic Comments'
          type: multi_value
          options:
            -
              id: priority_changed
              label: 'Priority changed'
              default: false
            -
              id: responsible_changed
              label: 'Responsible changed'
              default: false
            -
              id: description_changed
              label: 'Description changed'
              default: false
            -
              id: date_changed
              label: 'Date changed'
              default: false
            -
              id: status_changed
              label: 'Status changed'
              default: false
        -
          id: email
          label: 'Email notifications'
          type: multi_value
          options:
            -
              id: added
              label: 'When an incidence is added'
              default: false
            -
              id: priority_changed
              label: 'When priority is changed'
              default: false
            -
              id: responsible_changed
              label: 'When responsible is changed'
              default: false
            -
              id: description_changed
              label: 'When description is changed'
              default: false
            -
              id: date_changed
              label: 'When date is changed'
              default: false
            -
              id: status_changed
              label: 'When status is changed'
              default: false
            -
              id: file_added
              label: 'When a file is uploaded'
              default: false
    webdav:
      actions:
        -  
          label: 'Access'
          id: webdav_access
          options:
            - 
              label: 'Write'
              id: webdav_write
      id: webdav
      control: webdav
      label: 'Webdav'
    repository:
      actions:
        -  
          label: 'Access'
          id: repository_access
          options:
            - 
              label: 'Write'
              id: repository_write
      id: repository
      control: repository
      label: 'Repository'
    wiki:
      actions:
        -  
          label: 'Access'
          id: wiki_access
          options:
            - 
              label: 'Write'
              id: wiki_write
      id: wiki
      control: wiki
      label: 'Wiki'
    calendar:
      actions:
        -  
          label: 'Access'
          id: calendar_access
      id: calendar
      control: calendar
      label: 'Calendar'
"""

def get_configuration():
    try:
        return yaml.load(DOCUMENT)
    except Exception, e:
        raise(e)
