# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from mil.tools import get_in_fixed_list

from gluon import current, HTTP

from core.tools import is_email

from project import ProjectHandler

def rest_project_get(data):        
    service=service_project()
    try:
        return service.select(data)
    except Exception, e:
        raise HTTP(500)
    
           

class service_project:
    def __init__(self):
        self.max_items = 10
        self.user_requested = None
            
    def select(self, data):
        [id, action, action_id, params] = get_in_fixed_list(data, 3)
                
        # Control: User have been received in first place
        if (is_email(action)):
            self.user_requested = action
            action = action_id
            action_id = None
        
        if len(params) == 1:
            if (is_email(params[0])):
                self.user_requested = params[0]
        
        if action is None:
            if id is None:
                # All for user
                return self._get_all_by_user()
            else:
                # Project data
                return self._get_project(id)
        else:
            # Project specified subset of data
            
            return [] #self._get_project_data(id, action, action_id)

        return None
            
    def _get_all_by_user(self):
        list = []
        if self.user_requested is not None:
            if self.user_requested != current._user.email:
                return list
            
        rows = ProjectHandler.get_all_by_user(current._user.id)
        for row in rows:
            tmp = {}
            tmp['profile'] = row.project_team.profile
            tmp['id'] = row.project_card.identifier
            tmp['name'] = row.project_card.name
            tmp['title'] = row.project_card.title
            tmp['rest_url'] = "%s/api/project/%s" % (current._config.app_url, 
                                                     row.project_card.identifier) # Must be identifier
            tmp['web_url'] = "%s/project/resume/index/%s" % (current._config.app_url, 
                                                             row.project_card.name) # Must be name
            list.append(tmp)
        data={'json_data':list} 
        return data
    
    def _get_project(self, id):
        list = []
        project = ProjectHandler.get_by_identifier(id)
        if project:
            tmp = {}
            tmp['id'] = project.identifier
            tmp['name'] = project.name
            tmp['title'] = project.title
            tmp['description'] = project.description
            tmp['start'] = project.start_date
            tmp['end'] = project.end_date
            tmp['rest_url'] = "%s/api/project/%s" % (current._config.app_url, 
                                                     project.identifier) # Must be identifier
            tmp['web_url'] = "%s/project/resume/index/%s" % (current._config.app_url, 
                                                             project.name) # Must be name
            list.append(tmp)
        return {'json_data': list} 
#        
#    def _get_project_data(self, id, action, action_id):        
#        # Actions supported
#        actions = {'task': self._get_task,
#                   'bug': self._get_bug,
#                   'incidence': self._get_incidence,
#                   'calendar': self._get_calendar
#                   }
#        # Call specific function
#        return actions[action](id, action_id)
#        
#    
#    def _get_task(self, pid, id):
#        if id is None:
#            rows = self._get_project_tasks(pid)
#        else:
#            rows = self._get_project_task(pid, id)
#        
#        list = []
#        for row in rows:
#            tmp = {}
#            tmp['id'] = row.task.identifier
#            tmp['name'] = row.task.sumary
#            tmp['responsible'] = row.task.responsible
#            tmp['start'] = row.task.start_date
#            tmp['end'] = row.task.close_date
#            tmp['rest_url'] = "%s/api/task/%s" % (APP_URL, row.task.identifier)
#            tmp['web_url'] = "%s/task/read/%s/%s" % (APP_URL, row.project_card.name,row.task.id)
#            list.append(tmp)
#        return {'json_data':list} 
#    
#    def _get_project_tasks(self, id):
#        if self.user_requested is not None:
#            query =((db.project_card.identifier==db.element_has_element.parent)&(db.task.responsible==self.user_requested)&(db.task.identifier==db.element_has_element.child)&(db.element_has_element.type=='task')&(db.element_has_element.parent==id))
#        else:
#            query = ((db.project_card.identifier==db.element_has_element.parent)&(db.task.identifier==db.element_has_element.child)&(db.element_has_element.type=='task')&(db.element_has_element.parent==id))
#        return db(query).select(db.task.id,db.task.identifier,db.task.sumary,db.task.responsible,db.task.start_date,db.task.close_date, db.project_card.name)
#    
#    def _get_project_task(self, pid, tid):
#        query = ((db.project_card.identifier==db.element_has_element.parent)&(db.element_has_element.child==db.task.identifier)&(db.task.identifier==tid))
#        return db(query).select(db.task.id,db.task.identifier,db.task.sumary,db.task.responsible,db.task.start_date,db.task.close_date, db.project_card.name)
#    
#    def _get_bug(self, pid, id):
#        if id is None:
#            rows = self._get_project_bugs(pid)
#        else:
#            rows = self._get_project_bug(pid, id)
#        
#        list = []
#        for row in rows:
#            tmp = {}
#            tmp['id'] = row.bug.identifier
#            tmp['name'] = row.bug.sumary
#            tmp['responsible'] = row.bug.responsible
#            tmp['start'] = row.bug.start_date
#            tmp['end'] = row.bug.close_date
#            tmp['rest_url'] = "%s/api/bug/%s" % (APP_URL, row.bug.identifier)
#            tmp['web_url'] = "%s/bug/read/%s/%s" % (APP_URL, row.project_card.name,row.bug.id)
#            list.append(tmp)
#        return {'json_data':list} 
#    
#    def _get_project_bugs(self, id):
#        if self.user_requested is not None:
#            query =((db.project_card.identifier==db.element_has_element.parent)&(db.bug.responsible==self.user_requested)&(db.bug.identifier==db.element_has_element.child)&(db.element_has_element.type=='bug')&(db.element_has_element.parent==id))
#        else:
#            query = ((db.project_card.identifier==db.element_has_element.parent)&(db.bug.identifier==db.element_has_element.child)&(db.element_has_element.type=='bug')&(db.element_has_element.parent==id))
#        return db(query).select(db.bug.id,db.bug.identifier,db.bug.sumary,db.bug.responsible,db.bug.start_date,db.bug.close_date,db.project_card.name)
#    
#    def _get_project_bug(self, pid, tid):
#        query = ((db.project_card.identifier==db.element_has_element.parent)&(db.element_has_element.child==db.bug.identifier)&(db.bug.identifier==tid))
#        return db(query).select(db.bug.id,db.bug.identifier,db.bug.sumary,db.bug.responsible,db.bug.start_date,db.bug.close_date,db.project_card.name)
#    
#    def _get_incidence(self, pid, id):
#        if id is None:
#            rows = self._get_project_incidences(pid)
#        else:
#            rows = self._get_project_incidence(pid, id)
#        
#        list = []
#        for row in rows:
#            tmp = {}
#            tmp['id'] = row.incidence.identifier
#            tmp['name'] = row.incidence.sumary
#            tmp['responsible'] = row.incidence.responsible
#            tmp['start'] = row.incidence.start_date
#            tmp['end'] = row.incidence.close_date
#            tmp['rest_url'] = "%s/api/incidence/%s" % (APP_URL, row.incidence.identifier)
#            tmp['web_url'] = "%s/incidence/read/%s/%s" % (APP_URL, row.project_card.name,row.incidence.id)
#            list.append(tmp)
#        return {'json_data':list} 
#    
#    def _get_project_incidences(self, id):
#        if self.user_requested is not None:
#            query =((db.project_card.identifier==db.element_has_element.parent)&(db.incidence.responsible==self.user_requested)&(db.incidence.identifier==db.element_has_element.child)&(db.element_has_element.type=='incidence')&(db.element_has_element.parent==id))
#        else:
#            query = ((db.project_card.identifier==db.element_has_element.parent)&(db.incidence.identifier==db.element_has_element.child)&(db.element_has_element.type=='incidence')&(db.element_has_element.parent==id))
#        return db(query).select(db.incidence.id,db.incidence.identifier,db.incidence.sumary,db.incidence.responsible,db.incidence.start_date,db.incidence.close_date,db.project_card.name)
#    
#    def _get_project_incidence(self, pid, tid):
#        query = ((db.project_card.identifier==db.element_has_element.parent)&(db.element_has_element.child==db.incidence.identifier)&(db.incidence.identifier==tid))
#        return db(query).select(db.incidence.id,db.incidence.identifier,db.incidence.sumary,db.incidence.responsible,db.incidence.start_date,db.incidence.close_date,db.project_card.name)
#    
#    def _get_calendar(self, pid, id):
#        if id is None:
#            rows = self._get_project_calendars(pid)
#        else:
#            rows = self._get_project_calendars(pid, id)
#        
#        return rows 
#    
#    def _get_project_calendars(self, pid,id=None):
#        p_identifier = db(db.project_card.id==pid).select(db.project_card.identifier).first().identifier
#        
#        rdata = rest("%s/%s/calendar/project/%s" % (HOST,_config.get('calendar_rest'),p_identifier), 
#                     method='get', 
#                     headers={'ACCEPT': 'application/json',
#                              'AUTHORIZATION': current._user.get_http_auth()} )
#        if rdata.rfind('[ERROR]') == -1:
#            if len(rdata) >=0:        
#                data = json.loads(rdata)
#            else:
#                data = []
#        else:
#            data=[]
#        return {'json_data':data}    