 # -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import current, URL
from gluon.storage import Storage

from mil.tools import get_request_args

from core.subapplication import SubApp, SubAppModule

from project.visual import VisualResume

class ProjectResume(SubAppModule):
    def __init__(self, controller='project'):
        SubAppModule.__init__(self, 'project', 'resume')
    
#    @show_menu
#    @is_in_project
    def index(self):    
        from project import ProjectHandler
        
        current._menu.show()
        
        [p_name, temp] = get_request_args()
        
        project = ProjectHandler.get_by_name(p_name)
        
        # Maximun number of items
        max_num = 5
        
        # Select project's tasks
        rows = ProjectHandler.get_project_tasks(project.identifier, max_num)
        tasks = []
        for row in rows:
            tmp=Storage()
            tmp['id'] = row.id
            tmp['name'] = row.sumary
            tmp['start'] = str(row.start_date).split(' ')[0]
            tmp['href'] = URL(r=current.request,  c='project',  f='task', 
                              args=['read', project.name, row.id])
            tasks.append(tmp)
        
#        # Select project's bugs
#        query=((db.bug.identifier==db.element_has_element.child)&(db.element_has_element.type=='bug')&(db.element_has_element.parent==vip_user.p_identifier))
#        rows=db(query).select(db.bug.id,db.bug.sumary,db.bug.start_date,db.bug.close_date,limitby=(0,max_num),orderby="bug.update_date DESC")
        bugs=[]
#        for row in rows:
#            tmp=Storage()
#            tmp['id']=row.id
#            tmp['name']=row.sumary
#            tmp['start']=str(row.start_date).split(' ')[0]
#            tmp['href']=URL(r=request, c='bug', f='read',args=[vip_user.p_name,row.id])
#            bugs.append(tmp)
#        
#        # Select project's incidences
#        query=((db.incidence.identifier==db.element_has_element.child)&(db.element_has_element.type=='incidence')&(db.element_has_element.parent==vip_user.p_identifier))
#        rows=db(query).select(db.incidence.id,db.incidence.sumary,db.incidence.start_date,db.incidence.close_date,limitby=(0,max_num),orderby="incidence.update_date DESC")
        incidences=[]
#        for row in rows:
#            tmp=Storage()
#            tmp['id']=row.id
#            tmp['name']=row.sumary
#            tmp['start']=str(row.start_date).split(' ')[0]
#            tmp['href']=URL(r=request, c='incidence', f='read',args=[vip_user.p_name,row.id])
#            incidences.append(tmp)
    
        vip_user = Storage(write=True)
        return dict(project=ProjectHandler.prepare_resume(p_name),
                    user=vip_user,
                    tasks=tasks,
                    bugs=bugs,
                    incidences=incidences)
    
    
#    @is_in_project 
#    @auth.requires_login()   
    def printer(self):
        [pname, params] = get_request_args(1)       
        
        # Visual element
        printer = VisualResume(pname)
    
        if current.request.extension=="pdf":    
            return printer.pdf()
        elif current.request.extension=="txt":  
            return printer.txt()
        elif  current.request.extension=="rtf": 
            return printer.rtf()
        else:
            return printer.txt()
    