# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import *
from gluon.storage import Storage

from mil.tools import get_request_args
from mil.tools.common import return2json, escape2html

from core.subapplication import SubApp, SubAppModule

from project import ProjectHandler
from project.visual import VisualRoadmap
from project.visual.element_roadmap import RoadmapElement

class ProjectRoadmap(SubAppModule):
    def __init__(self, controller='project'):
        SubAppModule.__init__(self, 'project', 'roadmap')
  
    def index(self):
        current._menu.show()
        
        # Handle project
        [name, params] = get_request_args(1)
        ProjectHandler.user_set_project(name)
        
        # Visual element
        visual = VisualRoadmap()
        
        return dict(title=current.T('Roadmap'),
                    title_elements=[],
                    list_items=visual.get_data(),
                    name=current._user.identifier)
        
    
    def modal(self):
        [name, action, params] = get_request_args(2)
        
#        # Handle project
#        ProjectHandler.user_set_project(name)
    
        if action is None:
            action = 'add'
            
        if action == 'add':
            load = LOAD('project', 
                        'roadmap.load', 
                        args=['add',name], 
                        ajax=True)
        elif action == 'edit':
            load = LOAD('project', 
                        'roadmap.load', 
                        args=['edit',name,params[0]],
                        ajax=True)
        elif action == 'tag':
            load = LOAD('project', 
                        'roadmap', 
                        args=['tag', name,params[1], params[2]],
                        ajax=True)
        elif action == 'comment':
            load = LOAD('project', 
                        'roadmap.load', 
                        args=['comment', name,params[1], params[2]],
                        ajax=True)
        elif action == 'add_child':
            load = LOAD('project', 
                        'roadmap.load', 
                        args=['add', name, params[0], params[1]],
                        ajax=True)
        
        return dict(load_action=load)
    

    
    def add(self):  
        # Checks parameters
        [name, id, vid, params] = get_request_args(3)
        
        # Handle project
        ProjectHandler.user_set_project(name)
        
        # Visual element
        visual=VisualRoadmap()  
        # Default values
        visual.add_db_parameters()
    
        # Define identifier in add item case.
        # In add child case, identifier is given from url
        if id is None:
            is_parent=True
        else:
             is_parent=False    
            
        # Form
        form = SQLFORM(current.db.roadmap)
        bol_ok=True
        if form.accepts(current.request.vars, current.session):
            tid=form.vars.id    
            if is_parent:    
                (id,num)=visual.add_child(current.db.roadmap.identifier.default)
            else:
                elem=RoadmapElement(id)
                (id,num)=elem.insert_child(current.db.roadmap.identifier.default,'roadmap')
            
            from mil.tools import get_date_from_string
            init_date=get_date_from_string(current.request.vars.init_date, 
                                           "%Y-%m-%d")
            end_date=get_date_from_string(current.request.vars.end_date, 
                                          "%Y-%m-%d")
            
            # Visual data
            tmp = Storage()
            tmp['id']="item_%s"%tid
            tmp['pid']=current._user.project.id
            tmp['tid']=id
            if is_parent:
                tmp['vid']=0
            else:
                tmp['vid']=vid
            tmp['order']=num
            tmp['creator']=current._user.email if current._user else ''
            tmp['responsible']=''
            tmp['title']=current.request.vars.sumary            
            tmp['description'] = escape2html(return2json(current.request.vars.content))
            tmp['priority']=1
            tmp['tags']=[]
            tmp['ini_date']=init_date.strftime('%d-%m-%Y')
            tmp['end_date']=end_date.strftime('%d-%m-%Y')
            
            params={}
            params['div_id']='mm_add'
            params['data']=tmp
    
            bol_ok=False
            import gluon.contrib.simplejson
            form=XML(gluon.contrib.simplejson.dumps(params))
            
        return dict(form=form,show=bol_ok)

    def read(self):
        [name,id,params] = get_request_args(2)
        
        # Handle project
        ProjectHandler.user_set_project(name)
        
        # Visual element
        visual=VisualRoadmap()
        
        return dict(header_title=current.T('Roadmap'),
                    header_params=[],
                    list_items=visual.get_data(id),
                    item_id=id)
 
    
    def box(self):
        current.db.roadmap.id.represent = lambda id: DIV(A(current.T("Show"), _href="%s%s" % (HOST, URL(r=current.request, f='read', args=(id), extension=""))))
        form = current.db().select(current.db.roadmap.id, current.db.roadmap.name)
        return dict(form=form)
  
    
    def sort(self):   
        [name,identifier,params] = get_request_args(2)
        
        # Handle project
        ProjectHandler.user_set_project(name)
        
        elem=RoadmapElement(identifier)
        elem.sort(current.request.vars,'roadmap')
    #    
    #    # Visual element
    #    visual=VisualRoadmap()
    #    visual.sort(current.request.vars)
  
    
    def edit(self):  
        [name,identifier,params] = get_request_args(2)
        
        # Handle project
        ProjectHandler.user_set_project(name)
           
        # Visual element
        visual=VisualRoadmap()  
        # Default values
        visual.edit_db_parameters()
    
        # Form
        form = SQLFORM(current.db.roadmap, current.request.args(1))
        bol_ok=True
        if form.accepts(current.request.vars, current.session):
            tid=current.request.args(1) #form.vars.id
            params={}
            
            data=Storage()
            data.sumary=current.request.vars.sumary
            data.content=current.request.vars.content
            from mil.tools import get_date_from_string
            data.init_date=get_date_from_string(current.request.vars.init_date)
            data.end_date=get_date_from_string(current.request.vars.end_date)
            
            visual.edit_parent(data)
            
            # Visual data
            tmp = Storage()
            tmp['tid']=tid
            tmp['type']='roadmap'
            tmp['title']=data.sumary      
            tmp['description'] = escape2html(return2json(data.content))
            tmp['ini_date']=data.init_date.strftime('%d-%m-%Y %H:%M:%S')
            tmp['end_date']=data.end_date.strftime('%d-%m-%Y %H:%M:%S')
            
            params['div_id']='mm_add'
            params['data']=tmp
    
            bol_ok=False
            import gluon.contrib.simplejson
            form=XML(gluon.contrib.simplejson.dumps(params))
            
        return dict(form=form,show=bol_ok)
  
    
    def delete_roadmap(self):
        [pname,tid,id,params] = get_request_args(3)        
        
        # Handle project
        ProjectHandler.user_set_project(pname)
        
        elem=RoadmapElement(tid)    
        return elem.delete()
   
    
    def tag(self):
        [pid,tid,id,params] = get_request_args(3)

        # Handle project
        ProjectHandler.user_set_project(pid)
        
        elem = RoadmapElement(tid)    
        return elem.add_tag(pid,tid,id)
   
    
    def delete_tag(self):
        [pid,tid,id,params] = get_request_args(3)

        # Handle project
        ProjectHandler.user_set_project(pid)
        
        elem=RoadmapElement(tid)    
        return elem.delete_tag(id)
 
    
    def comment(self):
        [pid,tid,id,params] = get_request_args(3)  

        # Handle project
        ProjectHandler.user_set_project(pid)
          
        elem=RoadmapElement(tid)    
        return elem.add_comment(pid,tid,id)
 
    
    def printer(self):
        [pname,id,params] = get_request_args(2)     

        # Handle project
        ProjectHandler.user_set_project(pname)        
        
        # Visual element
        visual=VisualRoadmap()
        project=ProjectHandler.get_by_name(pname)
    
        if current.request.extension=="pdf":    
            return visual.printer(project,current.T('Roadmap list'),id)
        elif current.request.extension=="txt":  
            return visual.writer_txt(project,current.T('Roadmap list'),id)
        elif  current.request.extension=="rtf": 
            return visual.writer_rtf(project,current.T('Roadmap list'),id)
        else:
            return visual.writer_txt(project,current.T('Roadmap list'),id)
    