 # -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import current
from gluon.storage import Storage

from mil.tools import get_request_args

from core.subapplication import SubApp, SubAppModule

from project import ProjectHandler
from project.subapp import ProjectInstance

from project.models.service import ProjectServicesModel
from project.models.type import ProjectTypeModel
from project.models.team import ProjectTeamModel
from project.models.type_service import ProjectTypeServicesModel

class ProjectService(SubAppModule):
    def __init__(self, controller='project'):
        SubAppModule.__init__(self, 'project', 'service')
        
    def index(self):        
        current._menu.show()
        
        try:
            [p_name, temp] = get_request_args()                
        
            # Current Project Data
            from project import ProjectHandler
            project = ProjectHandler.get_by_name(p_name)
            ProjectHandler.user_set_project(p_name)
                    
            # Sub app data
            sapp_prj = ProjectInstance()
            services = sapp_prj.get_services()
            
            # Model instance
            psm = ProjectServicesModel(current.db)
            
            # Save form
            if current.request.vars.commit:
                c_list = psm.get_one_by_project(project.id).services
                add_list = []
                activate_actions =[]
                for service in services:
                    add = current.request.vars["service_%s" % service.id] or None
                    
                    if add is not None:
                        # Add service to db store
                        add_list.append(service.id)
                        
                        # Checks if service was active
                        if service.id in c_list:
                            activate_actions.append({'id': service.id, 'action': 'none'})
                        else:
                            activate_actions.append({'id': service.id, 'action': 'enable'})
                    else:
                        # Checks if service was active
                        if service.id in c_list:
                            activate_actions.append({'id': service.id, 'action': 'disable'})
                        else:
                            activate_actions.append({'id': service.id, 'action': 'none'})
                        
                psm.set(project.id, add_list)
                
                #get leader
                ptmKlass = ProjectTeamModel(current.db)
                leader = ptmKlass.get_leader(project.id)
                
                # Activate services
                ProjectHandler.activate_services(project.id, 
                                                 activate_actions,
                                                 leader)
                #                                 ProjectHandler.get_leader(project.id))
                    
                # Checks flow redirection
                self.redirect(project.name)
            
            # Get Services
            project_services_row = psm.get_one_by_project(project.id)
            if project_services_row:
                selected = project_services_row.services
            else:
                prj_type_service = ProjectTypeServicesModel(current.db)
                selected = prj_type_service.get_by_type(project.type).services
                
        except Exception, e:
            current._syslog.error('ProjectService.index: %s' % str(e), 'warning')
            selected = []
            services = []
            current.response.flash = current.T('Project must be defined')
        
        return dict(selected=selected, services=services)
 