# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import current, URL
from gluon.storage import Storage

from core.subapplication import SubAppInstance

class ProjectInstance(SubAppInstance):
    def __init__(self):
        SubAppInstance.__init__(self, 'project')
        
    def get_menu(self):
        return [(current.T('Projects'), False, None, [
                    (current.T('List'), False, 
                     URL(current.request.application,'project','index'), []),
                    (current.T('Create'), False, 
                     URL(current.request.application,'project','create'), []),
                    (current.T('Project Type'), False, 
                     URL(current.request.application,'project','type', args=('index')), []),
                    (current.T('Project Type Service'), False, 
                     URL(current.request.application,'project','typeservices', args=('index')), []),
#                    (current.T('Project configuration'), False, 
#                     URL(current.request.application,'government',
#                         'subapplication', args=['project']), []),
#                    (current.T('Project Team Profile'), False, 
#                     URL(current.request.application,'government',
#                         'profiles', args=['project']), []),
#                    (current.T('Project Team Profile'), False, 
#                     URL(current.request.application,'project','profile'), []),
         
                 ])
               ]
        
    def set_services(self):
        # default: True is service is enable always. False if can be configured
        data = []
        data.append(Storage(id='card', 
                            function='card',
                            label='Card', 
                            default=True))
        data.append(Storage(id='board',
                            function='board', 
                            label='Board', 
                            default=True))
        data.append(Storage(id='team',
                            function='team', 
                            label='Team', 
                            default=True))
        data.append(Storage(id='request',
                            function='request', 
                            label='Requests', 
                            default=True))                  
        data.append(Storage(id='service',
                            function='service', 
                            label='Services', 
                            default=True))
        
        data.append(Storage(id='roadmap',
                            function='roadmap', 
                            label='Roadmap', 
                            default=False))
        data.append(Storage(id='task',
                            function='task', 
                            label='Tasks', 
                            default=False))
        data.append(Storage(id='bug',
                            function='bug', 
                            label='Bugs', 
                            default=False))
        data.append(Storage(id='incidence',
                            function='incidence', 
                            label='Incidences', 
                            default=False))
        data.append(Storage(id='webdav',
                            function='webdav', 
                            label='Webdav', 
                            default=False))
        data.append(Storage(id='repository',
                            function='repository', 
                            label='Repository', 
                            default=False))
        data.append(Storage(id='wiki',
                            function='wiki', 
                            label='Wiki', 
                            default=False))
        data.append(Storage(id='calendar',
                            function='calendar', 
                            label='Calendar', 
                            default=False))  
        return data  
        
    def get_configuration(self):
        try:
            from project.request import get_configuration
            config = get_configuration()
            
            data = config
            data['controller'] = self.name
        except Exception, e:
            data = None
            
        return data