 # -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import current, IS_IN_SET, redirect, URL
from gluon.storage import Storage
from gluon.sql import SQLField
from gluon.sqlhtml import form_factory

from mil.tools import get_request_args

from core.subapplication import SubApp, SubAppModule
from masters.users import Users
from masters.profiles import Profiles

from project import ProjectHandler
from project.models.team import ProjectTeamModel

class ProjectTeam(SubAppModule):
    def __init__(self, controller='project'):
        SubAppModule.__init__(self, 'project', 'team')
        
    def index(self):        
        current._menu.show()
        
        try:
            [p_name, temp] = get_request_args(1)
            project = ProjectHandler.get_by_name(p_name)
            
            ProjectHandler.user_set_project(p_name)
        except:
            project = Storage(id=None)
        
        insKlss = ProjectTeamModel(current.db) 
        return dict(form=insKlss.grid(project.id), project_id=project.id)
    
    def edit(self):        
        try:
            [p_id, u_id, pf_id, temp] = get_request_args(3)
            project = ProjectHandler.get_by_id(p_id)
            
            current._menu.show()
            
            # Team members            
            cur_user = Users(current._config, current.db).get_by_id(u_id)
            user_lit = "%s %s (%s)" % (cur_user.first_name, 
                                       cur_user.last_name, 
                                       cur_user.email)
                        
            # Profile
            insKlss = Profiles(current._config, current.db)
            profiles = insKlss.get_subapp_profiles('project')
        
            # Form
            options_p = [(str(x.id), "%s" % (x.name) ) for x in profiles]
            
            form = form_factory(SQLField('member',
                                         default=user_lit,
                                         writable=False),
                                SQLField('profile',
                                         default=int(pf_id),
                                         requires=IS_IN_SET(options_p,
                                                            multiple=False)))
            
            if form.accepts(current.request.vars, current.session):
                # Check args
                profile_id = int(current.request.vars.profile)
                if u_id > 0:
                    # Deactivate services
                    ProjectHandler.unjoin_member(project.id, u_id)
        
                    # Set member
                    ProjectTeamModel(current.db).change_to(project.id, 
                                                           u_id,
                                                           profile_id)
                    # Activate services
                    ProjectHandler.join_member_as(project.id, 
                                                  u_id, 
                                                  profile_id)
                    
                    redirect(URL(r=current.request,
                                 c='project',
                                 f='team', 
                                 args=['index', project.name]))
                else:
                    current.response.flash = current.T('You must select a user')
            
        except ValueError:
            form = None
            current.response.flash = current.T('Project must be defined')
        
        return dict(form=form)
    
    def join(self):        
        try:
            [p_id, temp] = get_request_args(1)
            project = ProjectHandler.get_by_id(p_id)
            
            current._menu.show()
            
            # Team members            
            all_users = Users(current._config, current.db).get_all()
            team_list = ProjectHandler.get_team_list(p_id)
            team_members = []
            for user in all_users:
                if not user.email in team_list:
                    team_members.append(user)
                        
            # Profile
            insKlss = Profiles(current._config, current.db)
            profiles = insKlss.get_subapp_profiles('project')
        
            # Form
            options = [(str(x.id), "%s %s (%s)" % (x.first_name, x.last_name, x.email) ) for x in team_members]
            options_p = [(str(x.id), "%s" % (x.name) ) for x in profiles]

            form = form_factory(SQLField('member',
                                         requires=IS_IN_SET(options,
                                                            multiple=False)),
                                SQLField('profile',
                                         requires=IS_IN_SET(options_p,
                                                            multiple=False)))
            
            if form.accepts(current.request.vars, current.session):
                # Check args
                member_id = int(current.request.vars.member)
                profile_id = int(current.request.vars.profile)
                if member_id > 0:
                    # Set member
                    ProjectTeamModel(current.db).set_member(project.id, 
                                                            member_id,
                                                            profile_id)
                    # Activate services
                    ProjectHandler.join_member_as(project.id, 
                                                  member_id, 
                                                  profile_id)
                    
                    redirect(URL(r=current.request,
                                 c='project',
                                 f='team', 
                                 args=['index', project.name]))
                else:
                    current.response.flash = current.T('You must select a user')
            
        except ValueError:
            form = None
            current.response.flash = current.T('Project must be defined')
        
        return dict(form=form)
        
    
    def unjoin(self):
        [p_id, user_id, temp] = get_request_args(2)
        project = ProjectHandler.get_by_id(p_id)
        
        # Profile
        ProjectTeamModel(current.db).delete_user_project(user_id, project.id)
        
        # Activate services
        ProjectHandler.unjoin_member(project.id, user_id)
        
        redirect(URL(r=current.request,
                     c='project',
                     f='team', 
                     args=['index', project.name]))
    
    def leader(self):      
        current._menu.show()
        
        try:
            [p_name, temp] = get_request_args()
            
            # Team leaders            
            team_leaders = Users(current._config, current.db).get_all()
        
            # Form
            options = [(str(x.id), "%s %s (%s)" % (x.first_name, x.last_name, x.email) ) for x in team_leaders]

            form = form_factory(SQLField('leader',
                                         requires=IS_IN_SET(options,
                                                            multiple=False)))
            
            if form.accepts(current.request.vars, current.session):
                # Check args
                leader_id = int(current.request.vars.leader)
                if leader_id > 0:
                    # Project
                    project = ProjectHandler.get_by_name(p_name)
                    
                    # Set leader
                    ProjectTeamModel(current.db).set_leader(project.id, 
                                                            leader_id)
                    # Activate services
                    ProjectHandler.join_member_as(project.id, leader_id)
                    
                    # Checks flow redirection
                    self.redirect(project.name)
                else:
                    current.response.flash = current.T('You must select a user')
            
        except ValueError:
            form = None
            current.response.flash = current.T('Project must be defined')
        
        return dict(form=form)
        
    