# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from datetime import datetime, timedelta

from gluon import current
from gluon.storage import Storage
from gluon.compileapp import local_import_aux

from mil.tools.common import return2json, escape2html

from project.visual.element import Element

class OLElement(Element):
    def __init__(self, type, identifier, module=None):
        Element.__init__(self, identifier)
        
        self.type = type
        self.literal = type.capitalize()
        if module is None:
            module_name = '%sModel' % type.capitalize()
        else:
            module_name = module
        
        try:
            module = local_import_aux("project",
                                      app=current.request.application)
            inst = getattr(module, module_name)
        except Exception, e:
            raise Exception('Error importing Model handler: %s' % str(e))
            
        inst.define_table(self.db)
        
        self.table = self.db[type]
        self.query = self.default_query(type)
        
    def default_query(self,type):
        return (self.table.identifier==self.db.element_has_element.child) & \
               (self.db.element_has_element.type==type) & \
               (self.db.element_has_element.parent==self.identifier)
               
    def identifier_query(self):
        return (self.table.identifier==self.identifier)
    
    def get(self, id):
        query = (self.db.element_has_element.type == self.type) & \
                (self.db.element_has_element.child == self.table.identifier) & \
                (self.table.identifier==id)
        child = self.db(query).select().first() 
        return self._get_item_child(child)        
        
    def get_data(self):
        return self.db(self.table.identifier==self.identifier).select().first()
    
    def get_id(self):
        return self.db(self.table.identifier==self.identifier).select().first().id 
                    
    def prepare_child(self,child,level=0):
        return self._get_item_child(child,level)
        
    def get_children_from_db(self,child,limit):
        if child is not None:
            self.query = (self.db.element_has_element.type == self.type) & \
                         (self.db.element_has_element.child == self.table.identifier) & \
                         (self.table.id==child)
        if limit is not None:
            children=self.db(self.query).select(self.db.element_has_element.id,
                                                self.db.element_has_element.numeral,
                                                self.table.id,
                                                self.table.identifier,
                                                self.table.sumary,
                                                self.table.content,
                                                self.table.start_date,
                                                self.table.close_date,
                                                self.table.creator,
                                                self.table.responsible,
                                                self.table.priority,
                                                self.table.status,
                                                orderby=self.db.element_has_element.sortable,
                                                limitby=limit)
        else:
            children=self.db(self.query).select(self.db.element_has_element.id,
                                                self.db.element_has_element.numeral,
                                                self.table.id,
                                                self.table.identifier,
                                                self.table.sumary,
                                                self.table.content,
                                                self.table.start_date,
                                                self.table.close_date,
                                                self.table.creator,
                                                self.table.responsible,
                                                self.table.priority,
                                                self.table.status,
                                                orderby=self.db.element_has_element.sortable)
        return children 
    
    def _get_item_child(self,child,level=0):
        tmp=Storage()
        tmp['numeral'] = child.element_has_element.numeral
        tmp['id'] = child[self.type].id
        tmp['sid'] = child.element_has_element.id
        tmp['identifier'] = child[self.type].identifier
        tmp['name'] = escape2html(child[self.type].sumary)
        tmp['priority'] = child[self.type].priority
        if child[self.type].content == '':
            tmp['desc'] = current.T('empty')
        else:
            tmp['desc'] = escape2html(return2json(child[self.type].content))
        try:
            if child[self.type].date:
                tmp['date'] = child[self.type].date.strftime(current._config.default_datetime_format)
        except:            
            if child[self.type].start_date is not None:
                tmp['date'] = child[self.type].start_date.strftime(current._config.default_datetime_format)
            else:
                tmp['date'] = current.T("Date")
        if child[self.type].close_date is None:
            tmp['close'] = False
        elif child[self.type].close_date == current._config.default_date:
            tmp['close'] = False
        else:
            tmp['close'] = True
        tmp['creator'] = child[self.type].creator
        tmp['responsible'] = child[self.type].responsible or ''
        tmp['pid'] = self.identifier
        tmp['level'] = level
        # Comments
        tmp['comments'] = self.get_comments_row(child[self.type].identifier)
        # Tags
        tmp['tags'] = self.get_tags_row(child[self.type].identifier)
        # Docs
        tmp['docs'] = self.get_docs_row(child[self.type].identifier)
        # Status
        tmp['status'] = child[self.type].status
        
        return tmp    
    
    def menu_numbers(self):
        query = (self.query) & (self.table.close_date ==None)
        rows = self.db(query).select(self.db.element_has_element.id,
                                self.table.identifier,
                                self.table.creator,
                                self.table.start_date,
                                self.table.close_date,
                                self.table.responsible)
        
        numbers={}
        
        all={}
        all['num']=len(rows)
        all['no_num']=0 
        all['l10']=0
        all['w1']=0 
        all['w2']=0 
        all['m1']=0 
        all['m2']=0
        
        mine={}
        mine['assigned']=0 
        mine['created']=0 
        mine['w1']=0 
        mine['w2']=0 
        mine['m1']=0 
        mine['m2']=0 
        
        # Date range
        today=datetime.today()
        lapse_1w=timedelta(days=7)
        lapse_2w=timedelta(days=14)
        lapse_1m=timedelta(days=30)
        
        for row in rows:
            is_mine=False        
            # My items
            if row[self.type].responsible==current._user.email and \
               row[self.type].close_date is None:
                mine['assigned'] +=1
                is_mine=True
            else:
                if row[self.type].responsible=='' or \
                   row[self.type].responsible==None:
                    all['no_num'] +=1
            if row[self.type].creator == current._user.email:
                mine['created']+=1 
                
            if all['l10'] < 10:
                all['l10']+=1                 
                
            if (today-row[self.type].start_date) > lapse_1m:
                all['m2']+=1
                if is_mine:
                    mine['m2']+=1
            else:
                all['m1']+=1
                if is_mine:
                    mine['m1']+=1
                
            if (today-row[self.type].start_date) < lapse_2w:
                all['w2']+=1
                if is_mine:
                    mine['w2']+=1
                
            if (today-row[self.type].start_date) < lapse_1w:
                all['w1']+=1
                if is_mine:
                    mine['w1']+=1
        
        numbers['all']=all
        numbers['mine']=mine
        return numbers
        
    def change_priority(self,pid,tid,value): 
        try:
            # Update database row
            self.db(self.identifier_query()).update(priority=value) 
                
            # add comment            
            if current._government.check_configuration('project', 
                                                       'add_comment', 
                                                       'enabled'):  
                self.insert_comment(self.identifier,
                                    current.T('Priority changed: %s')%value,
                                    current._user.email,
                                    datetime.now())
                
            # Return element to view
            return current.T('Priority changed')
        except Exception, e:
            current._syslog.error(e, 'warning')
            return current.T('Error: Priority not changed in database')
        
    def change_responsible(self,value): 
        try:
            # Update database row
            self.db(self.identifier_query()).update(responsible=value) 
                
            # add comment
            if current._government.check_configuration('project', 
                                                       'add_comment', 
                                                       'enabled'):  
                self.insert_comment(self.identifier,
                                    current.T('Responsible changed: %s')%value,
                                    current._user.email,
                                    datetime.now())
            # Notify
            if current._government.check_configuration('project', 
                                                       'email_assign_to', 
                                                       'enabled'):          
                # Prepare data
                elem = self.get_data()
                url = "%s/project/bug.html/read/%s/%s" % (current._config.app_url, 
                                                         current._user.project.name, 
                                                         elem.id)
                data= Storage(type=self.literal,
                              sumary=elem.sumary,
                              description=elem.content,
                              url=url,
                              forge=current._config.get('title'))
                try:
                    current.mail.send_from_template(to=value, 
                                                   template='email_assign_to', 
                                                   data=data,
                                                   lang=current._config.get('language'))
                except Exception, e:
                    current._syslog.error('OLElement.change_responsible: %s' % str(e), 
                                          'warning')
                
                
            # Return element to view
            return current.T('Responsible changed')
        except Exception, e:
            current._syslog.error(e, 'warning')
            return current.T('Error: Responsible not changed in database')
        
    def close(self):
        try:
            # Update database row
            self.db(self.identifier_query()).update(close_date=datetime.now()) 
                
            # add comment
            if current._government.check_configuration('project', 
                                                       'add_comment', 
                                                       'enabled'):  
                self.insert_comment(self.identifier,
                                    current.T('%s closed'),
                                    current._user.email,
                                    datetime.now())
            # Notify
            if current._government.check_configuration('project', 
                                                       'email_inform_leader', 
                                                       'enabled'):          
                # Prepare data
                elem = self.get_data()
                url = "%s/project/bug.html/read/%s/%s" % (current._config.app_url, 
                                                         current._user.project.name, 
                                                         elem.id)
                # Leaders
                from project import ProjectHandler
                leaders = ProjectHandler.get_team_leader(current._user.project.id)
                data= Storage(type=self.literal,
                              sumary=elem.sumary,
                              description=elem.content,
                              url=url,
                              forge=current._config.get('title'))
                try:
                    current.mail.send_from_template(to=leaders, 
                                                   template='email_inform_leader', 
                                                   data=data,
                                                   lang=current._config.get('language'))
                except Exception, e:
                    current._syslog.error('OLElement.change_responsible: %s' % str(e), 
                                          'warning')
                
                
            # Return element to view
            return current.T('%s closed' % self.literal)
        except Exception, e:
            current._syslog.error(e, 'warning')
            return current.T('Error: %s not changed in database' % self.literal)
        
    def delete(self):
        try:
            # Update database row
            self.db(self.identifier_query()).update(close_date=datetime.now(),
                                                    status=True) 
                
            # add comment
            if current._government.check_configuration('project', 
                                                       'add_comment', 
                                                       'enabled'):  
                self.insert_comment(self.identifier,
                                    current.T('%s deleted' % self.literal),
                                    current._user.email,
                                    datetime.now())
                
            # Return element to view
            return current.T('%s deleted' % self.literal)
        except Exception, e:
            current._syslog.error(e, 'warning')
            return current.T('Error: %s not changed in database' % self.literal)
        
    def edit(self,data):
        try:
            # Update database row
            self.db(self.identifier_query()).update(**data) 
                
            # add comment
            if current._government.check_configuration('project', 
                                                       'add_comment', 
                                                       'enabled'):  
                self.insert_comment(self.identifier,
                                    current.T('%s edited' % self.literal),
                                    current._user.email,
                                    datetime.now())
        except Exception, e:
            current._syslog.error(e, 'warning')
        
        