# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import datetime

from gluon import current
from gluon.storage import Storage

from mil.tools.common import return2json, escape2html

from project.models.roadmap import RoadmapModel

from project.visual.element import Element

class RoadmapElement(Element):
    def __init__(self,identifier):
        Element.__init__(self,identifier)
        
        RoadmapModel.define_table(self.db)
        
        self.query=self.default_query('roadmap')
        
    def default_query(self,type):
        return (self.db.roadmap.identifier==self.db.element_has_element.child)&(self.db.element_has_element.type==type)&(self.db.element_has_element.parent==self.identifier)
                    
    def child_query_by_id(self,type,id):
        return (self.db.roadmap.id==id)&(self.db.element_has_element.type==type)&(self.db.element_has_element.parent==self.identifier)
                     
    def child_query_by_identifier(self,type,identifier):
        return (self.db.roadmap.identifier==identifier)&(self.db.element_has_element.type==type)&(self.db.element_has_element.parent==self.identifier)
                    
    def prepare_child(self,child,level=0):
        return self._get_roadmap_child(child,level)
        
    def get_children_from_db(self,child,limit):
        if child is not None:
            self.query=(self.db.element_has_element.type=='roadmap')&(self.db.element_has_element.child==self.db.roadmap.identifier)&(self.db.roadmap.id==child)
        if limit is not None:
            children=self.db(self.query).select(self.db.element_has_element.id,
                                                self.db.element_has_element.numeral,
                                                self.db.roadmap.id,
                                                self.db.roadmap.identifier,
                                                self.db.roadmap.sumary,
                                                self.db.roadmap.content,
                                                self.db.roadmap.start_date,
                                                self.db.roadmap.close_date,
                                                self.db.roadmap.creator,
                                                self.db.roadmap.responsible,
                                                self.db.roadmap.priority,
                                                self.db.roadmap.init_date,
                                                self.db.roadmap.end_date,
                                                orderby=self.db.element_has_element.sortable,
                                                limitby=limit)
        else:
            children=self.db(self.query).select(self.db.element_has_element.id,
                                                self.db.element_has_element.numeral,
                                                self.db.roadmap.id,
                                                self.db.roadmap.identifier,
                                                self.db.roadmap.sumary,
                                                self.db.roadmap.content,
                                                self.db.roadmap.start_date,
                                                self.db.roadmap.close_date,
                                                self.db.roadmap.creator,
                                                self.db.roadmap.responsible,
                                                self.db.roadmap.priority,
                                                self.db.roadmap.init_date,
                                                self.db.roadmap.end_date,
                                                orderby=self.db.element_has_element.sortable)
        return children 
    
    def get_child_by_id(self,id):
        query=self.child_query_by_id('roadmap',id)
        child=self.db(self.query).select(self.db.element_has_element.id,
                                         self.db.element_has_element.numeral,
                                         self.db.roadmap.id,
                                         self.db.roadmap.identifier,
                                         self.db.roadmap.sumary,
                                         self.db.roadmap.content,
                                         self.db.roadmap.start_date,
                                         self.db.roadmap.close_date,
                                         self.db.roadmap.creator,
                                         self.db.roadmap.responsible,
                                         self.db.roadmap.priority,
                                         self.db.roadmap.init_date,
                                         self.db.roadmap.end_date,
                                         orderby=self.db.element_has_element.sortable).first()
        
        return self.prepare_child(child)
    
    def _get_roadmap_child(self,child,level):
        tmp=Storage()
        tmp['numeral'] = child.element_has_element.numeral
        tmp['id'] = child.roadmap.id
        tmp['sid'] = child.element_has_element.id
        tmp['identifier'] = child.roadmap.identifier
        tmp['name'] = child.roadmap.sumary
        tmp['priority'] = child.roadmap.priority
        if child.roadmap.content == '':
            tmp['desc'] = current.T('empty')
        else:
            tmp['desc'] = escape2html(return2json(child.roadmap.content))
        if child.roadmap.close_date is None:
            tmp['close'] = False
        elif child.roadmap.close_date==DEFAULT_DATE:
            tmp['close'] = False
        else:
            tmp['close'] = True
        if child.roadmap.init_date is not None:
            tmp['ini_date'] = child.roadmap.init_date.strftime(current._config.default_date_format)
        else:
            tmp['ini_date'] = current.T("Date")
        if child.roadmap.end_date is not None:
            tmp['end_date'] = child.roadmap.end_date.strftime(current._config.default_date_format)
        else:
            tmp['end_date'] = current.T("Date")
        tmp['creator'] = child.roadmap.creator
        tmp['responsible'] = child.roadmap.responsible
        tmp['pid'] = self.identifier
        tmp['level'] = level
        # Comments
        tmp['comments'] = self.get_comments_row(child.roadmap.identifier)
        # Tags
        tmp['tags'] = self.get_tags_row(child.roadmap.identifier)
        
        return tmp
        
    def delete(self):
        try:
            # Update database row
            self.db(self.db.roadmap.identifier==self.identifier).delete() 
                
            # Return element to view
            return current.T('Roadmap deleted')
        except:
            return current.T('Error:Roadmap not changed in database')
        
    def edit(self,data):
        try:
            # Update database row
            self.db(self.db.roadmap.identifier==self.identifier).update(**data) 
                
            # add comment
            self.insert_comment(self.identifier,
                                current.T('Bug edited'),
                                vip_user.email,
                                datetime.datetime.now())
        except:
            pass
        