# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import datetime

from gluon import current
from gluon.storage import Storage

from project.models.task import TaskModel

from project.visual.element import Element

class TaskElement(Element):
    def __init__(self,identifier):
        Element.__init__(self,identifier)
        
        TaskModel.define_table(self.db)
        
        self.query=self.default_query('task')
        
    def default_query(self,type):
        return (self.db.task.identifier==self.db.element_has_element.child)&(self.db.element_has_element.type==type)&(self.db.element_has_element.parent==self.identifier)
                    
    def prepare_child(self,child,level=0):
        return self._get_task_child(child,level)
        
    def get_children_from_db(self,child,limit):
        if child is not None:
            self.query=(self.db.element_has_element.type=='task')&(self.db.element_has_element.child==self.db.task.identifier)&(self.db.task.id==child)
        if limit is not None:
            children=self.db(self.query).select(self.db.element_has_element.id,
                                                self.db.element_has_element.numeral,
                                                self.db.task.id,
                                                self.db.task.identifier,
                                                self.db.task.sumary,
                                                self.db.task.content,
                                                self.db.task.start_date,
                                                self.db.task.close_date,
                                                self.db.task.creator,
                                                self.db.task.responsible,
                                                self.db.task.priority,
                                                orderby=self.db.element_has_element.sortable,
                                                limitby=limit)
        else:
            children=self.db(self.query).select(self.db.element_has_element.id,
                                                self.db.element_has_element.numeral,
                                                self.db.task.id,
                                                self.db.task.identifier,
                                                self.db.task.sumary,
                                                self.db.task.content,
                                                self.db.task.start_date,
                                                self.db.task.close_date,
                                                self.db.task.creator,
                                                self.db.task.responsible,
                                                self.db.task.priority,
                                                orderby=self.db.element_has_element.sortable)
        return children 
    
    def _get_task_child(self,child,level):
        tmp=Storage()
        tmp['numeral'] = child.element_has_element.numeral
        tmp['id'] = child.task.id
        tmp['sid'] = child.element_has_element.id
        tmp['identifier'] = child.task.identifier
        tmp['name'] = child.task.sumary
        tmp['priority'] = child.task.priority
        if child.task.content == '':
            tmp['desc'] = current.T('empty')
        else:
            tmp['desc'] = child.task.content
        if child.task.start_date is not None:
            tmp['date'] = child.task.start_date.strftime(current._config.default_date_format)
        else:
            tmp['date'] = current.T("Date")
        if child.task.close_date is None:
            tmp['close'] = False
        elif child.task.close_date == current._config.default_date:
            tmp['close'] = False
        else:
            tmp['close'] = True
        tmp['creator'] = child.task.creator
        tmp['responsible'] = child.task.responsible or ''
        tmp['pid'] = self.identifier
        tmp['level'] = level
        # Comments
        tmp['comments'] = self.get_comments_row(child.task.identifier)
        # Tags
        tmp['tags'] = self.get_tags_row(child.task.identifier)
        # Docs
        tmp['docs'] = self.get_docs_row(child.task.identifier)
        
        return tmp    
    
    def menu_numbers(self):
        from datetime import datetime, timedelta
        
        TaskModel.define_table(self.db)
        query = (self.query) & (self.db.task.close_date ==None)
        rows = self.db(query).select(self.db.element_has_element.id,
                                self.db.task.identifier,
                                self.db.task.creator,
                                self.db.task.start_date,
                                self.db.task.close_date,
                                self.db.task.responsible)
        
        numbers={}
        
        all={}
        all['num']=len(rows)
        all['no_num']=0 
        all['l10']=0
        all['w1']=0 
        all['w2']=0 
        all['m1']=0 
        all['m2']=0
        
        mine={}
        mine['assigned']=0 
        mine['created']=0 
        mine['w1']=0 
        mine['w2']=0 
        mine['m1']=0 
        mine['m2']=0 
        
        # Date range
        today=datetime.today()
        lapse_1w=timedelta(days=7)
        lapse_2w=timedelta(days=14)
        lapse_1m=timedelta(days=30)
        
        for row in rows:
            is_mine=False        
            # My tasks
            if row.task.responsible==current._user.email and row.task.close_date is None:
                mine['assigned'] +=1
                is_mine=True
            else:
                if row.task.responsible=='' or row.task.responsible==None:
                    all['no_num'] +=1
            if row.task.creator==current._user.email:
                mine['created']+=1 
                
            if all['l10'] < 10:
                all['l10']+=1                 
                
            if (today-row.task.start_date) > lapse_1m:
                all['m2']+=1
                if is_mine:
                    mine['m2']+=1
            else:
                all['m1']+=1
                if is_mine:
                    mine['m1']+=1
                
            if (today-row.task.start_date) < lapse_2w:
                all['w2']+=1
                if is_mine:
                    mine['w2']+=1
                
            if (today-row.task.start_date) < lapse_1w:
                all['w1']+=1
                if is_mine:
                    mine['w1']+=1
        
        numbers['all']=all
        numbers['mine']=mine
        return numbers
        
    def change_priority(self,pid,tid,value): 
        try:
            # Update database row
            self.db(self.db.task.identifier==self.identifier).update(priority=value) 
                
            # add comment
            self.insert_comment(self.identifier,
                                current.T('Priority changed: %s')%value,
                                current._user.email,
                                datetime.datetime.now())
                
            # Return element to view
            return current.T('Priority changed')
        except:
            return current.T('Error:Priority not changed in database')
        
    def change_responsible(self,value): 
        try:
            # Update database row
            self.db(self.db.task.identifier==self.identifier).update(responsible=value) 
                
            # add comment
            self.insert_comment(self.identifier,
                                current.T('Responsible changed: %s')%value,
                                current._user.email,
                                datetime.datetime.now())
                
            # Return element to view
            return current.T('Responsible changed')
        except:
            return current.T('Error:Responsible not changed in database')
        
    def close(self):
        try:
            # Update database row
            self.db(self.db.task.identifier==self.identifier).update(close_date=datetime.datetime.now()) 
                
            # add comment
            self.insert_comment(self.identifier,
                                current.T('Task closed'),
                                current._user.email,
                                datetime.datetime.now())
                
            # Return element to view
            return current.T('Task closed')
        except:
            return current.T('Error:Task not changed in database')
        
    def delete(self):
        try:
            # Update database row
            self.db(self.db.task.identifier==self.identifier).update(close_date=datetime.datetime.now(),
                                                           status=True) 
                
            # add comment
            self.insert_comment(self.identifier,
                                current.T('Task deleted'),
                                current._user.email,
                                datetime.datetime.now())
                
            # Return element to view
            return current.T('Task deleted')
        except:
            return current.T('Error:Task not changed in database')
        
    def edit(self,data):
        try:
            # Update database row
            self.db(self.db.task.identifier==self.identifier).update(**data) 
                
            # add comment
            self.insert_comment(self.identifier,
                                current.T('Task edited'),
                                current._user.email,
                                datetime.datetime.now())
        except:
            pass
        
        