# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import current

# Base class
class VisualBase:
    def __init__(self,module,id):
        self.module=module
        self.id=id
        self.colspan='10'
    
    def header(self,header):    
        return ol_list_header(header)
    
    def container(self):
        if vip_user.write==False:
            commons=[]
        else:
            commons=[DIV(DIV('',
                             _class="ol-item-slider ui-slider ui-slider-vertical",
                             _id="olSliderSlide"),
                         _class="ol-item-slider ol-hidden",
                         _id="olSliderItem"),
                     self.pattern(),
                     ol_tag_pattern(),
                     ol_comment_pattern()]
    
        subheader=''
        
        return ol_list_container(subheader,
                                 self.ol_list(),
                                 ol_add_btn(self.module,self.id),
                                 commons)
    
    def element(self,id):
        f=algo_de_id(id)
        return self.box(f)
        
    def get_rows(self,query): 
        return []
      
    def row(self,row):
        # Prepare data to generate box
        tmp = Storage()
        tmp['numeral'] = 0
        tmp['id'] = 0
        tmp['name'] = current.T('empty')
        tmp['priority'] = 0
        tmp['desc'] = current.T('empty')
        tmp['date'] = current.T("Date")
        tmp['close'] = True
        tmp['creator'] = current.T('empty')
        tmp['responsible'] = current.T('empty')
        tmp['pid'] = 0
        tmp['level'] = 0
        tmp['comments'] = []
        tmp['tags'] = []
        # Prepare data to view
        return self.box(tmp)
      
    def printer(self,project,title,id=None):
        from project.writers import MyFPDF
        self.pdf=MyFPDF("%s\n%s\n%s"%(project.title,title,''))
        self.pdf.alias_nb_pages()
         
        # Visual element
        list_items=self.get_all_data(id)
        for item in list_items:
            self._printer_item(item)
            
        # prepare PDF to download:
        current.response.headers['Content-Type'] = 'application/pdf'
        current.response.headers['Content-Disposition'] = \
                        "attachment; filename='%s.pdf'"%project.name;
        return self.pdf.output(dest='S')
    
    def _printer_item(self,item):   
        # Add page        
        self.pdf.add_page()
        # Title
        (x,y)=self.pdf.get_xy()
        self.pdf.bold_font()
        literal="%s - %s (%s) [%s]"%(item.numeral,
                                     self.pdf.encode(item.name),
                                     item.creator,
                                     item.responsible)
        self.pdf.paragraph(10,y+5,0,5,literal,0,0)
        
        self.pdf.normal_font()  
        # Tags
        try:
            if len(item.tags) >0:
                tag_list=[]
                for tag in item.tags:
                     tag_list.append(tag['data'])
                literal=", ".join(["%s " %  k  for k  in tag_list])
                (x,y)=self.pdf.get_xy()
                self.pdf.italic_font()
                self.pdf.paragraph(15,y+3,0,5,"%s: %s"%(current.T("Tags"),literal),0,0)
                self.pdf.normal_font()   
        except:
            pass                  
        
        # Description
        (x,y)=self.pdf.get_xy()
        self.pdf.italic_font()
        self.pdf.paragraph(15,y+3,0,5,current.T('Description'),0,0)  
        self.pdf.normal_font()          
        (x,y)=self.pdf.get_xy()
        self.pdf.paragraph(20,y+3,0,5,self.pdf.encode(item.desc),0,0)
        
        # Comments
        try:
            if len(item.comments) >0:
                (x,y)=self.pdf.get_xy()
                self.pdf.italic_font()
                self.pdf.paragraph(15,y+3,0,5,current.T('Comments'),0,0)
                self.pdf.normal_font()  
                for comm in item.comments:
                    (x,y)=self.pdf.get_xy()
                    self.pdf.paragraph(25, y+3, 0, 5,
                                       self.pdf.encode(comm['content']),0,0)
                    (x,y)=self.pdf.get_xy()
                    self.pdf.paragraph(100, y+3, 0, 5,
                                       "%s (%s)"%(comm['author'],
                                                  comm['date']),0,0) 
        except:
            pass                  
                
        # Files
        try:
            if len(item.docs) >0:
                tmp_list=[]
                for elem in item.docs:
                     tmp_list.append("%s (%s - %s)"%(self.pdf.encode(elem['content']),
                                                     elem['creator'],
                                                     elem['date']))
                literal=", ".join(["%s " %  k  for k  in tmp_list])
                (x,y)=self.pdf.get_xy()
                self.pdf.italic_font()
                self.pdf.paragraph(15,y+3,0,5,
                                   "%s: %s"%(current.T("Files"),literal),
                                   0,0)
                self.pdf.normal_font()  
        except:
            pass       
                
        try:
            if len(item.children)>0:
                for child in item.children:
                    self._printer_item(child)
        except:
            pass
    
    def writer_txt(self,project,title,id=None):   
        from project.writers import MyTEXTER
        self.writer=MyTEXTER(project.name)
        self.writer.header(project.title,title)
         
        # Visual element
        list_items=self.get_all_data(id)
        for item in list_items:
            self._write_txt_item(item)

        return self.writer.output()
    
    def _write_txt_item(self,item): 
        self.writer.nl()        
        # Title
        literal="%s - %s (%s) [%s]"%(item.numeral,
                                     self.writer.encode(item.name),
                                     item.creator,
                                     item.responsible)
        self.writer.line(len(literal)+1)
        self.writer.paragraph(literal)
        self.writer.line(len(literal)+1)
        
        # Tags
        try:
            if len(item.tags) >0:
                tag_list=[]
                for tag in item.tags:
                     tag_list.append(tag['data'])
                literal=", ".join(["%s " %  k  for k  in tag_list])
                self.writer.paragraph("%s: %s"%(current.T("Tags"),literal))
                self.writer.nl()
        except:
            pass                  
        
        # Description
        self.writer.paragraph(current.T('Description'))  
        self.writer.line(len(current.T('Description'))+1)
        self.writer.paragraph(self.writer.encode(item.desc))
        self.writer.nl()
        
        # Comments
        try:
            if len(item.comments) >0:
                self.writer.paragraph(current.T('Comments'))
                self.writer.line(len(current.T('Comments'))+1)
                for comm in item.comments:
                    self.writer.paragraph(self.writer.encode(comm['content']))
                    self.writer.paragraph(self.writer.ident("%s (%s)"%(comm['author'],
                                                                       comm['date'])))
        except:
            pass                  
                
        # Files
        try:
            if len(item.docs) >0:
                tmp_list=[]
                for elem in item.docs:
                     tmp_list.append("%s (%s - %s)"%(self.writer.encode(elem['content']),
                                                     elem['creator'],
                                                     elem['date']))
                literal=", ".join(["%s " %  k  for k  in tmp_list])
                self.writer.paragraph(self.writer.ident("%s: %s"%(current.T("Files"),literal)))
                self.writer.nl()
        except:
            pass       
                
        try:
            if len(item.children)>0:
                for child in item.children:
                    self._write_txt_item(child)
        except:
            pass
    
    def writer_rtf(self,project,title,id=None):
        from project.writers import MyRTF
        self.writer=MyRTF(project.name)
        self.writer.header(project.title,title)
         
        # Visual element
        list_items=self.get_all_data(id)
        for item in list_items:
            self._write_rtf_item(item)

        return self.writer.output()
    
    def _write_rtf_item(self,item):  
        # Add page        
        self.writer.add_page()
        
        # Title
        literal="%s - %s (%s) [%s]"%(item.numeral,
                                     item.name,
                                     item.creator,
                                     item.responsible)
        self.writer.paragraph(self.writer.bold(literal))

        # Tags
        try:
            if len(item.tags) >0:
                tag_list=[]
                for tag in item.tags:
                     tag_list.append(tag['data'])
                literal=", ".join(["%s " %  k  for k  in tag_list])
                self.writer.paragraph(self.writer.italic("%s: %s"%(current.T("Tags"),
                                                                   literal)))
                self.writer.normal_font()   
        except:
            pass                  
        
        # Description
        self.writer.paragraph(self.writer.italic(current.T('Description')))  
        self.writer.paragraph(item.desc)
        # Comments
        try:
            if len(item.comments) >0:
                self.writer.paragraph(self.writer.italic(current.T('Comments')))
                for comm in item.comments:
                    self.writer.paragraph(self.writer.normal(comm['content']))
                    self.writer.paragraph(self.writer.normal("%s (%s)"%(comm['author'],
                                                                        comm['date'])))
        except:
            pass                  
                
        # Files
        try:
            if len(item.docs) >0:
                tmp_list=[]
                for elem in item.docs:
                     tmp_list.append("%s (%s - %s)"%(self.writer.encode(elem['content']),
                                                     elem['creator'],elem['date']))
                literal=", ".join(["%s " %  k  for k  in tmp_list])
                self.writer.paragraph(self.writer.italic("%s: %s"%(current.T("Files"),
                                                                   literal)))
        except:
            pass       
                
        try:
            if len(item.children)>0:
                for child in item.children:
                    self._write_rtf_item(child)
        except:
            pass
