# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import datetime

from gluon import *

from project import ProjectHandler

from project.visual.element_bug import BugElement

from project.visual.visual_base import VisualBase

class VisualBug(VisualBase):
    def __init__(self):
        self.module = 'bug'
        self.id = current._user.project.id
        self.db = current.db
                
        #self.ident = ProjectHandler.get_identifier(self.id)
        self.ident = current._user.project.ident
        self.parent = BugElement(self.ident)  
    
    def get_query(self):
        query=self.parent.default_query(self.module)
        try:  
            if current.session[self.module+'_type']:           
                if current.session[self.module+'_type']=='open':
                    query = query & (self.db.bug.close_date==None)
                elif current.session[self.module+'_type']=='close':        
                    query =  query & (self.db.bug.close_date!=None)
        except:
            pass
    
        flag_type=True
        try:
            if current.session[self.module+'_filter']:
                if current.session[self.module+'_filter']=='my':
                    if current.session[self.module+'_range']!='created':
                        query=query & (self.db.bug.responsible==current._user.email)
                    flag_type=False  
        except:
            pass
        
        try:
            if current.session[self.module+'_type']:
                if flag_type:
                    if current.session[self.module+'_type']=='open':
                        query = query & (self.db.bug.close_date==None)
                    elif current.session[self.module+'_type']=='close':        
                        query =  query & (self.db.bug.close_date!=None)
        except:
            pass
        
        # Date range
        from datetime import datetime,timedelta
        today=datetime.today()
        lapse_1w=today-timedelta(days=7)
        lapse_2w=today-timedelta(days=14)
        lapse_1m=today-timedelta(days=30)
        
        try:
            if current.session[self.module+'_range']:
                if current.session[self.module+'_range']=='oneweek':            
                    query=query & (self.db.bug.start_date>=lapse_1w)
                elif current.session[self.module+'_range']=='twoweeks':      
                    query=query & (self.db.bug.start_date>=lapse_2w)
                elif current.session[self.module+'_range']=='onemonth':
                    query=query & (self.db.bug.start_date>=lapse_1m)
                elif current.session[self.module+'_range']=='older':
                    query=query & (self.db.bug.start_date<=lapse_1m)
                elif current.session[self.module+'_range']=='assigned':
                    query=query & (self.db.bug.responsible!=None)
                elif current.session[self.module+'_range']=='created':
                    query=query & (self.db.bug.creator==current._user.email)
                elif current.session[self.module+'_range']=='unassigned':
                    query=query & (self.db.bug.responsible==None)
        except:
            pass
        
        self.parent.set_query(query)   
           
    def get_data(self,child_id=None): 
        # Prepare query
        self.get_query()
        
        # Prepare number of rows selected
        last_10=False
        try:       
           if current.session[self.module+'_range']=='last10':
                last_10=True 
        except:
            last_10=False
      
        if last_10:
            rows = self.parent.get_children(limit=(0,10))
        else:
            rows = self.parent.get_children(child=child_id)
        
        list = []
        for row in rows:
            list.append(self.prepare_data_child(row))  
            
        return list 
    
    def prepare_data_child(self,child): 
        child['config']=self._configure_element(child) 
        child_element=BugElement(child.identifier)
        level_child=child.level + 1
#        child_element.set_query(self.query) # WARN this assigment fails
        children=child_element.get_children(level=level_child)
        child['children']=[]
        for item in children:
            child['children'].append(self.prepare_data_child(item))
        return child
    
    def _configure_element(self,f): 
        params={'id': 'mm_add',
                'pid': f.pid,
                'sid': f.sid,
                'pname': current._user.project.name,
                'tid': f.id,
                'ident': f.identifier,
                'priority': f.priority,
                'callback': URL(r=current.request, 
                                c=current.request.controller, 
                                f=self.module),
                'modal': URL(r=current.request, 
                             c=current.request.controller, 
                             f='%s.load'%self.module,
                             args=['modal',current._user.project.name]),
                'delete': "delete_%s"%self.module,
                'close': "close_%s"%self.module}
        configuration="{div_id:'%(id)s', sid:'%(sid)s', pid:'%(pid)s', pname:'%(pname)s', tid:'%(tid)s', id:'%(ident)s', priority:'%(priority)s', callback:'%(callback)s', modal:'%(modal)s', delete_item:'%(delete)s', close_item:'%(close)s'}"%params
        return configuration 
                  
    def sort(self,vars):
        self.parent.sort(vars,'bug')
        
    def edit_parent(self,data):
        return self.parent.edit(data)
        
    def add_child(self,identifier):
        return self.parent.insert_child(identifier,'bug')
                
    def add_db_parameters(self):
        # Default values
        if current._user.id is not None:
            self.db.bug.creator.default=current._user.email
            self.db.bug.creator.writable=False
        self.db.bug.status.default=0
        self.db.bug.update_date.default=datetime.datetime.now()
        self.db.bug.update_date.writable=False
        self.db.bug.update_date.readable=False
        self.db.bug.responsible.writable=False
        self.db.bug.responsible.readable=False
        self.db.bug.close_date.writable=False
        self.db.bug.close_date.readable=False
        self.db.bug.status.writable=False
        self.db.bug.status.readable=False
        self.db.bug.priority.writable=False
        self.db.bug.priority.readable=False
        self.db.bug.priority.default=1
        self.db.bug.project.writable=False
        self.db.bug.project.readable=False
        self.db.bug.project.default=self.ident

    def edit_db_parameters(self):
        # Default values
        self.db.bug.id.readable=False
        self.db.bug.id.writable=False
        self.db.bug.creator.readable=False
        self.db.bug.creator.writable=False
        self.db.bug.status.default=0
        self.db.bug.update_date.default=datetime.datetime.now()
        self.db.bug.update_date.writable=False
        self.db.bug.update_date.readable=False
        self.db.bug.responsible.writable=False
        self.db.bug.responsible.readable=False
        self.db.bug.close_date.writable=False
        self.db.bug.close_date.readable=False
        self.db.bug.status.writable=False
        self.db.bug.status.readable=False
        self.db.bug.priority.writable=False
        self.db.bug.priority.readable=False
        self.db.bug.priority.default=1 
        self.db.bug.project.writable=False
        self.db.bug.project.readable=False
        self.db.bug.project.default=self.ident
    
    def element_by_id(self,id):
        row=self.parent.get_child_by_id(id)
        return self.box(row)
    
    def header(self):
        name=current._user.project.name
        
        if current.session.bug_filter:
            filter=current.session.bug_filter
        else:
            filter='all'
            
        if current.session.bug_type=='open':
            if filter=='all':
                title=current.T('Open Bugs')
            else:
                title=current.T('My Open Bugs')
            params=[{'title':current.T('All'),
                     'params':"{url:'%s'}"%URL(r=current.request,
                                               f='index', 
                                               args=(name,'all','all'))},
                    {'title':current.T('Closed'),
                     'params':"{url:'%s'}"%URL(r=current.request,
                                               f='index', 
                                               args=(name,'close',filter))}]
        elif current.session.bug_type=='close':
            if filter=='all':
                title=current.T('Closed bugs')
            else:
                title=current.T('My Closed Bugs')
            params=[{'title':current.T('All'),
                     'params':"{url:'%s'}"%URL(r=current.request,
                                               f='index', 
                                               args=(name,'all','all'))},
                    {'title':current.T('Open'),
                     'params':"{url:'%s'}"%URL(r=current.request,
                                               f='index', 
                                               args=(name,'open',filter))}]
        elif current.session.bug_type=='all':
            if filter=='all':
                title=current.T('All bugs')
            else:
                title=current.T('All My Bugs')
            params=[{'title':current.T('Close'),
                     'params':"{url:'%s'}"%URL(r=current.request,
                                               f='index', 
                                               args=(name,'close',filter))},
                    {'title':current.T('Open'),
                     'params':"{url:'%s'}"%URL(r=current.request,
                                               f='index', 
                                               args=(name,'open',filter))}]
        else:
            title=current.T('Bugs')
            params=[{'title':current.T('All'),
                     'params':"{url:'%s'}"%URL(r=current.request,
                                               f='index', 
                                               args=(name,'all','all'))},
                    {'title':current.T('Close'),
                     'params':"{url:'%s'}"%URL(r=current.request,
                                               f='index', 
                                               args=(name,'close',filter))},
                    {'title':current.T('Open'),
                     'params':"{url:'%s'}"%URL(r=current.request,
                                               f='index', 
                                               args=(name,'open',filter))}]
        
        return (title,params)
       
    def menu(self):
        # Get numbers
        numbers=self.parent.menu_numbers()
        name=current._user.project.name
        pid=current._user.project.id
        
        # Session type
        if current.session.bug_type:
            if current.session.bug_type=='open':
                type='open'
            elif current.session.bug_type=='close':
                type='close'
            else:
                type='all'
        else:
            type='all'
            
        return (type,numbers)

