# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import datetime

from gluon import *
import gluon.contrib.simplejson as json

from project import ProjectHandler

from project.visual.element_ol import OLElement

from project.visual.visual_base import VisualBase

class VisualOL(VisualBase):
    def __init__(self, type):
        self.type = type
        
        self.id = current._user.project.id
        self.db = current.db
                
        self.ident = current._user.project.ident
        self.parent = OLElement(self.type, self.ident)  
    
    def get_query(self):
        query=self.parent.default_query(self.type)
        try:  
            if current.session[self.type].type:           
                if current.session[self.type].type=='open':
                    query = query & (self.parent.table.close_date==None)
                elif current.session[self.type].type=='close':        
                    query =  query & (self.parent.table.close_date!=None)
        except:
            pass
    
        flag_type=True
        try:
            if current.session[self.type].filter:
                if current.session[self.type].filter=='my':
                    if current.session[self.type].range!='created':
                        query=query & (self.parent.table.responsible==current._user.email)
                    flag_type=False  
        except:
            pass
        
        try:
            if current.session[self.type].type:
                if flag_type:
                    if current.session[self.type].type=='open':
                        query = query & (self.parent.table.close_date==None)
                    elif current.session[self.type].type=='close':        
                        query =  query & (self.parent.table.close_date!=None)
        except:
            pass
        
        # Date range
        from datetime import datetime,timedelta
        today=datetime.today()
        lapse_1w=today-timedelta(days=7)
        lapse_2w=today-timedelta(days=14)
        lapse_1m=today-timedelta(days=30)
        
        try:
            if current.session[self.type].range:
                if current.session[self.type].range=='oneweek':            
                    query=query & (self.parent.table.start_date>=lapse_1w)
                elif current.session[self.type].range=='twoweeks':      
                    query=query & (self.parent.table.start_date>=lapse_2w)
                elif current.session[self.type].range=='onemonth':
                    query=query & (self.parent.table.start_date>=lapse_1m)
                elif current.session[self.type].range=='older':
                    query=query & (self.parent.table.start_date<=lapse_1m)
                elif current.session[self.type].range=='assigned':
                    query=query & (self.parent.table.responsible!=None)
                elif current.session[self.type].range=='created':
                    query=query & (self.parent.table.creator==current._user.email)
                elif current.session[self.type].range=='unassigned':
                    query=query & (self.parent.table.responsible==None)
        except:
            pass
        
        self.parent.set_query(query)   
           
    def get_all_data(self,child_id=None): 
        # Prepare query
        self.get_query()
        
        # Prepare number of rows selected
        last_10=False
        try:       
           if current.session[self.type].range=='last10':
                last_10=True 
        except:
            last_10=False
      
        if last_10:
            rows = self.parent.get_children(limit=(0,10))
        else:
            rows = self.parent.get_children(child=child_id)
            
        return rows
    
    def get_data(self,child_id=None): 
        rows = self.get_all_data(child_id)   
        list = []
        for row in rows:
            list.append(self.prepare_data_child(row))            
        return XML(json.dumps(list), sanitize=False)    
    
    def prepare_data_child(self,child): 
        r_child = self._configure_element(child) 
        
        child_element = OLElement(self.type, child.identifier)
        
        level_child = child.level + 1
        children = child_element.get_children(level=level_child)
        for item in children:
            r_child['children'].append(self.prepare_data_child(item))
        return r_child
    
    
#    def prepare_data_child(self,child): 
#        child['config'] = self._configure_element(child) 
#        child_element = OLElement(self.type, child.identifier)
#        level_child = child.level + 1
##        child_element.set_query(self.query) # WARN this assigment fails
#        children = child_element.get_children(level=level_child)
#        child['children'] = []
#        for item in children:
#            child['children'].append(self.prepare_data_child(item))
#        return child
    
    def _configure_element(self,f):
        params={'id': 'mm_add',
                'pid': f.pid,
                'sid': f.sid,
                'pname': current._user.project.name,
                'tid': f.identifier,
                'ident': f.identifier,
                'priority': f.priority,
                'callback': URL(r=current.request, 
                                c=current.request.controller, 
                                f=self.type),
                'modal': URL(r=current.request, 
                             c=current.request.controller, 
                             f='%s.load'%self.type,
                             args=['modal',current._user.project.name]),
                'delete': "delete_item",
                'close': "close_item"}
        
        config = {'div_id': params['id'], 
                  'sid': params['sid'], 
                  'pid': params['pid'], 
                  'pname': params['pname'], 
                  'tid': params['tid'], 
                  'id': params['ident'], 
                  'priority': params['priority'], 
                  'callback': params['callback'], 
                  'modal': params['modal'], 
                  'delete_item': params['delete'], 
                  'close_item': params['close'],
                  'numeral': f.numeral,
                  'description': f.desc,
                  'creator': f.creator,
                  'responsible': f.responsible,
                  'name': f.name,
                  'tags': f.tags,
                  'comments': f.comments,
                  'documents': f.docs,
                  'date': f.date,
                  'status': f.status,
                  'close': f.close,
                  'children': []}
        return config
    
    def sort(self,vars):
        self.parent.sort(vars, self.type)
        
    def edit_parent(self,data):
        return self.parent.edit(data)
        
    def get_child(self,id):
        return self._configure_element(self.parent.get(id))
    
    def add_child(self,identifier):
        return self.parent.insert_child(identifier, self.type)
                
    def add_db_parameters(self):
        # Default values
        if current._user.id is not None:
            self.parent.table.creator.default=current._user.email
            self.parent.table.creator.writable=False
        self.parent.table.status.default=0
        self.parent.table.update_date.default=datetime.datetime.now()
        self.parent.table.update_date.writable=False
        self.parent.table.update_date.readable=False
        self.parent.table.responsible.writable=False
        self.parent.table.responsible.readable=False
        self.parent.table.close_date.writable=False
        self.parent.table.close_date.readable=False
        self.parent.table.status.writable=False
        self.parent.table.status.readable=False
        self.parent.table.priority.writable=False
        self.parent.table.priority.readable=False
        self.parent.table.priority.default=1
        self.parent.table.project.writable=False
        self.parent.table.project.readable=False
        self.parent.table.project.default=self.ident

    def edit_db_parameters(self):
        # Default values
        self.parent.table.id.readable=False
        self.parent.table.id.writable=False
        self.parent.table.creator.readable=False
        self.parent.table.creator.writable=False
        self.parent.table.status.default=0
        self.parent.table.update_date.default=datetime.datetime.now()
        self.parent.table.update_date.writable=False
        self.parent.table.update_date.readable=False
        self.parent.table.responsible.writable=False
        self.parent.table.responsible.readable=False
        self.parent.table.close_date.writable=False
        self.parent.table.close_date.readable=False
        self.parent.table.status.writable=False
        self.parent.table.status.readable=False
        self.parent.table.priority.writable=False
        self.parent.table.priority.readable=False
        self.parent.table.priority.default=1 
        self.parent.table.project.writable=False
        self.parent.table.project.readable=False
        self.parent.table.project.default=self.ident
    
    def element_by_id(self,id):
        row=self.parent.get_child_by_id(id)
        return self.box(row)
    
    def header(self):
        name=current._user.project.name
        
        # Get data from session
        curr_sess = current.session[self.type]
        
        if curr_sess.filter:
            filter=curr_sess.filter
        else:
            filter='all'
            
        if curr_sess.type=='open':
            if filter=='all':
                title=current.T('Open %s' % self.parent.literal)
            else:
                title=current.T('My Open %s' % self.parent.literal)
            params=[{'title':current.T('All'),
                     'params':"{url:'%s'}"%URL(r=current.request,
                                               c='project',
                                               f=self.type, 
                                               args=(name,
                                                     'all',
                                                     'all'))},
                    {'title':current.T('Closed'),
                     'params':"{url:'%s'}"%URL(r=current.request,
                                               c='project',
                                               f=self.type, 
                                               args=(name,
                                                     'close',
                                                     filter))}]
        elif curr_sess.type=='close':
            if filter=='all':
                title=current.T('Closed %s' % self.parent.literal)
            else:
                title=current.T('My Closed %s' % self.parent.literal)
            params=[{'title':current.T('All'),
                     'params':"{url:'%s'}"%URL(r=current.request,
                                               c='project',
                                               f=self.type, 
                                               args=(name,
                                                     'all',
                                                     'all'))},
                    {'title':current.T('Open'),
                     'params':"{url:'%s'}"%URL(r=current.request,
                                               c='project',
                                               f=self.type, 
                                               args=(name,
                                                     'open',
                                                     filter))}]
        elif curr_sess.type=='all':
            if filter=='all':
                title=current.T('All %s' % self.parent.literal)
            else:
                title=current.T('All My %s' % self.parent.literal)
            params=[{'title':current.T('Close'),
                     'params':"{url:'%s'}"%URL(r=current.request,
                                               c='project',
                                               f=self.type, 
                                               args=(name,
                                                     'close',
                                                     filter))},
                    {'title':current.T('Open'),
                     'params':"{url:'%s'}"%URL(r=current.request,
                                               c='project',
                                               f=self.type, 
                                               args=(name,
                                                     'open',
                                                     filter))}]
        else:
            title=current.T('%s')
            params=[{'title':current.T('All'),
                     'params':"{url:'%s'}"%URL(r=current.request,
                                               c='project',
                                               f=self.type, 
                                               args=(name,
                                                     'all',
                                                     'all'))},
                    {'title':current.T('Close'),
                     'params':"{url:'%s'}"%URL(r=current.request,
                                               c='project',
                                               f=self.type, 
                                               args=(name,
                                                     'close',
                                                     filter))},
                    {'title':current.T('Open'),
                     'params':"{url:'%s'}"%URL(r=current.request,
                                               c='project',
                                               f=self.type, 
                                               args=(name,
                                                     'open',
                                                     filter))}]
        
        return (title,params)
       
    def menu(self):
        # Get numbers
        numbers=self.parent.menu_numbers()
        name=current._user.project.name
        pid=current._user.project.id
        
        # Get data from session
        curr_sess = current.session[self.type]
        # Session type
        if curr_sess.type:
            if curr_sess.type=='open':
                type='open'
            elif curr_sess.type=='close':
                type='close'
            else:
                type='all'
        else:
            type='all'
            
        return (type,numbers)

