# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import datetime

from gluon import *

from project import ProjectHandler


class VisualResume(object):
    def __init__(self, project):
        self.project = ProjectHandler.prepare_resume(project, True)
        self.title = current.T('Project Card')
    
    def pdf(self):        
        from project.writers import MyFPDF
        self.writer=MyFPDF("%s\n%s\n%s"%(self.project.name, self.title, ''))
        self.writer.alias_nb_pages()
         
        # Add page        
        self.writer.add_page()
        
        # Title
        (x,y)=self.writer.get_xy()
        self.writer.bold_font()
        literal="[%s](%s) %s [%s]"%(self.project.private,
                                    self.project.type,
                                    self.project.title,
                                    self.project.status)
        self.writer.paragraph(10,y+5,0,5,literal,0,0)
        
        self.writer.normal_font() 
        # Date
        (x,y)=self.writer.get_xy()
        literal="%s: %s - %s: %s"%(current.T('Start Date'),
                                   self.project.start,
                                   current.T('End Date'),
                                   self.project.end)
        self.writer.paragraph(15,y+5,0,5,literal,0,0)
        
        # Description
        (x,y)=self.writer.get_xy()
        self.writer.italic_font()
        self.writer.paragraph(15,y+3,0,5,current.T('Description'),0,0)  
        self.writer.normal_font()
        for paragr in self.project.description.split('\n'):
            (x,y)=self.writer.get_xy()
            self.writer.paragraph(20,y+3,0,5,paragr,0,0)
        
        # Objetive
        (x,y)=self.writer.get_xy()
        self.writer.italic_font()
        self.writer.paragraph(15,y+3,0,5,current.T('Objetive'),0,0)  
        self.writer.normal_font()  
        for paragr in self.project.objetive.split('\n'):        
            (x,y)=self.writer.get_xy()
            self.writer.paragraph(20,y+3,0,5,paragr,0,0)
        
        # Team
        try:
            if len(self.project.team)>0:
                (x,y)=self.writer.get_xy()
                self.writer.italic_font()
                self.writer.paragraph(15,y+3,0,5,current.T('Team'),0,0)
                self.writer.normal_font()  
                for item in self.project.team:
                    (x,y)=self.writer.get_xy()
                    literal="[%s] %s (%s)"%(item.profile,item.name,item.email)
                    self.writer.paragraph(25,y+3,0,5,literal,0,0)
        except:
            pass             
            
        # prepare PDF to download:
        current.response.headers['Content-Type'] = 'application/pdf'
        current.response.headers['Content-Disposition'] = \
                        "attachment; filename=%s.pdf"%self.project.name
        return self.writer.output(dest='S')
    
    
    def txt(self):       
        from project.writers import MyTEXTER   
        self.writer=MyTEXTER(self.project.name)
        self.writer.header(self.project.name,self.title)
         
        self.writer.nl()        
        # Title
        literal="[%s](%s) %s [%s]"%(self.project.private,
                                    self.project.type,
                                    self.project.title,
                                    self.project.status)
        self.writer.line(len(literal)+1)
        self.writer.paragraph(literal)
        self.writer.line(len(literal)+1)

        # Date
        literal="%s: %s - %s: %s"%(current.T('Start Date'),
                                   self.project.start,
                                   current.T('End Date'),
                                   self.project.end)
        self.writer.paragraph(literal)
        
        # Description
        self.writer.paragraph(current.T('Description'))  
        self.writer.line(len(current.T('Description'))+1)
        self.writer.paragraph(self.writer.encode(self.project.description))
        
        # Objetive
        self.writer.paragraph(current.T('Objetive'))  
        self.writer.line(len(current.T('Objetive'))+1)
        self.writer.paragraph(self.writer.encode(self.project.objetive))
        
        # Team
        try:
            if len(self.project.team)>0:
                self.writer.paragraph(current.T('Team'))
                self.writer.line(len(current.T('Team'))+1)
                for item in self.project.team:
                    literal="[%s] %s (%s)"%(item.profile,item.name,item.email)
                    self.writer.paragraph(literal)
        except:
            pass             

        return self.writer.output()
    
    def rtf(self):
        from project.writers import MyRTF
        self.writer=MyRTF(self.project.name)
        self.writer.header(self.project.name,self.title)
                
        # Title
        literal="[%s](%s) %s [%s]"%(self.project.private,
                                    self.project.type,
                                    self.project.title,
                                    self.project.status)
        self.writer.paragraph(self.writer.bold(literal))

        # Date
        literal="%s: %s - %s: %s"%(current.T('Start Date'),
                                   self.project.start,
                                   current.T('End Date'),
                                   self.project.end)
        self.writer.paragraph(self.writer.italic(literal))
        
        # Description
        self.writer.paragraph(self.writer.italic(current.T('Description'))) 
        for paragr in self.project.description.split('\n'):   
            self.writer.paragraph(self.writer.encode(paragr))
        
        # Objetive
        self.writer.paragraph(self.writer.italic(current.T('Objetive')))  
        for paragr in self.project.objetive.split('\n'):  
            self.writer.paragraph(self.writer.encode(paragr))
        
        # Team
        try:
            if len(self.project.team)>0:
                self.writer.paragraph(self.writer.italic(current.T('Team')))
                for item in self.project.team:
                    literal="[%s] %s (%s)"%(self.writer.encode(item.profile),
                                            self.writer.encode(item.name),
                                            item.email)
                    self.writer.paragraph(literal)
        except:
            pass             

        return self.writer.output()