# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import datetime

from gluon import *

from project import ProjectHandler

from project.visual.element_roadmap import RoadmapElement

from project.visual.visual_base import VisualBase

class VisualRoadmap(VisualBase):
    def __init__(self):
        self.module = 'roadmap'
        self.id = current._user.project.id
        self.db = current.db
                
        #self.ident = ProjectHandler.get_identifier(self.id)
        self.ident = current._user.project.ident
        self.parent = RoadmapElement(self.ident)  
          
    def get_query(self):
        query=self.parent.default_query(self.module)
        try:  
            if session[self.module+'_type']:           
                if session[self.module+'_type']=='open':
                    query = query & (self.db.roadmap.close_date==None)
                elif session[self.module+'_type']=='close':        
                    query =  query & (self.db.roadmap.close_date!=None)
        except:
            pass
    
        flag_type=True
        try:
            if session[self.module+'_filter']:
                if session[self.module+'_filter']=='my':
                    if session[self.module+'_range']!='created':
                        query=query & (self.db.roadmap.responsible==current._user.email)
                    flag_type=False  
        except:
            pass
        
        try:
            if session[self.module+'_type']:
                if flag_type:
                    if session[self.module+'_type']=='open':
                        query = query & (self.db.roadmap.close_date==None)
                    elif session[self.module+'_type']=='close':        
                        query =  query & (self.db.roadmap.close_date!=None)
        except:
            pass
        
        # Date range
        from datetime import datetime,timedelta
        today=datetime.today()
        lapse_1w=today-timedelta(days=7)
        lapse_2w=today-timedelta(days=14)
        lapse_1m=today-timedelta(days=30)
        
        try:
            if session[self.module+'_range']:
                if session[self.module+'_range']=='oneweek':            
                    query=query & (self.db.roadmap.start_date>=lapse_1w)
                elif session[self.module+'_range']=='twoweeks':      
                    query=query & (self.db.roadmap.start_date>=lapse_2w)
                elif session[self.module+'_range']=='onemonth':
                    query=query & (self.db.roadmap.start_date>=lapse_1m)
                elif session[self.module+'_range']=='older':
                    query=query & (self.db.roadmap.start_date<=lapse_1m)
                elif session[self.module+'_range']=='assigned':
                    query=query & (self.db.roadmap.responsible!=None)
                elif session[self.module+'_range']=='created':
                    query=query & (self.db.roadmap.creator==current._user.email)
                elif session[self.module+'_range']=='unassigned':
                    query=query & (self.db.roadmap.responsible==None)
        except:
            pass
        
        self.parent.set_query(query)
           
    def get_data(self,child_id=None): 
        # Prepare query
        self.get_query()
        
        # Prepare number of rows selected
        try:       
           if session[self.module+'_range']=='last10':
                last_10=True 
        except:
            last_10=False
      
        if last_10:
            rows = self.parent.get_children(limit=(0,10))
        else:
            rows = self.parent.get_children(child=child_id)
        
        list = []
        for row in rows:
            list.append(self.prepare_data_child(row))  
            
        return list 
    
    def prepare_data_child(self,child):
        child['config'] = self._configure_element(child)
        child_element = RoadmapElement(child.identifier)
        level_child = child.level + 1
#        child_element.set_query(self.query) # WARN this assigment fails
        children = child_element.get_children(level=level_child)
        child['children']=[]
        for item in children:
            child['children'].append(self.prepare_data_child(item))
        return child
    
    def _configure_element(self,f): 
        params={'id': 'mm_add',
                'pid': f.pid,
                'sid': f.sid,
                'pname': current._user.project.name,
                'tid': f.id,
                'ident': f.identifier,
                'priority': f.priority,
                'callback': URL(r=current.request, 
                                c=current.request.controller, 
                                f=self.module),
                'modal': URL(r=current.request, 
                             c=current.request.controller, 
                             f='%s.load'%self.module,
                             args=['modal',current._user.project.name]),
                'delete': "delete_%s"%self.module,
                'close': "close_%s"%self.module}
        configuration="{div_id:'%(id)s', sid:'%(sid)s', pid:'%(pid)s', pname:'%(pname)s', tid:'%(tid)s', id:'%(ident)s', priority:'%(priority)s', callback:'%(callback)s', modal:'%(modal)s', delete_item:'%(delete)s', close_item:'%(close)s'}"%params
        return configuration
              
    def sort(self,vars):
        self.parent.sort(vars,'roadmap')
        
    def edit_parent(self,data):
        return self.parent.edit(data)
        
    def add_child(self,identifier):
        return self.parent.insert_child(identifier,'roadmap')
                
    def add_db_parameters(self):
        # Default values
        if current._user.id is not None:
            self.db.roadmap.creator.default=current._user.email
            self.db.roadmap.creator.writable=False
        self.db.roadmap.status.default=0
        self.db.roadmap.update_date.default=datetime.datetime.now()
        self.db.roadmap.update_date.writable=False
        self.db.roadmap.update_date.readable=False
        self.db.roadmap.responsible.writable=False
        self.db.roadmap.responsible.readable=False
        self.db.roadmap.close_date.writable=False
        self.db.roadmap.close_date.readable=False
        self.db.roadmap.status.writable=False
        self.db.roadmap.status.readable=False
        self.db.roadmap.priority.writable=False
        self.db.roadmap.priority.readable=False
        self.db.roadmap.priority.default=1
        self.db.roadmap.init_date.writable=True
        self.db.roadmap.init_date.readable=True
        self.db.roadmap.end_date.writable=True
        self.db.roadmap.end_date.readable=True
        self.db.roadmap.project.default=self.ident
        self.db.roadmap.project.writable=False
        self.db.roadmap.project.readable=False

    def edit_db_parameters(self):
        # Default values
        self.db.roadmap.id.readable=False
        self.db.roadmap.id.writable=False
        self.db.roadmap.creator.readable=False
        self.db.roadmap.creator.writable=False
        self.db.roadmap.status.default=0
        self.db.roadmap.update_date.default=datetime.datetime.now()
        self.db.roadmap.update_date.writable=False
        self.db.roadmap.update_date.readable=False
        self.db.roadmap.responsible.writable=False
        self.db.roadmap.responsible.readable=False
        self.db.roadmap.close_date.writable=False
        self.db.roadmap.close_date.readable=False
        self.db.roadmap.status.writable=False
        self.db.roadmap.status.readable=False
        self.db.roadmap.priority.writable=False
        self.db.roadmap.priority.readable=False
        self.db.roadmap.priority.default=1
        self.db.roadmap.init_date.writable=True
        self.db.roadmap.init_date.readable=True
        self.db.roadmap.end_date.writable=True
        self.db.roadmap.end_date.readable=True
        self.db.roadmap.project.writable=False
        self.db.roadmap.project.readable=False
    
    def element_by_id(self,id):
        row=self.parent.get_child_by_id(id)
        return self.box(row)