# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import os.path

from gluon import current
from gluon.languages import lazyT
from gluon.contrib.pyfpdf import FPDF, HTMLMixin  
        
class MyFPDF(FPDF):
    def __init__(self, header):
        FPDF.__init__(self)
        
        self.my_font_family='Times'
        self.my_font_size=12
        
        self.cell_h=5
        
        self.header_title=header
        
        self.alias_nb_pages()
        self.set_font(self.my_font_family,'',self.my_font_size)

    def header(self):
        (x,y)=self.get_xy()
        
        # Arial bold 15
        self.set_font(self.my_font_family,'B',13) 
        
        # Logo
        self.image(os.path.join(current._config.app_path, "static", "base",
                                "img","logo_255x40_pdf.jpg")
                   ,10,8,30)
        self.ln(2)        
        (x,y)=self.get_xy()      
        self.paragraph(x+20,y,152,5,self.header_title)
        self.ln()
        
        self.normal_font()  

    # Page footer
    def footer(self):
        # Position at 1.5 cm from bottom
        self.set_y(-15)
        # Arial italic 8
        self.set_font('Arial','I',8)
        # Page number
        self.cell(0,10,self.encode(current.T('pag')+' '+str(self.page_no())),0,0,'C')

    def normal_font(self):
        self.set_font(self.my_font_family,'',self.my_font_size)    

    def bold_font(self):
        self.set_font(self.my_font_family,'B',self.my_font_size)     

    def italic_font(self):
        self.set_font(self.my_font_family,'I',self.my_font_size)    
        
    def paragraph(self,x,y,w,h,literal,border=0,justify='C'):
        self.set_xy(x,y)
        self.multi_cell(w,h,self.encode(literal),border,justify)
    
    def encode(self,txt):
        if isinstance(txt, lazyT):
            txt=txt.read()
        return unicode(txt,'utf-8').encode('iso-8859-1') 
    
    def get_xy(self):
        return (self.get_x(),self.get_y())
    
    def _parsejpg(self, filename):
        # Extract info from a JPEG file
        from PIL import Image
        try:
            f = open(filename, 'rb')
            im = Image.open(f)
        except Exception, e:
            self.error('Missing or incorrect image file: %s. error: %s' % (filename, str(e)))
        else:
            a = im.size
        # We shouldn't get into here, as Jpeg is RGB=8bpp right(?), but, just in case...
        bpc=8
        if im.mode == 'RGB':
            colspace='DeviceRGB'
        elif im.mode == 'CMYK':
            colspace='DeviceCMYK'
        else:
            colspace='DeviceGray'

        # Read whole file from the start
        f.seek(0)
        data = f.read()
        f.close()
        return {'w':a[0],'h':a[1],'cs':colspace,'bpc':bpc,'f':'DCTDecode','data':data}