# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

# NOTA 1
# Los márgenes de las páginas deben tener en cuenta los margenes de las celdas,
# de ahí su valor: top=1000, left=1200, bottom=1000, right=1200
#
# NOTA 2
# El cálculo se realiza:
#    valor = centimetros x 0.78125
#    
# Igual que antes, la celda añade 2mm de margen a cada lado, por eso:
#    default_size es equivalente a 8.4 cm
#    minimu_size es equivalente a 0.2 cm

import os

from gluon import current
from gluon.languages import lazyT
from gluon.contrib.pyrtf import *

class MyRTF:
    def __init__(self,name=None,header=None):
        self.name=name
        
        self.doc = Document()
        self.ss = self.doc.StyleSheet
        
        #Ver NOTA1
        margin_props = MarginsPropertySet( top=1000, left=1200, 
                                           bottom=1000, right=1200 )
        self.section = Section(StandardPaper.A4, margin_props)
        
        self.doc.Sections.append( self.section )
        
        # Header      
        p = Paragraph()
        p.append(Image(os.path.join(current._config.app_path, "static", "base",
                                    "img", "logo_255x40_pdf.jpg"), 
                       scale_x=30, 
                       scale_y=30), 
                 LINE)
        self.section.Header.append(p)
        #Footer
        p = Paragraph()
        p.append( PAGE_NUMBER, ' of ', TOTAL_PAGES )
        self.section.Footer.append( p )
        
    def header(self, title, subtitle):  
        self.paragraph(self.encode(title),'Heading1')
        self.paragraph(self.encode(subtitle),'Heading2')

    def write(self,string):
        self.paragraph(string)
        
    def paragraph(self, string, style="Normal"):
        p = Paragraph(self.checkParagraphStyles(style))
        p.append(string)
        self.section.append(p)

    def normal(self, txt):
        return TEXT(self.encode(txt))

    def bold(self,txt):
        return TEXT(self.encode(txt), bold=True)  

    def italic(self,txt):
        return TEXT(self.encode(txt), italic=True)   
        
    def checkParagraphStyles(self, style):
        """Controla si el formato es uno de los validos."""
        style_names = ['Normal', 'Heading1', 'Heading2']
        if style in style_names:
            if style == 'Normal':
                ret_val = self.ss.ParagraphStyles.Normal
            if style == 'Heading1':
                ret_val = self.ss.ParagraphStyles.Heading1
            if style == 'Heading2':
                ret_val = self.ss.ParagraphStyles.Heading2   
            if style == 'Heading3':
                ret_val = self.ss.ParagraphStyles.Heading3               
        else:
            ret_val = self.ss.ParagraphStyles.Normal

        return ret_val
    
    def add_page(self):
        p = Paragraph(self.checkParagraphStyles('Normal'),
                      ParagraphPS().SetPageBreakBefore(True))
        p.append('')
        self.section.append(p)
        
    def nl(self):
        p = Paragraph(self.checkParagraphStyles('Normal'))
        p.append(LINE)
        self.section.append(p)
        
        
    def output(self,dest='S'):
        rtf_renderer = Renderer()
        
        if dest=='F':            
            if self.name is not None:
                rtf_renderer.Write(self.doc, file(self.name, 'w'))
        else:
            current.response.headers['Content-Type']='text/rtf'
            if self.name is not None:
                current.response.headers['Content-Disposition'] = \
                                  "attachment; filename=%s.rtf"%self.name;
            import cStringIO
            s=cStringIO.StringIO()
            rtf_renderer.Write(self.doc,s)
            return s.getvalue()
    
    def encode(self,txt):
        if isinstance(txt, lazyT):
            txt=txt.read()
        txt=unicode(txt,'utf-8').encode('iso-8859-1') 
        return  ''.join(['\u%s?' % str(ord(b)) for b in txt])
#        return unicode(str,'utf-8').encode('iso-8859-1') 