# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import datetime

from gluon import *
from gluon.storage import Storage

from iservice import IService

from mil.tools.common import get_unique_id

class FileModel(IService):
    _actions = ['download', 'upload']
    _controller = 'file'
    _literal = 'Files'
    _permissions = ['read', 'write']
    
    @staticmethod
    def define_table(db):
        db.define_table('file',
            Field('identifier', 'string', length=20, default=get_unique_id(2,4), 
                  notnull=True, label=current.T('Identifier'), writable=False, 
                  readable=False),
            Field('name',requires=IS_NOT_EMPTY(),label=current.T('Name')),
            Field('creator', 'string', length=200, default='', notnull=True, 
                  label=current.T('Creator')),
            Field('created','datetime',default=datetime.datetime.today()),
            Field('deleted','datetime',default=None),
            Field('file_name','string',label=current.T('File')),
            Field('file_format','string',label=current.T('Extension')),
            Field('file_size','integer',label=current.T('Size')),
            Field('file_content','upload',autodelete=True,
                  label=current.T('Content'))
        )
        
    def __init__(self, db):
        self.db = db
        
        FileModel.define_table(db)
        
    def grid(self):
        form = SQLFORM.grid(self.db.file, 
                            searchable=False,
                            csv=False, 
                            user_signature = False,
                            maxtextlength=100)
        return form
    
    def get_all(self):
         # Get files
        query = (self.db.file.id > 0)
        rows = self.db(query).select(self.db.file.identifier,
                                     self.db.file.name,
                                     self.db.file.file_format)
        file_list=[]
        for row in rows:
            tmp=Storage()
            tmp.identifier=row.identifier
            tmp.name=row.name
            file_list.append(tmp)
        return file_list
    
    def get_load_form(self):
        form = SQLFORM(self.db.file, fields=['name', 'file_content'])
        
        if current.request.vars.file_content != None:
            form.vars.creator = current._user.identifier
            form.vars.file_name = current.request.vars.file_content.filename
            
        if form.accepts(current.request.vars, current.session):
            current.session.flash = 'File loaded'
            redirect(URL(r=current.request,c='file',f='index'))
        elif form.errors:
            current.response.flash = 'Error'
            
        return form
    
    def get_file(self, file):
        query = (self.db.file.identifier==file)
        return self.db(query).select(self.db.file.deleted,
                                     self.db.file.file_content,
                                     self.db.file.file_name,
                                     self.db.file.file_format).first()