# -*- coding: utf-8 -*- 

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import current
from gluon.storage import Storage

from mil.core.local_import import local_import

from core.configuration import get_configuration

class ServiceManager(object):    
    def __init__(self, system=False):
        if system:
            self.services = None
            self.system = get_configuration()
        else:
            self.services = self._load_services() 
            self.system = None
        
    def get(self, id):
        return self.services[id]
    
    def get_all_as_dict(self, auth_mng):
        ret_val = []
        for id in self.services:
            ret_val.append(self.get_as_dict(id, auth_mng))
        return ret_val
    
    def get_as_dict(self, id, auth_mng):
        all_data = self.services[id] 
        data = {}
        data['id'] = id
        data['controller'] = all_data['controller']
        data['literal'] = all_data['literal']
        data['permissions'] = all_data['permissions']
        data['configuration'] = []
        for action in all_data['configuration']:
            d_action = {}
            d_action['id'] = action['id']
            d_action['label'] = action['label'] 
            d_action['options'] = []
            value = auth_mng.get_value(id, action['id'])
            for option in action['options']:
                d_option = option 
                d_option['checked'] = False
                if value is None:
                    if option['default']:
                        d_option['checked'] = True
                else:
                    if option['id'] == value:
                        d_option['checked'] = True
                d_action['options'].append(d_option)
            data['configuration'].append(d_action)
        data['services'] = all_data['services']
        return data
        
    def _load_services(self):
        services  = {}
        try:
            for subapp in current._config.get('subapp'):
                mp = "applications.%s.modules.%s" % (current.request.application, 
                                                     subapp)
                klass_name = "%sInstance" % subapp.capitalize()
                try:
                    mod = __import__(mp, fromlist=[klass_name])
                    klass = getattr(mod, klass_name)
                    services[subapp] = klass().get_configuration()
                except Exception, e:
                    current._syslog.error('ServiceManager: %s' % str(e), 
                                  'warning')
        except:
            pass
        
        return services
      