# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import *

class AuthConfiguration(object):
    @staticmethod
    def define_table(db):
		if 'auth_conf' in db.tables:
			return
		
		db.define_table('auth_conf',
                        Field('application', 'string', length=200, required=True, 
                              notnull=True, label=current.T('Application')),
                        Field('module', 'string', length=50, required=True, 
                              notnull=True, label=current.T('Module')),
                        Field('role', 'integer', default=0,
                              label=current.T('Role')),
                        Field('action', 'string', length=50, required=True, 
                              notnull=True, label=current.T('Title')),
                        Field('value', 'string', length=50, required=False, 
                              notnull=False, label=current.T('Name')),
                        format='%(application)s -%(module)s - %(action)s : %(value)s')       
        
    def __init__(self, db, role=None):
        self.db = db
        self.role = role
        
        AuthConfiguration.define_table(db)
        
    def get_all(self):
        if self.role is None:
            return  self.db(self.db.auth_conf.id > 0).select()
        else:
            return  self.db(self.db.auth_conf.role==self.role).select()
    
    def get(self, application, module, action):
        query = ((self.db.auth_conf.application==application) & \
                 (self.db.auth_conf.module==module) & \
                 (self.db.auth_conf.action==action))
        if self.role:
            query = query&(self.db.auth_conf.role==self.role)
        return self.db(query).select().first()
    
    def get_value(self, application, module, action, value=None):
        query = ((self.db.auth_conf.application==application) & \
                 (self.db.auth_conf.module==module) & \
                 (self.db.auth_conf.action==action))
        if value is not None:
            query = query&(self.db.auth_conf.value==value)
        if self.role:
            query = query&(self.db.auth_conf.role==self.role)
        
        try:
            return self.db(query).select().first().value
        except:
            return self._get_value(application, module, action, value)
    
    def _get_value(self, application, module, action, value=None):
        query = ((self.db.auth_conf.application==application) & \
                 (self.db.auth_conf.module==module) & \
                 (self.db.auth_conf.action==action))
        if value is not None:
            query = query&(self.db.auth_conf.value==value)
        query = query&(self.db.auth_conf.role==0)
        try:
            return self.db(query).select().first().value
        except:
            return None
    
    def add(self, application, module, action, value):
        if self.role:
            role = self.role
        else:
            role = 0
            
        query = ((self.db.auth_conf.application==application) & \
                 (self.db.auth_conf.module==module) & \
                 (self.db.auth_conf.action==action) & \
                 (self.db.auth_conf.role==role))
        self.db.auth_conf.update_or_insert(query,
                                           application=application,
                                           module=module, 
                                           role=role,
                                           action=action, 
                                           value=value)
    
    def delete(self, id):
        try:
            query = (self.db.auth_conf.module==id)
            if self.role:
                query = query&(self.db.auth_conf.role==self.role)
            self.db(query).delete()
        except Exception, e:
            current._syslog.log(str(e))
            pass
        
    def delete_by_all(self, application, module, action):
        try:
            query = (self.db.auth_conf.application==application) & \
                    (self.db.auth_conf.module==module) & \
                    (self.db.auth_conf.action==action)
            if self.role:
                query = query&(self.db.auth_conf.role==self.role)
            self.db(query).delete()
        except Exception, e:
            current._syslog.log(str(e))
            pass
        
    def delete_by_profile(self):
        try:
            if self.role:
                self.db(self.db.auth_conf.role==self.role).delete()
        except Exception, e:
            current._syslog.log(str(e))
            pass
        
    def export_data(self):
        return str(self.db(self.db.auth_conf.id).select())
    
    def import_data(self, file_data):
        # Clean table
        self.db.auth_conf.truncate()
        # Import data
        self.db.auth_conf.import_from_csv_file(file_data)

    def get_by_application_and_role(self, application, role):
        query = ((self.db.auth_conf.application==application) & \
                 (self.db.auth_conf.role==role))
        return  self.db(query).select()
    