# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import *

def _decode_list(lst):
    newlist = []
    for i in lst:
        if isinstance(i, unicode):
            i = i.encode('utf-8')
        elif isinstance(i, list):
            i = _decode_list(i)
        newlist.append(i)
    return newlist

def _decode_dict(dct):
    newdict = {}
    for k, v in dct.iteritems():
        if isinstance(k, unicode):
            k = k.encode('utf-8')
        if isinstance(v, unicode):
             v = v.encode('utf-8')
        elif isinstance(v, list):
            v = _decode_list(v)
        newdict[k] = v
    return newdict

class DashBoard(object):
    def __init__(self, db):
        self.db = db
        
    def define_table(self):
        self.db.define_table('dashboard',
                             Field('user_identifier', 'string', length=250),
                             Field('dashboard_boxes', 'text'))

    def get_new_boxes_list(self, new_boxes, user):
        from gluon.contrib import simplejson as json
        query = (self.db.dashboard.user_identifier == user.identifier)
        row = self.db(query).select(self.db.dashboard.ALL).first()
    
        item = json.loads(row["dashboard_boxes"], object_hook=_decode_dict)
        
        center_column =item["center_column"]
        right_column =item["right_column"]
        
        user_boxes = []
        
        new_right_column = []
        for item in right_column:
            if item["box"] in new_boxes:
                user_boxes.append(item["box"])
                new_right_column.append(item) 
        del(right_column)
        
        new_center_column = []
        for item in center_column:
            if item["box"] in new_boxes:
                user_boxes.append(item["box"])
                new_center_column.append(item)
        
        del(center_column)
            
        for item in new_boxes:
            if item not in user_boxes:
                new_center_column.insert(0, {"box": item, "min_max": "max"})
                
                
        new_data = dict(center_column = new_center_column, 
                        right_column = new_right_column)
        return json.dumps(new_data)
