# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import current, URL, redirect
from gluon.storage import Storage

from mil.base import File

class Environment(object):
    def __init__(self):
        self.subapp = []
        self.current = None
        
        self.settings = Storage()
        self.settings.controller = 'default'
        self.settings.default = 'public'
        self.settings.on_failed_authorization = \
                                        self.url('user',args='not_authorized')
        self.messages = Storage()                                     
        self.messages.access_denied = current.T('Not in current application')
        
        self.add(self.settings.default)
        
    def url(self, f=None, args=None, vars=None):
        if args is None: 
            args=[]
        if vars is None: 
            vars={}
        return URL(c=self.settings.controller, f=f, args=args, vars=vars)
    
    def add(self, subapp):
        if subapp in self.subapp:
            self.subapp.append(subapp)
        
    def set(self, subapp=None):
        if subapp is None:
            subapp = self.settings.default
        if not subapp in self.subapp:
            self.current = subapp
            current.session.environ = subapp
            
    def is_in(self, subapp):
        if subapp == self.current:
            return True
        else:
            if subapp == self.settings.default:
                return True
            else:
                return False
        
    def is_in_app(self, application):
        """
        decorator that prevents access to action if not in current app
        """
        def wrapper(action,self=self):
            def f(_action=action,_self=self,*a,**b):
                if not self.is_in(application):
                    self.set()
                    current.session.flash = self.messages.access_denied
                    return redirect(self.settings.on_failed_authorization)
                return _action()
            f.__doc__ = action.__doc__
            f.__name__ = action.__name__
            return f
        return wrapper
        
        
    @staticmethod
    def get_dal_ddbb(name):
        ddbb_name = "%s%s%s" % (current._config.get('db_prefix'), 
                                name,
                                current._config.get('db_sufix'))
        return "%s%s" % (current._config.get('db_dal_base'),
                         ddbb_name)
        
    def get_ddbb_name(name):
        ddbb_name = "%s%s%s" % (current._config.get('db_prefix'), 
                                name,
                                current._config.get('db_sufix'))
        
    @staticmethod
    def get_table_name(name):
        return "%s%s%s" % (current._config.get('db_table_prefix'), 
                           name,
                           current._config.get('db_table_sufix'))
        
    @staticmethod
    def create_ddbb(name):
        # ddbb name 
        ddbb_name = Environment.get_ddbb_name(name)
        
        # Create ddbb
        if current._config.get('db_type')!='sqlite':
            try:
                query = 'CREATE DATABASE %s' % ddbb_name
                current.db.executesql(query)
            except Exception, e:
                raise e
    
        # Connect to new ddbb
        db_tmp= DAL("%s%s"%(current._config.get('db_dal_base'), ddbb_name))

        # Close connection
        del(db_tmp)
        
        # returns ddbb name
        return ddbb_name
    
#    @staticmethod
#    def create_filesystem(name):
#        try:
#            dir_name=os.path.join(APP_PATH,'private','data','project','grupo_%i'%name)
#            File.make_dir(dir_name)
#            File.make_dir(os.path.join(dir_name,'backups'))
#        except Exception, e:
#            raise e