# -*- coding: utf-8 -*- 

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import current
from mil.system import INotifierListener

def run_flow(name):
    from masters.workflows import WorkFlows
    from mil.statemachine import SMEngine, SMFlow
    
    try:
        flow_data = WorkFlows(current.db).get(name)
        wflow = SMFlow(name)
        wflow.fromString(flow_data.content)    
            
        # Initialize sm
        rel_path = "applications.%s.modules.workflow.%s" % (current.request.application,
                                                            name)
        sm_engine = SMEngine(wflow, rel_path)
        
        # Run StateMachine engine
        sm_engine.run()
    except Exception, e:
        pass

def run_instance(name, identifier):
    from masters.workflows import WorkFlows
    from mil.statemachine import SMEngine, SMFlow, SMInstance, END_OF_FLOW
    
    try:
        flow_data = WorkFlows(current.db).get(name)
        if flow_data is None:
            return '<sm flow unknown/>'            
        wflow = SMFlow(name)
        wflow.fromString(flow_data.content)         
            
        # Initialize sm
        rel_path = "applications.%s.modules.workflow.%s" % (current.request.application,
                                                            name)
        sm_engine = SMEngine(wflow, rel_path)
        
        # Handler
        from system import SMHandler
        db_handler = SMHandler(db=current.db, table='sm_instance')
        
        sm_instance = SMInstance(sm_engine, 
                                 identifier=identifier,
                                 handler=db_handler)
        # Run StateMachine engine
        value = sm_instance.run()
        if value == END_OF_FLOW:
            return '<sm delete/>'
        else:
            return sm_instance.identifier
    except Exception, e:
        pass
    
def run_flow_2(name):
    from masters.workflows import WorkFlows
    from mil.statemachine import SMWorkFlow
    
    try:
        flow_data = WorkFlows(current.db).get(name)
        wflow = SMWorkFlow(flow_data.content, current)
        
        # Run StateMachine Workflow engine
        wflow.run()
    except Exception, e:
        pass
    
def run_instance_2(name, identifier):
    from masters.workflows import WorkFlows
    from mil.statemachine import SMWorkFlow, SMInstance, END_OF_FLOW
    
    try:
        flow_data = WorkFlows(current.db).get(name)
        if flow_data is None:
            return '<sm flow unknown/>'   
        wflow = SMWorkFlow(flow_data.content, current)  
        
        # Handler
        from system import SMHandler
        db_handler = SMHandler(db=current.db, table='sm_instance')
        
        sm_instance = SMInstance(wflow, 
                                 identifier=identifier,
                                 handler=db_handler)
        # Run StateMachine engine
        value = sm_instance.run()
        if value == END_OF_FLOW:
            return '<sm delete/>'
        else:
            return sm_instance.identifier
    except Exception, e:
        raise e
        pass

class Flow(INotifierListener):
    @staticmethod
    def receipt(data):
        try:
            if data.event == 'flow':
                # Workflow validation
                from masters.workflows import WorkFlows
                flow_data = WorkFlows(current.db).get(data.message.flow)
                if flow_data is None:
                    return
                    #raise KeyError(data.message.flow)
                
                # Prepare workflow instance
                from system import FlowInstance
                fi = FlowInstance(current.db)
                fi.add(data.message.user, 
                       flow_data.identifier, 
                       data.message.data['status'])
        except Exception, e:
            current._syslog.error(e, 'error')
            
            