# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import *

from mil.tools.common import get_unique_id


class FlowInstance:
    @staticmethod
    def define_table(db):
        # Check if table is handled
        if 'flow_instance' in db.tables:
            return
        
        db.define_table('flow_instance',
            Field('identifier', 'string', length=20, 
              default=get_unique_id(2,4), required=False, 
              label=current.T('Identifier')),
            Field('user', 'string', length=36, 
                  label=current.T('User')),
            Field('flow', 'string', length=20, 
                  label=current.T('Flow')),
            Field('active', 'boolean', default=False,
                  label=current.T('Active')),
            Field('waiting', 'boolean', default=False,
                  label=current.T('Waiting')),
            format='')
            
    def __init__(self, db):
        self.db = db
        FlowInstance.define_table(db)
        
    def grid(self):
        form = SQLFORM.grid(self.db.flow_instance, 
                            searchable=False,
                            csv=False, 
                            user_signature = False,
                            maxtextlength=100)
        return form
    
    def get_all(self):
        return self.db(self.db.flow_instance.id > 0).select()
    
    def get_all_user(self, user):
        return self.db(self.db.flow_instance.user == user).select() or []
    
    def get_all_user_active(self, user):
        query = ((self.db.flow_instance.user == user) &
                 (self.db.flow_instance.active == True))
        return self.db(query).select() or []
    
    def get_flow_user_active(self, user):
        query = ((self.db.flow_instance.user == user) &
                 (self.db.flow_instance.active == True) &
                 (self.db.flow_instance.waiting == False))
        return self.db(query).select().first() or None
    
    def get_user_flow(self, user, flow):
        query = ((self.db.flow_instance.user == user) & \
                 (self.db.flow_instance.flow == flow))
        return self.db(query).select().first()
    
    def delete(self, user, flow):
        query = ((self.db.flow_instance.user == user) & 
                 (self.db.flow_instance.flow == flow))
        return self.db(query).delete()
    
    def update(self, user, flow, status):
        query = ((self.db.flow_instance.user == user) & 
                 (self.db.flow_instance.flow == flow))
        return self.db(query).update(active=status)
    
    def add(self, user, flow, status):
        return self.db.flow_instance.insert(user=user, 
                                            flow=flow, 
                                            active=status)
        
    def activate(self, id, status=True):
        query = (self.db.flow_instance.identifier == id)
        return self.db(query).update(active=status)
    