# -*- coding: utf-8 -*- 

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import yaml
import datetime

from gluon import *

from mil.handler import DBTableHandler

class SMHandler(DBTableHandler):    
    def define_table(self, db): 
        if not self.table_name in db.tables:
            db.define_table(self.table_name,
                Field('identifier', 'string', length=20, 
                      label=current.T('Identifier')),
                Field('data', 'text', required=True, 
                      label=current.T('Data')),
                Field('timestamp', 'datetime', label=current.T('Timestamp'), 
                      writable=False, readable=True, 
                      default=datetime.datetime.now(),
                      requires=IS_DATE(format=current.T("%Y-%m-%d %H:%M:%S"))), 
                format='%(value)s') 
            
    def _get_one(self, id):
        query = (self.db[self.table_name].identifier==id)
        row = self.db(query).select().last()    
        if row is None:
            return None
        else:
            return yaml.load(row.data)
    
    def _get_all(self):
        return None
    
    def insert(self, id, data):
        self._insert_or_update(id, data)
        
    def update(self, id, data):
        self._insert_or_update(id, data)
        
    def _insert_or_update(self, id, data):
        yaml_data = yaml.dump(data, default_flow_style=False)
        self.db[self.table_name].update_or_insert(dict(identifier=id),
                                                  identifier=id,
                                                  data=yaml_data)
        
    def delete(self, id):
        query = (self.db[self.table_name].identifier==id)
        row = self.db(query).delete()
 