# -*- coding: utf-8 -*-

# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import os.path
from gluon import current

import pysvn

from mil.repos.core.svn import Repository

class SvnHandler(object):
    def __init__(self, server, path='', port='80', realm=''):
        
        # Webdav connection
        if path != '':
            self.main_url = server + '/' + path
        else:
            self.main_url = server
            
        self.main_path = os.path.join(current._config.get('filesystem_path'),
                                      'svn')
            
        self._username = None
        self._passwd = None
        
    def set_auth(self, user, passwd):
        self._username = user
        self._passwd = passwd
            
    def __set_connection(self):          
        try:   
            if self._username is None:
                return False
                 
            self.client = pysvn.Client() 
            self.client.set_default_username(self._username)
            self.client.set_default_password(self._passwd)
            return True
        except Exception, e:
            return False
        
    def add_repo(self, name):        
        # Create repository
        try:
            repo = Repository(os.path.join(self.main_path, name))
            repo.create()
        except Exception, e:
            raise Exception("(SvnHandler:add_repo) %s" % str(e))
    
    def del_repo(self, name):      
        # Delete repository
        try:
            repo = Repository(os.path.join(self.main_path, name))
            repo.delete()
        except Exception, e:
            raise Exception("(SvnHandler:del_repo) %s" % str(e))
                    
#    def mkdir(self, name):
#        try:            
#            path = "%s/%s" % (self.main_url, name)  
#            if self.__set_connection():        
#                self.client.mkdir(path, 'Added project')
#        except Exception, e:
#            return False
#        
#    def rmdir(self, name):
#        try:            
#            path = "%s/%s" % (self.main_url, name)  
#            if self.__set_connection():        
#                def get_log_message():
#                    return True, 'Removed project'
#                self.client.callback_get_log_message = get_log_message                        
#                self.client.remove(path, force=True)            
#        except Exception, e:
#            return False
        
#    def assign_user_as_writer(self, name, user):
#        from system import AuthZHandler
#        auhtzHdl = AuthZHandler(current._config.get('authz_file'))
#        auhtzHdl.add_repo(name, user, 'rw')
#        
#    def assign_user_as_reader(self, name, user):
#        from system import AuthZHandler
#        auhtzHdl = AuthZHandler(current._config.get('authz_file'))
#        auhtzHdl.add_repo(name, user, 'r')
#        
#    def remove_auth(self, name):
#        from system import AuthZHandler
#        auhtzHdl = AuthZHandler(current._config.get('authz_file'))
#        auhtzHdl.del_repo(name)
        