# -*- coding: utf-8 -*-

# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

import sys

from gluon import current

from pywebdav.lib.dbconn import Mconn

from pywebdav.server.fileauth import DAVAuthHandler

class W2PAuthHandler(DAVAuthHandler):
    """
    Provides authentication based on a web2py application
    """
    w2p_auth = None

    def get_userinfo(self, user, pw, command):
        """ Authenticate user """

        # Commands that need write access
        nowrite=['OPTIONS','PROPFIND','GET']
        
        # Main dav route
        main_route = self.path.split('/')[1]
        if self.w2p_auth:
            access = self.w2p_auth.external_login(user, pw, main_route)
            if access:
                if access.read is False:
                    self._log('Authentication failed for user %s' % user)
                    return 0
                                
                can_write = access.write
                if not can_write and not command in nowrite:
                    self._log('Authentication failed for user %s using command %s' %(user,command))
                    return 0
                else:
                    self._log('Successfully authenticated user %s writable=%s' % (user,can_write))
                    return 1

            self._log('Authentication failed for user %s' % user)
            return 0
        
        self._log('Not defined Authentication system. Access locked ')
        return 0
    