# -*- coding: utf-8 -*-

# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import current

from system.webdav_client import WebdavClient

import python_webdav.connection as conn

class WebdavHandler(object):
    def __init__(self, server, path='', port='80', realm=''):
        
        # Webdav connection
        if path != '':
            self.main_path = server + '/' + path
        else:
            self.main_path = server
        self._connection_settings = dict(host=self.main_path,
                                         path=path,
                                         port=port, 
                                         realm=realm,
                                         username='',
                                         password='')   
        
    def set_auth(self, user, passwd):
        self._connection_settings['username'] = user
        self._connection_settings['password'] = passwd
        
    def __set_connection(self, path=None):          
        try:
            if self._connection_settings['username'] == '':
                return False
            
            if path is not None:
                self._connection_settings['path'] = path
                
            self.connection = conn.Connection(self._connection_settings)
            self.client = conn.Client()
            return True
        except Exception, e:
            return False
                    
    def mkdir(self, name):
        try:            
            path = "%s/%s" % (self.main_path, name)  
            if self.__set_connection():  
                response,content = self.connection.send_mkcol(path)
                if response['status'] != '200':
                    if response['status'] == '401':
                        raise Exception('Authentication required')   
                    if response['status'] == '405':
                        raise Exception('Method Not Allowed')          
        except Exception, e:
            return False
        
    def rmdir(self, name):
        try:            
            path = "%s/%s/" % (self.main_path, name)  
            if self.__set_connection():  
                response,content = self.connection.send_delete(path)
                if response['status'] != '200':
                    if response['status'] == '401':
                        self.response['error'] = 'Authentication required'   
                    if response['status'] == '405':
                        raise Exception('Method Not Allowed')                   
        except Exception, e:
            return False