# -*- coding: utf-8 -*- 

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import current, redirect, URL
from gluon.storage import Storage

from mil.statemachine import SMWorkFlow, SMInstance, END_OF_FLOW

from system import SMHandler, FlowInstance

from masters.workflows import WorkFlows

def workflow_redirect(values):
    # Checks if a workflow is active
    if current.session.workflow is not None:   
        try:     
            # Assign new step in active flow
            WorkFlowRunner(current.auth.user.identifier, 
                           current.session.workflow.id).next_step(current.session.workflow.next_node,
                                                                  values)
            
            # Calls to active flow
            path = URL(c='flow', 
                       f='execute', 
                       args=(current.session.workflow.id))
    
            redirect(path)
        except:
            pass
    
class WorkFlowRunner(object):
    def __init__(self, user_ident, flow_id):        
        # Data handler
        self.handler = SMHandler(db=current.db, table='sm_instance')
        
        # Checks if an instance exists
        self.fins_handler = FlowInstance(current.db)
        self.fins = self.fins_handler.get_user_flow(user_ident, flow_id)
        
        if self.fins is None:
            raise Exception('Unknown flow')
        
    def run(self): 
        try:
            from gluon import current
            # Activate workflow instance   
            if self.fins.active is False:
                self.fins_handler.activate(self.fins.identifier)
            
            # Get workflow data
            flow_data = WorkFlows(current.db).get_by_ident(self.fins.flow)
            if flow_data is None:
                return ('<sm flow unknown/>', None)  
            # WorkFlow 
            wflow = SMWorkFlow(flow_data.content, current)  
            
            # Workflow instance
            sm_instance = SMInstance(wflow, 
                                     identifier=self.fins.identifier,
                                     handler=self.handler)
            
            current._syslog.error('WorkFlowRunner: Run SM engine', 'info')
            # Run StateMachine engine
            (next_step, page) = sm_instance.run()
            
            current._syslog.error('WorkFlowRunner: next step', 'info')
            if next_step is False:
                self.fins_handler.delete(self.fins.user, self.fins.flow)
            else:
                self.fins_handler.update(self.fins.user, self.fins.flow, 
                                         next_step)            

            if page is None:
                # WorkFlow continues
                self.run()
                
            # Session control system
            # See models/flow.py
            if next_step == END_OF_FLOW or next_step is False:
                wf_session = None
            else:
                wf_session = Storage(id=self.fins.flow, 
                                     node=sm_instance.step,
                                     next_node=sm_instance.step,
                                     page=page)
            current.session.workflow = wf_session
                
            redirect(page)
        except Exception, e:
            current._syslog.error('WorkFlowRunner.run: %s' % str(e), 'warning')
            raise e
        
    def next_step(self, step, values=None): 
        try:               
            # Get workflow data
            flow_data = WorkFlows(current.db).get_by_ident(self.fins.flow)
            if flow_data is None:
                return ('<sm flow unknown/>', None)  
            # WorkFlow 
            wflow = SMWorkFlow(flow_data.content, current)  
            
            # Workflow instance
            sm_instance = SMInstance(wflow, 
                                     identifier=self.fins.identifier,
                                     handler=self.handler)
            # New step in StateMachine engine
            sm_instance.update_step(step, values)
        except Exception, e:
            current._syslog.error('WorkFlowRunner.next_step: %s' % stre(e), 
                                  'info')
            raise e
        
class FlowRunner(object):
    def __init__(self, flow_id): 
        self.flow = flow_id
        
        # Get workflow data
        self.flow_data = None
        flow_data = WorkFlows(current.db).get_by_name(self.flow)
        if flow_data:
            self.flow_data= flow_data.content
            
        # Session control
        if current.session.running_flow is None:
            current.session.running_flow = self.flow
            self.first = True
        else:
            self.first = False
            
    def run(self): 
        try:
            # Session control
            if self.first:            
                if self.flow_data:            
                    # WorkFlow 
                    wflow = SMWorkFlow(self.flow_data, current) 
                    # Run StateMachine Workflow engine
                    wflow.run()
                else:
                    current.session.running_flow = None
            else:
                current.session.running_flow = None
                
        except Exception, e:
            current._syslog.error('FlowRunner: %s' % str(e), 'info')
#            raise e
            pass
  