# -*- coding: utf-8 -*-

# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'LICENSE' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

###############################################################################
# Please try to conform to PEP-8 as much as possible
# http://www.python.org/dev/peps/pep-0008/
###############################################################################

from gluon import *

class WikiPageModel(object):
    @staticmethod
    def define_table(db):        
        if 'wiki_page' in db.tables:
            return

        db.define_table('wiki_page',
                        Field('ns',writable=False,default=''),
                        Field('slug',writable=False,
                              requires=(IS_SLUG(),IS_NOT_IN_DB(db,'wiki_page.slug'))),
                        Field('title',default='',requires=(IS_NOT_EMPTY())),
                        Field('active','boolean',default=True),
                        Field('public','boolean',default=True),
                        Field('body','text',default=''),
                        Field('changelog',default=''),
                        Field('created_by',
                              db.auth_user,default=current.auth.user_id,
                              writable=False,readable=False),
                        Field('created_on','datetime',
                              default=current.request.now,
                              writable=False,readable=False),
                        Field('modified_by',
                              db.auth_user, default=current.auth.user_id,
                              update=current.auth.user_id,
                              writable=False,readable=False),
                        Field('modified_on','datetime',
                              default=current.request.now,
                              update=current.request.now,
                              writable=False, readable=False),
                        format = '%(slug)s', 
                        migrate=current._config.migrate)

    def __init__(self, db):
        self.db = db
        
        WikiPageModel.define_table(db)
        
    def get_code_page(self, ns):
        query = (self.db.wiki_page.ns == ns) & (self.db.wiki_page.slug=='meta-code')
        return self.db(query).select().first()
        
    def get_menu_page(self, ns):
        query = (self.db.wiki_page.ns == ns) & (self.db.wiki_page.slug=='meta-menu')
        return self.db(query).select().first()
        
    def get_page(self, ns, slug):
        query = (self.db.wiki_page.ns == ns) & (self.db.wiki_page.slug==slug)
        return self.db(query).select().first()
    