# coding: utf8
# Este fichero forma parte de openMyWork
#
# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'licence' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import sys
import os

#get current folder
curr_path = os.path.dirname(os.path.abspath(__file__))
sys.path = [curr_path] + sys.path
from base_auth import Debugger, Auth, get_config #, get_auth, close_auth

#get the gluon path for extending the pythonpath
gluon_path = os.path.abspath(os.path.join(curr_path,"..","..", ".."))

#get the app path for using it
app_path = os.path.abspath(os.path.dirname(curr_path))

#get the modules path for extending the pythonpath
modules_path = os.path.abspath(os.path.join(app_path,"modules"))

#extend the pythonpath with gluon and modules path
sys.path = [gluon_path, modules_path] + sys.path

#WSGI mandatory functions
#Authentication: only check if the user exists
def check_password(environ, user, password):
    # Configuration
    config = get_config(app_path)
    # Debug system
    dbg = Debugger(config['filesystem_path'])
    
    # check access
    try: 
        from gluon.storage import Storage
        settings = Storage(hmac_key=config['hmac_key'],
                           password_min_length=config['register']['min_password_length'])
        auth = Auth(config['dal'], settings)
        ret_val = auth.check_user(user, password)
    except Exception, e:
        auth = None
        ret_val = None
        dbg.set('check_password: %s' % str(e))
        
    dbg.close()
    return ret_val
 
def groups_for_user(environ, user):
    # Configuration
    config = get_config(app_path)
    # Debug system
    dbg = Debugger(config['filesystem_path'])
    
    # Get Auth
    try: 
        from gluon.storage import Storage
        settings = Storage(hmac_key=config['hmac_key'],
                           password_min_length=config['register']['min_password_length'])
        auth = Auth(config['dal'], settings)
    except Exception, e:
        dbg.set(e)
        ret_val = ['']
        
    # Users Admin has always permission
    # Checks permission    
    if auth.has_membership(user, 'admin'):
        return ['access']
    
    # Prepare search path
    request = environ['REQUEST_URI']
    method = environ['REQUEST_METHOD']
    parts = request[1:].split('/')
    root_path = parts[0]
    main_element = parts[1]
    path = '/'.join(parts[2:])
    try: 
        # Prepare module and action
        if root_path.find('dav') != -1:
            module = 'webdav'
        else:
            if root_path.find('svn') != -1:
                module = 'repository'
            else:
                return False
        
        if method in ['OPTIONS', 'PROPFIND', 'GET', 'REPORT']:
            action = '%s_access' % module
        elif method in ['MKACTIVITY', 'PROPPATCH', 'PUT', 'CHECKOUT', 'MKCOL', \
                        'MOVE', 'COPY', 'DELETE', 'LOCK', 'UNLOCK', 'MERGE']:
            action = '%s_write' % module
        else:
            return False
            
        has_perms = auth.check_project_permission(user, 
                                                  main_element, 
                                                  module, 
                                                  action)
        if has_perms:
            ret_val = ['access']
        else:
            ret_val = ['']
    except Exception, e:
        dbg.set(e)
        ret_val = ['']
    
    dbg.close()
    
    return ret_val
 
