# coding: utf8
# Este fichero forma parte de openMyWork
#
# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'licence' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import os
import cStringIO
import re

class Debugger(object):
    def __init__(self, file):
        file_path = os.path.abspath(os.path.join(file, 'logs', 'wsgi.log'))
        self.fh = open(file_path, 'a')
        
    def set(self, message):
        output = cStringIO.StringIO()
        print >> output, message
        try:
            self.fh.write(output.getvalue())
        except:
            pass
        output.close()
        
    def close(self):
        self.fh.close()
        
def get_config(app_path):
    # Config files
    static_file = os.path.join(app_path, 
                               'private', 
                               'configuration', 
                               'forge_config.yaml')
    dynamic_file = os.path.join(app_path, 
                                'private', 
                                'configuration', 
                                'config.yaml')
       
    # Configuration system
    from gluon.storage import Storage
    try:
        from mil.base import File, YAMLFile
        config = YAMLFile.load(static_file)
        config.update(YAMLFile.load(dynamic_file))
        return config
    except Exception, e:
        raise Exception('get_auth: %s' % str(e))
    
def get_mysql_params(uri):
        uri = uri.split('://')[1]
        m = re.compile('^(?P<user>[^:@]+)(\:(?P<password>[^@]*))?@(?P<host>[^\:/]+)(\:(?P<port>[0-9]+))?/(?P<db>[^?]+)(\?set_encoding=(?P<charset>\w+))?$').match(uri)
        if not m:
            raise SyntaxError, \
                "Invalid URI string in DAL: %s" % uri
        user = m.group('user')
        if not user:
            raise SyntaxError, 'User required'
        password = m.group('password')
        if not password:
            password = ''
        host = m.group('host')
        if not host:
            raise SyntaxError, 'Host name required'
        db = m.group('db')
        if not db:
            raise SyntaxError, 'Database name required'
        port = int(m.group('port') or '3306')
        charset = m.group('charset') or 'utf8'
        return dict(db=db,
                    user=user,
                    passwd=password,
                    host=host,
                    port=port,
                    charset=charset)

class Auth(object):
    def __init__(self, uri, settings):
        self.db_params = get_mysql_params(uri)
        self.settings = settings
        
    def check_user(self, username, password):
        authorized = False
        
        # Connect to mysql
        import gluon.contrib.pymysql as pymysql 
        conn = pymysql.connect(**self.db_params)
        
        # Get data
        cur = conn.cursor()
        query = "SELECT password FROM auth_user WHERE email = '%s'" % username
        cur.execute(query)
        
        data = cur.fetchall()
        if len(data) == 1:
            crypt_password = data[0][0]
            try:
                # Checks password
                from gluon import CRYPT
                crypt = CRYPT(key=self.settings.hmac_key, 
                              min_length=self.settings.password_min_length)
                if crypt_password == crypt(password)[0]:
                    authorized = True
            except Exception, e:
                raise Exception(e)
            authorized = True
        else:
            if username == 'annonymous@host.ext' and password =='annonymous':
                return True
        
        # Close connection
        cur.close()
        conn.close()

        return authorized
    
    def has_membership(self, username, role):        
        try:      
            # Connect to mysql
            import gluon.contrib.pymysql as pymysql 
            conn = pymysql.connect(**self.db_params)
            
            # Prepare cursor
            cur = conn.cursor()
            
            # Get user 
            query = "SELECT id FROM auth_user WHERE email = '%s'" % username
            cur.execute(query)
            # Get user 
            user = cur.fetchone()[0]
            
            # Get group 
            query = "SELECT id FROM auth_group WHERE role = '%s'" % role
            cur.execute(query)
            # Get user 
            try:
                group = cur.fetchone()[0]
            except:
                return False
            
            # Checks membership
            query = "SELECT id FROM auth_membership WHERE user_id = '%s' AND group_id = '%s'" % (user, group)
            cur.execute(query)
            data = cur.fetchall()
            if len(data):
                return True
            else:
                return False
        except:
                return False
        
    def check_project_permission(self, username, element, module, action):
        if element == '':
            # TODO: test if user is admin
            return self.has_membership(username, 'admin')
              
        # Connect to mysql
        import gluon.contrib.pymysql as pymysql 
        conn = pymysql.connect(**self.db_params)
        
        # Prepare cursor
        cur = conn.cursor()
        
        # Get user 
        query = "SELECT id FROM auth_user WHERE email = '%s'" % username
        cur.execute(query)
        # Get user 
        try:
            user = cur.fetchone()[0]
        except:
            user = None

        # TODO: detect type of element {project}
        query = "SELECT id, private FROM project_card WHERE name = '%s'" % element
        cur.execute(query)
        # Get user 
        try:
            project = cur.fetchone()
        except:
            return False
        
        if user is None:
            # If project is public, access granted
            if project[1] is False:
                if username == 'annonymous@host.ext':
                    return True
        
        # Get user profile in current project
        query = "SELECT profile FROM project_team WHERE user = '%s' AND project = '%s'" % (user, project[0])
        cur.execute(query)
        profile = cur.fetchone()[0]
        
        # Use Government to check permission
        try:
            query = "SELECT value FROM auth_conf WHERE application = 'project' AND module = '%s' AND action = '%s' AND role = '%s'" % (module, action, profile)
            cur.execute(query)
            perms = cur.fetchone()[0]
            return perms
        except:
            return False
