# coding: utf8
# Este fichero forma parte de openMyWork
#
# Copyright Daniel Gonzalez     <dgzaballos@gmail.com>
# Copyright Jesus Martinez      <jamarcer@gmail.com>
#
# Este fichero se distribuye bajo la licencia GPL según las
# condiciones que figuran en el fichero 'licence' que se acompaña.
# Si se distribuyera este fichero individualmente, DEBE incluirse aquí
# las condiciones expresadas allí.
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

# From web2py directory, console mode, write...
# $ ./web2py.py -S forge -M -R applications/forge/scripts/test_profiles.py

import os.path

from mil.base import YAMLFile

from gluon.storage import Storage

def load_config(app_path):    
    filename = os.path.join(app_path, 'private', 'government', 
                            'government_project_1.0.yaml')
    if filename:
        conf = YAMLFile.load(filename)        
        return conf
    
    return None

def prepare_profiles():
    # Get data from configuration file
    data = load_config(app_path)
    
    # Process data
    if data is not None:        
        try:
            # Get metadata
            metadata = data['metadata']
            application = metadata['application']
            
            # Checks if application is in environment
            if not application in _config.get('subapp'):
                print 'Application "%s" not in environment' % application
                return False
                            
            from system import AuthConfiguration
            
#            # Process Configuration
#            configuration = data['configuration']
#            
#            if configuration:
#                auth_mng = AuthConfiguration(db)     # Role == 0
#                
#                # Clean application data
#                auth_mng.delete(application)       # Module == application
#                
#                for item in configuration:
#                    print "Inserting %s '%s': %s" % (item['module'], 
#                                                     item['action'], 
#                                                     item['value'])
#                    # Add new row (update or insert)
#                    auth_mng.add(application,
#                                 item['module'], 
#                                 item['action'], 
#                                 item['value'])        
            
            # Process Government Profiles
            profiles = data['government']['profiles']
            
            if profiles:
                # Import model
                from masters.profiles import Profiles
                insKlss = Profiles(_config, db)
                
                profiles_id = {}
                for item in profiles:
                    # Test if role exists
                    pid = insKlss.get_id_by_subapp_and_role(application, 
                                                            item['role'])
                    
                    if pid is not None:                      
                        print "Exists %s '%s'" % (item['role'], 
                                                  item['description'])
                        profiles_id[item['role']] = pid
                    else:                        
                        print "Inserting %s '%s'" % (item['role'], 
                                                     item['description'])
                        # Add web2py group
                        gp_id = auth.add_group(item['role'], 
                                               item['description'])
                        
                        # Add profile to subapplication
                        pid = insKlss.add_profile(application, gp_id, True)
                        profiles_id[item['role']] = pid


            # Process Government Authorization
            permissions = data['government']['permissions']
            if permissions:
                for role in permissions:
                    role_data = permissions[role]
                    auth_mng = AuthConfiguration(db, profiles_id[role])
                    for item in role_data:
                        print "Inserting %s - %s '%s': %s" % (profiles_id[role],
                                                              item['module'], 
                                                              item['action'], 
                                                              item['value'])
                        # Add new row (update or insert)
                        auth_mng.add(application,
                                     item['module'], 
                                     item['action'], 
                                     item['value'])        
                
        except Exception, e:
            print"Error: '%s'" % str(e)         
    else:
        print 'Configuration file is empty'

def launch(*args):
    # Parameters
    param=Storage()
    args_name=['app']
    for i in range(len(args)):
        try:
            param[args_name[i]]=args[i]
        except:     # No se tiene en cuenta el parametro
            pass

    # Main process
    prepare_profiles()
    
    print 'Done!'

try:
    args=sys.argv[1:] 
except:
    args=[]
launch(*args) 
