# coding: utf8

import datetime

from gluon import *

from mil.tools.common import get_unique_id, normalize

def populate_database(db):
    # user 
    if not 'auth_user' in db.tables:
        db.define_table('auth_user',
                    Field('first_name', length=128, default=''),
                    Field('last_name', length=128, default=''),
                    Field('email', length=512, default=''),
                    Field('username', length=128, default='',),
                    Field('password', 'password', length=512, readable=False),
                    Field('registration_key', length=512,
                          writable=False, readable=False, default=''),
                    Field('reset_password_key', length=512,
                          writable=False, readable=False, default=''),
                    Field('registration_id', length=512,
                          writable=False, readable=False, default=''),
                    format='%(username)s')

    if not 'user' in db.tables:
        db.define_table('user',
            Field('name', 'string', length=100, required=True, notnull=True, 
                  unique=True, label=current.T('Name')),
            Field('email', 'string', length=400, required=True, notnull=True, 
                  label=current.T('Email')),
            Field('status', 'boolean', required=True, notnull=True, 
                  label=current.T('Status')),
            Field('is_admin', 'boolean', required=True, 
                  label=current.T('Administrator')),
            Field('password', 'string', length=100, 
                  label=current.T('Password')),
            Field('preferences', 'string', length=400, 
                  label=current.T('Preferences')),
            format='%(name)s <%(email)s>')
        
    # Project
    if not 'project_type' in db.tables:
        db.define_table('project_type',
                        Field('name', 'string', length=200, required=True, 
                              notnull=True, 
                              label=current.T('Name')),
                        Field('description', 'text', 
                              label=current.T('Description')),
                        format='%(name)s')
        
    if not 'project_card' in db.tables:
        db.define_table('project_card',
        Field('identifier', 'string', length=20, default=get_unique_id(2,4), 
              required=False, notnull=True, writable=False, 
              label=current.T('Identifier')),
        Field('name', 'string', length=200, requires=IS_ALPHANUMERIC(), 
              required=False, notnull=False, unique=True, 
              compute=lambda r:normalizar(r.title), 
              label=current.T('Name')),
        Field('title', 'string', length=200, required=True, notnull=True, 
              label=current.T('Title')),
        Field('type', 'reference project_type', required=True, notnull=True, 
              label=current.T('Type')),
        Field('private', 'boolean', required=True, notnull=True, default=False, 
              label=current.T('Private')),
        Field('description', 'text', 
              label=current.T('Description')),
        Field('start_date', 'date', required=True, notnull=True, 
              requires=IS_DATE(format=current.T("%Y-%m-%d")), 
              label=current.T('Start')),
        Field('end_date', 'date', required=True, notnull=True, 
              label=current.T('End'), 
              requires=IS_DATE(format=current.T("%Y-%m-%d"))),
        Field('objetive', 'text', 
              label=current.T('Objetive')),
        Field('resume', 'text', 
              label=current.T('Resume')),
        Field('deleted', 'boolean', readable=False, writable=False, 
              notnull=True, default=False, 
              label=current.T('Check to delete')),
        format='%(name)s')
       
    if not 'element_has_element' in db.tables:
        db.define_table('element_has_element',
                        Field('parent', 'string', length=20, required=True, 
                              notnull=True),
                        Field('child', 'string', length=20, required=True, 
                              notnull=True),
                        Field('type', 'string', length=20, required=True, 
                              notnull=True),
                        Field('numeral', 'integer', default=0, writable=False, 
                              readable=False),
                        Field('sortable', 'integer', default=0, writable=False, 
                              readable=False),
                        format='')
        
        
    if not 'bug' in db.tables:
        db.define_table('bug',
                        Field('identifier', 'string', length=20, 
                              default=get_unique_id(2,4), notnull=True, 
                              label=current.T('Identifier'), 
                              writable=False, readable=False),
                        Field('project', 'string', length=20, notnull=True, 
                              label=current.T('Project')),
                        Field('sumary', 'string', length=250, required=True, 
                              notnull=True, label=current.T('Sumary')),
                        Field('content', 'text', required=True, notnull=True, 
                              label=current.T('Description')),
                        Field('start_date', 'datetime', 
                              default=datetime.datetime.now(), notnull=True, 
                              label=current.T('Creation Date'), writable=False, 
                              readable=False, 
                              requires=IS_DATE(format=current.T("%Y-%m-%d %H:%M:%S"))),
                        Field('creator', 'string', length=200, default='', 
                              notnull=True, label=current.T('Creator')),
                        Field('update_date', 'datetime', 
                              label=current.T('Update Date'), 
                              requires=IS_DATE(format=current.T("%Y-%m-%d %H:%M:%S"))),
                        Field('responsible', 'string', length=150, 
                              label=current.T('Assigned to')),
                        Field('close_date', 'datetime', label=current.T('Close Date'), 
                              requires=IS_DATE(format=current.T("%Y-%m-%d %H:%M:%S"))),
                        Field('status', 'boolean', default=False, notnull=True, 
                              label=current.T('Finished')),
                        Field('priority', 'integer', default=0, notnull=True, 
                              label=current.T('Priority')),
                        format='%(sumary)s')

    if not 'comment' in db.tables:
        db.define_table('comment',
                        Field('element', 'string', length=20, required=True, 
                              writable=False),
                        Field('content', 'text', required=True, notnull=True, 
                              label=current.T('Comment')),
                        Field('author', 'string', length=200, required=True, 
                              notnull=True, label=current.T('Author')),
                        Field('create_date', 'datetime', required=True, 
                              notnull=True, label=current.T('Date'), 
                              requires=IS_DATE(format=current.T("%Y-%m-%d %H:%M:%S"))),
                        Field('project','string',length=20,
                              requires=IS_NOT_EMPTY()),
                        format='')

    if not 'file' in db.tables:
        db.define_table('file',
                        Field('identifier', 'string', length=20, 
                              default=get_unique_id(2,4), notnull=True, 
                              label=current.T('Identifier'),
                              writable=False, readable=False),
                        Field('name',requires=IS_NOT_EMPTY(),
                              label=current.T('Name')),
                        Field('project','string',length=20,
                              requires=IS_NOT_EMPTY()),
                        Field('creator', 'string', length=200, default='', 
                              notnull=True, label=current.T('Creator')),
                        Field('created','datetime',
                              default=datetime.datetime.today()),
                        Field('deleted','datetime',default=None),
                        Field('file_name','string',
                              label=current.T('File')),
                        Field('file_format','string',
                              label=current.T('Extension')),
                        Field('file_size','integer',
                              label=current.T('Size')),
                        Field('file_content','upload',autodelete=True,
                              label=current.T('Content')),
                        format='')

    if not 'incidence' in db.tables:
        db.define_table('incidence',
                        Field('identifier', 'string', length=20, 
                              default=get_unique_id(2,4), notnull=True, 
                              label=current.T('Identifier'), 
                              writable=False, readable=False),
                        Field('project', 'string', length=20, notnull=True, 
                              label=current.T('Project')),
                        Field('sumary', 'string', length=250, required=True, 
                              notnull=True, label=current.T('Sumary')),
                        Field('content', 'text', required=True, notnull=True, 
                              label=current.T('Description')),
                        Field('start_date', 'datetime', 
                              default=datetime.datetime.now(), notnull=True, 
                              label=current.T('Creation Date'), 
                              writable=False, readable=False, 
                              requires=IS_DATE(format=current.T("%Y-%m-%d %H:%M:%S"))),
                        Field('creator', 'string', length=200, default='', 
                              notnull=True, label=current.T('Creator')),
                        Field('update_date', 'datetime', 
                              label=current.T('Update Date'), 
                              requires=IS_DATE(format=current.T("%Y-%m-%d %H:%M:%S"))),
                        Field('responsible', 'string', length=150, 
                              label=current.T('Assigned to')),
                        Field('close_date', 'datetime', 
                              label=current.T('Close Date'), 
                              requires=IS_DATE(format=current.T("%Y-%m-%d %H:%M:%S"))),
                        Field('status', 'boolean', default=False, notnull=True, 
                              label=current.T('Finished')),
                        Field('priority', 'integer', default=0, notnull=True, 
                              label=current.T('Priority')),
                        format='%(sumary)s')

    if not 'roadmap' in db.tables:
        db.define_table('roadmap',
                        Field('identifier', 'string', length=20, 
                              default=get_unique_id(2,4), notnull=True, 
                              label=current.T('Identifier'), 
                              writable=False, readable=False),
                        Field('project', 'string', length=20, notnull=True, 
                              label=current.T('Project')),
                        Field('sumary', 'string', length=250, required=True, 
                              notnull=True, label=current.T('Sumary')),
                        Field('content', 'text', required=True, notnull=True, 
                              label=current.T('Description')),
                        Field('start_date', 'datetime', 
                              default=datetime.datetime.now(), notnull=True, 
                              label=current.T('Creation Date'), 
                              writable=False, readable=False, 
                              requires=IS_DATE(format=current.T("%Y-%m-%d %H:%M:%S"))),
                        Field('creator', 'string', length=200, default='', 
                              notnull=True, label=current.T('Creator')),
                        Field('update_date', 'datetime', 
                              label=current.T('Update Date'), 
                              requires=IS_DATE(format=current.T("%Y-%m-%d %H:%M:%S"))),
                        Field('responsible', 'string', length=150, 
                              label=current.T('Assigned to')),
                        Field('close_date', 'datetime', 
                              label=current.T('Close Date'), 
                              requires=IS_DATE(format=current.T("%Y-%m-%d %H:%M:%S"))),
                        Field('status', 'boolean', default=False, notnull=True, 
                              label=current.T('Finished')),
                        Field('priority', 'integer', default=0, notnull=True, 
                              label=current.T('Priority')),
                        Field('init_date', 'date', 
                              default=datetime.datetime.now(), notnull=True, 
                              label=current.T('Initial Date'), 
                              writable=False, readable=False, 
                              requires=IS_DATE(format=current.T("%Y-%m-%d"))),
                        Field('end_date', 'date', 
                              default=datetime.datetime.now(), notnull=True, 
                              label=current.T('End Date'), 
                              writable=False, readable=False, 
                              requires=IS_DATE(format=current.T("%Y-%m-%d"))),
                        format='%(sumary)s')


    if not 'tag' in db.tables:
        db.define_table('tag',
                        Field('element', 'string', length=20, required=True, 
                              label=current.T('Tag')),
                        Field('value', 'string', length=50, required=True, 
                              label=current.T('Value')),
                        Field('project','string',length=20,
                              requires=IS_NOT_EMPTY()),
                        format='%(value)s')

    if not 'task' in db.tables:
        db.define_table('task',
                        Field('identifier', 'string', length=20, 
                              default=get_unique_id(2,4), notnull=True, 
                              label=current.T('Identifier'), 
                              writable=False, readable=False),
                        Field('project', 'string', length=20, notnull=True, 
                              label=current.T('Project')),
                        Field('sumary', 'string', length=250, required=True, 
                              notnull=True, label=current.T('Sumary')),
                        Field('content', 'text', required=True, notnull=True, 
                              label=current.T('Description')),
                        Field('start_date', 'datetime', 
                              default=datetime.datetime.now(), notnull=True, 
                              label=current.T('Creation Date'), 
                              writable=False, readable=False, 
                              requires=IS_DATE(format=current.T("%Y-%m-%d %H:%M:%S"))),
                        Field('creator', 'string', length=200, default='', 
                              notnull=True, label=current.T('Creator')),
                        Field('update_date', 'datetime', 
                              label=current.T('Update Date'), 
                              requires=IS_DATE(format=current.T("%Y-%m-%d %H:%M:%S"))),
                        Field('responsible', 'string', length=150, 
                              label=current.T('Assigned to')),
                        Field('close_date', 'datetime', 
                              label=current.T('Close Date'), 
                              requires=IS_DATE(format=current.T("%Y-%m-%d %H:%M:%S"))),
                        Field('status', 'boolean', default=False, notnull=True, 
                              label=current.T('Finished')),
                        Field('priority', 'integer', default=0, notnull=True, 
                              label=current.T('Priority')),
                        format='%(sumary)s')
     
    # Project Profile   
    if not 'project_user_profile' in db.tables:
        db.define_table('project_user_profile',
                        Field('name', 'string', length=100, required=True, 
                              notnull=True, label=current.T('Profile')),
                        Field('read_auth', 'boolean', required=True, 
                              notnull=True, label=current.T('Read')),
                        Field('write_auth', 'boolean', required=True, 
                              notnull=True, label=current.T('Write')),
                        Field('exec_auth', 'boolean', required=True, 
                              notnull=True, label=current.T('Execution')),
                        format='%(name)s')
        
    # Project team 
    if not 'project_team' in db.tables:
        db.define_table('project_team',
                        Field('project', 'reference project_card', 
                              required=False, 
                              label=current.T('Project')),
                        Field('user', 'reference user', required=False, 
                              label=current.T('User')),
                        Field('profile', 'reference project_user_profile', 
                              required=False, 
                              label=current.T('Profile')),
                        format='')
        
        
    # Wiki
    wiki_migrate = False
    if not 'wiki_page' in db.tables:        
        db.define_table('wiki_page',
                    Field('ns',writable=False,default=''),
                    Field('slug',writable=False,
                          requires=(IS_SLUG(),IS_NOT_IN_DB(db,'wiki_page.slug'))),
                    Field('title',default='',requires=(IS_NOT_EMPTY())),
                    Field('active','boolean',default=True),
                    Field('public','boolean',default=True),
                    Field('body','text',default=''),
                    Field('changelog',default=''),
                    Field('created_by',
                          db.auth_user,default=current.auth.user_id,
                          writable=False,readable=False),
                    Field('created_on','datetime',
                          default=current.request.now,
                          writable=False,readable=False),
                    Field('modified_by',
                          db.auth_user,default=current.auth.user_id,update=current.auth.user_id,
                          writable=False,readable=False),
                    Field('modified_on','datetime',
                          default=current.request.now,update=current.request.now,
                          writable=False,readable=False),
                    format = '%(slug)s', migrate=wiki_migrate)
    
    
    if not 'wiki_page_archive' in db.tables:      
        db.define_table('wiki_page_archive',                                    
                    Field('current_record',db.wiki_page),                               
                    db.wiki_page,
                    format = '%(slug) %(modified_on)s', migrate=wiki_migrate)
    
    
    if not 'wiki_attachment' in db.tables:        
        db.define_table('wiki_attachment',
                    Field('tablename',writable=False,readable=False),
                    Field('record_id','integer',writable=False,readable=False),
                    Field('name',requires=IS_NOT_EMPTY()),
                    Field('file','upload',requires=IS_NOT_EMPTY(),autodelete=True),
                    Field('created_by',
                          db.auth_user,default=current.auth.user_id or 1,
                          writable=False,readable=False),
                    Field('created_on','datetime',
                          default=current.request.now,
                          writable=False,readable=False),
                    Field('modified_by',
                          db.auth_user,default=current.auth.user_id,update=current.auth.user_id,
                          writable=False,readable=False),
                    Field('modified_on','datetime',
                          default=current.request.now,update=current.request.now,
                          writable=False,readable=False),
                    format='%(name)s', migrate=wiki_migrate)
    
    if not 'wiki_comment' in db.tables:        
        db.define_table('wiki_comment',
                    Field('tablename',
                          writable=False,readable=False),
                    Field('record_id','integer',
                          writable=False,readable=False),
                    Field('body',requires=IS_NOT_EMPTY(),label='Your comment'),
                    Field('created_by',db.auth_user,default=current.auth.user_id,
                          readable=False,writable=False),
                    Field('created_on','datetime',default=current.request.now,
                          readable=False,writable=False), migrate=wiki_migrate)
    
    if not 'wiki_tag' in db.tables:
        db.define_table('wiki_tag',
                    Field('name',requires=IS_NOT_IN_DB(db,'wiki_tag.name')),
                    Field('links','integer',default=0,writable=False),
                    Field('created_by',db.auth_user,writable=False,readable=False,
                          default=current.auth.user_id),
                    Field('created_on','datetime',
                          default=current.request.now,writable=False,readable=False),
                    format='%(name)s', migrate=wiki_migrate)
    
    
    if not 'wiki_link' in db.tables:
        db.define_table('wiki_link', 
                    Field('tag',db.wiki_tag),   
                    Field('table_name'),
                    Field('record_id','integer'), migrate=wiki_migrate)