# coding: utf8

from gluon import *
from gluon.storage import Storage

from masters.file import FileModel
from masters.profiles import Profiles
from project.models.type import ProjectTypeModel
from project.models.bug import BugModel
from project.models.card import ProjectCardModel
from project.models.comment import CommentModel
from project.models.element import ElementModel
from project.models.incidence import IncidenceModel
from project.models.roadmap import RoadmapModel
from project.models.service import ProjectServicesModel
from project.models.tag import TagModel
from project.models.task import TaskModel
from project.models.team import ProjectTeamModel
from project.models.type_service import ProjectTypeServicesModel

def check_by_name(name):
    rows = current.db(current.db.project_card.name == name).select()
    if len(rows) > 0:
        return True
    else:
        return False
    
def get_id_by_name(name):
    return current.db(current.db.project_card.name == name).select().first().id

def add_project(data):
    # Prepare elements
    row = Storage(identifier=data.identifier,
                  name=data.name,
                  title=data.title,
                  type=data.type,
                  private=data.private,
                  description=data.description,
                  start_date=data.start_date,
                  end_date=data.end_date,
                  objetive=data.objetive,
                  resume=data.resume,
                  deleted=data.deleted,
                  active=False,
                  create_lib=False)
    
    # Insert element
    if check_by_name(row.name) is False:
        return current.db.project_card.insert(**row)

def get_project_types(db_orig):  
    tmp = {}
    
    # Get 2.0 project types
    project_types = {}
    types_20 = ProjectTypeModel(current.db).get_all()
    for type in types_20:
        project_types[type.name] = type.id
    
    # Prepare translation matrix
    rows = db_orig(db_orig.project_type.id > 0).select()
    for row in rows:
        # Checks if name exits in 2.0
        if row.name in project_types.keys():
            type = project_types[row.name]
        else:
            # Default first type in list
            type = project_types[project_types.keys()[0]]
            
        tmp[row.id] = type        
     
    return tmp  

def get_project_profiles(db_orig):  
    tmp = {}
    
    # Get 2.0 project profiles 
    rows = Profiles(current._config, current.db).get_subapp_profiles('project')
    # dict: {'Collaborator': 3, 'Public': 5, 'Leader': 2, 'Guest': 4}
    profiles_20 = {}
    for row in rows:
        profiles_20[row.name] = row.id
        
    # Get 1.5 project profiles
    rows = db_orig(db_orig.project_user_profile.id > 0).select()
    # dict: {'Colaborador': 2, 'Invitado': 3, 'L\xc3\xadder': 1}
    profiles_15 = {}
    for row in rows:
        profiles_15[row.name] = row.id
                
    # Matrix
    tmp = {}
    for item in profiles_15.keys():
        if item == 'Líder':
            value = profiles_20['Leader']
        elif item == 'Colaborador':
            value = profiles_20['Collaborator']
        elif item == 'Invitado':
            value = profiles_20['Guest']
        else:
            value = profiles_20['Public']
        tmp[profiles_15[item]] = value
        
    return tmp  

def migrate_projects(db_orig, user_array):    
    ret_val = []
    # Prepare db
    BugModel.define_table(current.db)
    ProjectCardModel.define_table(current.db)
    CommentModel.define_table(current.db)
    ElementModel.define_table(current.db)
    FileModel.define_table(current.db)
    IncidenceModel.define_table(current.db)
    RoadmapModel.define_table(current.db)
    ProjectServicesModel.define_table(current.db)
    TagModel.define_table(current.db)
    TaskModel.define_table(current.db)
    ProjectTypeServicesModel.define_table(current.db)
    
    # Projects
    project_types = get_project_types(db_orig)
    project_array = {}
    rows = db_orig(db_orig.project_card.id > 0).select()        
    for row in rows:
        row.type = project_types[row.type]
        if check_by_name(row.name) is False:
            id = add_project(row)
            # Add services
            if row.type == 1:   # Consultoría
                services = ['bug', 'calendar', 'incidence', 'repository', 'roadmap', 'task', 'webdav', 'wiki']
            elif row.type == 2: # Publicación
                services = ['bug', 'calendar', 'incidence', 'repository', 'roadmap', 'task', 'webdav', 'wiki']
            else:               # Liberación
                services = ['bug', 'calendar', 'incidence', 'repository', 'roadmap', 'task', 'webdav', 'wiki']
                
            ProjectServicesModel(current.db).set(id, services)
#            type_service = ProjectTypeServicesModel(current.db).get_by_type(row.type)
#            ProjectServicesModel(current.db).set(id, type_service.services)
        else:
            id = get_id_by_name(row.name)
        
        if not row.id in project_array.keys():
            project_array[row.id] = id
            
    # Extra entities
    entities = ['bug', 'comment', 'element_has_element', 'file', 'incidence', 'roadmap', 'tag', 'task']
    for entity in entities:
        rows = db_orig(db_orig[entity].id > 0).select()        
        for row in rows:
            tmp = {}
            for item in row.keys():
                if not item in ['id', 'update_record', 'delete_record']:
                    tmp[item] = row[item]
            current.db[entity].insert(**tmp)
            
    # Prepare profile matrix
    profiles = get_project_profiles(db_orig)
    
    # Project teams
    rows = db_orig(db_orig.project_team.id > 0).select()
    for row in rows:
        # Set member
        ProjectTeamModel(current.db).set_member(project_array[row.project],  
                                                user_array[row.user], 
                                                profiles[row.profile])

    # Activate services
    # Join member is not necessary because 1.5 application use dav and svn 
    # authorization using mysql
    #from project import ProjectHandler
    #ProjectHandler.join_member_as(project.id, member_id, profile_id)
   