// http://wiki.jqueryui.com/w/page/12138135/Widget%20factory
// http://stackoverflow.com/questions/3631300/jquery-nested-droppables
// http://jsfiddle.net/mekwall/pPs34/

(function( $ ) {
	var links = {
		util: {
			// Data
			repo: {	'elements':{},
					'viewport': {
								'elements': {},
								'links': {}
						}
				},
			elem_counter: 0,
			vp_elem_counter: 0,
 
			links: [],
			links_counter: 0,
			lines: {},
 
			// Visual elements
			status: $("<div>").attr('id', 'base_status'),
			element_area: null,
			viewport: null,
			svg: null,
 
			debug: true,
			
			// API
			_add_repo_element: function(id, literal, type, top, left) {
				// Validations
				if (typeof type === 'undefined') {type = '';}
				if (typeof top === 'undefined') {top = 0;}
				if (typeof left === 'undefined') {left = 0;}
				
				links.util.repo['elements'][id] = {'label': literal,
												   'type': type};
			},
			
			_get_repo_element: function(id) {
				return links.util.repo['elements'][id];
			},
			
			_delete_repo_element: function(id) {
				delete links.util.repo['elements'][id];
			},
			
			_add_viewport_element: function(id, elem, top, left) {
				// Validations
				if (typeof type === 'undefined') {type = '';}
				if (typeof top === 'undefined') {top = 0;}
				if (typeof left === 'undefined') {left = 0;}
				
				links.util.repo['viewport']['elements'][id] = {'id': elem,
														       'pos': {'top': top,
																      'left': left}
															   };	
			},
			
			_get_viewport_element: function(id) {
				return links.util.repo['viewport']['elements'][id];
			},
			
			_delete_viewport_element: function(id) {
				delete links.util.repo['viewport']['elements'][id];
			},
			
			_add_viewport_link: function(id, origin, target, label) {
				// Validations
				if (id == null) {
						links.util.links_counter++;
						id = 'link_' + links.util.links_counter;
				}
				if ((typeof label === 'undefined') || (label == null)) {label = '';}
				
				links.util.repo['viewport']['links'][id] = {'origin': origin,
												'target': target,
												'label': label
				};
			},
			
			_delete_viewport_element_links: function(element) {
				for (item in links.util.repo['viewport']['links']) {
						link = links.util.repo['viewport']['links'][item];
						var delete_me = false;
						if (link['origin'] == element) {
								delete_me = true;
						};
						if (link['target'] == element) {
								delete_me = true;
						}
						if (delete_me == true) {
								delete links.util.repo['viewport']['links'][item];
						}
				}
			},
			
			_delete_repo_element_from_viewport: function(element) {
				for (item in links.util.repo['viewport']['elements']) {
						links.util.set_status('In element ' + item);
						var elem = links.util.repo['viewport']['elements'][item];
						if (elem['id'] == element) {
								links.util.set_status('Delete ' + item);
								links.util.delete_item(item);
						}
				}				
			},
			
			draw_element: function(id, element, type) {
				var v_elem = $("<div>").attr('class', 'pl_'+links.util._get_visual_class(type));
				links.util.elem_counter++;
				if (id == null) {
					id = 'elem_' + links.util.elem_counter;
				}
				links.util._add_repo_element(id, element, type);
				v_elem.attr('id', id);
				v_elem.addClass("pl_static");
				v_elem.html(element);
				
				// Trash
				var drag_trash =  $("<div>").attr('class', 'pl_drag_trash');
				drag_trash.click(function () {links.util.delete_element(id);});
				v_elem.append(drag_trash);
				
				// Draggable
				v_elem.draggable({helper: 'clone'});
				
				if (links.util.debug == true) {links.util.set_status('Added element: ' + element);}
				return v_elem;
			},
			
			draw_viewport_element: function(id, element, type) {
				var v_elem = $("<div>").attr('class', 'pl_'+links.util._get_visual_class(type));
				links.util.vp_elem_counter++;
				if (id == null) {
					id = 'item_' + links.util.vp_elem_counter;
				}
				
				v_elem.attr('id', id);
				v_elem.addClass("pl_movable");
				v_elem.append(element);
				
				// Trash
				var drag_trash =  $("<div>").attr('class', 'pl_drag_trash');
				drag_trash.click(function () {links.util.delete_item(id);});
				v_elem.append(drag_trash);
				
				// Linker
				var drag_linker =  $("<div>").attr('id', id);
				drag_linker.attr('class', 'pl_drag_linker');
				drag_linker.draggable({helper: 'clone'});
				v_elem.append(drag_linker);
				
				// Activate item
				v_elem.draggable({stop:links.util.element_drag});
				v_elem.droppable({accept: "div.pl_drag_linker",
						  greedy: true,
						  drop: links.util.element_drop});
				
				return v_elem;
			},
			
			viewport_drop: function( event, ui ) { 
				var vport = $(this);
				var old_elem =  ui.draggable.clone();
				
				if (old_elem.hasClass('pl_static')) {
					var id = old_elem.attr('id');
					var pos = ui.position;
					links.util.new_vp_elem(null, id, pos.top, pos.left);
				}
			},
			
			element_drop: function( event, ui ) { 
				var origin =  ui.draggable.attr('id');
				var target = $(this).attr('id');
				
				links.util._add_viewport_link(null, origin, target);
				
				links.util._draw_line_link(origin, target);
				
				// Connect to source
				links.util.send()
				return false;
			},
 
			element_drag: function() {
				var elem_id = $(this).attr('id')
				links.util.draw_links(elem_id);
			},
 
			draw_links: function(id) {
				for (link in links.util.links[id]) {
						links.util.svg_draw_line(id, links.util.links[id][link]);
				}
			},
 
			new_element: function(id, label, type) {
				var n_elem = links.util.draw_element(id, label, type);
				links.util.element_area.append(n_elem);
				
				// Connect to source
				links.util.send();
			},
 
			new_vp_elem: function(id, elem, top, left) {
				var repo_elem = links.util._get_repo_element(elem);
				
				if (typeof repo_elem === 'undefined') {
						if (links.util.debug == true) {links.util.set_status('Element not in repository: '+id);}
				} else {
						var n_elem = links.util.draw_viewport_element(id, repo_elem['label'], repo_elem['type']);
						if ((typeof top === 'undefined') || (top == 0)) {
								top = 40 * (links.util.vp_elem_counter - 1) + links.util.viewport.offset().top + 10;
						}
						if ((typeof left === 'undefined') || (left == 0)) {
								left = 20 + links.util.viewport.offset().left;
						}
						if (id == null) {
								id = n_elem.attr('id');
						}
						links.util._add_viewport_element(id, elem, top, left);
						links.util.viewport.append(n_elem);
						n_elem.offset({ 'top': top, 'left': left })
						
						// Viewport element links
						links.util.links[id] = [];
						if (links.util.debug == true) {links.util.set_status('[Viewport] Added element: ' + repo_elem['label']);}
				}
				
				// Connect to source
				links.util.send();
			},
 
			draw_link: function(origin, destiny, label) {
				links.util._draw_line_link(origin, destiny, label);
			},
 
			_draw_line_link: function(origin, destiny, label) {
				// Links
				var orig_num = links.util.links[origin].length
				links.util.links[origin][orig_num] = destiny ;
				var dest_num = links.util.links[destiny].length
				links.util.links[destiny][dest_num] = origin;
				links.util.links_counter++;
				links.util.svg_draw_line(destiny, origin, label);
			},
 
			delete_item: function(id) {
				// Visual
				for (link in links.util.links[id]) {
						var line_id = id+':'+links.util.links[id][link];
						try {
								links.util.svg_clear_line(line_id);
						} catch (err) {
								if (links.util.debug == true) {links.util.set_status('Deleted line: '+line_id);}
						}
				}
				for (item in links.util.lines) {
						var delete_me = false;
						var data = item.split(':');
						if (data[0] == id) {
							delete_me = true;
						}
						if (data[1] == id) {
							delete_me = true;
						}
						if (delete_me) { delete links.util.lines[item];}
				}
				var elem = $('#'+id).remove();
				
				// Data
				links.util._delete_viewport_element(id);
				links.util._delete_viewport_element_links(id);
				
				links.util.console();
				if (links.util.debug == true) {links.util.set_status('Delete: '+id);}
			},
 
			delete_element: function(id) {
				// NOTE: in this method 'links.util' object must be 'this' instance				
				this._delete_repo_element_from_viewport(id);
				this._delete_repo_element(id);
				
				var elem = $('#'+id).remove();
				this.console();
				if (this.debug == true) {this.set_status('Delete: '+id);}
			},
 
			_get_visual_class: function(type) {
				if (typeof type === 'undefined') { type = ''; }
				var v_class = '';
				switch( type ) {
						case "bool":
							v_class = 'drag_boolean';
							break;
						default:
							v_class = 'drag_simple';
				}
				return v_class;
			},
			
			get_status: function() {
				return links.util.status;
			},
 
			set_status: function(message) {
				links.util.status.prepend(message+"<br />");
			},
 
			svg_clear: function() {
				links.util.svg.clear();
			},

			svg_draw_line: function(eTarget, eSource, sLabel) {
				var $viewport = links.util.viewport;
				var $source = $("#"+eSource);
				var $target = $("#"+eTarget);
				
				var line_id = eSource+':'+eTarget;
				var line_id_2 = eTarget+':'+eSource;
				if (line_id in links.util.lines) {
					links.util.svg_clear_line(line_id);
				}

				var originX = $source.offset().left + ($source.width() / 2) - $viewport.offset().left;
				var originY = $source.offset().top + ($source.height() / 2) - $viewport.offset().top;

				var endingX = $target.offset().left  + ($target.width() / 2)- $viewport.offset().left;
				var endingY = $target.offset().top + ($target.height() / 2) - $viewport.offset().top;

				var stroke_width = 2;
				var color = "black";
		
				var g = links.util.svg.group({stroke: color}); 
				var n_line = links.util.svg.line(g, originX, originY, endingX, endingY, {stroke: color, strokeWidth: stroke_width});
				
				links.util.lines[line_id] = n_line;
				links.util.lines[line_id_2] = n_line;
			},

			svg_clear_line: function(id) {
				var line = links.util.lines[id];
				links.util.svg.remove(line);
			},
 
			add_vp_item: function(elem, data) {
				var add_me = true;
				for (var ind in links.util.vp_elements) {
					if (elem == ind) { add_me = false;}
				}
				if (add_me) {links.util.vp_elements[elem] = data;}
			},
			
			console: function() {
				if (links.util.debug == true) {
						$('#pl_console_data').html(JSON.stringify(links.util.repo, null, '\t')); 
				}				
			},
			
			send: function() {
				links.util.console();  //CHANGEME
			},
			
			receive: function() {
				links.util.console();  //CHANGEME
			}
		},
 
		literals: {
			status: 'Status message',
			dialog: {
				title: 'New element',
				accept: 'Accept',
				cancel: 'Cancel'
			},
			link: 'new element',
			console: 'console'
		},
 
		remove_item: function(ar_data, ind_item) {
			// Find the index
			var idx = ar_data.indexOf(ind_item); 
			// Remove it if really found!
			if (idx != -1) { ar_data.splice(idx, 1);}
		},
		
		process_data: function(data) {
			try {
				if (typeof data === 'undefined') {
						throw "Data structure undefined";
				}
				if (data == null) {
						throw "Data structure is null";
				}
				
				// Process data
				var n_data = JSON.parse(data);
				if (typeof n_data['elements'] === 'undefined') {
						throw "Data structure 'elements' is wrong";
				} else {
						for (elem in n_data['elements']) {
								links.util.new_element(elem,
														n_data['elements'][elem]['label'],
														n_data['elements'][elem]['type']);
						}
				}
				if (typeof n_data['viewport'] === 'undefined') {
						throw "Data structure 'viewport' is wrong";
				} else {
						for (elem in n_data['viewport']['elements']) {
								var item = n_data['viewport']['elements'][elem];
								links.util.new_vp_elem(elem,
														item['id'],
														item['pos']['top'],
														item['pos']['left']);
						}
						for (link in n_data['viewport']['links']) {
								var item = n_data['viewport']['links'][link];
								links.util.draw_link(item['origin'], item['target'], item['label']);
								links.util._add_viewport_link(link, item['origin'], item['target'], item['label']);
						}
				}
				links.util.console();
			} catch(err) {
				links.util.set_status('[ERROR] '+err);
			}
		},

		format_data: function(beauty) {
				if (typeof beauty === 'undefined') {
						beauty = false;
				}
				
			return JSON.parse(links.util.repo);
		}
	}
	
	$.widget( "plexxoo.links", {
		// These options will be used as defaults
		options: { 
			title: 'Sample',
			data: null
		},
		
		_create: function() {
			// Main area
			var main_div = $("<div>").attr('id', 'plexxoo_links');
			// Element area
			this.element_area = $("<div>").attr('id', 'element_area');
			
			// Vieport area
			var vieport_area = $("<div>").attr('id', 'viewport_area');
			// Header
			var header_viewport = $("<div>").attr('id', 'header_viewport');
			this.title = $("<div>").attr('id', 'base_header');
			header_viewport.append(this.title );
			// Canvas
			this.viewport = $("<div>").attr('id', 'viewport');
			// Status
			var status_viewport = $("<div>").attr('id', 'status_viewport');
			//this.status = $("<div>").attr('id', 'base_status');
			status_viewport.append(links.util.get_status());
			
			// Add elements
			vieport_area.append(header_viewport);
			vieport_area.append(this.viewport);
			vieport_area.append(status_viewport);
			
			main_div.append(this.element_area);
			main_div.append(vieport_area);
			
			this.element.empty();
			this.element.append(main_div);
			
			// Dialog
			var new_dialog = $("<div>").attr('id', 'pl_new_dialog');
			new_dialog.attr('title', links.literals.dialog.title);
			var new_form = $("<form>");
			new_form.append($("<input>").attr('id', 'pl_element').attr('type', 'text'));
			new_dialog.append(new_form);
			var new_link = $("<a>").attr('id', 'pl_new_link').attr('href', '#').html(links.literals.link);

			this.element.append(new_dialog);
			this.element.append(new_link);
			
			// Console
			if (links.util.debug == true) {
				var new_dialog = $("<div>").attr('id', 'pl_console_dialog');
				new_dialog.attr('title', links.literals.console);
				var console_data = $("<pre>").attr('id', 'pl_console_data');
				new_dialog.append(console_data);
				var new_link = $("<a>").attr('id', 'pl_console_link').attr('href', '#').html(links.literals.console);
	
				this.element.append(new_dialog);
				this.element.append(new_link);
			}
			if (links.util.debug == true) {this.set_status('Create visual interface');}
		},
		
		_init: function () {
			if (links.util.debug == true) {this.set_status('Initializing elements');}
			
			this.title.html(this.options.title);
			this.set_status(links.literals.status);
			
			// Elements
			links.util.element_area = this.element_area;
			
			// Viewport and SVG
			this.viewport.svg();
			links.util.viewport = this.viewport;
			links.util.svg = this.viewport.svg('get'); 			
			// Droppable
			this.viewport.droppable({drop: links.util.viewport_drop});
			
			// Dialogs
			// New element
			$('#pl_new_dialog').dialog({
						autoOpen: false,
						modal: false,
						overlay: {
							backgroundColor: '#000',
							opacity: 0.5
						},
						buttons: [
						{
							text: links.literals.dialog.cancel,
							click: function() { $(this).dialog("close"); }
						},
						{
							text: links.literals.dialog.accept,
							click: function() { 
								var new_elem = $('#pl_element').val();
								links.util.new_element(null, new_elem);
								$('#pl_element').val('');
								$(this).dialog("close");
							}
						}]
						});   
			$('#pl_new_link').click(function () {
				$('#pl_new_dialog').dialog("open");
			}); 
			// Console
			if (links.util.debug == true) {
				$('#pl_console_dialog').dialog({
							autoOpen: false,
							modal: false,
							position: ['right','top'],
							height: 400,
							width: 400,
							overlay: {
								backgroundColor: '#000',
								opacity: 0.5
							}
							});   
				$('#pl_console_link').click(function () {
					$('#pl_console_dialog').dialog("open");
				});
			}
			
			// Process data
			links.process_data(this.options.data);
			
			// Debug
			if (links.util.debug == true) {this.set_status('Initialized elements');}
		},
		
		set_status: function(message) {	    
			links.util.set_status(message);
		},
			
		_setOptions: function() {
			$.Widget.prototype._setOptions.apply( this, arguments );
			this.refresh();
		},    
	
		_setOption: function( key, value ) {
			$.Widget.prototype._setOption.apply( this, arguments );
	
		},
		
		destroy: function() {
			$.Widget.prototype.destroy.call( this );
		}
	});
})( jQuery );
