/* http://...
   WorkFlow 
   Written by Jesus Martinez (jamarcer{at}gmail.com) January 2012.
   Dual licensed under the GPL (http://dev.jquery.com/browser/trunk/jquery/GPL-LICENSE.txt) and 
   MIT (http://dev.jquery.com/browser/trunk/jquery/MIT-LICENSE.txt) licenses. 
   Please attribute the author if you use it. */

(function($) { // Hide scope, no $ conflict
	/* Utils */	
	/* Determine whether an object is an array. */
	function isArray(a) {
		return (a && a.constructor == Array);
	}
	
	/* Determine whether an object is an object. */
	function isObject(a) {
		return (a && (typeof a  == 'object'));
	}
	
	/* i18n*/
	/* WorkFlow manager i18n.*/
	function WorkFlowI18n(lang) {
		this.regional = ['en', 'es']; // Localisations, indexed by language, 'en' for default (English)
		this.regional['en'] = {
			accept: 'Accept',
			addNode: 'Add Node',
			cancel: 'Cancel',
			console: 'Console',
			done: 'Done',			
			errorLoading: 'Error loading',
			errorNodeConnectorNotAvailable: 'Node conector is not available',
			errorSettings: 'Wrong Settings format, it must be an array object',
			errorSettingNotAllowed: 'Setting element is not allowed',
			errorSameNode: 'Target node can not be origin node',			
			infoDeleteNode: '[Info] Delete Node ',
			label: 'Label',			
			nodeOpen: 'Open',
			nodeBoolean: 'Boolean',
			nodeConnector: 'Connector',
			nodePageConnector: 'Page Connector',
			nodePageview: 'Show Page',
			nodePagecall: 'Call Page',
			nodeEnd: 'End',
			notAllowed: 'Not allowed',
			output: 'Output',
			params: 'Parameter',
			remove: 'Remove',
			save: 'Save',	
			showConsole: 'Console',
			type: 'Type',
			workflowLoaded: 'Workflow loaded',
			workflowSaved: 'Workflow saved'
		};
		this.regional['es'] = {
			accept: 'Aceptar',
			addNode: 'Añade nodo',
			cancel: 'Cancela',
			console: 'Consola',
			done: 'Hecho',			
			errorLoading: 'Error en carga',
			errorNodeConnectorNotAvailable: 'Conector de nodo no disponible',
			errorSettings: 'Formato de Configuración incorrecto, debe ser un objeto tipo array',
			errorSettingNotAllowed: 'Elemento de Configuración no permitido',
			errorSameNode: 'El nodo destino no puede ser el nodo origen',			
			infoDeleteNode: '[Info] Nodo borrado ',
			label: 'Etiqueta',	
			nodeOpen: 'Apertura',
			nodeBoolean: 'Booleano',
			nodeConnector: 'Conector',
			nodePageConnector: 'Página Conector',
			nodePageview: 'Mostrar Página',
			nodePagecall: 'Llamar Página',
			nodeEnd: 'Cierre',
			notAllowed: 'No permitido',
			output: 'Salida',
			params: 'Parámetro',
			remove: 'Quitar',	
			save: 'Guardar',
			showConsole: 'Consola',	
			type: 'Tipo',
			workflowLoaded: 'Flujo cargado',
			workflowSaved: 'Flujo guardado'
		};
		
		if(typeof lang === 'undefined') {
			lang = 'en';
		}
		if(this.regional.indexOf(lang) == -1) {
			lang = 'en';
		}
		this.local = this.regional[lang]; // Current localisation
	}
	
	$.extend(WorkFlowI18n.prototype, {
		get: function(literal) {
			return this.local[literal];
		},
		
		set: function(lang) {
			if(typeof lang === 'undefined') {
				lang = 'en';
			}
			if(this.regional.indexOf(lang) == -1) {
				lang = 'en';
			}
			this.local = this.regional[lang]; // Current localisation
		},
		
		add_regional: function(lang, data) {
			this.regional[lang] = data;
		}
	});
	$.workflow_i18n = new WorkFlowI18n();
	
	/* Visual elements */
	/* Status manager.*/
	function StatusInformer() {
		this._widget = $("<div>").attr('id', 'pl_workflow_status');
	}
	
	$.extend(StatusInformer.prototype, {
		widget: function() {
			return this._widget;
		},
		
		set: function(message) {
			this._widget.html(message);
		}
	});
	
	/* Console */
	function Console() {
		this._data = [];
		this._widget = $("<div>").attr('id', 'pl_workflow_console');
	}
	
	$.extend(Console.prototype, {
		widget: function() {
			this._widget.hide();
			return this._widget;
		},
		
		get: function() {
			return this._data;
		},		
		
		set: function(message) {
			this._data[this._data.length + 1] = message;
			this._widget.prepend(message+"<br />");
		},
		
		toggle: function() {
			this.widget.toggle();
		}
	});
	
	/* Menu Popup*/
	function WorkflowMenu() {
		this._data = [];
		this._widget = null;
		this.debug = false;
	}	
	
	$.extend(WorkflowMenu.prototype, {
		widget: function(type, debug) {
			if (type == 'delete') {
				this.delete_menu();
			} else {				
				this.main_menu();
			}
			this.debug = debug;
			this._widget.hide();
			return this._widget;
		},
		
		main_menu: function() {
			this._widget = $("<ul>").attr('id', 'pl_workflow_menu');
			this._widget.append($("<li>").append($("<a>").attr('href', '#add').html($.workflow_i18n.get('addNode'))));
			this._widget.append($("<li>").append($("<a>").attr('href', '#save').html($.workflow_i18n.get('save'))));
			if (this.debug) {
				this._widget.append($("<li>").append($("<a>").attr('href', '#console').html($.workflow_i18n.get('console'))));
				this._widget.append($("<li>").append($("<a>").attr('href', '#output').html($.workflow_i18n.get('output'))));
			}			
		},
		
		delete_menu: function() {
			this._widget = $("<ul>").attr('id', 'pl_workflow_menu2');
			this._widget.append($("<li>").append($("<a>").attr('href', '#remove').html($.workflow_i18n.get('remove'))));
		}
	});
		
	
	/* WorkFlow Output */
	function WorkFlowOutput() {
		this._data = [];
		this._widget = null;
	}
	
	$.extend(WorkFlowOutput.prototype, {
		widget: function() {
			var new_dialog = $("<div>").attr('id', 'pl_output_dialog');
			new_dialog.attr('title', $.workflow_i18n.get('output'));
			var console_data = $("<pre>").attr('id', 'pl_output_data');
			new_dialog.append(console_data);
			
			this._widget = new_dialog;
			this._widget.hide();
			return this._widget;
		},
		
		get: function() {
			return this._data;
		},		
		
		set: function(message) {
			this._data = message;
			$('#pl_output_data').html(this._data);
		},
		
		toggle: function() {
			this.widget.toggle();
		}		
	});
	
	$.workflow_status = new StatusInformer();
	$.workflow_console = new Console();
	$.workflow_output = new WorkFlowOutput();
	
	/* The WorkFlow Node */
	function WorkFlowNode(id, label, type, param) {
		this._id = id; // The identifier of the node 
		this._label = label; // The label of the node 
		this._type = type; // The type of node
		this._param = param; // The parameter of node
		this._connector = {'input': {}, 'output': {}} ;
		this._pos = {'top': 0, 'left': 0};
		this._widget = null; // Widget 'code'
	}
	
	$.extend(WorkFlowNode.prototype, {
		widget: function() {
			if (this._widget == null) {
				this._widget = this._build();
			}
			return this._widget;
		},
		
		output: function(){
			var out_data = {'id': this._id,
							'label': this._label,
							'type': this._type, 
							'param': this._param, 
							'pos': this._pos,
							'connector': this._connector
						};
			return out_data;
		},
		
		connect: function(type, id, target) {
			this._connector[type][id]['connection'] = target;		
		},
		
		disconnect: function(type, id) {
			this._connector[type][id]['connection'] = "";		
		},
		
		connector_available: function(type, id) {
			try {
				var target = this._connector[type][id]['connection'];
			} catch(err) {
				var target = "";
			}			
			if (target == "") {
				return true;
			} else {
				return false;
			}
		},
		
		set_pos: function(top, left) {
			this._pos.top = top;
			this._pos.left = left;
		},
		
		set_connections: function (data) {
			this._connector['input'] = data['input'];
			this._connector['output'] = data['output'];
		},
		
		get_connection: function(type, id) {
			return this._connector[type][id]['connection'];
		},
		
		get_connections: function() {
			var conections = [];
			// In
			for (item in this._connector['input']) {
				if (this._connector['input'][item]['connection'] != "") {
					var item_data = this._connector['input'][item]['connection'].split('.');
				
					conections[conections.length + 1] = {'tid': this._id,
														'tcid': item,
														'sid': item_data[0],
														'scid': item_data[1]};
				}				
			}
			// Out
			for (item in this._connector['output']) {
				if (this._connector['output'][item]['connection'] != "") {
					var item_data = this._connector['output'][item]['connection'].split('.');
				
					conections[conections.length + 1] = {'tid': this._id,
														'tcid': item,
														'sid': item_data[0],
														'scid': item_data[1]};
				}
			}
			return conections;
		},
		
		_add_connector: function(id, type, label) {
			var connector = $("<div>").attr('id', id);
			if (typeof label === 'undefined') {label = type;}
			
			if (type == 'output') {
				connector.attr('class', 'pl_fb_out');
				connector.append($("<span>").attr('class', 'pl_fb_out_label').html(label));				
				var out_elem = $("<span>").attr('id', id + '_l').attr('class', 'blue circle pl_fb_out_action');
				connector.append(out_elem);
				out_elem.contextMenu({menu: 'pl_workflow_menu2'},
					function(action, el, pos) {
						switch(action) {
							case 'remove':
								var id = $(el).parents('.pl_flow_box').attr('id');
								var conn_id = $(el).parent().attr('id');
								$.workflow.node_disconnection(id, conn_id);
								break;
							default:
								alert(
									'Action: ' + action + '\n\n' +
									'Element ID: ' + $(el).attr('id') + '\n\n' + 
									'X: ' + pos.x + '  Y: ' + pos.y + ' (relative to element)\n\n' + 
									'X: ' + pos.docX + '  Y: ' + pos.docY+ ' (relative to document)'
								);
							}
				});
				out_elem.droppable({
						  greedy: true,
						  over: function(event, ui) {$(this).parents('.pl_flow_box').addClass('red');},
						  out: function(event, ui) {$(this).parents('.pl_flow_box').removeClass('red');},
						  drop: function(event, ui) {							
							var target = $(this);
							target.parents('.pl_flow_box').removeClass('red');
							
							var target_conn_id = target.parent().attr('id');
							var target_id = target.parents('.pl_flow_box').attr('id');	
							
							var drag = ui.draggable.clone();
							var drag_id = drag.attr('pid'); 
							var drag_conn_id = ui.draggable.parent().attr('id');
							
							$.workflow.node_connection(target_id, target_conn_id, drag_id, drag_conn_id );
							}});
			} else {
				connector.attr('class', 'pl_fb_in');
				var in_elem = $("<span>").attr('id', id + '_l').attr('class', 'green circle pl_fb_in_action').attr('pid', this._id);				
				in_elem.draggable({helper: 'clone'});	
				connector.append(in_elem);
				connector.append($("<span>").attr('class', 'pl_fb_in_label').html(label));
			}
			
			this._connector[type][id] = {'label': label, 'connection': ""};
			return connector;
		},
		
		_build: function() {
			var box = $("<div>").attr('id', this._id).attr('class', 'pl_flow_box');
			var label_box = $("<div>").attr('class', 'pl_fb_label_area');
			label_box.append($("<span>").attr('class', 'pl_fb_label').html(this._label));
			
			var trash = $("<span>").attr('class', 'red square pl_fb_trash');
			trash.click(function () {$.workflow.node_delete(this);});
			label_box.append(trash);
			
			var editable = $("<span>").attr('class', 'green square pl_fb_trash');
			editable.click(function () {$.workflow.node_edit(this);});
			label_box.append(editable);
			
			var inout_box = $("<div>").attr('class', 'pl_fb_inout_area');
			box.append(label_box);
			box.append(inout_box);
			var in_box = $("<div>").attr('class', 'pl_fb_in_area');
			var out_box = $("<div>").attr('class', 'pl_fb_out_area');
			inout_box.append(in_box);
			inout_box.append(out_box);
			switch(this._type) {
				case 'open':
					out_box.append(this._add_connector('con_out1', 'output', 'out'));
					break;
				case 'boolean':
					in_box.append(this._add_connector('con_in1', 'input', 'in'));
					out_box.append(this._add_connector('con_outt', 'output', 'true'));
					out_box.append(this._add_connector('con_outf', 'output', 'false'));					
					break;
				case 'connector':
					box.addClass('gold');
					in_box.append(this._add_connector('con_in1', 'input', 'in'));
					out_box.append(this._add_connector('con_out1', 'output', 'out'));
					break;
//				case 'pageconnector':
//					box.addClass('gold');
//					in_box.append(this._add_connector('con_in1', 'input', 'in'));
//					out_box.append(this._add_connector('con_out1', 'output', 'out'));
//					break;
				case 'end':
					box.addClass('pink');
					in_box.append(this._add_connector('con_in1', 'input', 'in'));
					break;
//				case 'pageview':
//					box.addClass('gold');
//					in_box.append(this._add_connector('con_in1', 'input', 'in'));
//					break;
//				case 'pagecall':
//					box.addClass('pink');
//					in_box.append(this._add_connector('con_in1', 'input', 'in'));
//					break;
				default:					
					out_box.append(this._add_connector('con_out1', 'output', 'out'));
			}
			box.draggable({stop:$.workflow.node_drag}); // connect to draw all lines
			return box;
		}
	});
	
	$.workflow_node = new WorkFlowNode();
	
	/* Drawer */
	function WorkFlowDrawer() {
		this._viewport = null;
		this._svg = null;
	};
	
	$.extend(WorkFlowDrawer.prototype, {
		init: function(viewport, svg) {
			this._viewport = viewport;
			this._svg = svg;
		},
		
		clear: function() {
			this._svg.clear();
		},

		draw_line: function(eTarget, eSource) {
			var $viewport = this._viewport;
			
			// Source
			var $source = eSource;
			var $target = eTarget;

			var originX = $source.offset().left + ($source.width() / 2) - $viewport.offset().left;
			var originY = $source.offset().top + ($source.height() / 2) - $viewport.offset().top;

			var endingX = $target.offset().left  + ($target.width() / 2)- $viewport.offset().left;
			var endingY = $target.offset().top + ($target.height() / 2) - $viewport.offset().top;

			var stroke_width = 2;
			var color = "black";
	
			var g = this._svg.group({stroke: color}); 
			var n_line = this._svg.line(g, originX, originY, endingX, endingY, {stroke: color, strokeWidth: stroke_width});
			
			return n_line;
		},

		clear_line: function(line) {
			try {
				this._svg.remove(line);
			} catch(err) {
				$.workflow_console.set(err);
			}
			
		} 
	});
	
	$.drawer = new WorkFlowDrawer();
	
	/* WorkFlow manager.
	   Use the singleton instance of this class, $.workflow, 
	   to interact with the WorkFlow functionality. */
	function WorkFlowManager() {
		this._settings = []; // Settings to be remembered 
		this._uuid = new Date().getTime();
		
		this._viewport = null;
		this._svg = null;
		
		this._nodes = [];
		this._node_counter = 0;
		
		this.lines = {};
		this.debug = false;
		
		this.url = null;
	}
	
	$.extend(WorkFlowManager.prototype, {
		/* Class name added to elements to indicate already configured with SVG. */
		_markerClassName: 'hasWorkFlow',
		
		_nodeName: 'node',
				
		/* Add the WorkFlow object to its container. */
		_attach: function(container, settings) {
			// Detect if container is workflow object
			if ($(container).hasClass(this._markerClassName)) { return; }
			
			// Add workflow mark
			$(container).addClass(this._markerClassName);
			
			// Settings
			var allowed_settings = ['language', 'data'];
			if (isObject(settings)) {
				for (setting in settings) {
					if ($.inArray(setting, allowed_settings) == -1) {
						throw ($.workflow_i18n.get('errorSettingNotAllowed') + ': "' + setting + '"');
					} else {
						this._settings[this._settings-length + 1] = settings[setting];
					}
				}
			} else {
				throw ($.workflow_i18n.get('errorSettings'));
			}
			
			// Set language
			if(settings['language'] != null) {
				$.workflow_i18n.set(settings['language']);
			}
			//this._workflow_node_types = ['open', 'boolean', 'connector', 'pageconnector', 'pageview', 'pagecall', 'end'];
			this._workflow_node_types = ['open', 'boolean', 'connector', 'end'];
			this._workflow_node_types_label = { 'open': $.workflow_i18n.get('nodeOpen'),
												'boolean': $.workflow_i18n.get('nodeBoolean'),
												'connector': $.workflow_i18n.get('nodeConnector'),
												//'pageconnector': $.workflow_i18n.get('nodePageConnector'),
												//'pageview': $.workflow_i18n.get('nodePageview'),
												//'pagecall': $.workflow_i18n.get('nodePagecall'),
												'end': $.workflow_i18n.get('nodeEnd')
											   };
			
			// Main Loop
			try {
				// Create main widget and attach it to container
				var main_wdg = $("<div>").attr('id', 'pl_workflow_main');
				
				var viewport = $("<div>").attr('id', 'pl_workflow_viewport');
				main_wdg.append(viewport);
				
				// Status bar
				main_wdg.append($.workflow_status.widget());
				main_wdg.append($.workflow_console.widget());
				main_wdg.append(this._new_node_widget());
				main_wdg.append($.workflow_output.widget());
				
				// Menu
				var menu = new WorkflowMenu();
				main_wdg.append(menu.widget());	
				main_wdg.append(menu.widget('delete'));				
				
				// Attach main widget to container
				$(container).html(main_wdg);
				
				// Initialize widget
				this._initialize();
				
				if(settings['data'] != null) {
					if(typeof settings['data'] == 'string') {
						this.url = settings['data'];
						var data = {};
					    $.ajax({type: "GET", 
					    	url: settings['data'], 
					    	success: function(msg) {
								$.workflow._process_workflow(msg);
					    	} 
					    });
					} else {
						var data = JSON.parse(settings['data']);
						$.workflow._process_workflow(data);
					} 
				} else {
					$.workflow_status.set($.workflow_i18n.get('done'));
				}				
			} catch (err) {
				var err_literal = '<p class="pl_workflow_error"><span class="pl_small">' + $.workflow_i18n.get('errorLoading') + '</span> ' + err + '</p>'
				$(container).html(err_literal);
			}
		},
		
		_process_workflow: function(data) {
			// Draw nodes
			for (node in data) {
				this.node_append(node, data[node]);
				this._nodes[node].set_connections(data[node]['connector']);
			}
			// Draw connections
			for (node in data) {
				var node_w = this._nodes[node];
				
				var connects = node_w.get_connections();			
				this._draw_lines(connects);
			}
			this._output();
			$.workflow_status.set($.workflow_i18n.get('workflowLoaded'));
		},
		
		_new_node_widget: function () {
			// Dialog
			var new_dialog = $("<div>").attr('id', 'pl_workflow_new_node_dialog');
			new_dialog.attr('title', $.workflow_i18n.get('console'));
			
			var new_form = $("<form>");
			new_form.append($("<label>").attr('for', 'pl_element').html($.workflow_i18n.get('label')));
			new_form.append($("<input>").attr('id', 'pl_element').attr('type', 'text'));
			
			new_form.append($("<br>"));
			
			new_form.append($("<label>").attr('for', 'pl_type').html($.workflow_i18n.get('type')));
			var selector = $("<select>").attr('id', 'pl_type');
			
			for(var i=0; i < this._workflow_node_types.length; i++) {
				var type = this._workflow_node_types[i];
				var label = this._workflow_node_types_label[type];
				selector.append($("<option>").attr('value', type).html(label));
			}
			
			new_form.append(selector);
			
			new_form.append($("<label>").attr('for', 'pl_params').html($.workflow_i18n.get('params')));
			new_form.append($("<textarea>").attr('id', 'pl_params').attr('class', 'pl_small'));
			
			new_dialog.append(new_form);
			
			return new_dialog;
		},

		/* Post-processing once loaded. */
		_initialize: function() {
			$('#pl_workflow_action_console').click(function () {
				$('#pl_workflow_console').toggle();
			});
			
			// Viewport and SVG
			this._viewport = $('#pl_workflow_viewport')
			this._viewport.svg();
			this._svg = this._viewport.svg('get');
			
			// Drawer
			$.drawer.init(this._viewport, this._svg);
			
			/* Menu */
			this._viewport.contextMenu({menu: 'pl_workflow_menu'},
				function(action, el, pos) {
					switch(action) {
						case 'add':
							//$.workflow.set_pos(pos.y, pos.x);
							$.workflow.set_pos(pos.docY, pos.docX);
							$('#pl_workflow_new_node_dialog').dialog("open");
							break;
						case 'save':
							$.workflow._send();
							break;
						case 'console':
							$('#pl_workflow_console').toggle();
							break;
						case 'output':
							$('#pl_output_dialog').dialog("open");
							break;
						default:
							alert(
								'Action: ' + action + '\n\n' +
								'Element ID: ' + $(el).attr('id') + '\n\n' + 
								'X: ' + pos.x + '  Y: ' + pos.y + ' (relative to element)\n\n' + 
								'X: ' + pos.docX + '  Y: ' + pos.docY+ ' (relative to document)'
							);
						}
			});
			
			// Droppable
			this._viewport.droppable({
						  drop: function( event, ui ) {
							var id = $(this).parents('.pl_flow_box').attr('id');
							event.stopPropagation();
							}
						});
			
			// Dialogs
			// New node
			$('#pl_workflow_new_node_dialog').dialog({
				autoOpen: false,
				modal: true,
				resizable: false,
				buttons: [{text: $.workflow_i18n.get('cancel'),
						  click: function() { $(this).dialog("close"); }},
						  {text: $.workflow_i18n.get('accept'), click: function() {
							var new_elem = $('#pl_element').val();
							var new_type = $('#pl_type').val();
							var new_param = $('#pl_params').attr("value");
							if (new_elem != '') {
								$.workflow.node_add(new_elem, new_type, new_param);
							}
							
							$('#pl_element').val('');
							$("#pl_type option[value="+$.workflow._workflow_node_types[0]+"]").attr("selected",true);
							$('#pl_params').attr("value", '');
							$(this).dialog("close");}}]
			});
			$('#pl_workflow_action_node').click(function () {
				$('#pl_workflow_new_node_dialog').dialog("open");
			});
			
			// Output
			if (this.debug) {
				$('#pl_output_dialog').dialog({
					autoOpen: false,
					modal: false,
					height: 350,
					width: 460,
					resizable: true,
					buttons: [{text: $.workflow_i18n.get('cancel'),
							  click: function() { $(this).dialog("close"); }}]
				});
				$('#pl_workflow_action_output').click(function () {
					$('#pl_output_dialog').dialog("open");
				});
			}
		},
		
		/* Remove the WorkFlow functionality from a div.
		   @param  container  (element) the container for the WorkFlow object */
		_destroy: function(container) {
			var $container = $(container);
			if (!$container.hasClass(this._markerClassName)) {
				return;
			}
			$container.removeClass(this._markerClassName);
			$container.empty();
		},
		
		/* Output widget */
		_output: function(out_data){
			if (this.debug) {
				if (typeof out_data === 'undefined') {
					out_data = {'workflow': {}, 'lines': this.lines};
					for (var node in this._nodes) {
						if (node != null) { 
							var node_data = this._nodes[node];
							out_data['workflow'][node] = {	'label': node_data._label,
															'type': node_data._type,
															'param': node_data._param,
															'pos': node_data._pos,
															'connector': node_data._connector};
						}	
					}
				}
				try {
					$.workflow_output.set('workflow\n'+JSON.stringify(out_data['workflow'], null, '\t')); 
				} catch(err) {
					$.workflow_output.set('ERROR');
				}
			}
		},
		
		_send: function() {
			if ($.workflow.url == null) {
				$.workflow_status.set($.workflow_i18n.get('notAllowed'));
			} else {
				out_data = {'workflow': {}};
				for (var node in this._nodes) {
					if (node != null) { 
						var node_data = this._nodes[node];
						out_data['workflow'][node] = {	'label': node_data._label,
														'type': node_data._type,
														'param': node_data._param,
														'pos': node_data._pos,
														'connector': node_data._connector};
					}	
				}
				var query = JSON.stringify(out_data['workflow']); 
			    $.ajax({type: "POST", 
			    	url: $.workflow.url, 
			    	data: query, 
			    	success: function(msg) { 
			    		$.workflow_status.set(msg);
			    		$.workflow_status.set($.workflow_i18n.get('workflowSaved'));
			    	}
			    });
			}
		},
		
		// Add node
		node_append: function(id, node_data) {			
			this._node_counter++;
			
			var node = new WorkFlowNode(id, node_data['label'], node_data['type'], node_data['param']);
			
			// Draw node
			var viewport = $('#pl_workflow_viewport');
			var w_node = node.widget();
			viewport.append(w_node);
			var pos = node_data['pos'];
			w_node.offset({ 'top': pos['top'], 'left': pos['left']})
			// Add node
			node.set_pos(pos['top'], pos['left']);
			this._nodes[id] = node;
		},
		
		// Public api
		node_add: function(label, type, param) {
			if ($.workflow_node._id != null) {
				$.workflow_node._label = label;
				$.workflow_node._type = type;
				$.workflow_node._param = param;
				$.workflow._nodes[$.workflow_node._id] = $.workflow_node;
				
				// Delete
				$.workflow._delete_lines($.workflow_node._id);				
				$('#' + $.workflow_node._id).remove()
				// Redraw
				var viewport = $('#pl_workflow_viewport');
				$.workflow_node._widget = null;
				var w_node = $.workflow_node.widget();
				viewport.append(w_node);
				var pos = $.workflow_node._pos;
				w_node.offset({ 'top': pos['top'], 'left': pos['left']})
				// Lines
				var connects = $.workflow_node.get_connections();			
				$.workflow._draw_lines(connects);
				
				// Restore tmp
				$.workflow_node = new WorkFlowNode();
			} else {
				this._node_counter++;
				var id = this._nodeName + '_' + this._node_counter;
				var node = new WorkFlowNode(id, label, type, param);
					
				// Draw node
				var viewport = $('#pl_workflow_viewport');
				var w_node = node.widget();
				viewport.append(w_node);
				
				// positioning node
				if (typeof this._tmp_pos === 'undefined') {
					var top = 40  + viewport.offset().top + 10;
					var left = 20 + viewport.offset().left;
				} else {
					var top = this._tmp_pos['top'];
					var left = this._tmp_pos['left'];				
				}
				
				w_node.offset({ 'top': top, 'left': left })
				
				// Add node
				node.set_pos(top, left);
				this._nodes[id] = node;
			}
				
			this._output();
		},
		
		node_delete: function(elem) {
			var id = $(elem).parents('.pl_flow_box').attr('id');
			
			try {
				$.workflow._delete_lines(id);
				
				$('#' + id).remove()
				delete $.workflow._nodes[id];
			} catch(err) {
				$.workflow_console.set($.workflow_i18n.get('infoDeleteNode') + id + ' :"' + err + '"');
			}
			
			this._output();
		},
		
		node_edit: function(elem) {			
			var id = $(elem).parents('.pl_flow_box').attr('id');
			$.workflow_node = $.workflow._nodes[id]
			
			$('#pl_element').val($.workflow_node._label);
			$("#pl_type option[value="+$.workflow_node._type+"]").attr("selected",true);
			$('#pl_params').attr("value", $.workflow_node._param);
			
			$('#pl_workflow_new_node_dialog').dialog("open");
			
		},
		
		node_drag: function() {
			// NOTE: use $.workflow instead this
			var id = $(this).attr('id');
			var pos =  $(this).offset();
			
			$.workflow._delete_lines(id);
			
			node = $.workflow._nodes[id];
			node.set_pos(pos.top, pos.left);
			
			var connects = node.get_connections();			
			$.workflow._draw_lines(connects);
			
			$.workflow._output($.workflow.lines);
		},
		
		node_connection: function(target_id , target_conn_id , drag_id, drag_conn_id) {
			// NOTE: use $.workflow instead this
			
			// Node target must not be origin node
			if (target_id == drag_id) {
				$.workflow_status.set($.workflow_i18n.get('errorSameNode'));
			} else {
				var t_connect = false;
				var node_t = $.workflow._nodes[target_id];
				if (node_t.connector_available('output', target_conn_id) == true) {
					t_connect = true;
				} else {
					$.workflow_status.set($.workflow_i18n.get('errorNodeConnectorNotAvailable'));
				}
				
				var o_connect = false;
				var node_o = $.workflow._nodes[drag_id];
				if (node_o.connector_available('input', drag_conn_id) == true) {
					o_connect = true;
				} else {
					$.workflow_status.set($.workflow_i18n.get('errorNodeConnectorNotAvailable'));
				}
				
				if ((t_connect == true) & (o_connect == true)) {
					// Draw Line
					var line = $.workflow.draw_node_line(target_id , target_conn_id , drag_id, drag_conn_id);
					
					node_t.connect('output', target_conn_id, drag_id + '.' + drag_conn_id);
					node_o.connect('input', drag_conn_id, target_id + '.' + target_conn_id);
					$.workflow._output();
				}
				
			}
		},
		node_disconnection: function(target_id , target_conn_id) {
			// Disconnect
			var node_t = $.workflow._nodes[target_id];
			
			if (node_t.connector_available('output', target_conn_id) == false) {
				// Get origin node
				var origin = node_t.get_connection('output', target_conn_id).split('.');
				var node_o = $.workflow._nodes[origin[0]];
				
				// Delete line
				var line_source = origin[0] + '.' + origin[1] + ':' + target_id + '.' + target_conn_id;
				var line_target = target_id + '.' + target_conn_id + ':' + origin[0] + '.' + origin[1];
				
				$.drawer.clear_line($.workflow.lines[line_target]);
				delete $.workflow.lines[line_source];
				delete $.workflow.lines[line_target];
				
				// Nodes disconnection
				node_t.disconnect('output', target_conn_id);
				node_o.disconnect('input', origin[1]);
				
				$.workflow._output();
			}
		},
		
		draw_node_line: function (target_id , target_conn_id , drag_id, drag_conn_id) {
			// NOTE: use $.workflow instead this	
			// Source
			var source_p = $("#" + drag_id)
			var source = source_p.find("#" + drag_conn_id + '_l')
			
			// Target
			var target_p = $("#" + target_id)
			var target = target_p.find("#" + target_conn_id + '_l')
			
			// Prepare identifiers
			var line_source = drag_id + '.' + drag_conn_id + ':' + target_id + '.' + target_conn_id;
			var line_target = target_id + '.' + target_conn_id + ':' + drag_id + '.' + drag_conn_id;
			// Check if line exists
			var line_exists = false;
			for (item in $.workflow.lines) {
				if (item == line_source) {
					line_exists = true;
				}
				if (item == line_target) {
					line_exists = true;
				}
			}
			if (line_exists == false) {
				var line = $.drawer.draw_line(target, source);
				
				// Assign line
				$.workflow.lines[line_target] = line;
				$.workflow.lines[line_source] = line;
			} else {
				var line = $.workflow.lines[line_target];
			}
			return line;
		},
		
		set_pos: function(top, left) {
			var position = $('#pl_workflow_viewport').position();
			var offset = $('#pl_workflow_viewport').offset();
			var x = left + offset.left - position.left;
			var y = top + offset.top - position.top;
			if (x < 0) { x = 0;}
			if (y < 0) { y = 0;}
			this._tmp_pos = {'top': y, 'left': x};
		},
		
		get_pos: function() {
			return this._tmp_pos;
		},
		
		_delete_lines: function(id) {		
			// NOTE: use $.workflow instead this
			for (item in $.workflow.lines) {
				var delete_me = false;
				var data = item.split(':');
				var item_1 = data[0].split('.');
				var item_2 = data[1].split('.');
				if (item_1[0] == id) {
					delete_me = true;
				}
				if (item_2[0] == id) {
					delete_me = true;
				}
				if (delete_me) {
					$.drawer.clear_line($.workflow.lines[item]);
					delete $.workflow.lines[item];
				}
			}
		},
		
		_draw_lines: function(connections) {
			// NOTE: use $.workflow instead this
			for(var i=0; i < connections.length; i++) {
				var conn = connections[i];
				if (conn != null) {
					$.workflow.draw_node_line(conn['tid'], conn['tcid'], conn['sid'], conn['scid']);
				}				
			}
		}
	});
	
	/* Attach the WorkFlow functionality to a jQuery selection.
	   @param  command  (string) the command to run (optional, default 'attach')
	   @param  options  (object) the new settings to use for these SVG instances
	   @return jQuery (object) for chaining further calls */
	$.fn.workflow = function(options) {
		var otherArgs = Array.prototype.slice.call(arguments, 1);
		return this.each(function() {
			if (typeof options == 'string') {
				$.workflow['_' + options].apply($.workflow, [this].concat(otherArgs));
			}
			else {
				try {
					$.workflow._attach(this, options || {});
				} catch (err) {
					$(this).html(err);
				}
				
			} 
		});
	};

	// Singleton primary WorkFlow interface
	$.workflow = new WorkFlowManager();
})( jQuery );
